/****************************************************************************
** Meta object code from reading C++ file 'shortcutpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/shortcutpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shortcutpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ShortcutViewer_t {
    QByteArrayData data[8];
    char stringdata0[91];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ShortcutViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ShortcutViewer_t qt_meta_stringdata_ShortcutViewer = {
    {
QT_MOC_LITERAL(0, 0, 14), // "ShortcutViewer"
QT_MOC_LITERAL(1, 15, 15), // "shortcutChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 9), // "setAction"
QT_MOC_LITERAL(4, 42, 8), // "QAction*"
QT_MOC_LITERAL(5, 51, 6), // "action"
QT_MOC_LITERAL(6, 58, 14), // "removeShortcut"
QT_MOC_LITERAL(7, 73, 17) // "onEditingFinished"

    },
    "ShortcutViewer\0shortcutChanged\0\0"
    "setAction\0QAction*\0action\0removeShortcut\0"
    "onEditingFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ShortcutViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   35,    2, 0x0a /* Public */,
       6,    0,   38,    2, 0x0a /* Public */,
       7,    0,   39,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ShortcutViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ShortcutViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shortcutChanged(); break;
        case 1: _t->setAction((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 2: _t->removeShortcut(); break;
        case 3: _t->onEditingFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ShortcutViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ShortcutViewer::shortcutChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ShortcutViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QKeySequenceEdit::staticMetaObject>(),
    qt_meta_stringdata_ShortcutViewer.data,
    qt_meta_data_ShortcutViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ShortcutViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShortcutViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ShortcutViewer.stringdata0))
        return static_cast<void*>(this);
    return QKeySequenceEdit::qt_metacast(_clname);
}

int ShortcutViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QKeySequenceEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void ShortcutViewer::shortcutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_ShortcutTree_t {
    QByteArrayData data[15];
    char stringdata0[178];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ShortcutTree_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ShortcutTree_t qt_meta_stringdata_ShortcutTree = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ShortcutTree"
QT_MOC_LITERAL(1, 13, 14), // "actionSelected"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 8), // "QAction*"
QT_MOC_LITERAL(4, 38, 8), // "searched"
QT_MOC_LITERAL(5, 47, 10), // "haveResult"
QT_MOC_LITERAL(6, 58, 20), // "onCurrentItemChanged"
QT_MOC_LITERAL(7, 79, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(8, 96, 7), // "current"
QT_MOC_LITERAL(9, 104, 8), // "previous"
QT_MOC_LITERAL(10, 113, 17), // "onShortcutChanged"
QT_MOC_LITERAL(11, 131, 13), // "onItemClicked"
QT_MOC_LITERAL(12, 145, 11), // "QModelIndex"
QT_MOC_LITERAL(13, 157, 15), // "onItemCollapsed"
QT_MOC_LITERAL(14, 173, 4) // "item"

    },
    "ShortcutTree\0actionSelected\0\0QAction*\0"
    "searched\0haveResult\0onCurrentItemChanged\0"
    "QTreeWidgetItem*\0current\0previous\0"
    "onShortcutChanged\0onItemClicked\0"
    "QModelIndex\0onItemCollapsed\0item"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ShortcutTree[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       4,    1,   47,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    2,   50,    2, 0x0a /* Public */,
      10,    0,   55,    2, 0x0a /* Public */,
      11,    1,   56,    2, 0x0a /* Public */,
      13,    1,   59,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Bool,    5,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 7,    8,    9,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,    2,
    QMetaType::Void, 0x80000000 | 7,   14,

       0        // eod
};

void ShortcutTree::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ShortcutTree *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->actionSelected((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 1: _t->searched((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->onCurrentItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 3: _t->onShortcutChanged(); break;
        case 4: _t->onItemClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 5: _t->onItemCollapsed((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ShortcutTree::*)(QAction * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ShortcutTree::actionSelected)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ShortcutTree::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ShortcutTree::searched)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ShortcutTree::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeWidget::staticMetaObject>(),
    qt_meta_stringdata_ShortcutTree.data,
    qt_meta_data_ShortcutTree,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ShortcutTree::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShortcutTree::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ShortcutTree.stringdata0))
        return static_cast<void*>(this);
    return QTreeWidget::qt_metacast(_clname);
}

int ShortcutTree::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void ShortcutTree::actionSelected(QAction * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ShortcutTree::searched(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_ShortcutPopup_t {
    QByteArrayData data[13];
    char stringdata0[151];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ShortcutPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ShortcutPopup_t qt_meta_stringdata_ShortcutPopup = {
    {
QT_MOC_LITERAL(0, 0, 13), // "ShortcutPopup"
QT_MOC_LITERAL(1, 14, 17), // "clearAllShortcuts"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 7), // "warning"
QT_MOC_LITERAL(4, 41, 19), // "onSearchTextChanged"
QT_MOC_LITERAL(5, 61, 4), // "text"
QT_MOC_LITERAL(6, 66, 15), // "onPresetChanged"
QT_MOC_LITERAL(7, 82, 14), // "onExportButton"
QT_MOC_LITERAL(8, 97, 9), // "TFilePath"
QT_MOC_LITERAL(9, 107, 2), // "fp"
QT_MOC_LITERAL(10, 110, 14), // "onDeletePreset"
QT_MOC_LITERAL(11, 125, 12), // "onSavePreset"
QT_MOC_LITERAL(12, 138, 12) // "onLoadPreset"

    },
    "ShortcutPopup\0clearAllShortcuts\0\0"
    "warning\0onSearchTextChanged\0text\0"
    "onPresetChanged\0onExportButton\0TFilePath\0"
    "fp\0onDeletePreset\0onSavePreset\0"
    "onLoadPreset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ShortcutPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x09 /* Protected */,
       1,    0,   62,    2, 0x29 /* Protected | MethodCloned */,
       4,    1,   63,    2, 0x09 /* Protected */,
       6,    0,   66,    2, 0x09 /* Protected */,
       7,    1,   67,    2, 0x09 /* Protected */,
       7,    0,   70,    2, 0x29 /* Protected | MethodCloned */,
      10,    0,   71,    2, 0x09 /* Protected */,
      11,    0,   72,    2, 0x09 /* Protected */,
      12,    0,   73,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ShortcutPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ShortcutPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clearAllShortcuts((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->clearAllShortcuts(); break;
        case 2: _t->onSearchTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onPresetChanged(); break;
        case 4: _t->onExportButton((*reinterpret_cast< TFilePath(*)>(_a[1]))); break;
        case 5: _t->onExportButton(); break;
        case 6: _t->onDeletePreset(); break;
        case 7: _t->onSavePreset(); break;
        case 8: _t->onLoadPreset(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ShortcutPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_ShortcutPopup.data,
    qt_meta_data_ShortcutPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ShortcutPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShortcutPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ShortcutPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int ShortcutPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
