/****************************************************************************
** Meta object code from reading C++ file 'tscenehandle.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/tscenehandle.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tscenehandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TSceneHandle_t {
    QByteArrayData data[18];
    char stringdata0[230];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TSceneHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TSceneHandle_t qt_meta_stringdata_TSceneHandle = {
    {
QT_MOC_LITERAL(0, 0, 12), // "TSceneHandle"
QT_MOC_LITERAL(1, 13, 13), // "sceneSwitched"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 12), // "sceneChanged"
QT_MOC_LITERAL(4, 41, 11), // "castChanged"
QT_MOC_LITERAL(5, 53, 15), // "castFolderAdded"
QT_MOC_LITERAL(6, 69, 9), // "TFilePath"
QT_MOC_LITERAL(7, 79, 4), // "path"
QT_MOC_LITERAL(8, 84, 16), // "nameSceneChanged"
QT_MOC_LITERAL(9, 101, 17), // "preferenceChanged"
QT_MOC_LITERAL(10, 119, 8), // "prefName"
QT_MOC_LITERAL(11, 128, 17), // "pixelUnitSelected"
QT_MOC_LITERAL(12, 146, 2), // "on"
QT_MOC_LITERAL(13, 149, 19), // "importPolicyChanged"
QT_MOC_LITERAL(14, 169, 6), // "policy"
QT_MOC_LITERAL(15, 176, 19), // "renamePolicyChanged"
QT_MOC_LITERAL(16, 196, 20), // "convertPolicyChanged"
QT_MOC_LITERAL(17, 217, 12) // "setDirtyFlag"

    },
    "TSceneHandle\0sceneSwitched\0\0sceneChanged\0"
    "castChanged\0castFolderAdded\0TFilePath\0"
    "path\0nameSceneChanged\0preferenceChanged\0"
    "prefName\0pixelUnitSelected\0on\0"
    "importPolicyChanged\0policy\0"
    "renamePolicyChanged\0convertPolicyChanged\0"
    "setDirtyFlag"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TSceneHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    0,   70,    2, 0x06 /* Public */,
       4,    0,   71,    2, 0x06 /* Public */,
       5,    1,   72,    2, 0x06 /* Public */,
       8,    0,   75,    2, 0x06 /* Public */,
       9,    1,   76,    2, 0x06 /* Public */,
      11,    1,   79,    2, 0x06 /* Public */,
      13,    1,   82,    2, 0x06 /* Public */,
      15,    1,   85,    2, 0x06 /* Public */,
      16,    1,   88,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      17,    0,   91,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::Int,   14,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void TSceneHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TSceneHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sceneSwitched(); break;
        case 1: _t->sceneChanged(); break;
        case 2: _t->castChanged(); break;
        case 3: _t->castFolderAdded((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 4: _t->nameSceneChanged(); break;
        case 5: _t->preferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->pixelUnitSelected((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->importPolicyChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->renamePolicyChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->convertPolicyChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->setDirtyFlag(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TSceneHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::sceneSwitched)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::sceneChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::castChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)(const TFilePath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::castFolderAdded)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::nameSceneChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::preferenceChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::pixelUnitSelected)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::importPolicyChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::renamePolicyChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (TSceneHandle::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TSceneHandle::convertPolicyChanged)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TSceneHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TSceneHandle.data,
    qt_meta_data_TSceneHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TSceneHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TSceneHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TSceneHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TSceneHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void TSceneHandle::sceneSwitched()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TSceneHandle::sceneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TSceneHandle::castChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TSceneHandle::castFolderAdded(const TFilePath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TSceneHandle::nameSceneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TSceneHandle::preferenceChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void TSceneHandle::pixelUnitSelected(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void TSceneHandle::importPolicyChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void TSceneHandle::renamePolicyChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void TSceneHandle::convertPolicyChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
