/****************************************************************************
** Meta object code from reading C++ file 'pltgizmopopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/pltgizmopopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pltgizmopopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ValueAdjuster_t {
    QByteArrayData data[6];
    char stringdata0[58];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ValueAdjuster_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ValueAdjuster_t qt_meta_stringdata_ValueAdjuster = {
    {
QT_MOC_LITERAL(0, 0, 13), // "ValueAdjuster"
QT_MOC_LITERAL(1, 14, 6), // "adjust"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 6), // "factor"
QT_MOC_LITERAL(4, 29, 13), // "onClickedPlus"
QT_MOC_LITERAL(5, 43, 14) // "onClickedMinus"

    },
    "ValueAdjuster\0adjust\0\0factor\0onClickedPlus\0"
    "onClickedMinus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ValueAdjuster[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   32,    2, 0x09 /* Protected */,
       5,    0,   33,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Double,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ValueAdjuster::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ValueAdjuster *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->adjust((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->onClickedPlus(); break;
        case 2: _t->onClickedMinus(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ValueAdjuster::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ValueAdjuster::adjust)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ValueAdjuster::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ValueAdjuster.data,
    qt_meta_data_ValueAdjuster,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ValueAdjuster::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ValueAdjuster::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ValueAdjuster.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ValueAdjuster::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void ValueAdjuster::adjust(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_ValueShifter_t {
    QByteArrayData data[6];
    char stringdata0[57];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ValueShifter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ValueShifter_t qt_meta_stringdata_ValueShifter = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ValueShifter"
QT_MOC_LITERAL(1, 13, 6), // "adjust"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 6), // "factor"
QT_MOC_LITERAL(4, 28, 13), // "onClickedPlus"
QT_MOC_LITERAL(5, 42, 14) // "onClickedMinus"

    },
    "ValueShifter\0adjust\0\0factor\0onClickedPlus\0"
    "onClickedMinus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ValueShifter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   32,    2, 0x09 /* Protected */,
       5,    0,   33,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Double,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ValueShifter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ValueShifter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->adjust((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->onClickedPlus(); break;
        case 2: _t->onClickedMinus(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ValueShifter::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ValueShifter::adjust)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ValueShifter::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ValueShifter.data,
    qt_meta_data_ValueShifter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ValueShifter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ValueShifter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ValueShifter.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ValueShifter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void ValueShifter::adjust(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_ColorFader_t {
    QByteArrayData data[5];
    char stringdata0[42];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColorFader_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColorFader_t qt_meta_stringdata_ColorFader = {
    {
QT_MOC_LITERAL(0, 0, 10), // "ColorFader"
QT_MOC_LITERAL(1, 11, 12), // "valueChanged"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 6), // "factor"
QT_MOC_LITERAL(4, 32, 9) // "onClicked"

    },
    "ColorFader\0valueChanged\0\0factor\0"
    "onClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColorFader[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   27,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Double,    3,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void ColorFader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorFader *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->onClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorFader::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorFader::valueChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ColorFader::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ColorFader.data,
    qt_meta_data_ColorFader,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColorFader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColorFader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColorFader.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ColorFader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void ColorFader::valueChanged(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_PltGizmoPopup_t {
    QByteArrayData data[15];
    char stringdata0[112];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PltGizmoPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PltGizmoPopup_t qt_meta_stringdata_PltGizmoPopup = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PltGizmoPopup"
QT_MOC_LITERAL(1, 14, 7), // "adjustV"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 1), // "p"
QT_MOC_LITERAL(4, 25, 7), // "adjustS"
QT_MOC_LITERAL(5, 33, 7), // "adjustH"
QT_MOC_LITERAL(6, 41, 7), // "adjustT"
QT_MOC_LITERAL(7, 49, 6), // "shiftV"
QT_MOC_LITERAL(8, 56, 6), // "shiftS"
QT_MOC_LITERAL(9, 63, 6), // "shiftH"
QT_MOC_LITERAL(10, 70, 6), // "shiftT"
QT_MOC_LITERAL(11, 77, 9), // "zeroMatte"
QT_MOC_LITERAL(12, 87, 9), // "fullMatte"
QT_MOC_LITERAL(13, 97, 7), // "onBlend"
QT_MOC_LITERAL(14, 105, 6) // "onFade"

    },
    "PltGizmoPopup\0adjustV\0\0p\0adjustS\0"
    "adjustH\0adjustT\0shiftV\0shiftS\0shiftH\0"
    "shiftT\0zeroMatte\0fullMatte\0onBlend\0"
    "onFade"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PltGizmoPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x0a /* Public */,
       4,    1,   77,    2, 0x0a /* Public */,
       5,    1,   80,    2, 0x0a /* Public */,
       6,    1,   83,    2, 0x0a /* Public */,
       7,    1,   86,    2, 0x0a /* Public */,
       8,    1,   89,    2, 0x0a /* Public */,
       9,    1,   92,    2, 0x0a /* Public */,
      10,    1,   95,    2, 0x0a /* Public */,
      11,    0,   98,    2, 0x0a /* Public */,
      12,    0,   99,    2, 0x0a /* Public */,
      13,    0,  100,    2, 0x0a /* Public */,
      14,    1,  101,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    3,

       0        // eod
};

void PltGizmoPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PltGizmoPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->adjustV((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->adjustS((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 2: _t->adjustH((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 3: _t->adjustT((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 4: _t->shiftV((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 5: _t->shiftS((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 6: _t->shiftH((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 7: _t->shiftT((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 8: _t->zeroMatte(); break;
        case 9: _t->fullMatte(); break;
        case 10: _t->onBlend(); break;
        case 11: _t->onFade((*reinterpret_cast< double(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PltGizmoPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_PltGizmoPopup.data,
    qt_meta_data_PltGizmoPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PltGizmoPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PltGizmoPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PltGizmoPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int PltGizmoPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
