/****************************************************************************
** Meta object code from reading C++ file 'colormodelviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/colormodelviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'colormodelviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ColorModelViewer_t {
    QByteArrayData data[10];
    char stringdata0[154];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColorModelViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColorModelViewer_t qt_meta_stringdata_ColorModelViewer = {
    {
QT_MOC_LITERAL(0, 0, 16), // "ColorModelViewer"
QT_MOC_LITERAL(1, 17, 16), // "refImageNotFound"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 16), // "showCurrentImage"
QT_MOC_LITERAL(4, 52, 16), // "loadCurrentFrame"
QT_MOC_LITERAL(5, 69, 16), // "removeColorModel"
QT_MOC_LITERAL(6, 86, 18), // "onRefImageNotFound"
QT_MOC_LITERAL(7, 105, 12), // "updateViewer"
QT_MOC_LITERAL(8, 118, 14), // "changePickType"
QT_MOC_LITERAL(9, 133, 20) // "repickFromColorModel"

    },
    "ColorModelViewer\0refImageNotFound\0\0"
    "showCurrentImage\0loadCurrentFrame\0"
    "removeColorModel\0onRefImageNotFound\0"
    "updateViewer\0changePickType\0"
    "repickFromColorModel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColorModelViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   55,    2, 0x09 /* Protected */,
       4,    0,   56,    2, 0x09 /* Protected */,
       5,    0,   57,    2, 0x09 /* Protected */,
       6,    0,   58,    2, 0x09 /* Protected */,
       7,    0,   59,    2, 0x09 /* Protected */,
       8,    0,   60,    2, 0x09 /* Protected */,
       9,    0,   61,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ColorModelViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorModelViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refImageNotFound(); break;
        case 1: _t->showCurrentImage(); break;
        case 2: _t->loadCurrentFrame(); break;
        case 3: _t->removeColorModel(); break;
        case 4: _t->onRefImageNotFound(); break;
        case 5: _t->updateViewer(); break;
        case 6: _t->changePickType(); break;
        case 7: _t->repickFromColorModel(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorModelViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorModelViewer::refImageNotFound)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ColorModelViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<FlipBook::staticMetaObject>(),
    qt_meta_stringdata_ColorModelViewer.data,
    qt_meta_data_ColorModelViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColorModelViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColorModelViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColorModelViewer.stringdata0))
        return static_cast<void*>(this);
    return FlipBook::qt_metacast(_clname);
}

int ColorModelViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FlipBook::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ColorModelViewer::refImageNotFound()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
