/****************************************************************************
** Meta object code from reading C++ file 'functionpanel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/functionpanel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'functionpanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FunctionPanel_t {
    QByteArrayData data[13];
    char stringdata0[158];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionPanel_t qt_meta_stringdata_FunctionPanel = {
    {
QT_MOC_LITERAL(0, 0, 13), // "FunctionPanel"
QT_MOC_LITERAL(1, 14, 16), // "keyframeSelected"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 5), // "frame"
QT_MOC_LITERAL(4, 38, 15), // "onFrameSwitched"
QT_MOC_LITERAL(5, 54, 7), // "BGColor"
QT_MOC_LITERAL(6, 62, 14), // "ValueLineColor"
QT_MOC_LITERAL(7, 77, 14), // "FrameLineColor"
QT_MOC_LITERAL(8, 92, 16), // "OtherCurvesColor"
QT_MOC_LITERAL(9, 109, 15), // "RulerBackground"
QT_MOC_LITERAL(10, 125, 9), // "TextColor"
QT_MOC_LITERAL(11, 135, 8), // "SubColor"
QT_MOC_LITERAL(12, 144, 13) // "SelectedColor"

    },
    "FunctionPanel\0keyframeSelected\0\0frame\0"
    "onFrameSwitched\0BGColor\0ValueLineColor\0"
    "FrameLineColor\0OtherCurvesColor\0"
    "RulerBackground\0TextColor\0SubColor\0"
    "SelectedColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       8,   28, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   27,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Double,    3,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
       5, QMetaType::QColor, 0x00095103,
       6, QMetaType::QColor, 0x00095103,
       7, QMetaType::QColor, 0x00095103,
       8, QMetaType::QColor, 0x00095103,
       9, QMetaType::QColor, 0x00095103,
      10, QMetaType::QColor, 0x00095103,
      11, QMetaType::QColor, 0x00095103,
      12, QMetaType::QColor, 0x00095103,

       0        // eod
};

void FunctionPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->keyframeSelected((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->onFrameSwitched(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FunctionPanel::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionPanel::keyframeSelected)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<FunctionPanel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getBGColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getValueLineColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getFrameLineColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getOtherCurvesColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getRulerBackground(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getSubColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->getSelectedColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<FunctionPanel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setValueLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setFrameLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setOtherCurvesColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setRulerBackground(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setSubColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setSelectedColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject FunctionPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_FunctionPanel.data,
    qt_meta_data_FunctionPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionPanel.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int FunctionPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void FunctionPanel::keyframeSelected(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
