/****************************************************************************
** Meta object code from reading C++ file 'previewfxmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/previewfxmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'previewfxmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PreviewFxManager_t {
    QByteArrayData data[17];
    char stringdata0[225];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreviewFxManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreviewFxManager_t qt_meta_stringdata_PreviewFxManager = {
    {
QT_MOC_LITERAL(0, 0, 16), // "PreviewFxManager"
QT_MOC_LITERAL(1, 17, 12), // "startedFrame"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 4), // "fxId"
QT_MOC_LITERAL(4, 36, 23), // "TRenderPort::RenderData"
QT_MOC_LITERAL(5, 60, 10), // "renderData"
QT_MOC_LITERAL(6, 71, 13), // "renderedFrame"
QT_MOC_LITERAL(7, 85, 16), // "refreshViewRects"
QT_MOC_LITERAL(8, 102, 14), // "onLevelChanged"
QT_MOC_LITERAL(9, 117, 11), // "onFxChanged"
QT_MOC_LITERAL(10, 129, 15), // "onXsheetChanged"
QT_MOC_LITERAL(11, 145, 15), // "onObjectChanged"
QT_MOC_LITERAL(12, 161, 10), // "isDragging"
QT_MOC_LITERAL(13, 172, 14), // "onStartedFrame"
QT_MOC_LITERAL(14, 187, 15), // "onRenderedFrame"
QT_MOC_LITERAL(15, 203, 18), // "onRefreshViewRects"
QT_MOC_LITERAL(16, 222, 2) // "id"

    },
    "PreviewFxManager\0startedFrame\0\0fxId\0"
    "TRenderPort::RenderData\0renderData\0"
    "renderedFrame\0refreshViewRects\0"
    "onLevelChanged\0onFxChanged\0onXsheetChanged\0"
    "onObjectChanged\0isDragging\0onStartedFrame\0"
    "onRenderedFrame\0onRefreshViewRects\0"
    "id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreviewFxManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   64,    2, 0x06 /* Public */,
       6,    2,   69,    2, 0x06 /* Public */,
       7,    1,   74,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,   77,    2, 0x09 /* Protected */,
       9,    0,   78,    2, 0x09 /* Protected */,
      10,    0,   79,    2, 0x09 /* Protected */,
      11,    1,   80,    2, 0x09 /* Protected */,
      13,    2,   83,    2, 0x09 /* Protected */,
      14,    2,   88,    2, 0x09 /* Protected */,
      15,    1,   93,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::ULong, 0x80000000 | 4,    3,    5,
    QMetaType::Void, QMetaType::ULong, 0x80000000 | 4,    3,    5,
    QMetaType::Void, QMetaType::ULong,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::ULong, 0x80000000 | 4,    3,    5,
    QMetaType::Void, QMetaType::ULong, 0x80000000 | 4,    3,    5,
    QMetaType::Void, QMetaType::ULong,   16,

       0        // eod
};

void PreviewFxManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PreviewFxManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startedFrame((*reinterpret_cast< ulong(*)>(_a[1])),(*reinterpret_cast< TRenderPort::RenderData(*)>(_a[2]))); break;
        case 1: _t->renderedFrame((*reinterpret_cast< ulong(*)>(_a[1])),(*reinterpret_cast< TRenderPort::RenderData(*)>(_a[2]))); break;
        case 2: _t->refreshViewRects((*reinterpret_cast< ulong(*)>(_a[1]))); break;
        case 3: _t->onLevelChanged(); break;
        case 4: _t->onFxChanged(); break;
        case 5: _t->onXsheetChanged(); break;
        case 6: _t->onObjectChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->onStartedFrame((*reinterpret_cast< ulong(*)>(_a[1])),(*reinterpret_cast< TRenderPort::RenderData(*)>(_a[2]))); break;
        case 8: _t->onRenderedFrame((*reinterpret_cast< ulong(*)>(_a[1])),(*reinterpret_cast< TRenderPort::RenderData(*)>(_a[2]))); break;
        case 9: _t->onRefreshViewRects((*reinterpret_cast< ulong(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PreviewFxManager::*)(unsigned long , TRenderPort::RenderData );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PreviewFxManager::startedFrame)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PreviewFxManager::*)(unsigned long , TRenderPort::RenderData );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PreviewFxManager::renderedFrame)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PreviewFxManager::*)(unsigned long );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PreviewFxManager::refreshViewRects)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PreviewFxManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_PreviewFxManager.data,
    qt_meta_data_PreviewFxManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreviewFxManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreviewFxManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreviewFxManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PreviewFxManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void PreviewFxManager::startedFrame(unsigned long _t1, TRenderPort::RenderData _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PreviewFxManager::renderedFrame(unsigned long _t1, TRenderPort::RenderData _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PreviewFxManager::refreshViewRects(unsigned long _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
