/****************************************************************************
** Meta object code from reading C++ file 'tapp.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/tapp.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tapp.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TApp_t {
    QByteArrayData data[26];
    char stringdata0[420];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TApp_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TApp_t qt_meta_stringdata_TApp = {
    {
QT_MOC_LITERAL(0, 0, 4), // "TApp"
QT_MOC_LITERAL(1, 5, 10), // "tabletLeft"
QT_MOC_LITERAL(2, 16, 0), // ""
QT_MOC_LITERAL(3, 17, 19), // "activeViewerChanged"
QT_MOC_LITERAL(4, 37, 20), // "activeLocatorChanged"
QT_MOC_LITERAL(5, 58, 15), // "onXsheetChanged"
QT_MOC_LITERAL(6, 74, 15), // "onSceneSwitched"
QT_MOC_LITERAL(7, 90, 16), // "onXsheetSwitched"
QT_MOC_LITERAL(8, 107, 20), // "onXsheetSoundChanged"
QT_MOC_LITERAL(9, 128, 15), // "onFrameSwitched"
QT_MOC_LITERAL(10, 144, 12), // "onFxSwitched"
QT_MOC_LITERAL(11, 157, 21), // "onColumnIndexSwitched"
QT_MOC_LITERAL(12, 179, 18), // "onXshLevelSwitched"
QT_MOC_LITERAL(13, 198, 10), // "TXshLevel*"
QT_MOC_LITERAL(14, 209, 17), // "onXshLevelChanged"
QT_MOC_LITERAL(15, 227, 16), // "onObjectSwitched"
QT_MOC_LITERAL(16, 244, 15), // "onSplineChanged"
QT_MOC_LITERAL(17, 260, 14), // "onSceneChanged"
QT_MOC_LITERAL(18, 275, 14), // "onImageChanged"
QT_MOC_LITERAL(19, 290, 16), // "onPaletteChanged"
QT_MOC_LITERAL(20, 307, 24), // "onLevelColorStyleChanged"
QT_MOC_LITERAL(21, 332, 25), // "onLevelColorStyleSwitched"
QT_MOC_LITERAL(22, 358, 8), // "autosave"
QT_MOC_LITERAL(23, 367, 21), // "onToolEditingFinished"
QT_MOC_LITERAL(24, 389, 15), // "onStartAutoSave"
QT_MOC_LITERAL(25, 405, 14) // "onStopAutoSave"

    },
    "TApp\0tabletLeft\0\0activeViewerChanged\0"
    "activeLocatorChanged\0onXsheetChanged\0"
    "onSceneSwitched\0onXsheetSwitched\0"
    "onXsheetSoundChanged\0onFrameSwitched\0"
    "onFxSwitched\0onColumnIndexSwitched\0"
    "onXshLevelSwitched\0TXshLevel*\0"
    "onXshLevelChanged\0onObjectSwitched\0"
    "onSplineChanged\0onSceneChanged\0"
    "onImageChanged\0onPaletteChanged\0"
    "onLevelColorStyleChanged\0"
    "onLevelColorStyleSwitched\0autosave\0"
    "onToolEditingFinished\0onStartAutoSave\0"
    "onStopAutoSave"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TApp[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  129,    2, 0x06 /* Public */,
       3,    0,  130,    2, 0x06 /* Public */,
       4,    0,  131,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  132,    2, 0x09 /* Protected */,
       6,    0,  133,    2, 0x09 /* Protected */,
       7,    0,  134,    2, 0x09 /* Protected */,
       8,    0,  135,    2, 0x09 /* Protected */,
       9,    0,  136,    2, 0x09 /* Protected */,
      10,    0,  137,    2, 0x09 /* Protected */,
      11,    0,  138,    2, 0x09 /* Protected */,
      12,    1,  139,    2, 0x09 /* Protected */,
      14,    0,  142,    2, 0x09 /* Protected */,
      15,    0,  143,    2, 0x09 /* Protected */,
      16,    0,  144,    2, 0x09 /* Protected */,
      17,    0,  145,    2, 0x09 /* Protected */,
      18,    0,  146,    2, 0x09 /* Protected */,
      19,    0,  147,    2, 0x09 /* Protected */,
      20,    0,  148,    2, 0x09 /* Protected */,
      21,    0,  149,    2, 0x09 /* Protected */,
      22,    0,  150,    2, 0x09 /* Protected */,
      23,    0,  151,    2, 0x09 /* Protected */,
      24,    0,  152,    2, 0x09 /* Protected */,
      25,    0,  153,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void TApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TApp *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->tabletLeft(); break;
        case 1: _t->activeViewerChanged(); break;
        case 2: _t->activeLocatorChanged(); break;
        case 3: _t->onXsheetChanged(); break;
        case 4: _t->onSceneSwitched(); break;
        case 5: _t->onXsheetSwitched(); break;
        case 6: _t->onXsheetSoundChanged(); break;
        case 7: _t->onFrameSwitched(); break;
        case 8: _t->onFxSwitched(); break;
        case 9: _t->onColumnIndexSwitched(); break;
        case 10: _t->onXshLevelSwitched((*reinterpret_cast< TXshLevel*(*)>(_a[1]))); break;
        case 11: _t->onXshLevelChanged(); break;
        case 12: _t->onObjectSwitched(); break;
        case 13: _t->onSplineChanged(); break;
        case 14: _t->onSceneChanged(); break;
        case 15: _t->onImageChanged(); break;
        case 16: _t->onPaletteChanged(); break;
        case 17: _t->onLevelColorStyleChanged(); break;
        case 18: _t->onLevelColorStyleSwitched(); break;
        case 19: _t->autosave(); break;
        case 20: _t->onToolEditingFinished(); break;
        case 21: _t->onStartAutoSave(); break;
        case 22: _t->onStopAutoSave(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TXshLevel* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TApp::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TApp::tabletLeft)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TApp::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TApp::activeViewerChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TApp::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TApp::activeLocatorChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TApp::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TApp.data,
    qt_meta_data_TApp,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TApp.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TTool::Application"))
        return static_cast< TTool::Application*>(this);
    return QObject::qt_metacast(_clname);
}

int TApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void TApp::tabletLeft()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TApp::activeViewerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TApp::activeLocatorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
