/****************************************************************************
** Meta object code from reading C++ file 'exportscenepopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/exportscenepopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'exportscenepopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ExportSceneTreeViewDelegate_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportSceneTreeViewDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportSceneTreeViewDelegate_t qt_meta_stringdata_ExportSceneTreeViewDelegate = {
    {
QT_MOC_LITERAL(0, 0, 27) // "ExportSceneTreeViewDelegate"

    },
    "ExportSceneTreeViewDelegate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportSceneTreeViewDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ExportSceneTreeViewDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ExportSceneTreeViewDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QItemDelegate::staticMetaObject>(),
    qt_meta_stringdata_ExportSceneTreeViewDelegate.data,
    qt_meta_data_ExportSceneTreeViewDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportSceneTreeViewDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportSceneTreeViewDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportSceneTreeViewDelegate.stringdata0))
        return static_cast<void*>(this);
    return QItemDelegate::qt_metacast(_clname);
}

int ExportSceneTreeViewDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ExportSceneTreeView_t {
    QByteArrayData data[4];
    char stringdata0[43];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportSceneTreeView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportSceneTreeView_t qt_meta_stringdata_ExportSceneTreeView = {
    {
QT_MOC_LITERAL(0, 0, 19), // "ExportSceneTreeView"
QT_MOC_LITERAL(1, 20, 7), // "focusIn"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 13) // "resizeToConts"

    },
    "ExportSceneTreeView\0focusIn\0\0resizeToConts"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportSceneTreeView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   25,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void ExportSceneTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExportSceneTreeView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->focusIn(); break;
        case 1: _t->resizeToConts(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ExportSceneTreeView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ExportSceneTreeView::focusIn)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ExportSceneTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<StyledTreeView::staticMetaObject>(),
    qt_meta_stringdata_ExportSceneTreeView.data,
    qt_meta_data_ExportSceneTreeView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportSceneTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportSceneTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportSceneTreeView.stringdata0))
        return static_cast<void*>(this);
    return StyledTreeView::qt_metacast(_clname);
}

int ExportSceneTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyledTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void ExportSceneTreeView::focusIn()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_ExportScenePopup_t {
    QByteArrayData data[8];
    char stringdata0[107];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportScenePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportScenePopup_t qt_meta_stringdata_ExportScenePopup = {
    {
QT_MOC_LITERAL(0, 0, 16), // "ExportScenePopup"
QT_MOC_LITERAL(1, 17, 10), // "switchMode"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 2), // "id"
QT_MOC_LITERAL(4, 32, 24), // "onProjectTreeViweFocusIn"
QT_MOC_LITERAL(5, 57, 20), // "onProjectNameFocusIn"
QT_MOC_LITERAL(6, 78, 19), // "onLonelyModeFocusIn"
QT_MOC_LITERAL(7, 98, 8) // "onExport"

    },
    "ExportScenePopup\0switchMode\0\0id\0"
    "onProjectTreeViweFocusIn\0onProjectNameFocusIn\0"
    "onLonelyModeFocusIn\0onExport"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportScenePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x09 /* Protected */,
       4,    0,   42,    2, 0x09 /* Protected */,
       5,    0,   43,    2, 0x09 /* Protected */,
       6,    0,   44,    2, 0x09 /* Protected */,
       7,    0,   45,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ExportScenePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExportScenePopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->switchMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onProjectTreeViweFocusIn(); break;
        case 2: _t->onProjectNameFocusIn(); break;
        case 3: _t->onLonelyModeFocusIn(); break;
        case 4: _t->onExport(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ExportScenePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_ExportScenePopup.data,
    qt_meta_data_ExportScenePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportScenePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportScenePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportScenePopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int ExportScenePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
