/****************************************************************************
** Meta object code from reading C++ file 'vectorizerpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/vectorizerpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'vectorizerpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_VectorizerPopup_t {
    QByteArrayData data[24];
    char stringdata0[288];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VectorizerPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VectorizerPopup_t qt_meta_stringdata_VectorizerPopup = {
    {
QT_MOC_LITERAL(0, 0, 15), // "VectorizerPopup"
QT_MOC_LITERAL(1, 16, 13), // "valuesChanged"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 4), // "onOk"
QT_MOC_LITERAL(4, 36, 12), // "onTypeChange"
QT_MOC_LITERAL(5, 49, 4), // "type"
QT_MOC_LITERAL(6, 54, 11), // "onFrameName"
QT_MOC_LITERAL(7, 66, 9), // "frameName"
QT_MOC_LITERAL(8, 76, 11), // "onFrameDone"
QT_MOC_LITERAL(9, 88, 10), // "frameCount"
QT_MOC_LITERAL(10, 99, 13), // "onPartialDone"
QT_MOC_LITERAL(11, 113, 7), // "partial"
QT_MOC_LITERAL(12, 121, 5), // "total"
QT_MOC_LITERAL(13, 127, 10), // "onFinished"
QT_MOC_LITERAL(14, 138, 19), // "updateSceneSettings"
QT_MOC_LITERAL(15, 158, 12), // "refreshPopup"
QT_MOC_LITERAL(16, 171, 13), // "onValueEdited"
QT_MOC_LITERAL(17, 185, 6), // "isDrag"
QT_MOC_LITERAL(18, 192, 8), // "TPixel32"
QT_MOC_LITERAL(19, 201, 22), // "populateVisibilityMenu"
QT_MOC_LITERAL(20, 224, 17), // "visibilityToggled"
QT_MOC_LITERAL(21, 242, 14), // "saveParameters"
QT_MOC_LITERAL(22, 257, 14), // "loadParameters"
QT_MOC_LITERAL(23, 272, 15) // "resetParameters"

    },
    "VectorizerPopup\0valuesChanged\0\0onOk\0"
    "onTypeChange\0type\0onFrameName\0frameName\0"
    "onFrameDone\0frameCount\0onPartialDone\0"
    "partial\0total\0onFinished\0updateSceneSettings\0"
    "refreshPopup\0onValueEdited\0isDrag\0"
    "TPixel32\0populateVisibilityMenu\0"
    "visibilityToggled\0saveParameters\0"
    "loadParameters\0resetParameters"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VectorizerPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,  100,    2, 0x0a /* Public */,
       4,    1,  101,    2, 0x08 /* Private */,
       6,    1,  104,    2, 0x08 /* Private */,
       8,    1,  107,    2, 0x08 /* Private */,
      10,    2,  110,    2, 0x08 /* Private */,
      13,    0,  115,    2, 0x08 /* Private */,
      14,    0,  116,    2, 0x08 /* Private */,
      15,    0,  117,    2, 0x08 /* Private */,
      16,    1,  118,    2, 0x08 /* Private */,
      16,    0,  121,    2, 0x28 /* Private | MethodCloned */,
      16,    2,  122,    2, 0x08 /* Private */,
      19,    0,  127,    2, 0x08 /* Private */,
      20,    0,  128,    2, 0x08 /* Private */,
      21,    0,  129,    2, 0x08 /* Private */,
      22,    0,  130,    2, 0x08 /* Private */,
      23,    0,  131,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   11,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18, QMetaType::Bool,    2,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void VectorizerPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<VectorizerPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valuesChanged(); break;
        case 1: _t->onOk(); break;
        case 2: _t->onTypeChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onFrameName((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->onFrameDone((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onPartialDone((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->onFinished(); break;
        case 7: _t->updateSceneSettings(); break;
        case 8: _t->refreshPopup(); break;
        case 9: _t->onValueEdited((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->onValueEdited(); break;
        case 11: _t->onValueEdited((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 12: _t->populateVisibilityMenu(); break;
        case 13: _t->visibilityToggled(); break;
        case 14: _t->saveParameters(); break;
        case 15: _t->loadParameters(); break;
        case 16: _t->resetParameters(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (VectorizerPopup::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&VectorizerPopup::valuesChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject VectorizerPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_VectorizerPopup.data,
    qt_meta_data_VectorizerPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VectorizerPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VectorizerPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VectorizerPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int VectorizerPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void VectorizerPopup::valuesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_Vectorizer_t {
    QByteArrayData data[8];
    char stringdata0[87];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Vectorizer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Vectorizer_t qt_meta_stringdata_Vectorizer = {
    {
QT_MOC_LITERAL(0, 0, 10), // "Vectorizer"
QT_MOC_LITERAL(1, 11, 9), // "frameName"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 9), // "frameDone"
QT_MOC_LITERAL(4, 32, 11), // "partialDone"
QT_MOC_LITERAL(5, 44, 14), // "transmitCancel"
QT_MOC_LITERAL(6, 59, 20), // "configurationChanged"
QT_MOC_LITERAL(7, 80, 6) // "cancel"

    },
    "Vectorizer\0frameName\0\0frameDone\0"
    "partialDone\0transmitCancel\0"
    "configurationChanged\0cancel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Vectorizer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       3,    1,   47,    2, 0x06 /* Public */,
       4,    2,   50,    2, 0x06 /* Public */,
       5,    0,   55,    2, 0x06 /* Public */,
       6,    0,   56,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   57,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    2,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void Vectorizer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Vectorizer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->frameName((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->frameDone((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->partialDone((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->transmitCancel(); break;
        case 4: _t->configurationChanged(); break;
        case 5: _t->cancel(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Vectorizer::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Vectorizer::frameName)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Vectorizer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Vectorizer::frameDone)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Vectorizer::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Vectorizer::partialDone)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Vectorizer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Vectorizer::transmitCancel)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Vectorizer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Vectorizer::configurationChanged)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Vectorizer::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_Vectorizer.data,
    qt_meta_data_Vectorizer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Vectorizer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Vectorizer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Vectorizer.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int Vectorizer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Vectorizer::frameName(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Vectorizer::frameDone(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Vectorizer::partialDone(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Vectorizer::transmitCancel()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Vectorizer::configurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
