/****************************************************************************
** Meta object code from reading C++ file 'xshcellviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/xshcellviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xshcellviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_XsheetGUI__RenameCellField_t {
    QByteArrayData data[4];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__RenameCellField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__RenameCellField_t qt_meta_stringdata_XsheetGUI__RenameCellField = {
    {
QT_MOC_LITERAL(0, 0, 26), // "XsheetGUI::RenameCellField"
QT_MOC_LITERAL(1, 27, 15), // "onReturnPressed"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 15) // "onXsheetChanged"

    },
    "XsheetGUI::RenameCellField\0onReturnPressed\0"
    "\0onXsheetChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__RenameCellField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x09 /* Protected */,
       3,    0,   25,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void XsheetGUI::RenameCellField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RenameCellField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onReturnPressed(); break;
        case 1: _t->onXsheetChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::RenameCellField::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__RenameCellField.data,
    qt_meta_data_XsheetGUI__RenameCellField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::RenameCellField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::RenameCellField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__RenameCellField.stringdata0))
        return static_cast<void*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int XsheetGUI::RenameCellField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__CellArea_t {
    QByteArrayData data[9];
    char stringdata0[108];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__CellArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__CellArea_t qt_meta_stringdata_XsheetGUI__CellArea = {
    {
QT_MOC_LITERAL(0, 0, 19), // "XsheetGUI::CellArea"
QT_MOC_LITERAL(1, 20, 8), // "openNote"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 10), // "deleteNote"
QT_MOC_LITERAL(4, 41, 13), // "onStepChanged"
QT_MOC_LITERAL(5, 55, 8), // "QAction*"
QT_MOC_LITERAL(6, 64, 22), // "onReplaceByCastedLevel"
QT_MOC_LITERAL(7, 87, 6), // "action"
QT_MOC_LITERAL(8, 94, 13) // "onSetCellMark"

    },
    "XsheetGUI::CellArea\0openNote\0\0deleteNote\0"
    "onStepChanged\0QAction*\0onReplaceByCastedLevel\0"
    "action\0onSetCellMark"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__CellArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x09 /* Protected */,
       3,    0,   40,    2, 0x09 /* Protected */,
       4,    1,   41,    2, 0x09 /* Protected */,
       6,    1,   44,    2, 0x09 /* Protected */,
       8,    0,   47,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, 0x80000000 | 5,    7,
    QMetaType::Void,

       0        // eod
};

void XsheetGUI::CellArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CellArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->openNote(); break;
        case 1: _t->deleteNote(); break;
        case 2: _t->onStepChanged((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 3: _t->onReplaceByCastedLevel((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 4: _t->onSetCellMark(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::CellArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__CellArea.data,
    qt_meta_data_XsheetGUI__CellArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::CellArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::CellArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__CellArea.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int XsheetGUI::CellArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
