/****************************************************************************
** Meta object code from reading C++ file 'levelsettingspopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/levelsettingspopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'levelsettingspopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LevelSettingsPopup_t {
    QByteArrayData data[25];
    char stringdata0[455];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LevelSettingsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LevelSettingsPopup_t qt_meta_stringdata_LevelSettingsPopup = {
    {
QT_MOC_LITERAL(0, 0, 18), // "LevelSettingsPopup"
QT_MOC_LITERAL(1, 19, 22), // "onCastSelectionChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 19), // "onSelectionSwitched"
QT_MOC_LITERAL(4, 63, 11), // "TSelection*"
QT_MOC_LITERAL(5, 75, 12), // "oldSelection"
QT_MOC_LITERAL(6, 88, 12), // "newSelection"
QT_MOC_LITERAL(7, 101, 19), // "updateLevelSettings"
QT_MOC_LITERAL(8, 121, 13), // "onNameChanged"
QT_MOC_LITERAL(9, 135, 13), // "onPathChanged"
QT_MOC_LITERAL(10, 149, 17), // "onScanPathChanged"
QT_MOC_LITERAL(11, 167, 16), // "onDpiTypeChanged"
QT_MOC_LITERAL(12, 184, 17), // "onDpiFieldChanged"
QT_MOC_LITERAL(13, 202, 19), // "onWidthFieldChanged"
QT_MOC_LITERAL(14, 222, 20), // "onHeightFieldChanged"
QT_MOC_LITERAL(15, 243, 20), // "onSquarePixelChanged"
QT_MOC_LITERAL(16, 264, 12), // "useCameraDpi"
QT_MOC_LITERAL(17, 277, 20), // "onSubsamplingChanged"
QT_MOC_LITERAL(18, 298, 22), // "onDoPremultiplyClicked"
QT_MOC_LITERAL(19, 321, 20), // "onDoAntialiasClicked"
QT_MOC_LITERAL(20, 342, 26), // "onAntialiasSoftnessChanged"
QT_MOC_LITERAL(21, 369, 20), // "onWhiteTranspClicked"
QT_MOC_LITERAL(22, 390, 29), // "onColorSpaceGammaFieldChanged"
QT_MOC_LITERAL(23, 420, 14), // "onSceneChanged"
QT_MOC_LITERAL(24, 435, 19) // "onPreferenceChanged"

    },
    "LevelSettingsPopup\0onCastSelectionChanged\0"
    "\0onSelectionSwitched\0TSelection*\0"
    "oldSelection\0newSelection\0updateLevelSettings\0"
    "onNameChanged\0onPathChanged\0"
    "onScanPathChanged\0onDpiTypeChanged\0"
    "onDpiFieldChanged\0onWidthFieldChanged\0"
    "onHeightFieldChanged\0onSquarePixelChanged\0"
    "useCameraDpi\0onSubsamplingChanged\0"
    "onDoPremultiplyClicked\0onDoAntialiasClicked\0"
    "onAntialiasSoftnessChanged\0"
    "onWhiteTranspClicked\0onColorSpaceGammaFieldChanged\0"
    "onSceneChanged\0onPreferenceChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LevelSettingsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  114,    2, 0x09 /* Protected */,
       3,    2,  115,    2, 0x09 /* Protected */,
       7,    0,  120,    2, 0x09 /* Protected */,
       8,    0,  121,    2, 0x09 /* Protected */,
       9,    0,  122,    2, 0x09 /* Protected */,
      10,    0,  123,    2, 0x09 /* Protected */,
      11,    1,  124,    2, 0x09 /* Protected */,
      12,    0,  127,    2, 0x09 /* Protected */,
      13,    0,  128,    2, 0x09 /* Protected */,
      14,    0,  129,    2, 0x09 /* Protected */,
      15,    1,  130,    2, 0x09 /* Protected */,
      16,    0,  133,    2, 0x09 /* Protected */,
      17,    0,  134,    2, 0x09 /* Protected */,
      18,    0,  135,    2, 0x09 /* Protected */,
      19,    0,  136,    2, 0x09 /* Protected */,
      20,    0,  137,    2, 0x09 /* Protected */,
      21,    0,  138,    2, 0x09 /* Protected */,
      22,    0,  139,    2, 0x09 /* Protected */,
      23,    0,  140,    2, 0x09 /* Protected */,
      24,    1,  141,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void LevelSettingsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LevelSettingsPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onCastSelectionChanged(); break;
        case 1: _t->onSelectionSwitched((*reinterpret_cast< TSelection*(*)>(_a[1])),(*reinterpret_cast< TSelection*(*)>(_a[2]))); break;
        case 2: _t->updateLevelSettings(); break;
        case 3: _t->onNameChanged(); break;
        case 4: _t->onPathChanged(); break;
        case 5: _t->onScanPathChanged(); break;
        case 6: _t->onDpiTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onDpiFieldChanged(); break;
        case 8: _t->onWidthFieldChanged(); break;
        case 9: _t->onHeightFieldChanged(); break;
        case 10: _t->onSquarePixelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->useCameraDpi(); break;
        case 12: _t->onSubsamplingChanged(); break;
        case 13: _t->onDoPremultiplyClicked(); break;
        case 14: _t->onDoAntialiasClicked(); break;
        case 15: _t->onAntialiasSoftnessChanged(); break;
        case 16: _t->onWhiteTranspClicked(); break;
        case 17: _t->onColorSpaceGammaFieldChanged(); break;
        case 18: _t->onSceneChanged(); break;
        case 19: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LevelSettingsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_LevelSettingsPopup.data,
    qt_meta_data_LevelSettingsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LevelSettingsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LevelSettingsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LevelSettingsPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int LevelSettingsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
