/****************************************************************************
** Meta object code from reading C++ file 'toolhandle.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/tools/toolhandle.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'toolhandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ToolHandle_t {
    QByteArrayData data[13];
    char stringdata0[183];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolHandle_t qt_meta_stringdata_ToolHandle = {
    {
QT_MOC_LITERAL(0, 0, 10), // "ToolHandle"
QT_MOC_LITERAL(1, 11, 23), // "toolComboBoxListChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 11), // "std::string"
QT_MOC_LITERAL(4, 48, 12), // "toolSwitched"
QT_MOC_LITERAL(5, 61, 11), // "toolChanged"
QT_MOC_LITERAL(6, 73, 21), // "toolOptionsBoxChanged"
QT_MOC_LITERAL(7, 95, 19), // "toolEditingFinished"
QT_MOC_LITERAL(8, 115, 21), // "toolCursorTypeChanged"
QT_MOC_LITERAL(9, 137, 14), // "onImageChanged"
QT_MOC_LITERAL(10, 152, 12), // "TImage::Type"
QT_MOC_LITERAL(11, 165, 4), // "type"
QT_MOC_LITERAL(12, 170, 12) // "updateMatrix"

    },
    "ToolHandle\0toolComboBoxListChanged\0\0"
    "std::string\0toolSwitched\0toolChanged\0"
    "toolOptionsBoxChanged\0toolEditingFinished\0"
    "toolCursorTypeChanged\0onImageChanged\0"
    "TImage::Type\0type\0updateMatrix"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    0,   57,    2, 0x06 /* Public */,
       5,    0,   58,    2, 0x06 /* Public */,
       6,    0,   59,    2, 0x06 /* Public */,
       7,    0,   60,    2, 0x06 /* Public */,
       8,    0,   61,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,   62,    2, 0x0a /* Public */,
      12,    0,   65,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,

       0        // eod
};

void ToolHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->toolComboBoxListChanged((*reinterpret_cast< std::string(*)>(_a[1]))); break;
        case 1: _t->toolSwitched(); break;
        case 2: _t->toolChanged(); break;
        case 3: _t->toolOptionsBoxChanged(); break;
        case 4: _t->toolEditingFinished(); break;
        case 5: _t->toolCursorTypeChanged(); break;
        case 6: _t->onImageChanged((*reinterpret_cast< TImage::Type(*)>(_a[1]))); break;
        case 7: _t->updateMatrix(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ToolHandle::*)(std::string );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToolHandle::toolComboBoxListChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ToolHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToolHandle::toolSwitched)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ToolHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToolHandle::toolChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ToolHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToolHandle::toolOptionsBoxChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ToolHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToolHandle::toolEditingFinished)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ToolHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToolHandle::toolCursorTypeChanged)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolHandle.data,
    qt_meta_data_ToolHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ToolHandle::toolComboBoxListChanged(std::string _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ToolHandle::toolSwitched()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ToolHandle::toolChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ToolHandle::toolOptionsBoxChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ToolHandle::toolEditingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ToolHandle::toolCursorTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
