/****************************************************************************
** Meta object code from reading C++ file 'functiontreeviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/functiontreeviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'functiontreeviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FunctionTreeModel_t {
    QByteArrayData data[9];
    char stringdata0[143];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionTreeModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionTreeModel_t qt_meta_stringdata_FunctionTreeModel = {
    {
QT_MOC_LITERAL(0, 0, 17), // "FunctionTreeModel"
QT_MOC_LITERAL(1, 18, 21), // "activeChannelsChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 13), // "curveSelected"
QT_MOC_LITERAL(4, 55, 13), // "TDoubleParam*"
QT_MOC_LITERAL(5, 69, 12), // "curveChanged"
QT_MOC_LITERAL(6, 82, 10), // "isDragging"
QT_MOC_LITERAL(7, 93, 21), // "currentChannelChanged"
QT_MOC_LITERAL(8, 115, 27) // "FunctionTreeModel::Channel*"

    },
    "FunctionTreeModel\0activeChannelsChanged\0"
    "\0curveSelected\0TDoubleParam*\0curveChanged\0"
    "isDragging\0currentChannelChanged\0"
    "FunctionTreeModel::Channel*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionTreeModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,
       3,    1,   35,    2, 0x06 /* Public */,
       5,    1,   38,    2, 0x06 /* Public */,
       7,    1,   41,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, 0x80000000 | 8,    2,

       0        // eod
};

void FunctionTreeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionTreeModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activeChannelsChanged(); break;
        case 1: _t->curveSelected((*reinterpret_cast< TDoubleParam*(*)>(_a[1]))); break;
        case 2: _t->curveChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->currentChannelChanged((*reinterpret_cast< FunctionTreeModel::Channel*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FunctionTreeModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionTreeModel::activeChannelsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FunctionTreeModel::*)(TDoubleParam * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionTreeModel::curveSelected)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FunctionTreeModel::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionTreeModel::curveChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FunctionTreeModel::*)(FunctionTreeModel::Channel * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionTreeModel::currentChannelChanged)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionTreeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeModel::staticMetaObject>(),
    qt_meta_stringdata_FunctionTreeModel.data,
    qt_meta_data_FunctionTreeModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionTreeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionTreeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionTreeModel.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TParamObserver"))
        return static_cast< TParamObserver*>(this);
    return TreeModel::qt_metacast(_clname);
}

int FunctionTreeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void FunctionTreeModel::activeChannelsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FunctionTreeModel::curveSelected(TDoubleParam * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FunctionTreeModel::curveChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void FunctionTreeModel::currentChannelChanged(FunctionTreeModel::Channel * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_FunctionTreeView_t {
    QByteArrayData data[14];
    char stringdata0[154];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionTreeView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionTreeView_t qt_meta_stringdata_FunctionTreeView = {
    {
QT_MOC_LITERAL(0, 0, 16), // "FunctionTreeView"
QT_MOC_LITERAL(1, 17, 19), // "switchCurrentObject"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 13), // "TStageObject*"
QT_MOC_LITERAL(4, 52, 3), // "obj"
QT_MOC_LITERAL(5, 56, 15), // "switchCurrentFx"
QT_MOC_LITERAL(6, 72, 4), // "TFx*"
QT_MOC_LITERAL(7, 77, 2), // "fx"
QT_MOC_LITERAL(8, 80, 11), // "onActivated"
QT_MOC_LITERAL(9, 92, 11), // "QModelIndex"
QT_MOC_LITERAL(10, 104, 5), // "index"
QT_MOC_LITERAL(11, 110, 9), // "updateAll"
QT_MOC_LITERAL(12, 120, 23), // "displayAnimatedChannels"
QT_MOC_LITERAL(13, 144, 9) // "TextColor"

    },
    "FunctionTreeView\0switchCurrentObject\0"
    "\0TStageObject*\0obj\0switchCurrentFx\0"
    "TFx*\0fx\0onActivated\0QModelIndex\0index\0"
    "updateAll\0displayAnimatedChannels\0"
    "TextColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionTreeView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       1,   50, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       5,    1,   42,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    1,   45,    2, 0x0a /* Public */,
      11,    0,   48,    2, 0x0a /* Public */,
      12,    0,   49,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      13, QMetaType::QColor, 0x00095103,

       0        // eod
};

void FunctionTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionTreeView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->switchCurrentObject((*reinterpret_cast< TStageObject*(*)>(_a[1]))); break;
        case 1: _t->switchCurrentFx((*reinterpret_cast< TFx*(*)>(_a[1]))); break;
        case 2: _t->onActivated((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 3: _t->updateAll(); break;
        case 4: _t->displayAnimatedChannels(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FunctionTreeView::*)(TStageObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionTreeView::switchCurrentObject)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FunctionTreeView::*)(TFx * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionTreeView::switchCurrentFx)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<FunctionTreeView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<FunctionTreeView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject FunctionTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeView::staticMetaObject>(),
    qt_meta_stringdata_FunctionTreeView.data,
    qt_meta_data_FunctionTreeView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionTreeView.stringdata0))
        return static_cast<void*>(this);
    return TreeView::qt_metacast(_clname);
}

int FunctionTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void FunctionTreeView::switchCurrentObject(TStageObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FunctionTreeView::switchCurrentFx(TFx * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
