/****************************************************************************
** Meta object code from reading C++ file 'palettecontroller.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/palettecontroller.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'palettecontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PaletteController_t {
    QByteArrayData data[11];
    char stringdata0[168];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PaletteController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PaletteController_t qt_meta_stringdata_PaletteController = {
    {
QT_MOC_LITERAL(0, 0, 17), // "PaletteController"
QT_MOC_LITERAL(1, 18, 21), // "colorAutoApplyEnabled"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 7), // "enabled"
QT_MOC_LITERAL(4, 49, 18), // "colorSampleChanged"
QT_MOC_LITERAL(5, 68, 8), // "TPixel32"
QT_MOC_LITERAL(6, 77, 16), // "checkPaletteLock"
QT_MOC_LITERAL(7, 94, 18), // "colorPassivePicked"
QT_MOC_LITERAL(8, 113, 18), // "stylePassivePicked"
QT_MOC_LITERAL(9, 132, 16), // "editLevelPalette"
QT_MOC_LITERAL(10, 149, 18) // "editCleanupPalette"

    },
    "PaletteController\0colorAutoApplyEnabled\0"
    "\0enabled\0colorSampleChanged\0TPixel32\0"
    "checkPaletteLock\0colorPassivePicked\0"
    "stylePassivePicked\0editLevelPalette\0"
    "editCleanupPalette"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PaletteController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       4,    1,   52,    2, 0x06 /* Public */,
       6,    0,   55,    2, 0x06 /* Public */,
       7,    1,   56,    2, 0x06 /* Public */,
       8,    3,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   66,    2, 0x0a /* Public */,
      10,    0,   67,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QColor,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    2,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PaletteController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PaletteController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->colorAutoApplyEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->colorSampleChanged((*reinterpret_cast< const TPixel32(*)>(_a[1]))); break;
        case 2: _t->checkPaletteLock(); break;
        case 3: _t->colorPassivePicked((*reinterpret_cast< const QColor(*)>(_a[1]))); break;
        case 4: _t->stylePassivePicked((*reinterpret_cast< const int(*)>(_a[1])),(*reinterpret_cast< const int(*)>(_a[2])),(*reinterpret_cast< const int(*)>(_a[3]))); break;
        case 5: _t->editLevelPalette(); break;
        case 6: _t->editCleanupPalette(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PaletteController::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteController::colorAutoApplyEnabled)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PaletteController::*)(const TPixel32 & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteController::colorSampleChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PaletteController::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteController::checkPaletteLock)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PaletteController::*)(const QColor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteController::colorPassivePicked)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (PaletteController::*)(const int , const int , const int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteController::stylePassivePicked)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PaletteController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_PaletteController.data,
    qt_meta_data_PaletteController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PaletteController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaletteController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PaletteController.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PaletteController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void PaletteController::colorAutoApplyEnabled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PaletteController::colorSampleChanged(const TPixel32 & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PaletteController::checkPaletteLock()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PaletteController::colorPassivePicked(const QColor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void PaletteController::stylePassivePicked(const int _t1, const int _t2, const int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
