/****************************************************************************
** Meta object code from reading C++ file 'paletteviewergui.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/paletteviewergui.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'paletteviewergui.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PaletteViewerGUI__PageViewer_t {
    QByteArrayData data[17];
    char stringdata0[290];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PaletteViewerGUI__PageViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PaletteViewerGUI__PageViewer_t qt_meta_stringdata_PaletteViewerGUI__PageViewer = {
    {
QT_MOC_LITERAL(0, 0, 28), // "PaletteViewerGUI::PageViewer"
QT_MOC_LITERAL(1, 29, 23), // "changeWindowTitleSignal"
QT_MOC_LITERAL(2, 53, 0), // ""
QT_MOC_LITERAL(3, 54, 12), // "switchToPage"
QT_MOC_LITERAL(4, 67, 11), // "computeSize"
QT_MOC_LITERAL(5, 79, 14), // "onFrameChanged"
QT_MOC_LITERAL(6, 94, 14), // "onStyleRenamed"
QT_MOC_LITERAL(7, 109, 11), // "addNewColor"
QT_MOC_LITERAL(8, 121, 10), // "addNewPage"
QT_MOC_LITERAL(9, 132, 9), // "TextColor"
QT_MOC_LITERAL(10, 142, 14), // "SeparatorColor"
QT_MOC_LITERAL(11, 157, 19), // "SelectedBorderColor"
QT_MOC_LITERAL(12, 177, 21), // "NumpadShortcutBgColor"
QT_MOC_LITERAL(13, 199, 25), // "NumpadShortcutBorderColor"
QT_MOC_LITERAL(14, 225, 16), // "CurrentCellColor"
QT_MOC_LITERAL(15, 242, 17), // "SelectedCellColor"
QT_MOC_LITERAL(16, 260, 29) // "ListNumpadShortcutBorderColor"

    },
    "PaletteViewerGUI::PageViewer\0"
    "changeWindowTitleSignal\0\0switchToPage\0"
    "computeSize\0onFrameChanged\0onStyleRenamed\0"
    "addNewColor\0addNewPage\0TextColor\0"
    "SeparatorColor\0SelectedBorderColor\0"
    "NumpadShortcutBgColor\0NumpadShortcutBorderColor\0"
    "CurrentCellColor\0SelectedCellColor\0"
    "ListNumpadShortcutBorderColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PaletteViewerGUI__PageViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       8,   58, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    1,   50,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   53,    2, 0x0a /* Public */,
       5,    0,   54,    2, 0x0a /* Public */,
       6,    0,   55,    2, 0x0a /* Public */,
       7,    0,   56,    2, 0x0a /* Public */,
       8,    0,   57,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       9, QMetaType::QColor, 0x00095103,
      10, QMetaType::QColor, 0x00095103,
      11, QMetaType::QColor, 0x00095103,
      12, QMetaType::QColor, 0x00095103,
      13, QMetaType::QColor, 0x00095103,
      14, QMetaType::QColor, 0x00095103,
      15, QMetaType::QColor, 0x00095103,
      16, QMetaType::QColor, 0x00095103,

       0        // eod
};

void PaletteViewerGUI::PageViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changeWindowTitleSignal(); break;
        case 1: _t->switchToPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->computeSize(); break;
        case 3: _t->onFrameChanged(); break;
        case 4: _t->onStyleRenamed(); break;
        case 5: _t->addNewColor(); break;
        case 6: _t->addNewPage(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageViewer::changeWindowTitleSignal)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageViewer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageViewer::switchToPage)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PageViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getSeparatorColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getSelectedBorderColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getNumpadShortcutBgColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getNumpadShortcutBorderColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getCurrentCellColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getSelectedCellColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->getListNumpadShortcutBorderColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<PageViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setSeparatorColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setSelectedBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setNumpadShortcutBgColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setNumpadShortcutBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setCurrentCellColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setSelectedCellColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setListNumpadShortcutBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject PaletteViewerGUI::PageViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_PaletteViewerGUI__PageViewer.data,
    qt_meta_data_PaletteViewerGUI__PageViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PaletteViewerGUI::PageViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaletteViewerGUI::PageViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PaletteViewerGUI__PageViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSelection::View"))
        return static_cast< TSelection::View*>(this);
    return QFrame::qt_metacast(_clname);
}

int PaletteViewerGUI::PageViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void PaletteViewerGUI::PageViewer::changeWindowTitleSignal()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PaletteViewerGUI::PageViewer::switchToPage(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_PaletteViewerGUI__PaletteTabBar_t {
    QByteArrayData data[8];
    char stringdata0[95];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PaletteViewerGUI__PaletteTabBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PaletteViewerGUI__PaletteTabBar_t qt_meta_stringdata_PaletteViewerGUI__PaletteTabBar = {
    {
QT_MOC_LITERAL(0, 0, 31), // "PaletteViewerGUI::PaletteTabBar"
QT_MOC_LITERAL(1, 32, 14), // "tabTextChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 5), // "index"
QT_MOC_LITERAL(4, 54, 8), // "movePage"
QT_MOC_LITERAL(5, 63, 8), // "srcIndex"
QT_MOC_LITERAL(6, 72, 8), // "dstIndex"
QT_MOC_LITERAL(7, 81, 13) // "updateTabName"

    },
    "PaletteViewerGUI::PaletteTabBar\0"
    "tabTextChanged\0\0index\0movePage\0srcIndex\0"
    "dstIndex\0updateTabName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PaletteViewerGUI__PaletteTabBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       4,    2,   32,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   37,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    5,    6,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void PaletteViewerGUI::PaletteTabBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PaletteTabBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->tabTextChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->movePage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->updateTabName(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PaletteTabBar::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteTabBar::tabTextChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PaletteTabBar::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteTabBar::movePage)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PaletteViewerGUI::PaletteTabBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabBar::staticMetaObject>(),
    qt_meta_stringdata_PaletteViewerGUI__PaletteTabBar.data,
    qt_meta_data_PaletteViewerGUI__PaletteTabBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PaletteViewerGUI::PaletteTabBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaletteViewerGUI::PaletteTabBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PaletteViewerGUI__PaletteTabBar.stringdata0))
        return static_cast<void*>(this);
    return QTabBar::qt_metacast(_clname);
}

int PaletteViewerGUI::PaletteTabBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void PaletteViewerGUI::PaletteTabBar::tabTextChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PaletteViewerGUI::PaletteTabBar::movePage(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_PaletteViewerGUI__PaletteIconWidget_t {
    QByteArrayData data[3];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PaletteViewerGUI__PaletteIconWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PaletteViewerGUI__PaletteIconWidget_t qt_meta_stringdata_PaletteViewerGUI__PaletteIconWidget = {
    {
QT_MOC_LITERAL(0, 0, 35), // "PaletteViewerGUI::PaletteIcon..."
QT_MOC_LITERAL(1, 36, 9), // "startDrag"
QT_MOC_LITERAL(2, 46, 0) // ""

    },
    "PaletteViewerGUI::PaletteIconWidget\0"
    "startDrag\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PaletteViewerGUI__PaletteIconWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void PaletteViewerGUI::PaletteIconWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PaletteIconWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startDrag(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PaletteIconWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PaletteIconWidget::startDrag)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PaletteViewerGUI::PaletteIconWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PaletteViewerGUI__PaletteIconWidget.data,
    qt_meta_data_PaletteViewerGUI__PaletteIconWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PaletteViewerGUI::PaletteIconWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaletteViewerGUI::PaletteIconWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PaletteViewerGUI__PaletteIconWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int PaletteViewerGUI::PaletteIconWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void PaletteViewerGUI::PaletteIconWidget::startDrag()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
