/****************************************************************************
** Meta object code from reading C++ file 'scenesettingspopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/scenesettingspopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scenesettingspopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CellMarksPopup_t {
    QByteArrayData data[5];
    char stringdata0[54];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CellMarksPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CellMarksPopup_t qt_meta_stringdata_CellMarksPopup = {
    {
QT_MOC_LITERAL(0, 0, 14), // "CellMarksPopup"
QT_MOC_LITERAL(1, 15, 14), // "onColorChanged"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 8), // "TPixel32"
QT_MOC_LITERAL(4, 40, 13) // "onNameChanged"

    },
    "CellMarksPopup\0onColorChanged\0\0TPixel32\0"
    "onNameChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CellMarksPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       4,    0,   29,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    2,
    QMetaType::Void,

       0        // eod
};

void CellMarksPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CellMarksPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onColorChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onNameChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CellMarksPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_CellMarksPopup.data,
    qt_meta_data_CellMarksPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CellMarksPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CellMarksPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CellMarksPopup.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int CellMarksPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ColorFiltersPopup_t {
    QByteArrayData data[6];
    char stringdata0[78];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColorFiltersPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColorFiltersPopup_t qt_meta_stringdata_ColorFiltersPopup = {
    {
QT_MOC_LITERAL(0, 0, 17), // "ColorFiltersPopup"
QT_MOC_LITERAL(1, 18, 14), // "onColorChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 8), // "TPixel32"
QT_MOC_LITERAL(4, 43, 13), // "onNameChanged"
QT_MOC_LITERAL(5, 57, 20) // "onClearButtonClicked"

    },
    "ColorFiltersPopup\0onColorChanged\0\0"
    "TPixel32\0onNameChanged\0onClearButtonClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColorFiltersPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x09 /* Protected */,
       4,    0,   34,    2, 0x09 /* Protected */,
       5,    0,   35,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ColorFiltersPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorFiltersPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onColorChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onNameChanged(); break;
        case 2: _t->onClearButtonClicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ColorFiltersPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ColorFiltersPopup.data,
    qt_meta_data_ColorFiltersPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColorFiltersPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColorFiltersPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColorFiltersPopup.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int ColorFiltersPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_SceneSettingsPopup_t {
    QByteArrayData data[16];
    char stringdata0[340];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SceneSettingsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SceneSettingsPopup_t qt_meta_stringdata_SceneSettingsPopup = {
    {
QT_MOC_LITERAL(0, 0, 18), // "SceneSettingsPopup"
QT_MOC_LITERAL(1, 19, 6), // "update"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 26), // "onFrameRateEditingFinished"
QT_MOC_LITERAL(4, 54, 31), // "onFieldGuideSizeEditingFinished"
QT_MOC_LITERAL(5, 86, 38), // "onFieldGuideAspectRatioEditin..."
QT_MOC_LITERAL(6, 125, 33), // "onFullColorSubsampEditingFini..."
QT_MOC_LITERAL(7, 159, 27), // "onTlvSubsampEditingFinished"
QT_MOC_LITERAL(8, 187, 25), // "onMakerInformationChanged"
QT_MOC_LITERAL(9, 213, 10), // "setBgColor"
QT_MOC_LITERAL(10, 224, 8), // "TPixel32"
QT_MOC_LITERAL(11, 233, 5), // "value"
QT_MOC_LITERAL(12, 239, 10), // "isDragging"
QT_MOC_LITERAL(13, 250, 28), // "onColorFilterOnRenderChanged"
QT_MOC_LITERAL(14, 279, 28), // "onEditCellMarksButtonClicked"
QT_MOC_LITERAL(15, 308, 31) // "onEditColorFiltersButtonClicked"

    },
    "SceneSettingsPopup\0update\0\0"
    "onFrameRateEditingFinished\0"
    "onFieldGuideSizeEditingFinished\0"
    "onFieldGuideAspectRatioEditingFinished\0"
    "onFullColorSubsampEditingFinished\0"
    "onTlvSubsampEditingFinished\0"
    "onMakerInformationChanged\0setBgColor\0"
    "TPixel32\0value\0isDragging\0"
    "onColorFilterOnRenderChanged\0"
    "onEditCellMarksButtonClicked\0"
    "onEditColorFiltersButtonClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SceneSettingsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x0a /* Public */,
       3,    0,   70,    2, 0x0a /* Public */,
       4,    0,   71,    2, 0x0a /* Public */,
       5,    0,   72,    2, 0x0a /* Public */,
       6,    0,   73,    2, 0x0a /* Public */,
       7,    0,   74,    2, 0x0a /* Public */,
       8,    0,   75,    2, 0x0a /* Public */,
       9,    2,   76,    2, 0x0a /* Public */,
      13,    0,   81,    2, 0x0a /* Public */,
      14,    0,   82,    2, 0x0a /* Public */,
      15,    0,   83,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Bool,   11,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SceneSettingsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SceneSettingsPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->update(); break;
        case 1: _t->onFrameRateEditingFinished(); break;
        case 2: _t->onFieldGuideSizeEditingFinished(); break;
        case 3: _t->onFieldGuideAspectRatioEditingFinished(); break;
        case 4: _t->onFullColorSubsampEditingFinished(); break;
        case 5: _t->onTlvSubsampEditingFinished(); break;
        case 6: _t->onMakerInformationChanged(); break;
        case 7: _t->setBgColor((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 8: _t->onColorFilterOnRenderChanged(); break;
        case 9: _t->onEditCellMarksButtonClicked(); break;
        case 10: _t->onEditColorFiltersButtonClicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SceneSettingsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_SceneSettingsPopup.data,
    qt_meta_data_SceneSettingsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SceneSettingsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SceneSettingsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SceneSettingsPopup.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int SceneSettingsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
