/****************************************************************************
** Meta object code from reading C++ file 'stageschematicnode.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/stageschematicnode.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'stageschematicnode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ColumnPainter_t {
    QByteArrayData data[3];
    char stringdata0[31];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColumnPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColumnPainter_t qt_meta_stringdata_ColumnPainter = {
    {
QT_MOC_LITERAL(0, 0, 13), // "ColumnPainter"
QT_MOC_LITERAL(1, 14, 15), // "onIconGenerated"
QT_MOC_LITERAL(2, 30, 0) // ""

    },
    "ColumnPainter\0onIconGenerated\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColumnPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void ColumnPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColumnPainter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onIconGenerated(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ColumnPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ColumnPainter.data,
    qt_meta_data_ColumnPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColumnPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColumnPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColumnPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int ColumnPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_GroupPainter_t {
    QByteArrayData data[1];
    char stringdata0[13];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GroupPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GroupPainter_t qt_meta_stringdata_GroupPainter = {
    {
QT_MOC_LITERAL(0, 0, 12) // "GroupPainter"

    },
    "GroupPainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GroupPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void GroupPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject GroupPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_GroupPainter.data,
    qt_meta_data_GroupPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GroupPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GroupPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GroupPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int GroupPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_PegbarPainter_t {
    QByteArrayData data[1];
    char stringdata0[14];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PegbarPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PegbarPainter_t qt_meta_stringdata_PegbarPainter = {
    {
QT_MOC_LITERAL(0, 0, 13) // "PegbarPainter"

    },
    "PegbarPainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PegbarPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void PegbarPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PegbarPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_PegbarPainter.data,
    qt_meta_data_PegbarPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PegbarPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PegbarPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PegbarPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int PegbarPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_CameraPainter_t {
    QByteArrayData data[1];
    char stringdata0[14];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CameraPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CameraPainter_t qt_meta_stringdata_CameraPainter = {
    {
QT_MOC_LITERAL(0, 0, 13) // "CameraPainter"

    },
    "CameraPainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CameraPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void CameraPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CameraPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CameraPainter.data,
    qt_meta_data_CameraPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CameraPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CameraPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CameraPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int CameraPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_TablePainter_t {
    QByteArrayData data[1];
    char stringdata0[13];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TablePainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TablePainter_t qt_meta_stringdata_TablePainter = {
    {
QT_MOC_LITERAL(0, 0, 12) // "TablePainter"

    },
    "TablePainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TablePainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void TablePainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TablePainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TablePainter.data,
    qt_meta_data_TablePainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TablePainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TablePainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TablePainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int TablePainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SplinePainter_t {
    QByteArrayData data[1];
    char stringdata0[14];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SplinePainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SplinePainter_t qt_meta_stringdata_SplinePainter = {
    {
QT_MOC_LITERAL(0, 0, 13) // "SplinePainter"

    },
    "SplinePainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SplinePainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SplinePainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SplinePainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_SplinePainter.data,
    qt_meta_data_SplinePainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SplinePainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SplinePainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SplinePainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int SplinePainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_StageSchematicNodeDock_t {
    QByteArrayData data[7];
    char stringdata0[91];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicNodeDock_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicNodeDock_t qt_meta_stringdata_StageSchematicNodeDock = {
    {
QT_MOC_LITERAL(0, 0, 22), // "StageSchematicNodeDock"
QT_MOC_LITERAL(1, 23, 12), // "sceneChanged"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 13), // "onPortClicked"
QT_MOC_LITERAL(4, 51, 14), // "onPortReleased"
QT_MOC_LITERAL(5, 66, 9), // "onTimeOut"
QT_MOC_LITERAL(6, 76, 14) // "onModifyHandle"

    },
    "StageSchematicNodeDock\0sceneChanged\0"
    "\0onPortClicked\0onPortReleased\0onTimeOut\0"
    "onModifyHandle"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicNodeDock[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   40,    2, 0x09 /* Protected */,
       4,    1,   41,    2, 0x09 /* Protected */,
       5,    0,   44,    2, 0x09 /* Protected */,
       6,    1,   45,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPointF,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void StageSchematicNodeDock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicNodeDock *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sceneChanged(); break;
        case 1: _t->onPortClicked(); break;
        case 2: _t->onPortReleased((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 3: _t->onTimeOut(); break;
        case 4: _t->onModifyHandle((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StageSchematicNodeDock::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicNodeDock::sceneChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicNodeDock::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicNodeDock.data,
    qt_meta_data_StageSchematicNodeDock,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicNodeDock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicNodeDock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicNodeDock.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int StageSchematicNodeDock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void StageSchematicNodeDock::sceneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_StageSchematicSplineDock_t {
    QByteArrayData data[3];
    char stringdata0[39];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicSplineDock_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicSplineDock_t qt_meta_stringdata_StageSchematicSplineDock = {
    {
QT_MOC_LITERAL(0, 0, 24), // "StageSchematicSplineDock"
QT_MOC_LITERAL(1, 25, 12), // "sceneChanged"
QT_MOC_LITERAL(2, 38, 0) // ""

    },
    "StageSchematicSplineDock\0sceneChanged\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicSplineDock[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void StageSchematicSplineDock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicSplineDock *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->sceneChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StageSchematicSplineDock::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicSplineDock::sceneChanged)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicSplineDock::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicSplineDock.data,
    qt_meta_data_StageSchematicSplineDock,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicSplineDock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicSplineDock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicSplineDock.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int StageSchematicSplineDock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void StageSchematicSplineDock::sceneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_StageSchematicNode_t {
    QByteArrayData data[11];
    char stringdata0[146];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicNode_t qt_meta_stringdata_StageSchematicNode = {
    {
QT_MOC_LITERAL(0, 0, 18), // "StageSchematicNode"
QT_MOC_LITERAL(1, 19, 20), // "currentObjectChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 14), // "TStageObjectId"
QT_MOC_LITERAL(4, 56, 2), // "id"
QT_MOC_LITERAL(5, 59, 8), // "isSpline"
QT_MOC_LITERAL(6, 68, 20), // "currentColumnChanged"
QT_MOC_LITERAL(7, 89, 5), // "index"
QT_MOC_LITERAL(8, 95, 10), // "editObject"
QT_MOC_LITERAL(9, 106, 22), // "onHandleReleasedPublic"
QT_MOC_LITERAL(10, 129, 16) // "onHandleReleased"

    },
    "StageSchematicNode\0currentObjectChanged\0"
    "\0TStageObjectId\0id\0isSpline\0"
    "currentColumnChanged\0index\0editObject\0"
    "onHandleReleasedPublic\0onHandleReleased"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   39,    2, 0x06 /* Public */,
       6,    1,   44,    2, 0x06 /* Public */,
       8,    0,   47,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   48,    2, 0x0a /* Public */,
      10,    0,   49,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StageSchematicNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentObjectChanged((*reinterpret_cast< const TStageObjectId(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->currentColumnChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->editObject(); break;
        case 3: _t->onHandleReleasedPublic(); break;
        case 4: _t->onHandleReleased(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StageSchematicNode::*)(const TStageObjectId & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicNode::currentObjectChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (StageSchematicNode::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicNode::currentColumnChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (StageSchematicNode::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicNode::editObject)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicNode::staticMetaObject = { {
    QMetaObject::SuperData::link<SchematicNode::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicNode.data,
    qt_meta_data_StageSchematicNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicNode.stringdata0))
        return static_cast<void*>(this);
    return SchematicNode::qt_metacast(_clname);
}

int StageSchematicNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void StageSchematicNode::currentObjectChanged(const TStageObjectId & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void StageSchematicNode::currentColumnChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void StageSchematicNode::editObject()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_StageSchematicPegbarNode_t {
    QByteArrayData data[3];
    char stringdata0[40];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicPegbarNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicPegbarNode_t qt_meta_stringdata_StageSchematicPegbarNode = {
    {
QT_MOC_LITERAL(0, 0, 24), // "StageSchematicPegbarNode"
QT_MOC_LITERAL(1, 25, 13), // "onNameChanged"
QT_MOC_LITERAL(2, 39, 0) // ""

    },
    "StageSchematicPegbarNode\0onNameChanged\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicPegbarNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void StageSchematicPegbarNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicPegbarNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onNameChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicPegbarNode::staticMetaObject = { {
    QMetaObject::SuperData::link<StageSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicPegbarNode.data,
    qt_meta_data_StageSchematicPegbarNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicPegbarNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicPegbarNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicPegbarNode.stringdata0))
        return static_cast<void*>(this);
    return StageSchematicNode::qt_metacast(_clname);
}

int StageSchematicPegbarNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StageSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_StageSchematicColumnNode_t {
    QByteArrayData data[9];
    char stringdata0[125];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicColumnNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicColumnNode_t qt_meta_stringdata_StageSchematicColumnNode = {
    {
QT_MOC_LITERAL(0, 0, 24), // "StageSchematicColumnNode"
QT_MOC_LITERAL(1, 25, 13), // "onNameChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 13), // "onChangedSize"
QT_MOC_LITERAL(4, 54, 6), // "expand"
QT_MOC_LITERAL(5, 61, 21), // "onRenderToggleClicked"
QT_MOC_LITERAL(6, 83, 8), // "isActive"
QT_MOC_LITERAL(7, 92, 26), // "onCameraStandToggleClicked"
QT_MOC_LITERAL(8, 119, 5) // "state"

    },
    "StageSchematicColumnNode\0onNameChanged\0"
    "\0onChangedSize\0expand\0onRenderToggleClicked\0"
    "isActive\0onCameraStandToggleClicked\0"
    "state"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicColumnNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x09 /* Protected */,
       3,    1,   35,    2, 0x09 /* Protected */,
       5,    1,   38,    2, 0x09 /* Protected */,
       7,    1,   41,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Int,    8,

       0        // eod
};

void StageSchematicColumnNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicColumnNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onNameChanged(); break;
        case 1: _t->onChangedSize((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->onRenderToggleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onCameraStandToggleClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicColumnNode::staticMetaObject = { {
    QMetaObject::SuperData::link<StageSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicColumnNode.data,
    qt_meta_data_StageSchematicColumnNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicColumnNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicColumnNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicColumnNode.stringdata0))
        return static_cast<void*>(this);
    return StageSchematicNode::qt_metacast(_clname);
}

int StageSchematicColumnNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StageSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_StageSchematicCameraNode_t {
    QByteArrayData data[3];
    char stringdata0[40];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicCameraNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicCameraNode_t qt_meta_stringdata_StageSchematicCameraNode = {
    {
QT_MOC_LITERAL(0, 0, 24), // "StageSchematicCameraNode"
QT_MOC_LITERAL(1, 25, 13), // "onNameChanged"
QT_MOC_LITERAL(2, 39, 0) // ""

    },
    "StageSchematicCameraNode\0onNameChanged\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicCameraNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void StageSchematicCameraNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicCameraNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onNameChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicCameraNode::staticMetaObject = { {
    QMetaObject::SuperData::link<StageSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicCameraNode.data,
    qt_meta_data_StageSchematicCameraNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicCameraNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicCameraNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicCameraNode.stringdata0))
        return static_cast<void*>(this);
    return StageSchematicNode::qt_metacast(_clname);
}

int StageSchematicCameraNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StageSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_StageSchematicSplineNode_t {
    QByteArrayData data[9];
    char stringdata0[111];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicSplineNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicSplineNode_t qt_meta_stringdata_StageSchematicSplineNode = {
    {
QT_MOC_LITERAL(0, 0, 24), // "StageSchematicSplineNode"
QT_MOC_LITERAL(1, 25, 20), // "currentObjectChanged"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 14), // "TStageObjectId"
QT_MOC_LITERAL(4, 62, 2), // "id"
QT_MOC_LITERAL(5, 65, 8), // "isSpline"
QT_MOC_LITERAL(6, 74, 13), // "onNameChanged"
QT_MOC_LITERAL(7, 88, 13), // "onChangedSize"
QT_MOC_LITERAL(8, 102, 8) // "expanded"

    },
    "StageSchematicSplineNode\0currentObjectChanged\0"
    "\0TStageObjectId\0id\0isSpline\0onNameChanged\0"
    "onChangedSize\0expanded"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicSplineNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   34,    2, 0x09 /* Protected */,
       7,    1,   35,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    8,

       0        // eod
};

void StageSchematicSplineNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicSplineNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentObjectChanged((*reinterpret_cast< const TStageObjectId(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onNameChanged(); break;
        case 2: _t->onChangedSize((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StageSchematicSplineNode::*)(const TStageObjectId & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicSplineNode::currentObjectChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicSplineNode::staticMetaObject = { {
    QMetaObject::SuperData::link<SchematicNode::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicSplineNode.data,
    qt_meta_data_StageSchematicSplineNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicSplineNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicSplineNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicSplineNode.stringdata0))
        return static_cast<void*>(this);
    return SchematicNode::qt_metacast(_clname);
}

int StageSchematicSplineNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void StageSchematicSplineNode::currentObjectChanged(const TStageObjectId & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StageSchematicGroupNode_t {
    QByteArrayData data[5];
    char stringdata0[62];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicGroupNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicGroupNode_t qt_meta_stringdata_StageSchematicGroupNode = {
    {
QT_MOC_LITERAL(0, 0, 23), // "StageSchematicGroupNode"
QT_MOC_LITERAL(1, 24, 13), // "onNameChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 13), // "onChangedSize"
QT_MOC_LITERAL(4, 53, 8) // "expanded"

    },
    "StageSchematicGroupNode\0onNameChanged\0"
    "\0onChangedSize\0expanded"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicGroupNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x09 /* Protected */,
       3,    1,   25,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,

       0        // eod
};

void StageSchematicGroupNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicGroupNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onNameChanged(); break;
        case 1: _t->onChangedSize((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicGroupNode::staticMetaObject = { {
    QMetaObject::SuperData::link<StageSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicGroupNode.data,
    qt_meta_data_StageSchematicGroupNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicGroupNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicGroupNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicGroupNode.stringdata0))
        return static_cast<void*>(this);
    return StageSchematicNode::qt_metacast(_clname);
}

int StageSchematicGroupNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StageSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
