/****************************************************************************
** Meta object code from reading C++ file 'filebrowserpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/filebrowserpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filebrowserpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FileBrowserPopup_t {
    QByteArrayData data[13];
    char stringdata0[198];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileBrowserPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileBrowserPopup_t qt_meta_stringdata_FileBrowserPopup = {
    {
QT_MOC_LITERAL(0, 0, 16), // "FileBrowserPopup"
QT_MOC_LITERAL(1, 17, 15), // "filePathClicked"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 9), // "TFilePath"
QT_MOC_LITERAL(4, 44, 11), // "onOkPressed"
QT_MOC_LITERAL(5, 56, 14), // "onApplyPressed"
QT_MOC_LITERAL(6, 71, 17), // "onFilePathClicked"
QT_MOC_LITERAL(7, 89, 19), // "onFilePathsSelected"
QT_MOC_LITERAL(8, 109, 19), // "std::set<TFilePath>"
QT_MOC_LITERAL(9, 129, 5), // "paths"
QT_MOC_LITERAL(10, 135, 33), // "std::list<std::vector<TFrameI..."
QT_MOC_LITERAL(11, 169, 4), // "fIds"
QT_MOC_LITERAL(12, 174, 23) // "onFilePathDoubleClicked"

    },
    "FileBrowserPopup\0filePathClicked\0\0"
    "TFilePath\0onOkPressed\0onApplyPressed\0"
    "onFilePathClicked\0onFilePathsSelected\0"
    "std::set<TFilePath>\0paths\0"
    "std::list<std::vector<TFrameId> >\0"
    "fIds\0onFilePathDoubleClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileBrowserPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   47,    2, 0x0a /* Public */,
       5,    0,   48,    2, 0x0a /* Public */,
       6,    1,   49,    2, 0x09 /* Protected */,
       7,    2,   52,    2, 0x09 /* Protected */,
      12,    1,   57,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 10,    9,   11,
    QMetaType::Void, 0x80000000 | 3,    2,

       0        // eod
};

void FileBrowserPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileBrowserPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->filePathClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 1: _t->onOkPressed(); break;
        case 2: _t->onApplyPressed(); break;
        case 3: _t->onFilePathClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 4: _t->onFilePathsSelected((*reinterpret_cast< const std::set<TFilePath>(*)>(_a[1])),(*reinterpret_cast< const std::list<std::vector<TFrameId> >(*)>(_a[2]))); break;
        case 5: _t->onFilePathDoubleClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileBrowserPopup::*)(const TFilePath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBrowserPopup::filePathClicked)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FileBrowserPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_FileBrowserPopup.data,
    qt_meta_data_FileBrowserPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileBrowserPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileBrowserPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileBrowserPopup.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int FileBrowserPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void FileBrowserPopup::filePathClicked(const TFilePath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_LoadScenePopup_t {
    QByteArrayData data[5];
    char stringdata0[55];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadScenePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadScenePopup_t qt_meta_stringdata_LoadScenePopup = {
    {
QT_MOC_LITERAL(0, 0, 14), // "LoadScenePopup"
QT_MOC_LITERAL(1, 15, 23), // "onFilePathDoubleClicked"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 9), // "TFilePath"
QT_MOC_LITERAL(4, 50, 4) // "path"

    },
    "LoadScenePopup\0onFilePathDoubleClicked\0"
    "\0TFilePath\0path"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadScenePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void LoadScenePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LoadScenePopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onFilePathDoubleClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LoadScenePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_LoadScenePopup.data,
    qt_meta_data_LoadScenePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadScenePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadScenePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadScenePopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int LoadScenePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_LoadSubScenePopup_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadSubScenePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadSubScenePopup_t qt_meta_stringdata_LoadSubScenePopup = {
    {
QT_MOC_LITERAL(0, 0, 17) // "LoadSubScenePopup"

    },
    "LoadSubScenePopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadSubScenePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void LoadSubScenePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject LoadSubScenePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_LoadSubScenePopup.data,
    qt_meta_data_LoadSubScenePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadSubScenePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadSubScenePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadSubScenePopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int LoadSubScenePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SaveSceneAsPopup_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SaveSceneAsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SaveSceneAsPopup_t qt_meta_stringdata_SaveSceneAsPopup = {
    {
QT_MOC_LITERAL(0, 0, 16) // "SaveSceneAsPopup"

    },
    "SaveSceneAsPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SaveSceneAsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SaveSceneAsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SaveSceneAsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_SaveSceneAsPopup.data,
    qt_meta_data_SaveSceneAsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SaveSceneAsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SaveSceneAsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SaveSceneAsPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int SaveSceneAsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SaveSubSceneAsPopup_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SaveSubSceneAsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SaveSubSceneAsPopup_t qt_meta_stringdata_SaveSubSceneAsPopup = {
    {
QT_MOC_LITERAL(0, 0, 19) // "SaveSubSceneAsPopup"

    },
    "SaveSubSceneAsPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SaveSubSceneAsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SaveSubSceneAsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SaveSubSceneAsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_SaveSubSceneAsPopup.data,
    qt_meta_data_SaveSubSceneAsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SaveSubSceneAsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SaveSubSceneAsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SaveSubSceneAsPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int SaveSubSceneAsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_LoadLevelPopup_t {
    QByteArrayData data[21];
    char stringdata0[336];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadLevelPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadLevelPopup_t qt_meta_stringdata_LoadLevelPopup = {
    {
QT_MOC_LITERAL(0, 0, 14), // "LoadLevelPopup"
QT_MOC_LITERAL(1, 15, 19), // "onFilePathsSelected"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 19), // "std::set<TFilePath>"
QT_MOC_LITERAL(4, 56, 5), // "paths"
QT_MOC_LITERAL(5, 62, 33), // "std::list<std::vector<TFrameI..."
QT_MOC_LITERAL(6, 96, 4), // "fIds"
QT_MOC_LITERAL(7, 101, 15), // "onFrameSwitched"
QT_MOC_LITERAL(8, 117, 24), // "onSubsequentFrameChanged"
QT_MOC_LITERAL(9, 142, 11), // "updatePosTo"
QT_MOC_LITERAL(10, 154, 16), // "onNameSetEditted"
QT_MOC_LITERAL(11, 171, 18), // "onSelectionChanged"
QT_MOC_LITERAL(12, 190, 11), // "TSelection*"
QT_MOC_LITERAL(13, 202, 9), // "selection"
QT_MOC_LITERAL(14, 212, 23), // "onFilePathDoubleClicked"
QT_MOC_LITERAL(15, 236, 9), // "TFilePath"
QT_MOC_LITERAL(16, 246, 4), // "path"
QT_MOC_LITERAL(17, 251, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(18, 271, 20), // "onDpiPolicyActivated"
QT_MOC_LITERAL(19, 292, 22), // "onDoPremultiplyClicked"
QT_MOC_LITERAL(20, 315, 20) // "onWhiteTranspClicked"

    },
    "LoadLevelPopup\0onFilePathsSelected\0\0"
    "std::set<TFilePath>\0paths\0"
    "std::list<std::vector<TFrameId> >\0"
    "fIds\0onFrameSwitched\0onSubsequentFrameChanged\0"
    "updatePosTo\0onNameSetEditted\0"
    "onSelectionChanged\0TSelection*\0selection\0"
    "onFilePathDoubleClicked\0TFilePath\0"
    "path\0onPreferenceChanged\0onDpiPolicyActivated\0"
    "onDoPremultiplyClicked\0onWhiteTranspClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadLevelPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   69,    2, 0x09 /* Protected */,
       7,    0,   74,    2, 0x09 /* Protected */,
       8,    0,   75,    2, 0x09 /* Protected */,
       9,    0,   76,    2, 0x09 /* Protected */,
      10,    0,   77,    2, 0x09 /* Protected */,
      11,    1,   78,    2, 0x09 /* Protected */,
      14,    1,   81,    2, 0x09 /* Protected */,
      17,    1,   84,    2, 0x09 /* Protected */,
      18,    0,   87,    2, 0x09 /* Protected */,
      19,    0,   88,    2, 0x09 /* Protected */,
      20,    0,   89,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void LoadLevelPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LoadLevelPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onFilePathsSelected((*reinterpret_cast< const std::set<TFilePath>(*)>(_a[1])),(*reinterpret_cast< const std::list<std::vector<TFrameId> >(*)>(_a[2]))); break;
        case 1: _t->onFrameSwitched(); break;
        case 2: _t->onSubsequentFrameChanged(); break;
        case 3: _t->updatePosTo(); break;
        case 4: _t->onNameSetEditted(); break;
        case 5: _t->onSelectionChanged((*reinterpret_cast< TSelection*(*)>(_a[1]))); break;
        case 6: _t->onFilePathDoubleClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 7: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->onDpiPolicyActivated(); break;
        case 9: _t->onDoPremultiplyClicked(); break;
        case 10: _t->onWhiteTranspClicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LoadLevelPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_LoadLevelPopup.data,
    qt_meta_data_LoadLevelPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadLevelPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadLevelPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadLevelPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int LoadLevelPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
struct qt_meta_stringdata_SaveLevelAsPopup_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SaveLevelAsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SaveLevelAsPopup_t qt_meta_stringdata_SaveLevelAsPopup = {
    {
QT_MOC_LITERAL(0, 0, 16) // "SaveLevelAsPopup"

    },
    "SaveLevelAsPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SaveLevelAsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SaveLevelAsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SaveLevelAsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_SaveLevelAsPopup.data,
    qt_meta_data_SaveLevelAsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SaveLevelAsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SaveLevelAsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SaveLevelAsPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int SaveLevelAsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ReplaceLevelPopup_t {
    QByteArrayData data[5];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ReplaceLevelPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ReplaceLevelPopup_t qt_meta_stringdata_ReplaceLevelPopup = {
    {
QT_MOC_LITERAL(0, 0, 17), // "ReplaceLevelPopup"
QT_MOC_LITERAL(1, 18, 18), // "onSelectionChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 11), // "TSelection*"
QT_MOC_LITERAL(4, 50, 9) // "selection"

    },
    "ReplaceLevelPopup\0onSelectionChanged\0"
    "\0TSelection*\0selection"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ReplaceLevelPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void ReplaceLevelPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ReplaceLevelPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSelectionChanged((*reinterpret_cast< TSelection*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ReplaceLevelPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_ReplaceLevelPopup.data,
    qt_meta_data_ReplaceLevelPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ReplaceLevelPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ReplaceLevelPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ReplaceLevelPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int ReplaceLevelPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_SavePaletteAsPopup_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SavePaletteAsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SavePaletteAsPopup_t qt_meta_stringdata_SavePaletteAsPopup = {
    {
QT_MOC_LITERAL(0, 0, 18) // "SavePaletteAsPopup"

    },
    "SavePaletteAsPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SavePaletteAsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SavePaletteAsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SavePaletteAsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_SavePaletteAsPopup.data,
    qt_meta_data_SavePaletteAsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SavePaletteAsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SavePaletteAsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SavePaletteAsPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int SavePaletteAsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_LoadColorModelPopup_t {
    QByteArrayData data[5];
    char stringdata0[67];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadColorModelPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadColorModelPopup_t qt_meta_stringdata_LoadColorModelPopup = {
    {
QT_MOC_LITERAL(0, 0, 19), // "LoadColorModelPopup"
QT_MOC_LITERAL(1, 20, 19), // "onFilePathsSelected"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 19), // "std::set<TFilePath>"
QT_MOC_LITERAL(4, 61, 5) // "paths"

    },
    "LoadColorModelPopup\0onFilePathsSelected\0"
    "\0std::set<TFilePath>\0paths"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadColorModelPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void LoadColorModelPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LoadColorModelPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onFilePathsSelected((*reinterpret_cast< const std::set<TFilePath>(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LoadColorModelPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_LoadColorModelPopup.data,
    qt_meta_data_LoadColorModelPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadColorModelPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadColorModelPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadColorModelPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int LoadColorModelPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ImportMagpieFilePopup_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ImportMagpieFilePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ImportMagpieFilePopup_t qt_meta_stringdata_ImportMagpieFilePopup = {
    {
QT_MOC_LITERAL(0, 0, 21) // "ImportMagpieFilePopup"

    },
    "ImportMagpieFilePopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ImportMagpieFilePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ImportMagpieFilePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ImportMagpieFilePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_ImportMagpieFilePopup.data,
    qt_meta_data_ImportMagpieFilePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ImportMagpieFilePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImportMagpieFilePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ImportMagpieFilePopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int ImportMagpieFilePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ReplaceParentDirectoryPopup_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ReplaceParentDirectoryPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ReplaceParentDirectoryPopup_t qt_meta_stringdata_ReplaceParentDirectoryPopup = {
    {
QT_MOC_LITERAL(0, 0, 27) // "ReplaceParentDirectoryPopup"

    },
    "ReplaceParentDirectoryPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ReplaceParentDirectoryPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ReplaceParentDirectoryPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ReplaceParentDirectoryPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_ReplaceParentDirectoryPopup.data,
    qt_meta_data_ReplaceParentDirectoryPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ReplaceParentDirectoryPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ReplaceParentDirectoryPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ReplaceParentDirectoryPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int ReplaceParentDirectoryPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_BrowserPopup_t {
    QByteArrayData data[1];
    char stringdata0[13];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BrowserPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BrowserPopup_t qt_meta_stringdata_BrowserPopup = {
    {
QT_MOC_LITERAL(0, 0, 12) // "BrowserPopup"

    },
    "BrowserPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BrowserPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void BrowserPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject BrowserPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_BrowserPopup.data,
    qt_meta_data_BrowserPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BrowserPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BrowserPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BrowserPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int BrowserPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_BrowserPopupController_t {
    QByteArrayData data[1];
    char stringdata0[23];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BrowserPopupController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BrowserPopupController_t qt_meta_stringdata_BrowserPopupController = {
    {
QT_MOC_LITERAL(0, 0, 22) // "BrowserPopupController"

    },
    "BrowserPopupController"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BrowserPopupController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void BrowserPopupController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject BrowserPopupController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_BrowserPopupController.data,
    qt_meta_data_BrowserPopupController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BrowserPopupController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BrowserPopupController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BrowserPopupController.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DVGui::FileField::BrowserPopupController"))
        return static_cast< DVGui::FileField::BrowserPopupController*>(this);
    return QObject::qt_metacast(_clname);
}

int BrowserPopupController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
