/****************************************************************************
** Meta object code from reading C++ file 'dvdirtreeview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/dvdirtreeview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dvdirtreeview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MyFileSystemWatcher_t {
    QByteArrayData data[4];
    char stringdata0[43];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MyFileSystemWatcher_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MyFileSystemWatcher_t qt_meta_stringdata_MyFileSystemWatcher = {
    {
QT_MOC_LITERAL(0, 0, 19), // "MyFileSystemWatcher"
QT_MOC_LITERAL(1, 20, 16), // "directoryChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 4) // "path"

    },
    "MyFileSystemWatcher\0directoryChanged\0"
    "\0path"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MyFileSystemWatcher[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

       0        // eod
};

void MyFileSystemWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MyFileSystemWatcher *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->directoryChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MyFileSystemWatcher::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MyFileSystemWatcher::directoryChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MyFileSystemWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_MyFileSystemWatcher.data,
    qt_meta_data_MyFileSystemWatcher,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MyFileSystemWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MyFileSystemWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MyFileSystemWatcher.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MyFileSystemWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void MyFileSystemWatcher::directoryChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StyledTreeView_t {
    QByteArrayData data[6];
    char stringdata0[106];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyledTreeView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyledTreeView_t qt_meta_stringdata_StyledTreeView = {
    {
QT_MOC_LITERAL(0, 0, 14), // "StyledTreeView"
QT_MOC_LITERAL(1, 15, 9), // "TextColor"
QT_MOC_LITERAL(2, 25, 17), // "SelectedTextColor"
QT_MOC_LITERAL(3, 43, 15), // "FolderTextColor"
QT_MOC_LITERAL(4, 59, 23), // "SelectedFolderTextColor"
QT_MOC_LITERAL(5, 83, 22) // "SelectedItemBackground"

    },
    "StyledTreeView\0TextColor\0SelectedTextColor\0"
    "FolderTextColor\0SelectedFolderTextColor\0"
    "SelectedItemBackground"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyledTreeView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       5,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QColor, 0x00095103,
       2, QMetaType::QColor, 0x00095103,
       3, QMetaType::QColor, 0x00095103,
       4, QMetaType::QColor, 0x00095103,
       5, QMetaType::QColor, 0x00095103,

       0        // eod
};

void StyledTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<StyledTreeView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getSelectedTextColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getFolderTextColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getSelectedFolderTextColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getSelectedItemBackground(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<StyledTreeView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setSelectedTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setFolderTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setSelectedFolderTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setSelectedItemBackground(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StyledTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeView::staticMetaObject>(),
    qt_meta_stringdata_StyledTreeView.data,
    qt_meta_data_StyledTreeView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyledTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyledTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyledTreeView.stringdata0))
        return static_cast<void*>(this);
    return QTreeView::qt_metacast(_clname);
}

int StyledTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_DvDirTreeView_t {
    QByteArrayData data[42];
    char stringdata0[805];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DvDirTreeView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DvDirTreeView_t qt_meta_stringdata_DvDirTreeView = {
    {
QT_MOC_LITERAL(0, 0, 13), // "DvDirTreeView"
QT_MOC_LITERAL(1, 14, 18), // "currentNodeChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 12), // "deleteFolder"
QT_MOC_LITERAL(4, 47, 14), // "setCurrentNode"
QT_MOC_LITERAL(5, 62, 15), // "DvDirModelNode*"
QT_MOC_LITERAL(6, 78, 4), // "node"
QT_MOC_LITERAL(7, 83, 9), // "TFilePath"
QT_MOC_LITERAL(8, 93, 2), // "fp"
QT_MOC_LITERAL(9, 96, 10), // "expandNode"
QT_MOC_LITERAL(10, 107, 13), // "resizeToConts"
QT_MOC_LITERAL(11, 121, 21), // "cleanupVersionControl"
QT_MOC_LITERAL(12, 143, 24), // "DvDirVersionControlNode*"
QT_MOC_LITERAL(13, 168, 19), // "purgeVersionControl"
QT_MOC_LITERAL(14, 188, 20), // "deleteVersionControl"
QT_MOC_LITERAL(15, 209, 17), // "putVersionControl"
QT_MOC_LITERAL(16, 227, 20), // "updateVersionControl"
QT_MOC_LITERAL(17, 248, 18), // "listVersionControl"
QT_MOC_LITERAL(18, 267, 16), // "lastExistingNode"
QT_MOC_LITERAL(19, 284, 12), // "relativePath"
QT_MOC_LITERAL(20, 297, 29), // "editCurrentVersionControlNode"
QT_MOC_LITERAL(21, 327, 31), // "unlockCurrentVersionControlNode"
QT_MOC_LITERAL(22, 359, 31), // "revertCurrentVersionControlNode"
QT_MOC_LITERAL(23, 391, 31), // "updateCurrentVersionControlNode"
QT_MOC_LITERAL(24, 423, 28), // "putCurrentVersionControlNode"
QT_MOC_LITERAL(25, 452, 31), // "deleteCurrentVersionControlNode"
QT_MOC_LITERAL(26, 484, 32), // "refreshCurrentVersionControlNode"
QT_MOC_LITERAL(27, 517, 32), // "cleanupCurrentVersionControlNode"
QT_MOC_LITERAL(28, 550, 30), // "purgeCurrentVersionControlNode"
QT_MOC_LITERAL(29, 581, 10), // "onInfoDone"
QT_MOC_LITERAL(30, 592, 10), // "onListDone"
QT_MOC_LITERAL(31, 603, 23), // "onCheckPartialLockError"
QT_MOC_LITERAL(32, 627, 22), // "onCheckPartialLockDone"
QT_MOC_LITERAL(33, 650, 15), // "onCheckOutError"
QT_MOC_LITERAL(34, 666, 14), // "onCheckOutDone"
QT_MOC_LITERAL(35, 681, 19), // "onRefreshStatusDone"
QT_MOC_LITERAL(36, 701, 20), // "onRefreshStatusError"
QT_MOC_LITERAL(37, 722, 10), // "onExpanded"
QT_MOC_LITERAL(38, 733, 11), // "QModelIndex"
QT_MOC_LITERAL(39, 745, 11), // "onCollapsed"
QT_MOC_LITERAL(40, 757, 27), // "onMonitoredDirectoryChanged"
QT_MOC_LITERAL(41, 785, 19) // "onPreferenceChanged"

    },
    "DvDirTreeView\0currentNodeChanged\0\0"
    "deleteFolder\0setCurrentNode\0DvDirModelNode*\0"
    "node\0TFilePath\0fp\0expandNode\0resizeToConts\0"
    "cleanupVersionControl\0DvDirVersionControlNode*\0"
    "purgeVersionControl\0deleteVersionControl\0"
    "putVersionControl\0updateVersionControl\0"
    "listVersionControl\0lastExistingNode\0"
    "relativePath\0editCurrentVersionControlNode\0"
    "unlockCurrentVersionControlNode\0"
    "revertCurrentVersionControlNode\0"
    "updateCurrentVersionControlNode\0"
    "putCurrentVersionControlNode\0"
    "deleteCurrentVersionControlNode\0"
    "refreshCurrentVersionControlNode\0"
    "cleanupCurrentVersionControlNode\0"
    "purgeCurrentVersionControlNode\0"
    "onInfoDone\0onListDone\0onCheckPartialLockError\0"
    "onCheckPartialLockDone\0onCheckOutError\0"
    "onCheckOutDone\0onRefreshStatusDone\0"
    "onRefreshStatusError\0onExpanded\0"
    "QModelIndex\0onCollapsed\0"
    "onMonitoredDirectoryChanged\0"
    "onPreferenceChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DvDirTreeView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      33,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  179,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,  180,    2, 0x0a /* Public */,
       4,    1,  181,    2, 0x0a /* Public */,
       4,    2,  184,    2, 0x0a /* Public */,
       4,    1,  189,    2, 0x2a /* Public | MethodCloned */,
      10,    0,  192,    2, 0x0a /* Public */,
      11,    1,  193,    2, 0x0a /* Public */,
      13,    1,  196,    2, 0x0a /* Public */,
      14,    1,  199,    2, 0x0a /* Public */,
      15,    1,  202,    2, 0x0a /* Public */,
      16,    1,  205,    2, 0x0a /* Public */,
      17,    2,  208,    2, 0x0a /* Public */,
      20,    0,  213,    2, 0x0a /* Public */,
      21,    0,  214,    2, 0x0a /* Public */,
      22,    0,  215,    2, 0x0a /* Public */,
      23,    0,  216,    2, 0x0a /* Public */,
      24,    0,  217,    2, 0x0a /* Public */,
      25,    0,  218,    2, 0x0a /* Public */,
      26,    0,  219,    2, 0x0a /* Public */,
      27,    0,  220,    2, 0x0a /* Public */,
      28,    0,  221,    2, 0x0a /* Public */,
      29,    1,  222,    2, 0x0a /* Public */,
      30,    1,  225,    2, 0x0a /* Public */,
      31,    1,  228,    2, 0x0a /* Public */,
      32,    1,  231,    2, 0x0a /* Public */,
      33,    1,  234,    2, 0x0a /* Public */,
      34,    1,  237,    2, 0x0a /* Public */,
      35,    1,  240,    2, 0x0a /* Public */,
      36,    1,  243,    2, 0x0a /* Public */,
      37,    1,  246,    2, 0x0a /* Public */,
      39,    1,  249,    2, 0x0a /* Public */,
      40,    1,  252,    2, 0x0a /* Public */,
      41,    1,  255,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 7, QMetaType::Bool,    8,    9,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,    6,
    QMetaType::Void, 0x80000000 | 12,    6,
    QMetaType::Void, 0x80000000 | 12,    6,
    QMetaType::Void, 0x80000000 | 12,    6,
    QMetaType::Void, 0x80000000 | 12,    6,
    QMetaType::Void, 0x80000000 | 12, QMetaType::QString,   18,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 38,    2,
    QMetaType::Void, 0x80000000 | 38,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void DvDirTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvDirTreeView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentNodeChanged(); break;
        case 1: _t->deleteFolder(); break;
        case 2: _t->setCurrentNode((*reinterpret_cast< DvDirModelNode*(*)>(_a[1]))); break;
        case 3: _t->setCurrentNode((*reinterpret_cast< const TFilePath(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->setCurrentNode((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 5: _t->resizeToConts(); break;
        case 6: _t->cleanupVersionControl((*reinterpret_cast< DvDirVersionControlNode*(*)>(_a[1]))); break;
        case 7: _t->purgeVersionControl((*reinterpret_cast< DvDirVersionControlNode*(*)>(_a[1]))); break;
        case 8: _t->deleteVersionControl((*reinterpret_cast< DvDirVersionControlNode*(*)>(_a[1]))); break;
        case 9: _t->putVersionControl((*reinterpret_cast< DvDirVersionControlNode*(*)>(_a[1]))); break;
        case 10: _t->updateVersionControl((*reinterpret_cast< DvDirVersionControlNode*(*)>(_a[1]))); break;
        case 11: _t->listVersionControl((*reinterpret_cast< DvDirVersionControlNode*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 12: _t->editCurrentVersionControlNode(); break;
        case 13: _t->unlockCurrentVersionControlNode(); break;
        case 14: _t->revertCurrentVersionControlNode(); break;
        case 15: _t->updateCurrentVersionControlNode(); break;
        case 16: _t->putCurrentVersionControlNode(); break;
        case 17: _t->deleteCurrentVersionControlNode(); break;
        case 18: _t->refreshCurrentVersionControlNode(); break;
        case 19: _t->cleanupCurrentVersionControlNode(); break;
        case 20: _t->purgeCurrentVersionControlNode(); break;
        case 21: _t->onInfoDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->onListDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 23: _t->onCheckPartialLockError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: _t->onCheckPartialLockDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 25: _t->onCheckOutError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: _t->onCheckOutDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 27: _t->onRefreshStatusDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 28: _t->onRefreshStatusError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->onExpanded((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 30: _t->onCollapsed((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 31: _t->onMonitoredDirectoryChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 32: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DvDirTreeView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvDirTreeView::currentNodeChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DvDirTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<StyledTreeView::staticMetaObject>(),
    qt_meta_stringdata_DvDirTreeView.data,
    qt_meta_data_DvDirTreeView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DvDirTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DvDirTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DvDirTreeView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSelection"))
        return static_cast< TSelection*>(this);
    return StyledTreeView::qt_metacast(_clname);
}

int DvDirTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyledTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 33;
    }
    return _id;
}

// SIGNAL 0
void DvDirTreeView::currentNodeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_DvDirTreeViewDelegate_t {
    QByteArrayData data[3];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DvDirTreeViewDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DvDirTreeViewDelegate_t qt_meta_stringdata_DvDirTreeViewDelegate = {
    {
QT_MOC_LITERAL(0, 0, 21), // "DvDirTreeViewDelegate"
QT_MOC_LITERAL(1, 22, 20), // "commitAndCloseEditor"
QT_MOC_LITERAL(2, 43, 0) // ""

    },
    "DvDirTreeViewDelegate\0commitAndCloseEditor\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DvDirTreeViewDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void DvDirTreeViewDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvDirTreeViewDelegate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->commitAndCloseEditor(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DvDirTreeViewDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QItemDelegate::staticMetaObject>(),
    qt_meta_stringdata_DvDirTreeViewDelegate.data,
    qt_meta_data_DvDirTreeViewDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DvDirTreeViewDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DvDirTreeViewDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DvDirTreeViewDelegate.stringdata0))
        return static_cast<void*>(this);
    return QItemDelegate::qt_metacast(_clname);
}

int DvDirTreeViewDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QItemDelegate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_NodeEditor_t {
    QByteArrayData data[4];
    char stringdata0[41];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NodeEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NodeEditor_t qt_meta_stringdata_NodeEditor = {
    {
QT_MOC_LITERAL(0, 0, 10), // "NodeEditor"
QT_MOC_LITERAL(1, 11, 15), // "editingFinished"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 12) // "emitFinished"

    },
    "NodeEditor\0editingFinished\0\0emitFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NodeEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   25,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void NodeEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NodeEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editingFinished(); break;
        case 1: _t->emitFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NodeEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NodeEditor::editingFinished)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject NodeEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_NodeEditor.data,
    qt_meta_data_NodeEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *NodeEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NodeEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_NodeEditor.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int NodeEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void NodeEditor::editingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
