/****************************************************************************
** Meta object code from reading C++ file 'tasksviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/tasksviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tasksviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TaskTreeModel_t {
    QByteArrayData data[11];
    char stringdata0[94];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TaskTreeModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TaskTreeModel_t qt_meta_stringdata_TaskTreeModel = {
    {
QT_MOC_LITERAL(0, 0, 13), // "TaskTreeModel"
QT_MOC_LITERAL(1, 14, 5), // "start"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 4), // "stop"
QT_MOC_LITERAL(4, 26, 6), // "remove"
QT_MOC_LITERAL(5, 33, 4), // "save"
QT_MOC_LITERAL(6, 38, 6), // "saveas"
QT_MOC_LITERAL(7, 45, 4), // "load"
QT_MOC_LITERAL(8, 50, 14), // "addCleanupTask"
QT_MOC_LITERAL(9, 65, 13), // "addRenderTask"
QT_MOC_LITERAL(10, 79, 14) // "setupModelData"

    },
    "TaskTreeModel\0start\0\0stop\0remove\0save\0"
    "saveas\0load\0addCleanupTask\0addRenderTask\0"
    "setupModelData"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TaskTreeModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       3,    1,   62,    2, 0x0a /* Public */,
       4,    1,   65,    2, 0x0a /* Public */,
       5,    1,   68,    2, 0x0a /* Public */,
       6,    1,   71,    2, 0x0a /* Public */,
       7,    1,   74,    2, 0x0a /* Public */,
       8,    1,   77,    2, 0x0a /* Public */,
       9,    1,   80,    2, 0x0a /* Public */,
      10,    0,   83,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void TaskTreeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TaskTreeModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->start((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->stop((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->remove((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->save((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->saveas((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->load((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->addCleanupTask((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->addRenderTask((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->setupModelData(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TaskTreeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeModel::staticMetaObject>(),
    qt_meta_stringdata_TaskTreeModel.data,
    qt_meta_data_TaskTreeModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TaskTreeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TaskTreeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TaskTreeModel.stringdata0))
        return static_cast<void*>(this);
    return TreeModel::qt_metacast(_clname);
}

int TaskTreeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
struct qt_meta_stringdata_TaskTreeView_t {
    QByteArrayData data[1];
    char stringdata0[13];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TaskTreeView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TaskTreeView_t qt_meta_stringdata_TaskTreeView = {
    {
QT_MOC_LITERAL(0, 0, 12) // "TaskTreeView"

    },
    "TaskTreeView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TaskTreeView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void TaskTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TaskTreeView::staticMetaObject = { {
    QMetaObject::SuperData::link<TreeView::staticMetaObject>(),
    qt_meta_stringdata_TaskTreeView.data,
    qt_meta_data_TaskTreeView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TaskTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TaskTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TaskTreeView.stringdata0))
        return static_cast<void*>(this);
    return TreeView::qt_metacast(_clname);
}

int TaskTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TreeView::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_TaskSheet_t {
    QByteArrayData data[22];
    char stringdata0[262];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TaskSheet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TaskSheet_t qt_meta_stringdata_TaskSheet = {
    {
QT_MOC_LITERAL(0, 0, 9), // "TaskSheet"
QT_MOC_LITERAL(1, 10, 7), // "onAdded"
QT_MOC_LITERAL(2, 18, 0), // ""
QT_MOC_LITERAL(3, 19, 9), // "onRemoved"
QT_MOC_LITERAL(4, 29, 26), // "onRemovedItemDoubleClicked"
QT_MOC_LITERAL(5, 56, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(6, 73, 4), // "item"
QT_MOC_LITERAL(7, 78, 24), // "onAddedItemDoubleClicked"
QT_MOC_LITERAL(8, 103, 9), // "onFocusIn"
QT_MOC_LITERAL(9, 113, 7), // "setName"
QT_MOC_LITERAL(10, 121, 7), // "setFrom"
QT_MOC_LITERAL(11, 129, 5), // "setTo"
QT_MOC_LITERAL(12, 135, 9), // "setShrink"
QT_MOC_LITERAL(13, 145, 7), // "setStep"
QT_MOC_LITERAL(14, 153, 9), // "setOutput"
QT_MOC_LITERAL(15, 163, 12), // "setChunkSize"
QT_MOC_LITERAL(16, 176, 10), // "setVisible"
QT_MOC_LITERAL(17, 187, 12), // "setOverwrite"
QT_MOC_LITERAL(18, 200, 13), // "setMultimedia"
QT_MOC_LITERAL(19, 214, 15), // "setThreadsCombo"
QT_MOC_LITERAL(20, 230, 19), // "setGranularityCombo"
QT_MOC_LITERAL(21, 250, 11) // "setPriority"

    },
    "TaskSheet\0onAdded\0\0onRemoved\0"
    "onRemovedItemDoubleClicked\0QListWidgetItem*\0"
    "item\0onAddedItemDoubleClicked\0onFocusIn\0"
    "setName\0setFrom\0setTo\0setShrink\0setStep\0"
    "setOutput\0setChunkSize\0setVisible\0"
    "setOverwrite\0setMultimedia\0setThreadsCombo\0"
    "setGranularityCombo\0setPriority"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TaskSheet[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x09 /* Protected */,
       3,    1,  107,    2, 0x09 /* Protected */,
       4,    1,  110,    2, 0x09 /* Protected */,
       7,    1,  113,    2, 0x09 /* Protected */,
       8,    0,  116,    2, 0x09 /* Protected */,
       9,    0,  117,    2, 0x09 /* Protected */,
      10,    0,  118,    2, 0x09 /* Protected */,
      11,    0,  119,    2, 0x09 /* Protected */,
      12,    0,  120,    2, 0x09 /* Protected */,
      13,    0,  121,    2, 0x09 /* Protected */,
      14,    0,  122,    2, 0x09 /* Protected */,
      15,    0,  123,    2, 0x09 /* Protected */,
      16,    1,  124,    2, 0x09 /* Protected */,
      17,    1,  127,    2, 0x09 /* Protected */,
      18,    1,  130,    2, 0x09 /* Protected */,
      19,    1,  133,    2, 0x09 /* Protected */,
      20,    1,  136,    2, 0x09 /* Protected */,
      21,    0,  139,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

       0        // eod
};

void TaskSheet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TaskSheet *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onAdded((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onRemoved((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->onRemovedItemDoubleClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 3: _t->onAddedItemDoubleClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 4: _t->onFocusIn(); break;
        case 5: _t->setName(); break;
        case 6: _t->setFrom(); break;
        case 7: _t->setTo(); break;
        case 8: _t->setShrink(); break;
        case 9: _t->setStep(); break;
        case 10: _t->setOutput(); break;
        case 11: _t->setChunkSize(); break;
        case 12: _t->setVisible((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->setOverwrite((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->setMultimedia((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->setThreadsCombo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->setGranularityCombo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->setPriority(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TaskSheet::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_TaskSheet.data,
    qt_meta_data_TaskSheet,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TaskSheet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TaskSheet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TaskSheet.stringdata0))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int TaskSheet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}
struct qt_meta_stringdata_TasksViewer_t {
    QByteArrayData data[3];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TasksViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TasksViewer_t qt_meta_stringdata_TasksViewer = {
    {
QT_MOC_LITERAL(0, 0, 11), // "TasksViewer"
QT_MOC_LITERAL(1, 12, 7), // "onTimer"
QT_MOC_LITERAL(2, 20, 0) // ""

    },
    "TasksViewer\0onTimer\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TasksViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void TasksViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TasksViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onTimer(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TasksViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_meta_stringdata_TasksViewer.data,
    qt_meta_data_TasksViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TasksViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TasksViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TasksViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "BatchesController::Observer"))
        return static_cast< BatchesController::Observer*>(this);
    return QSplitter::qt_metacast(_clname);
}

int TasksViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
