/****************************************************************************
** Meta object code from reading C++ file 'tframehandle.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/tframehandle.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tframehandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TFrameHandle_t {
    QByteArrayData data[18];
    char stringdata0[220];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TFrameHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TFrameHandle_t qt_meta_stringdata_TFrameHandle = {
    {
QT_MOC_LITERAL(0, 0, 12), // "TFrameHandle"
QT_MOC_LITERAL(1, 13, 13), // "frameSwitched"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 12), // "scrubStarted"
QT_MOC_LITERAL(4, 41, 12), // "scrubStopped"
QT_MOC_LITERAL(5, 54, 16), // "frameTypeChanged"
QT_MOC_LITERAL(6, 71, 22), // "isPlayingStatusChanged"
QT_MOC_LITERAL(7, 94, 19), // "triggerNextKeyframe"
QT_MOC_LITERAL(8, 114, 8), // "QWidget*"
QT_MOC_LITERAL(9, 123, 5), // "panel"
QT_MOC_LITERAL(10, 129, 19), // "triggerPrevKeyframe"
QT_MOC_LITERAL(11, 149, 9), // "nextFrame"
QT_MOC_LITERAL(12, 159, 8), // "TFrameId"
QT_MOC_LITERAL(13, 168, 9), // "prevFrame"
QT_MOC_LITERAL(14, 178, 10), // "firstFrame"
QT_MOC_LITERAL(15, 189, 9), // "lastFrame"
QT_MOC_LITERAL(16, 199, 10), // "setPlaying"
QT_MOC_LITERAL(17, 210, 9) // "isPlaying"

    },
    "TFrameHandle\0frameSwitched\0\0scrubStarted\0"
    "scrubStopped\0frameTypeChanged\0"
    "isPlayingStatusChanged\0triggerNextKeyframe\0"
    "QWidget*\0panel\0triggerPrevKeyframe\0"
    "nextFrame\0TFrameId\0prevFrame\0firstFrame\0"
    "lastFrame\0setPlaying\0isPlaying"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TFrameHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x06 /* Public */,
       3,    0,   80,    2, 0x06 /* Public */,
       4,    0,   81,    2, 0x06 /* Public */,
       5,    0,   82,    2, 0x06 /* Public */,
       6,    0,   83,    2, 0x06 /* Public */,
       7,    1,   84,    2, 0x06 /* Public */,
      10,    1,   87,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    1,   90,    2, 0x0a /* Public */,
      11,    0,   93,    2, 0x2a /* Public | MethodCloned */,
      13,    0,   94,    2, 0x0a /* Public */,
      14,    0,   95,    2, 0x0a /* Public */,
      15,    0,   96,    2, 0x0a /* Public */,
      16,    1,   97,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 8,    9,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 12,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   17,

       0        // eod
};

void TFrameHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TFrameHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->frameSwitched(); break;
        case 1: _t->scrubStarted(); break;
        case 2: _t->scrubStopped(); break;
        case 3: _t->frameTypeChanged(); break;
        case 4: _t->isPlayingStatusChanged(); break;
        case 5: _t->triggerNextKeyframe((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 6: _t->triggerPrevKeyframe((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 7: _t->nextFrame((*reinterpret_cast< TFrameId(*)>(_a[1]))); break;
        case 8: _t->nextFrame(); break;
        case 9: _t->prevFrame(); break;
        case 10: _t->firstFrame(); break;
        case 11: _t->lastFrame(); break;
        case 12: _t->setPlaying((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TFrameHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TFrameHandle::frameSwitched)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TFrameHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TFrameHandle::scrubStarted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TFrameHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TFrameHandle::scrubStopped)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TFrameHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TFrameHandle::frameTypeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TFrameHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TFrameHandle::isPlayingStatusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TFrameHandle::*)(QWidget * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TFrameHandle::triggerNextKeyframe)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TFrameHandle::*)(QWidget * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TFrameHandle::triggerPrevKeyframe)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TFrameHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TFrameHandle.data,
    qt_meta_data_TFrameHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TFrameHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TFrameHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TFrameHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TFrameHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void TFrameHandle::frameSwitched()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TFrameHandle::scrubStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TFrameHandle::scrubStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TFrameHandle::frameTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TFrameHandle::isPlayingStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TFrameHandle::triggerNextKeyframe(QWidget * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void TFrameHandle::triggerPrevKeyframe(QWidget * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
