/****************************************************************************
** Meta object code from reading C++ file 'functionsegmentviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/functionsegmentviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'functionsegmentviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FunctionSegmentViewer_t {
    QByteArrayData data[17];
    char stringdata0[270];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionSegmentViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionSegmentViewer_t qt_meta_stringdata_FunctionSegmentViewer = {
    {
QT_MOC_LITERAL(0, 0, 21), // "FunctionSegmentViewer"
QT_MOC_LITERAL(1, 22, 20), // "onSegmentTypeChanged"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 4), // "type"
QT_MOC_LITERAL(4, 49, 14), // "onCurveChanged"
QT_MOC_LITERAL(5, 64, 18), // "onStepFieldChanged"
QT_MOC_LITERAL(6, 83, 20), // "onApplyButtonPressed"
QT_MOC_LITERAL(7, 104, 24), // "onPrevCurveButtonPressed"
QT_MOC_LITERAL(8, 129, 24), // "onNextCurveButtonPressed"
QT_MOC_LITERAL(9, 154, 23), // "onPrevLinkButtonPressed"
QT_MOC_LITERAL(10, 178, 23), // "onNextLinkButtonPressed"
QT_MOC_LITERAL(11, 202, 10), // "setSegment"
QT_MOC_LITERAL(12, 213, 13), // "TDoubleParam*"
QT_MOC_LITERAL(13, 227, 5), // "curve"
QT_MOC_LITERAL(14, 233, 12), // "segmentIndex"
QT_MOC_LITERAL(15, 246, 17), // "setSegmentByFrame"
QT_MOC_LITERAL(16, 264, 5) // "frame"

    },
    "FunctionSegmentViewer\0onSegmentTypeChanged\0"
    "\0type\0onCurveChanged\0onStepFieldChanged\0"
    "onApplyButtonPressed\0onPrevCurveButtonPressed\0"
    "onNextCurveButtonPressed\0"
    "onPrevLinkButtonPressed\0onNextLinkButtonPressed\0"
    "setSegment\0TDoubleParam*\0curve\0"
    "segmentIndex\0setSegmentByFrame\0frame"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionSegmentViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x08 /* Private */,
       4,    0,   67,    2, 0x08 /* Private */,
       5,    1,   68,    2, 0x08 /* Private */,
       6,    0,   71,    2, 0x08 /* Private */,
       7,    0,   72,    2, 0x08 /* Private */,
       8,    0,   73,    2, 0x08 /* Private */,
       9,    0,   74,    2, 0x08 /* Private */,
      10,    0,   75,    2, 0x08 /* Private */,
      11,    2,   76,    2, 0x0a /* Public */,
      15,    2,   81,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int,   13,   14,
    QMetaType::Void, 0x80000000 | 12, QMetaType::Int,   13,   16,

       0        // eod
};

void FunctionSegmentViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionSegmentViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSegmentTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onCurveChanged(); break;
        case 2: _t->onStepFieldChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onApplyButtonPressed(); break;
        case 4: _t->onPrevCurveButtonPressed(); break;
        case 5: _t->onNextCurveButtonPressed(); break;
        case 6: _t->onPrevLinkButtonPressed(); break;
        case 7: _t->onNextLinkButtonPressed(); break;
        case 8: _t->setSegment((*reinterpret_cast< TDoubleParam*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: _t->setSegmentByFrame((*reinterpret_cast< TDoubleParam*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionSegmentViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_FunctionSegmentViewer.data,
    qt_meta_data_FunctionSegmentViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionSegmentViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionSegmentViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionSegmentViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TParamObserver"))
        return static_cast< TParamObserver*>(this);
    return QFrame::qt_metacast(_clname);
}

int FunctionSegmentViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
struct qt_meta_stringdata_FunctionSegmentPage_t {
    QByteArrayData data[3];
    char stringdata0[36];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionSegmentPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionSegmentPage_t qt_meta_stringdata_FunctionSegmentPage = {
    {
QT_MOC_LITERAL(0, 0, 19), // "FunctionSegmentPage"
QT_MOC_LITERAL(1, 20, 14), // "onFieldChanged"
QT_MOC_LITERAL(2, 35, 0) // ""

    },
    "FunctionSegmentPage\0onFieldChanged\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionSegmentPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void FunctionSegmentPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionSegmentPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onFieldChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FunctionSegmentPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_FunctionSegmentPage.data,
    qt_meta_data_FunctionSegmentPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionSegmentPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionSegmentPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionSegmentPage.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int FunctionSegmentPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_SpeedInOutSegmentPage_t {
    QByteArrayData data[8];
    char stringdata0[148];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SpeedInOutSegmentPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SpeedInOutSegmentPage_t qt_meta_stringdata_SpeedInOutSegmentPage = {
    {
QT_MOC_LITERAL(0, 0, 21), // "SpeedInOutSegmentPage"
QT_MOC_LITERAL(1, 22, 21), // "onFirstHandleXChanged"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 21), // "onFirstHandleYChanged"
QT_MOC_LITERAL(4, 67, 20), // "onLastHandleXChanged"
QT_MOC_LITERAL(5, 88, 20), // "onLastHandleYChanged"
QT_MOC_LITERAL(6, 109, 19), // "onFirstSpeedChanged"
QT_MOC_LITERAL(7, 129, 18) // "onLastSpeedChanged"

    },
    "SpeedInOutSegmentPage\0onFirstHandleXChanged\0"
    "\0onFirstHandleYChanged\0onLastHandleXChanged\0"
    "onLastHandleYChanged\0onFirstSpeedChanged\0"
    "onLastSpeedChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SpeedInOutSegmentPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x0a /* Public */,
       3,    0,   45,    2, 0x0a /* Public */,
       4,    0,   46,    2, 0x0a /* Public */,
       5,    0,   47,    2, 0x0a /* Public */,
       6,    0,   48,    2, 0x0a /* Public */,
       7,    0,   49,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SpeedInOutSegmentPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SpeedInOutSegmentPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onFirstHandleXChanged(); break;
        case 1: _t->onFirstHandleYChanged(); break;
        case 2: _t->onLastHandleXChanged(); break;
        case 3: _t->onLastHandleYChanged(); break;
        case 4: _t->onFirstSpeedChanged(); break;
        case 5: _t->onLastSpeedChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SpeedInOutSegmentPage::staticMetaObject = { {
    QMetaObject::SuperData::link<FunctionSegmentPage::staticMetaObject>(),
    qt_meta_stringdata_SpeedInOutSegmentPage.data,
    qt_meta_data_SpeedInOutSegmentPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SpeedInOutSegmentPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpeedInOutSegmentPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SpeedInOutSegmentPage.stringdata0))
        return static_cast<void*>(this);
    return FunctionSegmentPage::qt_metacast(_clname);
}

int SpeedInOutSegmentPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FunctionSegmentPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
struct qt_meta_stringdata_EaseInOutSegmentPage_t {
    QByteArrayData data[4];
    char stringdata0[52];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_EaseInOutSegmentPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_EaseInOutSegmentPage_t qt_meta_stringdata_EaseInOutSegmentPage = {
    {
QT_MOC_LITERAL(0, 0, 20), // "EaseInOutSegmentPage"
QT_MOC_LITERAL(1, 21, 14), // "onEase0Changed"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 14) // "onEase1Changed"

    },
    "EaseInOutSegmentPage\0onEase0Changed\0"
    "\0onEase1Changed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_EaseInOutSegmentPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    0,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void EaseInOutSegmentPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EaseInOutSegmentPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onEase0Changed(); break;
        case 1: _t->onEase1Changed(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject EaseInOutSegmentPage::staticMetaObject = { {
    QMetaObject::SuperData::link<FunctionSegmentPage::staticMetaObject>(),
    qt_meta_stringdata_EaseInOutSegmentPage.data,
    qt_meta_data_EaseInOutSegmentPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *EaseInOutSegmentPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *EaseInOutSegmentPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_EaseInOutSegmentPage.stringdata0))
        return static_cast<void*>(this);
    return FunctionSegmentPage::qt_metacast(_clname);
}

int EaseInOutSegmentPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FunctionSegmentPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_FunctionExpressionSegmentPage_t {
    QByteArrayData data[1];
    char stringdata0[30];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionExpressionSegmentPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionExpressionSegmentPage_t qt_meta_stringdata_FunctionExpressionSegmentPage = {
    {
QT_MOC_LITERAL(0, 0, 29) // "FunctionExpressionSegmentPage"

    },
    "FunctionExpressionSegmentPage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionExpressionSegmentPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FunctionExpressionSegmentPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FunctionExpressionSegmentPage::staticMetaObject = { {
    QMetaObject::SuperData::link<FunctionSegmentPage::staticMetaObject>(),
    qt_meta_stringdata_FunctionExpressionSegmentPage.data,
    qt_meta_data_FunctionExpressionSegmentPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionExpressionSegmentPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionExpressionSegmentPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionExpressionSegmentPage.stringdata0))
        return static_cast<void*>(this);
    return FunctionSegmentPage::qt_metacast(_clname);
}

int FunctionExpressionSegmentPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FunctionSegmentPage::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SimilarShapeSegmentPage_t {
    QByteArrayData data[1];
    char stringdata0[24];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SimilarShapeSegmentPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SimilarShapeSegmentPage_t qt_meta_stringdata_SimilarShapeSegmentPage = {
    {
QT_MOC_LITERAL(0, 0, 23) // "SimilarShapeSegmentPage"

    },
    "SimilarShapeSegmentPage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SimilarShapeSegmentPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SimilarShapeSegmentPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SimilarShapeSegmentPage::staticMetaObject = { {
    QMetaObject::SuperData::link<FunctionSegmentPage::staticMetaObject>(),
    qt_meta_stringdata_SimilarShapeSegmentPage.data,
    qt_meta_data_SimilarShapeSegmentPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SimilarShapeSegmentPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SimilarShapeSegmentPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SimilarShapeSegmentPage.stringdata0))
        return static_cast<void*>(this);
    return FunctionSegmentPage::qt_metacast(_clname);
}

int SimilarShapeSegmentPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FunctionSegmentPage::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FileSegmentPage_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileSegmentPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileSegmentPage_t qt_meta_stringdata_FileSegmentPage = {
    {
QT_MOC_LITERAL(0, 0, 15) // "FileSegmentPage"

    },
    "FileSegmentPage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileSegmentPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FileSegmentPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FileSegmentPage::staticMetaObject = { {
    QMetaObject::SuperData::link<FunctionSegmentPage::staticMetaObject>(),
    qt_meta_stringdata_FileSegmentPage.data,
    qt_meta_data_FileSegmentPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileSegmentPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileSegmentPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileSegmentPage.stringdata0))
        return static_cast<void*>(this);
    return FunctionSegmentPage::qt_metacast(_clname);
}

int FileSegmentPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FunctionSegmentPage::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
