/****************************************************************************
** Meta object code from reading C++ file 'fxschematicnode.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/fxschematicnode.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fxschematicnode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FxColumnPainter_t {
    QByteArrayData data[3];
    char stringdata0[33];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxColumnPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxColumnPainter_t qt_meta_stringdata_FxColumnPainter = {
    {
QT_MOC_LITERAL(0, 0, 15), // "FxColumnPainter"
QT_MOC_LITERAL(1, 16, 15), // "onIconGenerated"
QT_MOC_LITERAL(2, 32, 0) // ""

    },
    "FxColumnPainter\0onIconGenerated\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxColumnPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void FxColumnPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxColumnPainter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onIconGenerated(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxColumnPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FxColumnPainter.data,
    qt_meta_data_FxColumnPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxColumnPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxColumnPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxColumnPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int FxColumnPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_FxPalettePainter_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxPalettePainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxPalettePainter_t qt_meta_stringdata_FxPalettePainter = {
    {
QT_MOC_LITERAL(0, 0, 16) // "FxPalettePainter"

    },
    "FxPalettePainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxPalettePainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxPalettePainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxPalettePainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FxPalettePainter.data,
    qt_meta_data_FxPalettePainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxPalettePainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxPalettePainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxPalettePainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int FxPalettePainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxPainter_t {
    QByteArrayData data[1];
    char stringdata0[10];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxPainter_t qt_meta_stringdata_FxPainter = {
    {
QT_MOC_LITERAL(0, 0, 9) // "FxPainter"

    },
    "FxPainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FxPainter.data,
    qt_meta_data_FxPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int FxPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxXSheetPainter_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxXSheetPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxXSheetPainter_t qt_meta_stringdata_FxXSheetPainter = {
    {
QT_MOC_LITERAL(0, 0, 15) // "FxXSheetPainter"

    },
    "FxXSheetPainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxXSheetPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxXSheetPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxXSheetPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FxXSheetPainter.data,
    qt_meta_data_FxXSheetPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxXSheetPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxXSheetPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxXSheetPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int FxXSheetPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxOutputPainter_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxOutputPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxOutputPainter_t qt_meta_stringdata_FxOutputPainter = {
    {
QT_MOC_LITERAL(0, 0, 15) // "FxOutputPainter"

    },
    "FxOutputPainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxOutputPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxOutputPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxOutputPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FxOutputPainter.data,
    qt_meta_data_FxOutputPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxOutputPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxOutputPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxOutputPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int FxOutputPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxSchematicLink_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicLink_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicLink_t qt_meta_stringdata_FxSchematicLink = {
    {
QT_MOC_LITERAL(0, 0, 15) // "FxSchematicLink"

    },
    "FxSchematicLink"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicLink[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxSchematicLink::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicLink::staticMetaObject = { {
    QMetaObject::SuperData::link<SchematicLink::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicLink.data,
    qt_meta_data_FxSchematicLink,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicLink::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicLink::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicLink.stringdata0))
        return static_cast<void*>(this);
    return SchematicLink::qt_metacast(_clname);
}

int FxSchematicLink::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchematicLink::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxSchematicPort_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicPort_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicPort_t qt_meta_stringdata_FxSchematicPort = {
    {
QT_MOC_LITERAL(0, 0, 15) // "FxSchematicPort"

    },
    "FxSchematicPort"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicPort[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxSchematicPort::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicPort::staticMetaObject = { {
    QMetaObject::SuperData::link<SchematicPort::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicPort.data,
    qt_meta_data_FxSchematicPort,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicPort::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicPort::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicPort.stringdata0))
        return static_cast<void*>(this);
    return SchematicPort::qt_metacast(_clname);
}

int FxSchematicPort::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchematicPort::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxSchematicNode_t {
    QByteArrayData data[8];
    char stringdata0[88];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicNode_t qt_meta_stringdata_FxSchematicNode = {
    {
QT_MOC_LITERAL(0, 0, 15), // "FxSchematicNode"
QT_MOC_LITERAL(1, 16, 15), // "switchCurrentFx"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 4), // "TFx*"
QT_MOC_LITERAL(4, 38, 2), // "fx"
QT_MOC_LITERAL(5, 41, 20), // "currentColumnChanged"
QT_MOC_LITERAL(6, 62, 5), // "index"
QT_MOC_LITERAL(7, 68, 19) // "fxNodeDoubleClicked"

    },
    "FxSchematicNode\0switchCurrentFx\0\0TFx*\0"
    "fx\0currentColumnChanged\0index\0"
    "fxNodeDoubleClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       5,    1,   32,    2, 0x06 /* Public */,
       7,    0,   35,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,

       0        // eod
};

void FxSchematicNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSchematicNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->switchCurrentFx((*reinterpret_cast< TFx*(*)>(_a[1]))); break;
        case 1: _t->currentColumnChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->fxNodeDoubleClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FxSchematicNode::*)(TFx * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicNode::switchCurrentFx)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FxSchematicNode::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicNode::currentColumnChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FxSchematicNode::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicNode::fxNodeDoubleClicked)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicNode::staticMetaObject = { {
    QMetaObject::SuperData::link<SchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicNode.data,
    qt_meta_data_FxSchematicNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicNode.stringdata0))
        return static_cast<void*>(this);
    return SchematicNode::qt_metacast(_clname);
}

int FxSchematicNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void FxSchematicNode::switchCurrentFx(TFx * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FxSchematicNode::currentColumnChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FxSchematicNode::fxNodeDoubleClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_FxSchematicOutputNode_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicOutputNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicOutputNode_t qt_meta_stringdata_FxSchematicOutputNode = {
    {
QT_MOC_LITERAL(0, 0, 21) // "FxSchematicOutputNode"

    },
    "FxSchematicOutputNode"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicOutputNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxSchematicOutputNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicOutputNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicOutputNode.data,
    qt_meta_data_FxSchematicOutputNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicOutputNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicOutputNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicOutputNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxSchematicOutputNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxSchematicXSheetNode_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicXSheetNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicXSheetNode_t qt_meta_stringdata_FxSchematicXSheetNode = {
    {
QT_MOC_LITERAL(0, 0, 21) // "FxSchematicXSheetNode"

    },
    "FxSchematicXSheetNode"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicXSheetNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxSchematicXSheetNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicXSheetNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicXSheetNode.data,
    qt_meta_data_FxSchematicXSheetNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicXSheetNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicXSheetNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicXSheetNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxSchematicXSheetNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxSchematicNormalFxNode_t {
    QByteArrayData data[4];
    char stringdata0[61];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicNormalFxNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicNormalFxNode_t qt_meta_stringdata_FxSchematicNormalFxNode = {
    {
QT_MOC_LITERAL(0, 0, 23), // "FxSchematicNormalFxNode"
QT_MOC_LITERAL(1, 24, 13), // "onNameChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 21) // "onRenderToggleClicked"

    },
    "FxSchematicNormalFxNode\0onNameChanged\0"
    "\0onRenderToggleClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicNormalFxNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x09 /* Protected */,
       3,    1,   25,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void FxSchematicNormalFxNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSchematicNormalFxNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onNameChanged(); break;
        case 1: _t->onRenderToggleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicNormalFxNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicNormalFxNode.data,
    qt_meta_data_FxSchematicNormalFxNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicNormalFxNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicNormalFxNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicNormalFxNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxSchematicNormalFxNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_FxSchematicZeraryNode_t {
    QByteArrayData data[5];
    char stringdata0[86];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicZeraryNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicZeraryNode_t qt_meta_stringdata_FxSchematicZeraryNode = {
    {
QT_MOC_LITERAL(0, 0, 21), // "FxSchematicZeraryNode"
QT_MOC_LITERAL(1, 22, 21), // "onRenderToggleClicked"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 26), // "onCameraStandToggleClicked"
QT_MOC_LITERAL(4, 72, 13) // "onNameChanged"

    },
    "FxSchematicZeraryNode\0onRenderToggleClicked\0"
    "\0onCameraStandToggleClicked\0onNameChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicZeraryNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x09 /* Protected */,
       3,    1,   32,    2, 0x09 /* Protected */,
       4,    0,   35,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

       0        // eod
};

void FxSchematicZeraryNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSchematicZeraryNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onRenderToggleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onCameraStandToggleClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onNameChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicZeraryNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicZeraryNode.data,
    qt_meta_data_FxSchematicZeraryNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicZeraryNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicZeraryNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicZeraryNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxSchematicZeraryNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_FxSchematicColumnNode_t {
    QByteArrayData data[6];
    char stringdata0[100];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicColumnNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicColumnNode_t qt_meta_stringdata_FxSchematicColumnNode = {
    {
QT_MOC_LITERAL(0, 0, 21), // "FxSchematicColumnNode"
QT_MOC_LITERAL(1, 22, 21), // "onRenderToggleClicked"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 26), // "onCameraStandToggleClicked"
QT_MOC_LITERAL(4, 72, 13), // "onChangedSize"
QT_MOC_LITERAL(5, 86, 13) // "onNameChanged"

    },
    "FxSchematicColumnNode\0onRenderToggleClicked\0"
    "\0onCameraStandToggleClicked\0onChangedSize\0"
    "onNameChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicColumnNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x09 /* Protected */,
       3,    1,   37,    2, 0x09 /* Protected */,
       4,    1,   40,    2, 0x09 /* Protected */,
       5,    0,   43,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void FxSchematicColumnNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSchematicColumnNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onRenderToggleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onCameraStandToggleClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onChangedSize((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onNameChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicColumnNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicColumnNode.data,
    qt_meta_data_FxSchematicColumnNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicColumnNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicColumnNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicColumnNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxSchematicColumnNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_FxSchematicPaletteNode_t {
    QByteArrayData data[4];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicPaletteNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicPaletteNode_t qt_meta_stringdata_FxSchematicPaletteNode = {
    {
QT_MOC_LITERAL(0, 0, 22), // "FxSchematicPaletteNode"
QT_MOC_LITERAL(1, 23, 21), // "onRenderToggleClicked"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 13) // "onNameChanged"

    },
    "FxSchematicPaletteNode\0onRenderToggleClicked\0"
    "\0onNameChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicPaletteNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x09 /* Protected */,
       3,    0,   27,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void FxSchematicPaletteNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSchematicPaletteNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onRenderToggleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onNameChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicPaletteNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicPaletteNode.data,
    qt_meta_data_FxSchematicPaletteNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicPaletteNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicPaletteNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicPaletteNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxSchematicPaletteNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_FxGroupNode_t {
    QByteArrayData data[4];
    char stringdata0[49];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxGroupNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxGroupNode_t qt_meta_stringdata_FxGroupNode = {
    {
QT_MOC_LITERAL(0, 0, 11), // "FxGroupNode"
QT_MOC_LITERAL(1, 12, 13), // "onNameChanged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 21) // "onRenderToggleClicked"

    },
    "FxGroupNode\0onNameChanged\0\0"
    "onRenderToggleClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxGroupNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x09 /* Protected */,
       3,    1,   25,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void FxGroupNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxGroupNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onNameChanged(); break;
        case 1: _t->onRenderToggleClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxGroupNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxGroupNode.data,
    qt_meta_data_FxGroupNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxGroupNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxGroupNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxGroupNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxGroupNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_FxPassThroughPainter_t {
    QByteArrayData data[1];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxPassThroughPainter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxPassThroughPainter_t qt_meta_stringdata_FxPassThroughPainter = {
    {
QT_MOC_LITERAL(0, 0, 20) // "FxPassThroughPainter"

    },
    "FxPassThroughPainter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxPassThroughPainter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxPassThroughPainter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxPassThroughPainter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FxPassThroughPainter.data,
    qt_meta_data_FxPassThroughPainter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxPassThroughPainter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxPassThroughPainter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxPassThroughPainter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QGraphicsItem"))
        return static_cast< QGraphicsItem*>(this);
    return QObject::qt_metacast(_clname);
}

int FxPassThroughPainter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxSchematicPassThroughNode_t {
    QByteArrayData data[3];
    char stringdata0[42];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicPassThroughNode_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicPassThroughNode_t qt_meta_stringdata_FxSchematicPassThroughNode = {
    {
QT_MOC_LITERAL(0, 0, 26), // "FxSchematicPassThroughNode"
QT_MOC_LITERAL(1, 27, 13), // "onNameChanged"
QT_MOC_LITERAL(2, 41, 0) // ""

    },
    "FxSchematicPassThroughNode\0onNameChanged\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicPassThroughNode[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void FxSchematicPassThroughNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSchematicPassThroughNode *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onNameChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicPassThroughNode::staticMetaObject = { {
    QMetaObject::SuperData::link<FxSchematicNode::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicPassThroughNode.data,
    qt_meta_data_FxSchematicPassThroughNode,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicPassThroughNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicPassThroughNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicPassThroughNode.stringdata0))
        return static_cast<void*>(this);
    return FxSchematicNode::qt_metacast(_clname);
}

int FxSchematicPassThroughNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FxSchematicNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
