/****************************************************************************
** Meta object code from reading C++ file 'xshcolumnviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/xshcolumnviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xshcolumnviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_XsheetGUI__MotionPathMenu_t {
    QByteArrayData data[1];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__MotionPathMenu_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__MotionPathMenu_t qt_meta_stringdata_XsheetGUI__MotionPathMenu = {
    {
QT_MOC_LITERAL(0, 0, 25) // "XsheetGUI::MotionPathMenu"

    },
    "XsheetGUI::MotionPathMenu"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__MotionPathMenu[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void XsheetGUI::MotionPathMenu::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::MotionPathMenu::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__MotionPathMenu.data,
    qt_meta_data_XsheetGUI__MotionPathMenu,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::MotionPathMenu::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::MotionPathMenu::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__MotionPathMenu.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int XsheetGUI::MotionPathMenu::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__ChangeObjectWidget_t {
    QByteArrayData data[4];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__ChangeObjectWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__ChangeObjectWidget_t qt_meta_stringdata_XsheetGUI__ChangeObjectWidget = {
    {
QT_MOC_LITERAL(0, 0, 29), // "XsheetGUI::ChangeObjectWidget"
QT_MOC_LITERAL(1, 30, 14), // "onItemSelected"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 16) // "QListWidgetItem*"

    },
    "XsheetGUI::ChangeObjectWidget\0"
    "onItemSelected\0\0QListWidgetItem*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__ChangeObjectWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

       0        // eod
};

void XsheetGUI::ChangeObjectWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChangeObjectWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onItemSelected((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::ChangeObjectWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QListWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__ChangeObjectWidget.data,
    qt_meta_data_XsheetGUI__ChangeObjectWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::ChangeObjectWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::ChangeObjectWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__ChangeObjectWidget.stringdata0))
        return static_cast<void*>(this);
    return QListWidget::qt_metacast(_clname);
}

int XsheetGUI::ChangeObjectWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__ChangeObjectParent_t {
    QByteArrayData data[4];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__ChangeObjectParent_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__ChangeObjectParent_t qt_meta_stringdata_XsheetGUI__ChangeObjectParent = {
    {
QT_MOC_LITERAL(0, 0, 29), // "XsheetGUI::ChangeObjectParent"
QT_MOC_LITERAL(1, 30, 14), // "onItemSelected"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 16) // "QListWidgetItem*"

    },
    "XsheetGUI::ChangeObjectParent\0"
    "onItemSelected\0\0QListWidgetItem*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__ChangeObjectParent[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

       0        // eod
};

void XsheetGUI::ChangeObjectParent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChangeObjectParent *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onItemSelected((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::ChangeObjectParent::staticMetaObject = { {
    QMetaObject::SuperData::link<ChangeObjectWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__ChangeObjectParent.data,
    qt_meta_data_XsheetGUI__ChangeObjectParent,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::ChangeObjectParent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::ChangeObjectParent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__ChangeObjectParent.stringdata0))
        return static_cast<void*>(this);
    return ChangeObjectWidget::qt_metacast(_clname);
}

int XsheetGUI::ChangeObjectParent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ChangeObjectWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__ChangeObjectHandle_t {
    QByteArrayData data[4];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__ChangeObjectHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__ChangeObjectHandle_t qt_meta_stringdata_XsheetGUI__ChangeObjectHandle = {
    {
QT_MOC_LITERAL(0, 0, 29), // "XsheetGUI::ChangeObjectHandle"
QT_MOC_LITERAL(1, 30, 14), // "onItemSelected"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 16) // "QListWidgetItem*"

    },
    "XsheetGUI::ChangeObjectHandle\0"
    "onItemSelected\0\0QListWidgetItem*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__ChangeObjectHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

       0        // eod
};

void XsheetGUI::ChangeObjectHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChangeObjectHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onItemSelected((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::ChangeObjectHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<ChangeObjectWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__ChangeObjectHandle.data,
    qt_meta_data_XsheetGUI__ChangeObjectHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::ChangeObjectHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::ChangeObjectHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__ChangeObjectHandle.stringdata0))
        return static_cast<void*>(this);
    return ChangeObjectWidget::qt_metacast(_clname);
}

int XsheetGUI::ChangeObjectHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ChangeObjectWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__RenameColumnField_t {
    QByteArrayData data[3];
    char stringdata0[43];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__RenameColumnField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__RenameColumnField_t qt_meta_stringdata_XsheetGUI__RenameColumnField = {
    {
QT_MOC_LITERAL(0, 0, 28), // "XsheetGUI::RenameColumnField"
QT_MOC_LITERAL(1, 29, 12), // "renameColumn"
QT_MOC_LITERAL(2, 42, 0) // ""

    },
    "XsheetGUI::RenameColumnField\0renameColumn\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__RenameColumnField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void XsheetGUI::RenameColumnField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RenameColumnField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->renameColumn(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::RenameColumnField::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__RenameColumnField.data,
    qt_meta_data_XsheetGUI__RenameColumnField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::RenameColumnField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::RenameColumnField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__RenameColumnField.stringdata0))
        return static_cast<void*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int XsheetGUI::RenameColumnField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__ColumnTransparencyPopup_t {
    QByteArrayData data[10];
    char stringdata0[152];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__ColumnTransparencyPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__ColumnTransparencyPopup_t qt_meta_stringdata_XsheetGUI__ColumnTransparencyPopup = {
    {
QT_MOC_LITERAL(0, 0, 34), // "XsheetGUI::ColumnTransparency..."
QT_MOC_LITERAL(1, 35, 16), // "onSliderReleased"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 14), // "onSliderChange"
QT_MOC_LITERAL(4, 68, 3), // "val"
QT_MOC_LITERAL(5, 72, 20), // "onSliderValueChanged"
QT_MOC_LITERAL(6, 93, 14), // "onValueChanged"
QT_MOC_LITERAL(7, 108, 20), // "onFilterColorChanged"
QT_MOC_LITERAL(8, 129, 19), // "onLockButtonClicked"
QT_MOC_LITERAL(9, 149, 2) // "on"

    },
    "XsheetGUI::ColumnTransparencyPopup\0"
    "onSliderReleased\0\0onSliderChange\0val\0"
    "onSliderValueChanged\0onValueChanged\0"
    "onFilterColorChanged\0onLockButtonClicked\0"
    "on"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__ColumnTransparencyPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x09 /* Protected */,
       3,    1,   45,    2, 0x09 /* Protected */,
       5,    1,   48,    2, 0x09 /* Protected */,
       6,    1,   51,    2, 0x09 /* Protected */,
       7,    0,   54,    2, 0x09 /* Protected */,
       8,    1,   55,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    9,

       0        // eod
};

void XsheetGUI::ColumnTransparencyPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColumnTransparencyPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSliderReleased(); break;
        case 1: _t->onSliderChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onSliderValueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onValueChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->onFilterColorChanged(); break;
        case 5: _t->onLockButtonClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::ColumnTransparencyPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__ColumnTransparencyPopup.data,
    qt_meta_data_XsheetGUI__ColumnTransparencyPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::ColumnTransparencyPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::ColumnTransparencyPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__ColumnTransparencyPopup.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int XsheetGUI::ColumnTransparencyPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__SoundColumnPopup_t {
    QByteArrayData data[7];
    char stringdata0[101];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__SoundColumnPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__SoundColumnPopup_t qt_meta_stringdata_XsheetGUI__SoundColumnPopup = {
    {
QT_MOC_LITERAL(0, 0, 27), // "XsheetGUI::SoundColumnPopup"
QT_MOC_LITERAL(1, 28, 16), // "onSliderReleased"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 14), // "onSliderChange"
QT_MOC_LITERAL(4, 61, 3), // "val"
QT_MOC_LITERAL(5, 65, 20), // "onSliderValueChanged"
QT_MOC_LITERAL(6, 86, 14) // "onValueChanged"

    },
    "XsheetGUI::SoundColumnPopup\0"
    "onSliderReleased\0\0onSliderChange\0val\0"
    "onSliderValueChanged\0onValueChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__SoundColumnPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x09 /* Protected */,
       3,    1,   35,    2, 0x09 /* Protected */,
       5,    1,   38,    2, 0x09 /* Protected */,
       6,    1,   41,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void XsheetGUI::SoundColumnPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SoundColumnPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSliderReleased(); break;
        case 1: _t->onSliderChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onSliderValueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onValueChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::SoundColumnPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__SoundColumnPopup.data,
    qt_meta_data_XsheetGUI__SoundColumnPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::SoundColumnPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::SoundColumnPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__SoundColumnPopup.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int XsheetGUI::SoundColumnPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__ColumnArea_t {
    QByteArrayData data[12];
    char stringdata0[203];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__ColumnArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__ColumnArea_t qt_meta_stringdata_XsheetGUI__ColumnArea = {
    {
QT_MOC_LITERAL(0, 0, 21), // "XsheetGUI::ColumnArea"
QT_MOC_LITERAL(1, 22, 13), // "onSubSampling"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 8), // "QAction*"
QT_MOC_LITERAL(4, 46, 21), // "openTransparencyPopup"
QT_MOC_LITERAL(5, 68, 20), // "openSoundColumnPopup"
QT_MOC_LITERAL(6, 89, 21), // "openCameraColumnPopup"
QT_MOC_LITERAL(7, 111, 3), // "pos"
QT_MOC_LITERAL(8, 115, 30), // "onCameraColumnChangedTriggered"
QT_MOC_LITERAL(9, 146, 25), // "onCameraColumnLockToggled"
QT_MOC_LITERAL(10, 172, 20), // "onXsheetCameraChange"
QT_MOC_LITERAL(11, 193, 9) // "onSetMask"

    },
    "XsheetGUI::ColumnArea\0onSubSampling\0"
    "\0QAction*\0openTransparencyPopup\0"
    "openSoundColumnPopup\0openCameraColumnPopup\0"
    "pos\0onCameraColumnChangedTriggered\0"
    "onCameraColumnLockToggled\0"
    "onXsheetCameraChange\0onSetMask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__ColumnArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x09 /* Protected */,
       4,    0,   57,    2, 0x09 /* Protected */,
       5,    0,   58,    2, 0x09 /* Protected */,
       6,    1,   59,    2, 0x09 /* Protected */,
       8,    0,   62,    2, 0x09 /* Protected */,
       9,    1,   63,    2, 0x09 /* Protected */,
      10,    1,   66,    2, 0x09 /* Protected */,
      11,    1,   69,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void XsheetGUI::ColumnArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColumnArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSubSampling((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 1: _t->openTransparencyPopup(); break;
        case 2: _t->openSoundColumnPopup(); break;
        case 3: _t->openCameraColumnPopup((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 4: _t->onCameraColumnChangedTriggered(); break;
        case 5: _t->onCameraColumnLockToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->onXsheetCameraChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onSetMask((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::ColumnArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__ColumnArea.data,
    qt_meta_data_XsheetGUI__ColumnArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::ColumnArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::ColumnArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__ColumnArea.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int XsheetGUI::ColumnArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
