/****************************************************************************
** Meta object code from reading C++ file 'scriptbinding_centerline_vectorizer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/scriptbinding_centerline_vectorizer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scriptbinding_centerline_vectorizer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TScriptBinding__CenterlineVectorizer_t {
    QByteArrayData data[14];
    char stringdata0[183];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TScriptBinding__CenterlineVectorizer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TScriptBinding__CenterlineVectorizer_t qt_meta_stringdata_TScriptBinding__CenterlineVectorizer = {
    {
QT_MOC_LITERAL(0, 0, 36), // "TScriptBinding::CenterlineVec..."
QT_MOC_LITERAL(1, 37, 8), // "toString"
QT_MOC_LITERAL(2, 46, 12), // "QScriptValue"
QT_MOC_LITERAL(3, 59, 0), // ""
QT_MOC_LITERAL(4, 60, 9), // "vectorize"
QT_MOC_LITERAL(5, 70, 12), // "img_or_level"
QT_MOC_LITERAL(6, 83, 9), // "threshold"
QT_MOC_LITERAL(7, 93, 8), // "accuracy"
QT_MOC_LITERAL(8, 102, 11), // "despeckling"
QT_MOC_LITERAL(9, 114, 12), // "maxThickness"
QT_MOC_LITERAL(10, 127, 20), // "thicknessCalibration"
QT_MOC_LITERAL(11, 148, 20), // "preservePaintedAreas"
QT_MOC_LITERAL(12, 169, 9), // "addBorder"
QT_MOC_LITERAL(13, 179, 3) // "eir"

    },
    "TScriptBinding::CenterlineVectorizer\0"
    "toString\0QScriptValue\0\0vectorize\0"
    "img_or_level\0threshold\0accuracy\0"
    "despeckling\0maxThickness\0thicknessCalibration\0"
    "preservePaintedAreas\0addBorder\0eir"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TScriptBinding__CenterlineVectorizer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       8,   28, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    0,   24,    3, 0x02 /* Public */,
       4,    1,   25,    3, 0x02 /* Public */,

 // methods: parameters
    0x80000000 | 2,
    0x80000000 | 2, 0x80000000 | 2,    5,

 // properties: name, type, flags
       6, QMetaType::Int, 0x00095103,
       7, QMetaType::Int, 0x00095103,
       8, QMetaType::Int, 0x00095103,
       9, QMetaType::Double, 0x00095103,
      10, QMetaType::Double, 0x00095103,
      11, QMetaType::Bool, 0x00095103,
      12, QMetaType::Bool, 0x00095103,
      13, QMetaType::Bool, 0x00095103,

       0        // eod
};

void TScriptBinding::CenterlineVectorizer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CenterlineVectorizer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QScriptValue _r = _t->toString();
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QScriptValue _r = _t->vectorize((*reinterpret_cast< QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<CenterlineVectorizer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->getThreshold(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->getAccuracy(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->getDespeckling(); break;
        case 3: *reinterpret_cast< double*>(_v) = _t->getMaxThickness(); break;
        case 4: *reinterpret_cast< double*>(_v) = _t->getThicknessCalibration(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->getPreservePaintedAreas(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->getAddBorder(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->getEir(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<CenterlineVectorizer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setThreshold(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setAccuracy(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setDespeckling(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setMaxThickness(*reinterpret_cast< double*>(_v)); break;
        case 4: _t->setThicknessCalibration(*reinterpret_cast< double*>(_v)); break;
        case 5: _t->setPreservePaintedAreas(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setAddBorder(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setEir(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TScriptBinding::CenterlineVectorizer::staticMetaObject = { {
    QMetaObject::SuperData::link<Wrapper::staticMetaObject>(),
    qt_meta_stringdata_TScriptBinding__CenterlineVectorizer.data,
    qt_meta_data_TScriptBinding__CenterlineVectorizer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TScriptBinding::CenterlineVectorizer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TScriptBinding::CenterlineVectorizer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TScriptBinding__CenterlineVectorizer.stringdata0))
        return static_cast<void*>(this);
    return Wrapper::qt_metacast(_clname);
}

int TScriptBinding::CenterlineVectorizer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Wrapper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
