/****************************************************************************
** Meta object code from reading C++ file 'dvdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/dvdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dvdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__Separator_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__Separator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__Separator_t qt_meta_stringdata_DVGui__Separator = {
    {
QT_MOC_LITERAL(0, 0, 16) // "DVGui::Separator"

    },
    "DVGui::Separator"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__Separator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void DVGui::Separator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::Separator::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_DVGui__Separator.data,
    qt_meta_data_DVGui__Separator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::Separator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::Separator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__Separator.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int DVGui::Separator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_DVGui__Dialog_t {
    QByteArrayData data[3];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__Dialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__Dialog_t qt_meta_stringdata_DVGui__Dialog = {
    {
QT_MOC_LITERAL(0, 0, 13), // "DVGui::Dialog"
QT_MOC_LITERAL(1, 14, 12), // "dialogClosed"
QT_MOC_LITERAL(2, 27, 0) // ""

    },
    "DVGui::Dialog\0dialogClosed\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__Dialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void DVGui::Dialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Dialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->dialogClosed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Dialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Dialog::dialogClosed)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::Dialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_DVGui__Dialog.data,
    qt_meta_data_DVGui__Dialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::Dialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::Dialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__Dialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int DVGui::Dialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DVGui::Dialog::dialogClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_DVGui__MessageAndCheckboxDialog_t {
    QByteArrayData data[6];
    char stringdata0[78];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__MessageAndCheckboxDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__MessageAndCheckboxDialog_t qt_meta_stringdata_DVGui__MessageAndCheckboxDialog = {
    {
QT_MOC_LITERAL(0, 0, 31), // "DVGui::MessageAndCheckboxDialog"
QT_MOC_LITERAL(1, 32, 17), // "onCheckboxChanged"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 7), // "checked"
QT_MOC_LITERAL(4, 59, 15), // "onButtonClicked"
QT_MOC_LITERAL(5, 75, 2) // "id"

    },
    "DVGui::MessageAndCheckboxDialog\0"
    "onCheckboxChanged\0\0checked\0onButtonClicked\0"
    "id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__MessageAndCheckboxDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x0a /* Public */,
       4,    1,   27,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,

       0        // eod
};

void DVGui::MessageAndCheckboxDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MessageAndCheckboxDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onCheckboxChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onButtonClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::MessageAndCheckboxDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_DVGui__MessageAndCheckboxDialog.data,
    qt_meta_data_DVGui__MessageAndCheckboxDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::MessageAndCheckboxDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::MessageAndCheckboxDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__MessageAndCheckboxDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int DVGui::MessageAndCheckboxDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__RadioButtonDialog_t {
    QByteArrayData data[6];
    char stringdata0[62];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__RadioButtonDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__RadioButtonDialog_t qt_meta_stringdata_DVGui__RadioButtonDialog = {
    {
QT_MOC_LITERAL(0, 0, 24), // "DVGui::RadioButtonDialog"
QT_MOC_LITERAL(1, 25, 15), // "onButtonClicked"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 2), // "id"
QT_MOC_LITERAL(4, 45, 8), // "onCancel"
QT_MOC_LITERAL(5, 54, 7) // "onApply"

    },
    "DVGui::RadioButtonDialog\0onButtonClicked\0"
    "\0id\0onCancel\0onApply"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__RadioButtonDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x0a /* Public */,
       4,    0,   32,    2, 0x0a /* Public */,
       5,    0,   33,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void DVGui::RadioButtonDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RadioButtonDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onButtonClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onCancel(); break;
        case 2: _t->onApply(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::RadioButtonDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_DVGui__RadioButtonDialog.data,
    qt_meta_data_DVGui__RadioButtonDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::RadioButtonDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::RadioButtonDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__RadioButtonDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int DVGui::RadioButtonDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__ProgressDialog_t {
    QByteArrayData data[6];
    char stringdata0[59];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__ProgressDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__ProgressDialog_t qt_meta_stringdata_DVGui__ProgressDialog = {
    {
QT_MOC_LITERAL(0, 0, 21), // "DVGui::ProgressDialog"
QT_MOC_LITERAL(1, 22, 8), // "canceled"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 8), // "setValue"
QT_MOC_LITERAL(4, 41, 8), // "progress"
QT_MOC_LITERAL(5, 50, 8) // "onCancel"

    },
    "DVGui::ProgressDialog\0canceled\0\0"
    "setValue\0progress\0onCancel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__ProgressDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   30,    2, 0x0a /* Public */,
       5,    0,   33,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,

       0        // eod
};

void DVGui::ProgressDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ProgressDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->canceled(); break;
        case 1: _t->setValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onCancel(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ProgressDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ProgressDialog::canceled)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::ProgressDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_DVGui__ProgressDialog.data,
    qt_meta_data_DVGui__ProgressDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::ProgressDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::ProgressDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__ProgressDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int DVGui::ProgressDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void DVGui::ProgressDialog::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
