/****************************************************************************
** Meta object code from reading C++ file 'tpalettehandle.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/tpalettehandle.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tpalettehandle.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TPaletteHandle_t {
    QByteArrayData data[15];
    char stringdata0[328];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPaletteHandle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPaletteHandle_t qt_meta_stringdata_TPaletteHandle = {
    {
QT_MOC_LITERAL(0, 0, 14), // "TPaletteHandle"
QT_MOC_LITERAL(1, 15, 15), // "paletteSwitched"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 14), // "paletteChanged"
QT_MOC_LITERAL(4, 47, 19), // "paletteTitleChanged"
QT_MOC_LITERAL(5, 67, 18), // "colorStyleSwitched"
QT_MOC_LITERAL(6, 86, 17), // "colorStyleChanged"
QT_MOC_LITERAL(7, 104, 31), // "colorStyleChangedOnMouseRelease"
QT_MOC_LITERAL(8, 136, 23), // "paletteDirtyFlagChanged"
QT_MOC_LITERAL(9, 160, 18), // "paletteLockChanged"
QT_MOC_LITERAL(10, 179, 23), // "broadcastPaletteChanged"
QT_MOC_LITERAL(11, 203, 28), // "broadcastPaletteTitleChanged"
QT_MOC_LITERAL(12, 232, 27), // "broadcastColorStyleSwitched"
QT_MOC_LITERAL(13, 260, 26), // "broadcastColorStyleChanged"
QT_MOC_LITERAL(14, 287, 40) // "broadcastColorStyleChangedOnM..."

    },
    "TPaletteHandle\0paletteSwitched\0\0"
    "paletteChanged\0paletteTitleChanged\0"
    "colorStyleSwitched\0colorStyleChanged\0"
    "colorStyleChangedOnMouseRelease\0"
    "paletteDirtyFlagChanged\0paletteLockChanged\0"
    "broadcastPaletteChanged\0"
    "broadcastPaletteTitleChanged\0"
    "broadcastColorStyleSwitched\0"
    "broadcastColorStyleChanged\0"
    "broadcastColorStyleChangedOnMouseRelease"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPaletteHandle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      13,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x06 /* Public */,
       3,    0,   80,    2, 0x06 /* Public */,
       4,    0,   81,    2, 0x06 /* Public */,
       5,    0,   82,    2, 0x06 /* Public */,
       6,    1,   83,    2, 0x06 /* Public */,
       7,    0,   86,    2, 0x06 /* Public */,
       8,    0,   87,    2, 0x06 /* Public */,
       9,    0,   88,    2, 0x06 /* Public */,
      10,    0,   89,    2, 0x06 /* Public */,
      11,    0,   90,    2, 0x06 /* Public */,
      12,    0,   91,    2, 0x06 /* Public */,
      13,    1,   92,    2, 0x06 /* Public */,
      14,    0,   95,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void TPaletteHandle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TPaletteHandle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->paletteSwitched(); break;
        case 1: _t->paletteChanged(); break;
        case 2: _t->paletteTitleChanged(); break;
        case 3: _t->colorStyleSwitched(); break;
        case 4: _t->colorStyleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->colorStyleChangedOnMouseRelease(); break;
        case 6: _t->paletteDirtyFlagChanged(); break;
        case 7: _t->paletteLockChanged(); break;
        case 8: _t->broadcastPaletteChanged(); break;
        case 9: _t->broadcastPaletteTitleChanged(); break;
        case 10: _t->broadcastColorStyleSwitched(); break;
        case 11: _t->broadcastColorStyleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->broadcastColorStyleChangedOnMouseRelease(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::paletteSwitched)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::paletteChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::paletteTitleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::colorStyleSwitched)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::colorStyleChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::colorStyleChangedOnMouseRelease)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::paletteDirtyFlagChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::paletteLockChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::broadcastPaletteChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::broadcastPaletteTitleChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::broadcastColorStyleSwitched)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::broadcastColorStyleChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (TPaletteHandle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPaletteHandle::broadcastColorStyleChangedOnMouseRelease)) {
                *result = 12;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TPaletteHandle::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TPaletteHandle.data,
    qt_meta_data_TPaletteHandle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPaletteHandle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPaletteHandle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPaletteHandle.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TPaletteHandle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void TPaletteHandle::paletteSwitched()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TPaletteHandle::paletteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TPaletteHandle::paletteTitleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TPaletteHandle::colorStyleSwitched()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TPaletteHandle::colorStyleChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void TPaletteHandle::colorStyleChangedOnMouseRelease()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TPaletteHandle::paletteDirtyFlagChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void TPaletteHandle::paletteLockChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void TPaletteHandle::broadcastPaletteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void TPaletteHandle::broadcastPaletteTitleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TPaletteHandle::broadcastColorStyleSwitched()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void TPaletteHandle::broadcastColorStyleChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void TPaletteHandle::broadcastColorStyleChangedOnMouseRelease()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
