/****************************************************************************
** Meta object code from reading C++ file 'preferencespopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/preferencespopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'preferencespopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SizeField_t {
    QByteArrayData data[3];
    char stringdata0[27];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SizeField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SizeField_t qt_meta_stringdata_SizeField = {
    {
QT_MOC_LITERAL(0, 0, 9), // "SizeField"
QT_MOC_LITERAL(1, 10, 15), // "editingFinished"
QT_MOC_LITERAL(2, 26, 0) // ""

    },
    "SizeField\0editingFinished\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SizeField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void SizeField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SizeField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editingFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SizeField::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SizeField::editingFinished)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SizeField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SizeField.data,
    qt_meta_data_SizeField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SizeField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SizeField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SizeField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SizeField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void SizeField::editingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_PreferencesPopup_t {
    QByteArrayData data[25];
    char stringdata0[493];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreferencesPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreferencesPopup_t qt_meta_stringdata_PreferencesPopup = {
    {
QT_MOC_LITERAL(0, 0, 16), // "PreferencesPopup"
QT_MOC_LITERAL(1, 17, 8), // "onChange"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 19), // "onColorFieldChanged"
QT_MOC_LITERAL(4, 47, 8), // "TPixel32"
QT_MOC_LITERAL(5, 56, 20), // "onAnimateToolChanged"
QT_MOC_LITERAL(6, 77, 27), // "onAutoSaveExternallyChanged"
QT_MOC_LITERAL(7, 105, 33), // "onAutoSavePeriodExternallyCha..."
QT_MOC_LITERAL(8, 139, 20), // "onProjectRootChanged"
QT_MOC_LITERAL(9, 160, 26), // "onEditAdditionalStyleSheet"
QT_MOC_LITERAL(10, 187, 28), // "onAdditionalStyleSheetEdited"
QT_MOC_LITERAL(11, 216, 29), // "onPixelUnitExternallySelected"
QT_MOC_LITERAL(12, 246, 2), // "on"
QT_MOC_LITERAL(13, 249, 22), // "onInterfaceFontChanged"
QT_MOC_LITERAL(14, 272, 4), // "text"
QT_MOC_LITERAL(15, 277, 16), // "onLutPathChanged"
QT_MOC_LITERAL(16, 294, 19), // "onCheck30bitDisplay"
QT_MOC_LITERAL(17, 314, 16), // "onAddLevelFormat"
QT_MOC_LITERAL(18, 331, 19), // "onRemoveLevelFormat"
QT_MOC_LITERAL(19, 351, 17), // "onEditLevelFormat"
QT_MOC_LITERAL(20, 369, 19), // "onLevelFormatEdited"
QT_MOC_LITERAL(21, 389, 31), // "onImportPolicyExternallyChanged"
QT_MOC_LITERAL(22, 421, 6), // "policy"
QT_MOC_LITERAL(23, 428, 31), // "onRenamePolicyExternallyChanged"
QT_MOC_LITERAL(24, 460, 32) // "onConvertPolicyExternallyChanged"

    },
    "PreferencesPopup\0onChange\0\0"
    "onColorFieldChanged\0TPixel32\0"
    "onAnimateToolChanged\0onAutoSaveExternallyChanged\0"
    "onAutoSavePeriodExternallyChanged\0"
    "onProjectRootChanged\0onEditAdditionalStyleSheet\0"
    "onAdditionalStyleSheetEdited\0"
    "onPixelUnitExternallySelected\0on\0"
    "onInterfaceFontChanged\0text\0"
    "onLutPathChanged\0onCheck30bitDisplay\0"
    "onAddLevelFormat\0onRemoveLevelFormat\0"
    "onEditLevelFormat\0onLevelFormatEdited\0"
    "onImportPolicyExternallyChanged\0policy\0"
    "onRenamePolicyExternallyChanged\0"
    "onConvertPolicyExternallyChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreferencesPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  109,    2, 0x08 /* Private */,
       3,    2,  110,    2, 0x08 /* Private */,
       5,    0,  115,    2, 0x08 /* Private */,
       6,    0,  116,    2, 0x08 /* Private */,
       7,    0,  117,    2, 0x08 /* Private */,
       8,    0,  118,    2, 0x08 /* Private */,
       9,    0,  119,    2, 0x08 /* Private */,
      10,    0,  120,    2, 0x08 /* Private */,
      11,    1,  121,    2, 0x08 /* Private */,
      13,    1,  124,    2, 0x08 /* Private */,
      15,    0,  127,    2, 0x08 /* Private */,
      16,    0,  128,    2, 0x08 /* Private */,
      17,    0,  129,    2, 0x08 /* Private */,
      18,    0,  130,    2, 0x08 /* Private */,
      19,    0,  131,    2, 0x08 /* Private */,
      20,    0,  132,    2, 0x08 /* Private */,
      21,    1,  133,    2, 0x08 /* Private */,
      23,    1,  136,    2, 0x08 /* Private */,
      24,    1,  139,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Bool,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int,   22,

       0        // eod
};

void PreferencesPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PreferencesPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onChange(); break;
        case 1: _t->onColorFieldChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->onAnimateToolChanged(); break;
        case 3: _t->onAutoSaveExternallyChanged(); break;
        case 4: _t->onAutoSavePeriodExternallyChanged(); break;
        case 5: _t->onProjectRootChanged(); break;
        case 6: _t->onEditAdditionalStyleSheet(); break;
        case 7: _t->onAdditionalStyleSheetEdited(); break;
        case 8: _t->onPixelUnitExternallySelected((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->onInterfaceFontChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->onLutPathChanged(); break;
        case 11: _t->onCheck30bitDisplay(); break;
        case 12: _t->onAddLevelFormat(); break;
        case 13: _t->onRemoveLevelFormat(); break;
        case 14: _t->onEditLevelFormat(); break;
        case 15: _t->onLevelFormatEdited(); break;
        case 16: _t->onImportPolicyExternallyChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->onRenamePolicyExternallyChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->onConvertPolicyExternallyChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PreferencesPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_PreferencesPopup.data,
    qt_meta_data_PreferencesPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreferencesPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreferencesPopup.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int PreferencesPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}
struct qt_meta_stringdata_PreferencesPopup__FormatProperties_t {
    QByteArrayData data[3];
    char stringdata0[56];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreferencesPopup__FormatProperties_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreferencesPopup__FormatProperties_t qt_meta_stringdata_PreferencesPopup__FormatProperties = {
    {
QT_MOC_LITERAL(0, 0, 34), // "PreferencesPopup::FormatPrope..."
QT_MOC_LITERAL(1, 35, 19), // "updateEnabledStatus"
QT_MOC_LITERAL(2, 55, 0) // ""

    },
    "PreferencesPopup::FormatProperties\0"
    "updateEnabledStatus\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreferencesPopup__FormatProperties[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void PreferencesPopup::FormatProperties::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FormatProperties *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateEnabledStatus(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PreferencesPopup::FormatProperties::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_PreferencesPopup__FormatProperties.data,
    qt_meta_data_PreferencesPopup__FormatProperties,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreferencesPopup::FormatProperties::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesPopup::FormatProperties::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreferencesPopup__FormatProperties.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int PreferencesPopup::FormatProperties::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_PreferencesPopup__AdditionalStyleEdit_t {
    QByteArrayData data[5];
    char stringdata0[74];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreferencesPopup__AdditionalStyleEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreferencesPopup__AdditionalStyleEdit_t qt_meta_stringdata_PreferencesPopup__AdditionalStyleEdit = {
    {
QT_MOC_LITERAL(0, 0, 37), // "PreferencesPopup::AdditionalS..."
QT_MOC_LITERAL(1, 38, 21), // "additionalSheetEdited"
QT_MOC_LITERAL(2, 60, 0), // ""
QT_MOC_LITERAL(3, 61, 4), // "onOK"
QT_MOC_LITERAL(4, 66, 7) // "onApply"

    },
    "PreferencesPopup::AdditionalStyleEdit\0"
    "additionalSheetEdited\0\0onOK\0onApply"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreferencesPopup__AdditionalStyleEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   30,    2, 0x08 /* Private */,
       4,    0,   31,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PreferencesPopup::AdditionalStyleEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AdditionalStyleEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->additionalSheetEdited(); break;
        case 1: _t->onOK(); break;
        case 2: _t->onApply(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AdditionalStyleEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AdditionalStyleEdit::additionalSheetEdited)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PreferencesPopup::AdditionalStyleEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_PreferencesPopup__AdditionalStyleEdit.data,
    qt_meta_data_PreferencesPopup__AdditionalStyleEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreferencesPopup::AdditionalStyleEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesPopup::AdditionalStyleEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreferencesPopup__AdditionalStyleEdit.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int PreferencesPopup::AdditionalStyleEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void PreferencesPopup::AdditionalStyleEdit::additionalSheetEdited()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_PreferencesPopup__Display30bitChecker_t {
    QByteArrayData data[1];
    char stringdata0[38];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreferencesPopup__Display30bitChecker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreferencesPopup__Display30bitChecker_t qt_meta_stringdata_PreferencesPopup__Display30bitChecker = {
    {
QT_MOC_LITERAL(0, 0, 37) // "PreferencesPopup::Display30bi..."

    },
    "PreferencesPopup::Display30bitChecker"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreferencesPopup__Display30bitChecker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void PreferencesPopup::Display30bitChecker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PreferencesPopup::Display30bitChecker::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_PreferencesPopup__Display30bitChecker.data,
    qt_meta_data_PreferencesPopup__Display30bitChecker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreferencesPopup::Display30bitChecker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesPopup::Display30bitChecker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreferencesPopup__Display30bitChecker.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int PreferencesPopup::Display30bitChecker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_PreferencesPopup__Display30bitChecker__GLView_t {
    QByteArrayData data[1];
    char stringdata0[46];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PreferencesPopup__Display30bitChecker__GLView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PreferencesPopup__Display30bitChecker__GLView_t qt_meta_stringdata_PreferencesPopup__Display30bitChecker__GLView = {
    {
QT_MOC_LITERAL(0, 0, 45) // "PreferencesPopup::Display30bi..."

    },
    "PreferencesPopup::Display30bitChecker::GLView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PreferencesPopup__Display30bitChecker__GLView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void PreferencesPopup::Display30bitChecker::GLView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PreferencesPopup::Display30bitChecker::GLView::staticMetaObject = { {
    QMetaObject::SuperData::link<QOpenGLWidget::staticMetaObject>(),
    qt_meta_stringdata_PreferencesPopup__Display30bitChecker__GLView.data,
    qt_meta_data_PreferencesPopup__Display30bitChecker__GLView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PreferencesPopup::Display30bitChecker::GLView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PreferencesPopup::Display30bitChecker::GLView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PreferencesPopup__Display30bitChecker__GLView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QOpenGLFunctions"))
        return static_cast< QOpenGLFunctions*>(this);
    return QOpenGLWidget::qt_metacast(_clname);
}

int PreferencesPopup::Display30bitChecker::GLView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QOpenGLWidget::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
