/****************************************************************************
** Meta object code from reading C++ file 'styleeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/styleeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'styleeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StyleEditorGUI__HexagonalColorWheel_t {
    QByteArrayData data[8];
    char stringdata0[114];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__HexagonalColorWheel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__HexagonalColorWheel_t qt_meta_stringdata_StyleEditorGUI__HexagonalColorWheel = {
    {
QT_MOC_LITERAL(0, 0, 35), // "StyleEditorGUI::HexagonalColo..."
QT_MOC_LITERAL(1, 36, 12), // "colorChanged"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 10), // "ColorModel"
QT_MOC_LITERAL(4, 61, 5), // "color"
QT_MOC_LITERAL(5, 67, 10), // "isDragging"
QT_MOC_LITERAL(6, 78, 27), // "onContextAboutToBeDestroyed"
QT_MOC_LITERAL(7, 106, 7) // "BGColor"

    },
    "StyleEditorGUI::HexagonalColorWheel\0"
    "colorChanged\0\0ColorModel\0color\0"
    "isDragging\0onContextAboutToBeDestroyed\0"
    "BGColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__HexagonalColorWheel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       1,   30, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   29,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
       7, QMetaType::QColor, 0x00095103,

       0        // eod
};

void StyleEditorGUI::HexagonalColorWheel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HexagonalColorWheel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast< const ColorModel(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onContextAboutToBeDestroyed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HexagonalColorWheel::*)(const ColorModel & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HexagonalColorWheel::colorChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<HexagonalColorWheel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getBGColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<HexagonalColorWheel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBGColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::HexagonalColorWheel::staticMetaObject = { {
    QMetaObject::SuperData::link<GLWidgetForHighDpi::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__HexagonalColorWheel.data,
    qt_meta_data_StyleEditorGUI__HexagonalColorWheel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::HexagonalColorWheel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::HexagonalColorWheel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__HexagonalColorWheel.stringdata0))
        return static_cast<void*>(this);
    return GLWidgetForHighDpi::qt_metacast(_clname);
}

int StyleEditorGUI::HexagonalColorWheel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GLWidgetForHighDpi::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::HexagonalColorWheel::colorChanged(const ColorModel & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StyleEditorGUI__SquaredColorWheel_t {
    QByteArrayData data[8];
    char stringdata0[95];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__SquaredColorWheel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__SquaredColorWheel_t qt_meta_stringdata_StyleEditorGUI__SquaredColorWheel = {
    {
QT_MOC_LITERAL(0, 0, 33), // "StyleEditorGUI::SquaredColorW..."
QT_MOC_LITERAL(1, 34, 12), // "colorChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 10), // "ColorModel"
QT_MOC_LITERAL(4, 59, 5), // "color"
QT_MOC_LITERAL(5, 65, 10), // "isDragging"
QT_MOC_LITERAL(6, 76, 10), // "setChannel"
QT_MOC_LITERAL(7, 87, 7) // "channel"

    },
    "StyleEditorGUI::SquaredColorWheel\0"
    "colorChanged\0\0ColorModel\0color\0"
    "isDragging\0setChannel\0channel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__SquaredColorWheel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   29,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    7,

       0        // eod
};

void StyleEditorGUI::SquaredColorWheel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SquaredColorWheel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast< const ColorModel(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->setChannel((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SquaredColorWheel::*)(const ColorModel & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SquaredColorWheel::colorChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::SquaredColorWheel::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__SquaredColorWheel.data,
    qt_meta_data_StyleEditorGUI__SquaredColorWheel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::SquaredColorWheel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::SquaredColorWheel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__SquaredColorWheel.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int StyleEditorGUI::SquaredColorWheel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::SquaredColorWheel::colorChanged(const ColorModel & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StyleEditorGUI__ColorSlider_t {
    QByteArrayData data[1];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__ColorSlider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__ColorSlider_t qt_meta_stringdata_StyleEditorGUI__ColorSlider = {
    {
QT_MOC_LITERAL(0, 0, 27) // "StyleEditorGUI::ColorSlider"

    },
    "StyleEditorGUI::ColorSlider"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__ColorSlider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void StyleEditorGUI::ColorSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::ColorSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSlider::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__ColorSlider.data,
    qt_meta_data_StyleEditorGUI__ColorSlider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::ColorSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::ColorSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__ColorSlider.stringdata0))
        return static_cast<void*>(this);
    return QAbstractSlider::qt_metacast(_clname);
}

int StyleEditorGUI::ColorSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSlider::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_StyleEditorGUI__ArrowButton_t {
    QByteArrayData data[6];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__ArrowButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__ArrowButton_t qt_meta_stringdata_StyleEditorGUI__ArrowButton = {
    {
QT_MOC_LITERAL(0, 0, 27), // "StyleEditorGUI::ArrowButton"
QT_MOC_LITERAL(1, 28, 3), // "add"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 6), // "remove"
QT_MOC_LITERAL(4, 40, 9), // "onPressed"
QT_MOC_LITERAL(5, 50, 9) // "onRelease"

    },
    "StyleEditorGUI::ArrowButton\0add\0\0"
    "remove\0onPressed\0onRelease"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__ArrowButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,
       3,    0,   35,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   36,    2, 0x09 /* Protected */,
       5,    0,   37,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StyleEditorGUI::ArrowButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ArrowButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->add(); break;
        case 1: _t->remove(); break;
        case 2: _t->onPressed(); break;
        case 3: _t->onRelease(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ArrowButton::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ArrowButton::add)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ArrowButton::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ArrowButton::remove)) {
                *result = 1;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::ArrowButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolButton::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__ArrowButton.data,
    qt_meta_data_StyleEditorGUI__ArrowButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::ArrowButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::ArrowButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__ArrowButton.stringdata0))
        return static_cast<void*>(this);
    return QToolButton::qt_metacast(_clname);
}

int StyleEditorGUI::ArrowButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::ArrowButton::add()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void StyleEditorGUI::ArrowButton::remove()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_StyleEditorGUI__ColorSliderBar_t {
    QByteArrayData data[5];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__ColorSliderBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__ColorSliderBar_t qt_meta_stringdata_StyleEditorGUI__ColorSliderBar = {
    {
QT_MOC_LITERAL(0, 0, 30), // "StyleEditorGUI::ColorSliderBar"
QT_MOC_LITERAL(1, 31, 12), // "valueChanged"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 8), // "onRemove"
QT_MOC_LITERAL(4, 54, 5) // "onAdd"

    },
    "StyleEditorGUI::ColorSliderBar\0"
    "valueChanged\0\0onRemove\0onAdd"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__ColorSliderBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       1,    0,   37,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   38,    2, 0x09 /* Protected */,
       4,    0,   39,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StyleEditorGUI::ColorSliderBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorSliderBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->valueChanged(); break;
        case 2: _t->onRemove(); break;
        case 3: _t->onAdd(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorSliderBar::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorSliderBar::valueChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ColorSliderBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorSliderBar::valueChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::ColorSliderBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__ColorSliderBar.data,
    qt_meta_data_StyleEditorGUI__ColorSliderBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::ColorSliderBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::ColorSliderBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__ColorSliderBar.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int StyleEditorGUI::ColorSliderBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::ColorSliderBar::valueChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void StyleEditorGUI::ColorSliderBar::valueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_StyleEditorGUI__ChannelLineEdit_t {
    QByteArrayData data[1];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__ChannelLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__ChannelLineEdit_t qt_meta_stringdata_StyleEditorGUI__ChannelLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 31) // "StyleEditorGUI::ChannelLineEdit"

    },
    "StyleEditorGUI::ChannelLineEdit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__ChannelLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void StyleEditorGUI::ChannelLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::ChannelLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::IntLineEdit::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__ChannelLineEdit.data,
    qt_meta_data_StyleEditorGUI__ChannelLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::ChannelLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::ChannelLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__ChannelLineEdit.stringdata0))
        return static_cast<void*>(this);
    return DVGui::IntLineEdit::qt_metacast(_clname);
}

int StyleEditorGUI::ChannelLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::IntLineEdit::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_StyleEditorGUI__ColorChannelControl_t {
    QByteArrayData data[12];
    char stringdata0[170];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__ColorChannelControl_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__ColorChannelControl_t qt_meta_stringdata_StyleEditorGUI__ColorChannelControl = {
    {
QT_MOC_LITERAL(0, 0, 35), // "StyleEditorGUI::ColorChannelC..."
QT_MOC_LITERAL(1, 36, 12), // "colorChanged"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 10), // "ColorModel"
QT_MOC_LITERAL(4, 61, 5), // "color"
QT_MOC_LITERAL(5, 67, 10), // "isDragging"
QT_MOC_LITERAL(6, 78, 14), // "onFieldChanged"
QT_MOC_LITERAL(7, 93, 15), // "onSliderChanged"
QT_MOC_LITERAL(8, 109, 5), // "value"
QT_MOC_LITERAL(9, 115, 16), // "onSliderReleased"
QT_MOC_LITERAL(10, 132, 18), // "onAddButtonClicked"
QT_MOC_LITERAL(11, 151, 18) // "onSubButtonClicked"

    },
    "StyleEditorGUI::ColorChannelControl\0"
    "colorChanged\0\0ColorModel\0color\0"
    "isDragging\0onFieldChanged\0onSliderChanged\0"
    "value\0onSliderReleased\0onAddButtonClicked\0"
    "onSubButtonClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__ColorChannelControl[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   44,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   49,    2, 0x09 /* Protected */,
       7,    1,   50,    2, 0x09 /* Protected */,
       9,    0,   53,    2, 0x09 /* Protected */,
      10,    0,   54,    2, 0x09 /* Protected */,
      11,    0,   55,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StyleEditorGUI::ColorChannelControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorChannelControl *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast< const ColorModel(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onFieldChanged(); break;
        case 2: _t->onSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onSliderReleased(); break;
        case 4: _t->onAddButtonClicked(); break;
        case 5: _t->onSubButtonClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorChannelControl::*)(const ColorModel & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorChannelControl::colorChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::ColorChannelControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__ColorChannelControl.data,
    qt_meta_data_StyleEditorGUI__ColorChannelControl,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::ColorChannelControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::ColorChannelControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__ColorChannelControl.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int StyleEditorGUI::ColorChannelControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::ColorChannelControl::colorChanged(const ColorModel & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StyleEditorGUI__ColorParameterSelector_t {
    QByteArrayData data[3];
    char stringdata0[58];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__ColorParameterSelector_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__ColorParameterSelector_t qt_meta_stringdata_StyleEditorGUI__ColorParameterSelector = {
    {
QT_MOC_LITERAL(0, 0, 38), // "StyleEditorGUI::ColorParamete..."
QT_MOC_LITERAL(1, 39, 17), // "colorParamChanged"
QT_MOC_LITERAL(2, 57, 0) // ""

    },
    "StyleEditorGUI::ColorParameterSelector\0"
    "colorParamChanged\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__ColorParameterSelector[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void StyleEditorGUI::ColorParameterSelector::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorParameterSelector *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->colorParamChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorParameterSelector::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorParameterSelector::colorParamChanged)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::ColorParameterSelector::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__ColorParameterSelector.data,
    qt_meta_data_StyleEditorGUI__ColorParameterSelector,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::ColorParameterSelector::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::ColorParameterSelector::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__ColorParameterSelector.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int StyleEditorGUI::ColorParameterSelector::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::ColorParameterSelector::colorParamChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_StyleEditorGUI__PlainColorPage_t {
    QByteArrayData data[9];
    char stringdata0[123];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__PlainColorPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__PlainColorPage_t qt_meta_stringdata_StyleEditorGUI__PlainColorPage = {
    {
QT_MOC_LITERAL(0, 0, 30), // "StyleEditorGUI::PlainColorPage"
QT_MOC_LITERAL(1, 31, 12), // "colorChanged"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 10), // "ColorModel"
QT_MOC_LITERAL(4, 56, 10), // "isDragging"
QT_MOC_LITERAL(5, 67, 14), // "onWheelChanged"
QT_MOC_LITERAL(6, 82, 5), // "color"
QT_MOC_LITERAL(7, 88, 16), // "onControlChanged"
QT_MOC_LITERAL(8, 105, 17) // "toggleOrientation"

    },
    "StyleEditorGUI::PlainColorPage\0"
    "colorChanged\0\0ColorModel\0isDragging\0"
    "onWheelChanged\0color\0onControlChanged\0"
    "toggleOrientation"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__PlainColorPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    2,   39,    2, 0x09 /* Protected */,
       7,    2,   44,    2, 0x0a /* Public */,
       8,    0,   49,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    6,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    6,    4,
    QMetaType::Void,

       0        // eod
};

void StyleEditorGUI::PlainColorPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlainColorPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->colorChanged((*reinterpret_cast< const ColorModel(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onWheelChanged((*reinterpret_cast< const ColorModel(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->onControlChanged((*reinterpret_cast< const ColorModel(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->toggleOrientation(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlainColorPage::*)(const ColorModel & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlainColorPage::colorChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::PlainColorPage::staticMetaObject = { {
    QMetaObject::SuperData::link<StyleEditorPage::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__PlainColorPage.data,
    qt_meta_data_StyleEditorGUI__PlainColorPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::PlainColorPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::PlainColorPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__PlainColorPage.stringdata0))
        return static_cast<void*>(this);
    return StyleEditorPage::qt_metacast(_clname);
}

int StyleEditorGUI::PlainColorPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyleEditorPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::PlainColorPage::colorChanged(const ColorModel & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StyleEditorGUI__StyleChooserPage_t {
    QByteArrayData data[16];
    char stringdata0[253];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__StyleChooserPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__StyleChooserPage_t qt_meta_stringdata_StyleEditorGUI__StyleChooserPage = {
    {
QT_MOC_LITERAL(0, 0, 32), // "StyleEditorGUI::StyleChooserPage"
QT_MOC_LITERAL(1, 33, 13), // "styleSelected"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 11), // "TColorStyle"
QT_MOC_LITERAL(4, 60, 5), // "style"
QT_MOC_LITERAL(5, 66, 12), // "patternAdded"
QT_MOC_LITERAL(6, 79, 11), // "computeSize"
QT_MOC_LITERAL(7, 91, 14), // "togglePinToTop"
QT_MOC_LITERAL(8, 106, 14), // "doSetPinsToTop"
QT_MOC_LITERAL(9, 121, 14), // "doClrPinsToTop"
QT_MOC_LITERAL(10, 136, 17), // "doPinsToTopChange"
QT_MOC_LITERAL(11, 154, 18), // "CommonChipBoxColor"
QT_MOC_LITERAL(12, 173, 18), // "PinnedChipBoxColor"
QT_MOC_LITERAL(13, 192, 17), // "SolidChipBoxColor"
QT_MOC_LITERAL(14, 210, 20), // "SelectedChipBoxColor"
QT_MOC_LITERAL(15, 231, 21) // "SelectedChipBox2Color"

    },
    "StyleEditorGUI::StyleChooserPage\0"
    "styleSelected\0\0TColorStyle\0style\0"
    "patternAdded\0computeSize\0togglePinToTop\0"
    "doSetPinsToTop\0doClrPinsToTop\0"
    "doPinsToTopChange\0CommonChipBoxColor\0"
    "PinnedChipBoxColor\0SolidChipBoxColor\0"
    "SelectedChipBoxColor\0SelectedChipBox2Color"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__StyleChooserPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       5,   58, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   52,    2, 0x0a /* Public */,
       6,    0,   53,    2, 0x0a /* Public */,
       7,    0,   54,    2, 0x0a /* Public */,
       8,    0,   55,    2, 0x0a /* Public */,
       9,    0,   56,    2, 0x0a /* Public */,
      10,    0,   57,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      11, QMetaType::QColor, 0x00095103,
      12, QMetaType::QColor, 0x00095103,
      13, QMetaType::QColor, 0x00095103,
      14, QMetaType::QColor, 0x00095103,
      15, QMetaType::QColor, 0x00095103,

       0        // eod
};

void StyleEditorGUI::StyleChooserPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StyleChooserPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->styleSelected((*reinterpret_cast< const TColorStyle(*)>(_a[1]))); break;
        case 1: _t->patternAdded(); break;
        case 2: _t->computeSize(); break;
        case 3: _t->togglePinToTop(); break;
        case 4: _t->doSetPinsToTop(); break;
        case 5: _t->doClrPinsToTop(); break;
        case 6: _t->doPinsToTopChange(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StyleChooserPage::*)(const TColorStyle & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StyleChooserPage::styleSelected)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<StyleChooserPage *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getCommonChipBoxColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getPinnedChipBoxColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getSolidChipBoxColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getSelectedChipBoxColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getSelectedChipBox2Color(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<StyleChooserPage *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCommonChipBoxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setPinnedChipBoxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setSolidChipBoxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setSelectedChipBoxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setSelectedChipBox2Color(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::StyleChooserPage::staticMetaObject = { {
    QMetaObject::SuperData::link<StyleEditorPage::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__StyleChooserPage.data,
    qt_meta_data_StyleEditorGUI__StyleChooserPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::StyleChooserPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::StyleChooserPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__StyleChooserPage.stringdata0))
        return static_cast<void*>(this);
    return StyleEditorPage::qt_metacast(_clname);
}

int StyleEditorGUI::StyleChooserPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyleEditorPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::StyleChooserPage::styleSelected(const TColorStyle & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StyleEditorGUI__SettingsPage_t {
    QByteArrayData data[7];
    char stringdata0[105];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorGUI__SettingsPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorGUI__SettingsPage_t qt_meta_stringdata_StyleEditorGUI__SettingsPage = {
    {
QT_MOC_LITERAL(0, 0, 28), // "StyleEditorGUI::SettingsPage"
QT_MOC_LITERAL(1, 29, 17), // "paramStyleChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 10), // "isDragging"
QT_MOC_LITERAL(4, 59, 17), // "onAutofillChanged"
QT_MOC_LITERAL(5, 77, 14), // "onValueChanged"
QT_MOC_LITERAL(6, 92, 12) // "onValueReset"

    },
    "StyleEditorGUI::SettingsPage\0"
    "paramStyleChanged\0\0isDragging\0"
    "onAutofillChanged\0onValueChanged\0"
    "onValueReset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorGUI__SettingsPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   42,    2, 0x08 /* Private */,
       5,    1,   43,    2, 0x08 /* Private */,
       5,    0,   46,    2, 0x28 /* Private | MethodCloned */,
       6,    0,   47,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StyleEditorGUI::SettingsPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SettingsPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->paramStyleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onAutofillChanged(); break;
        case 2: _t->onValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onValueChanged(); break;
        case 4: _t->onValueReset(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SettingsPage::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SettingsPage::paramStyleChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorGUI::SettingsPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorGUI__SettingsPage.data,
    qt_meta_data_StyleEditorGUI__SettingsPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorGUI::SettingsPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorGUI::SettingsPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorGUI__SettingsPage.stringdata0))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int StyleEditorGUI::SettingsPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void StyleEditorGUI::SettingsPage::paramStyleChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StyleEditor_t {
    QByteArrayData data[43];
    char stringdata0[651];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditor_t qt_meta_stringdata_StyleEditor = {
    {
QT_MOC_LITERAL(0, 0, 11), // "StyleEditor"
QT_MOC_LITERAL(1, 12, 15), // "onStyleSwitched"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 14), // "onStyleChanged"
QT_MOC_LITERAL(4, 44, 10), // "isDragging"
QT_MOC_LITERAL(5, 55, 21), // "onCleanupStyleChanged"
QT_MOC_LITERAL(6, 77, 17), // "onOldStyleClicked"
QT_MOC_LITERAL(7, 95, 17), // "onNewStyleClicked"
QT_MOC_LITERAL(8, 113, 23), // "updateOrientationButton"
QT_MOC_LITERAL(9, 137, 16), // "checkPaletteLock"
QT_MOC_LITERAL(10, 154, 20), // "enableColorAutoApply"
QT_MOC_LITERAL(11, 175, 7), // "enabled"
QT_MOC_LITERAL(12, 183, 14), // "setColorSample"
QT_MOC_LITERAL(13, 198, 8), // "TPixel32"
QT_MOC_LITERAL(14, 207, 5), // "color"
QT_MOC_LITERAL(15, 213, 14), // "onColorChanged"
QT_MOC_LITERAL(16, 228, 10), // "ColorModel"
QT_MOC_LITERAL(17, 239, 11), // "selectStyle"
QT_MOC_LITERAL(18, 251, 11), // "TColorStyle"
QT_MOC_LITERAL(19, 263, 5), // "style"
QT_MOC_LITERAL(20, 269, 18), // "applyButtonClicked"
QT_MOC_LITERAL(21, 288, 16), // "autoCheckChanged"
QT_MOC_LITERAL(22, 305, 5), // "value"
QT_MOC_LITERAL(23, 311, 7), // "setPage"
QT_MOC_LITERAL(24, 319, 5), // "index"
QT_MOC_LITERAL(25, 325, 19), // "onColorParamChanged"
QT_MOC_LITERAL(26, 345, 19), // "onParamStyleChanged"
QT_MOC_LITERAL(27, 365, 12), // "onHexChanged"
QT_MOC_LITERAL(28, 378, 11), // "onHexEditor"
QT_MOC_LITERAL(29, 390, 15), // "onSearchVisible"
QT_MOC_LITERAL(30, 406, 2), // "on"
QT_MOC_LITERAL(31, 409, 22), // "onSpecialButtonToggled"
QT_MOC_LITERAL(32, 432, 21), // "onCustomButtonToggled"
QT_MOC_LITERAL(33, 454, 26), // "onVectorBrushButtonToggled"
QT_MOC_LITERAL(34, 481, 26), // "onSliderAppearanceSelected"
QT_MOC_LITERAL(35, 508, 8), // "QAction*"
QT_MOC_LITERAL(36, 517, 22), // "onPopupMenuAboutToShow"
QT_MOC_LITERAL(37, 540, 15), // "onTextureSearch"
QT_MOC_LITERAL(38, 556, 20), // "onTextureClearSearch"
QT_MOC_LITERAL(39, 577, 15), // "onVectorsSearch"
QT_MOC_LITERAL(40, 593, 20), // "onVectorsClearSearch"
QT_MOC_LITERAL(41, 614, 15), // "onMyPaintSearch"
QT_MOC_LITERAL(42, 630, 20) // "onMyPaintClearSearch"

    },
    "StyleEditor\0onStyleSwitched\0\0"
    "onStyleChanged\0isDragging\0"
    "onCleanupStyleChanged\0onOldStyleClicked\0"
    "onNewStyleClicked\0updateOrientationButton\0"
    "checkPaletteLock\0enableColorAutoApply\0"
    "enabled\0setColorSample\0TPixel32\0color\0"
    "onColorChanged\0ColorModel\0selectStyle\0"
    "TColorStyle\0style\0applyButtonClicked\0"
    "autoCheckChanged\0value\0setPage\0index\0"
    "onColorParamChanged\0onParamStyleChanged\0"
    "onHexChanged\0onHexEditor\0onSearchVisible\0"
    "on\0onSpecialButtonToggled\0"
    "onCustomButtonToggled\0onVectorBrushButtonToggled\0"
    "onSliderAppearanceSelected\0QAction*\0"
    "onPopupMenuAboutToShow\0onTextureSearch\0"
    "onTextureClearSearch\0onVectorsSearch\0"
    "onVectorsClearSearch\0onMyPaintSearch\0"
    "onMyPaintClearSearch"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      30,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  164,    2, 0x09 /* Protected */,
       3,    1,  165,    2, 0x09 /* Protected */,
       5,    1,  168,    2, 0x09 /* Protected */,
       6,    0,  171,    2, 0x09 /* Protected */,
       7,    0,  172,    2, 0x09 /* Protected */,
       8,    0,  173,    2, 0x09 /* Protected */,
       9,    0,  174,    2, 0x09 /* Protected */,
      10,    1,  175,    2, 0x09 /* Protected */,
      12,    1,  178,    2, 0x09 /* Protected */,
      15,    2,  181,    2, 0x09 /* Protected */,
      17,    1,  186,    2, 0x09 /* Protected */,
      20,    0,  189,    2, 0x09 /* Protected */,
      21,    1,  190,    2, 0x09 /* Protected */,
      23,    1,  193,    2, 0x09 /* Protected */,
      25,    0,  196,    2, 0x09 /* Protected */,
      26,    1,  197,    2, 0x09 /* Protected */,
      27,    0,  200,    2, 0x09 /* Protected */,
      28,    0,  201,    2, 0x09 /* Protected */,
      29,    1,  202,    2, 0x09 /* Protected */,
      31,    1,  205,    2, 0x09 /* Protected */,
      32,    1,  208,    2, 0x09 /* Protected */,
      33,    1,  211,    2, 0x09 /* Protected */,
      34,    1,  214,    2, 0x09 /* Protected */,
      36,    0,  217,    2, 0x09 /* Protected */,
      37,    1,  218,    2, 0x09 /* Protected */,
      38,    0,  221,    2, 0x09 /* Protected */,
      39,    1,  222,    2, 0x09 /* Protected */,
      40,    0,  225,    2, 0x09 /* Protected */,
      41,    1,  226,    2, 0x09 /* Protected */,
      42,    0,  229,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 16, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 18,   19,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   30,
    QMetaType::Void, QMetaType::Bool,   30,
    QMetaType::Void, QMetaType::Bool,   30,
    QMetaType::Void, QMetaType::Bool,   30,
    QMetaType::Void, 0x80000000 | 35,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,

       0        // eod
};

void StyleEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StyleEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onStyleSwitched(); break;
        case 1: _t->onStyleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->onCleanupStyleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onOldStyleClicked(); break;
        case 4: _t->onNewStyleClicked(); break;
        case 5: _t->updateOrientationButton(); break;
        case 6: _t->checkPaletteLock(); break;
        case 7: _t->enableColorAutoApply((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->setColorSample((*reinterpret_cast< const TPixel32(*)>(_a[1]))); break;
        case 9: _t->onColorChanged((*reinterpret_cast< const ColorModel(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 10: _t->selectStyle((*reinterpret_cast< const TColorStyle(*)>(_a[1]))); break;
        case 11: _t->applyButtonClicked(); break;
        case 12: _t->autoCheckChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->setPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->onColorParamChanged(); break;
        case 15: _t->onParamStyleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->onHexChanged(); break;
        case 17: _t->onHexEditor(); break;
        case 18: _t->onSearchVisible((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->onSpecialButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->onCustomButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->onVectorBrushButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->onSliderAppearanceSelected((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 23: _t->onPopupMenuAboutToShow(); break;
        case 24: _t->onTextureSearch((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 25: _t->onTextureClearSearch(); break;
        case 26: _t->onVectorsSearch((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 27: _t->onVectorsClearSearch(); break;
        case 28: _t->onMyPaintSearch((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->onMyPaintClearSearch(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_StyleEditor.data,
    qt_meta_data_StyleEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SaveLoadQSettings"))
        return static_cast< SaveLoadQSettings*>(this);
    return QWidget::qt_metacast(_clname);
}

int StyleEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
