/****************************************************************************
** Meta object code from reading C++ file 'tpanels.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/tpanels.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tpanels.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PaletteViewerPanel_t {
    QByteArrayData data[9];
    char stringdata0[135];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PaletteViewerPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PaletteViewerPanel_t qt_meta_stringdata_PaletteViewerPanel = {
    {
QT_MOC_LITERAL(0, 0, 18), // "PaletteViewerPanel"
QT_MOC_LITERAL(1, 19, 20), // "onColorStyleSwitched"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 17), // "onPaletteSwitched"
QT_MOC_LITERAL(4, 59, 21), // "onFreezeButtonToggled"
QT_MOC_LITERAL(5, 81, 8), // "isFrozen"
QT_MOC_LITERAL(6, 90, 15), // "onSceneSwitched"
QT_MOC_LITERAL(7, 106, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(8, 126, 8) // "prefName"

    },
    "PaletteViewerPanel\0onColorStyleSwitched\0"
    "\0onPaletteSwitched\0onFreezeButtonToggled\0"
    "isFrozen\0onSceneSwitched\0onPreferenceChanged\0"
    "prefName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PaletteViewerPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x09 /* Protected */,
       3,    0,   40,    2, 0x09 /* Protected */,
       4,    1,   41,    2, 0x09 /* Protected */,
       6,    0,   44,    2, 0x09 /* Protected */,
       7,    1,   45,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,

       0        // eod
};

void PaletteViewerPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PaletteViewerPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onColorStyleSwitched(); break;
        case 1: _t->onPaletteSwitched(); break;
        case 2: _t->onFreezeButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onSceneSwitched(); break;
        case 4: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PaletteViewerPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<StyleShortcutSwitchablePanel::staticMetaObject>(),
    qt_meta_stringdata_PaletteViewerPanel.data,
    qt_meta_data_PaletteViewerPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PaletteViewerPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaletteViewerPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PaletteViewerPanel.stringdata0))
        return static_cast<void*>(this);
    return StyleShortcutSwitchablePanel::qt_metacast(_clname);
}

int PaletteViewerPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyleShortcutSwitchablePanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
struct qt_meta_stringdata_StudioPaletteViewerPanel_t {
    QByteArrayData data[4];
    char stringdata0[65];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StudioPaletteViewerPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StudioPaletteViewerPanel_t qt_meta_stringdata_StudioPaletteViewerPanel = {
    {
QT_MOC_LITERAL(0, 0, 24), // "StudioPaletteViewerPanel"
QT_MOC_LITERAL(1, 25, 20), // "onColorStyleSwitched"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 17) // "onPaletteSwitched"

    },
    "StudioPaletteViewerPanel\0onColorStyleSwitched\0"
    "\0onPaletteSwitched"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StudioPaletteViewerPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x09 /* Protected */,
       3,    0,   25,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StudioPaletteViewerPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StudioPaletteViewerPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onColorStyleSwitched(); break;
        case 1: _t->onPaletteSwitched(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StudioPaletteViewerPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_StudioPaletteViewerPanel.data,
    qt_meta_data_StudioPaletteViewerPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StudioPaletteViewerPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StudioPaletteViewerPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StudioPaletteViewerPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int StudioPaletteViewerPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_StyleEditorPanel_t {
    QByteArrayData data[4];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleEditorPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleEditorPanel_t qt_meta_stringdata_StyleEditorPanel = {
    {
QT_MOC_LITERAL(0, 0, 16), // "StyleEditorPanel"
QT_MOC_LITERAL(1, 17, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 8) // "prefName"

    },
    "StyleEditorPanel\0onPreferenceChanged\0"
    "\0prefName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleEditorPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,

       0        // eod
};

void StyleEditorPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StyleEditorPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleEditorPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_StyleEditorPanel.data,
    qt_meta_data_StyleEditorPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleEditorPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleEditorPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleEditorPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int StyleEditorPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ColorFieldEditorController_t {
    QByteArrayData data[6];
    char stringdata0[78];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColorFieldEditorController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColorFieldEditorController_t qt_meta_stringdata_ColorFieldEditorController = {
    {
QT_MOC_LITERAL(0, 0, 26), // "ColorFieldEditorController"
QT_MOC_LITERAL(1, 27, 19), // "onColorStyleChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 14), // "onColorChanged"
QT_MOC_LITERAL(4, 63, 8), // "TPixel32"
QT_MOC_LITERAL(5, 72, 5) // "color"

    },
    "ColorFieldEditorController\0"
    "onColorStyleChanged\0\0onColorChanged\0"
    "TPixel32\0color"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColorFieldEditorController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x09 /* Protected */,
       3,    2,   27,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Bool,    5,    2,

       0        // eod
};

void ColorFieldEditorController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorFieldEditorController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onColorStyleChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onColorChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ColorFieldEditorController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ColorFieldEditorController.data,
    qt_meta_data_ColorFieldEditorController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColorFieldEditorController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColorFieldEditorController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColorFieldEditorController.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DVGui::ColorField::ColorFieldEditorController"))
        return static_cast< DVGui::ColorField::ColorFieldEditorController*>(this);
    return QObject::qt_metacast(_clname);
}

int ColorFieldEditorController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_CleanupColorFieldEditorController_t {
    QByteArrayData data[3];
    char stringdata0[55];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CleanupColorFieldEditorController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CleanupColorFieldEditorController_t qt_meta_stringdata_CleanupColorFieldEditorController = {
    {
QT_MOC_LITERAL(0, 0, 33), // "CleanupColorFieldEditorContro..."
QT_MOC_LITERAL(1, 34, 19), // "onColorStyleChanged"
QT_MOC_LITERAL(2, 54, 0) // ""

    },
    "CleanupColorFieldEditorController\0"
    "onColorStyleChanged\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CleanupColorFieldEditorController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void CleanupColorFieldEditorController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CleanupColorFieldEditorController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onColorStyleChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CleanupColorFieldEditorController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CleanupColorFieldEditorController.data,
    qt_meta_data_CleanupColorFieldEditorController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CleanupColorFieldEditorController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CleanupColorFieldEditorController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CleanupColorFieldEditorController.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DVGui::CleanupColorField::CleanupColorFieldEditorController"))
        return static_cast< DVGui::CleanupColorField::CleanupColorFieldEditorController*>(this);
    return QObject::qt_metacast(_clname);
}

int CleanupColorFieldEditorController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_SchematicScenePanel_t {
    QByteArrayData data[17];
    char stringdata0[249];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SchematicScenePanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SchematicScenePanel_t qt_meta_stringdata_SchematicScenePanel = {
    {
QT_MOC_LITERAL(0, 0, 19), // "SchematicScenePanel"
QT_MOC_LITERAL(1, 20, 13), // "onShowPreview"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 4), // "TFxP"
QT_MOC_LITERAL(4, 40, 2), // "fx"
QT_MOC_LITERAL(5, 43, 10), // "onCollapse"
QT_MOC_LITERAL(6, 54, 11), // "QList<TFxP>"
QT_MOC_LITERAL(7, 66, 21), // "QList<TStageObjectId>"
QT_MOC_LITERAL(8, 88, 14), // "onExplodeChild"
QT_MOC_LITERAL(9, 103, 12), // "onEditObject"
QT_MOC_LITERAL(10, 116, 11), // "onDeleteFxs"
QT_MOC_LITERAL(11, 128, 18), // "const FxSelection*"
QT_MOC_LITERAL(12, 147, 20), // "onDeleteStageObjects"
QT_MOC_LITERAL(13, 168, 27), // "const StageObjectSelection*"
QT_MOC_LITERAL(14, 196, 13), // "onColumnPaste"
QT_MOC_LITERAL(15, 210, 18), // "QList<TXshColumnP>"
QT_MOC_LITERAL(16, 229, 19) // "onPreferenceChanged"

    },
    "SchematicScenePanel\0onShowPreview\0\0"
    "TFxP\0fx\0onCollapse\0QList<TFxP>\0"
    "QList<TStageObjectId>\0onExplodeChild\0"
    "onEditObject\0onDeleteFxs\0const FxSelection*\0"
    "onDeleteStageObjects\0const StageObjectSelection*\0"
    "onColumnPaste\0QList<TXshColumnP>\0"
    "onPreferenceChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SchematicScenePanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x09 /* Protected */,
       5,    1,   67,    2, 0x09 /* Protected */,
       5,    1,   70,    2, 0x09 /* Protected */,
       8,    1,   73,    2, 0x09 /* Protected */,
       8,    1,   76,    2, 0x09 /* Protected */,
       9,    0,   79,    2, 0x09 /* Protected */,
      10,    1,   80,    2, 0x09 /* Protected */,
      12,    1,   83,    2, 0x09 /* Protected */,
      14,    1,   86,    2, 0x09 /* Protected */,
      16,    1,   89,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void SchematicScenePanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SchematicScenePanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onShowPreview((*reinterpret_cast< TFxP(*)>(_a[1]))); break;
        case 1: _t->onCollapse((*reinterpret_cast< const QList<TFxP>(*)>(_a[1]))); break;
        case 2: _t->onCollapse((*reinterpret_cast< QList<TStageObjectId>(*)>(_a[1]))); break;
        case 3: _t->onExplodeChild((*reinterpret_cast< const QList<TFxP>(*)>(_a[1]))); break;
        case 4: _t->onExplodeChild((*reinterpret_cast< QList<TStageObjectId>(*)>(_a[1]))); break;
        case 5: _t->onEditObject(); break;
        case 6: _t->onDeleteFxs((*reinterpret_cast< const FxSelection*(*)>(_a[1]))); break;
        case 7: _t->onDeleteStageObjects((*reinterpret_cast< const StageObjectSelection*(*)>(_a[1]))); break;
        case 8: _t->onColumnPaste((*reinterpret_cast< const QList<TXshColumnP>(*)>(_a[1]))); break;
        case 9: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SchematicScenePanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_SchematicScenePanel.data,
    qt_meta_data_SchematicScenePanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SchematicScenePanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SchematicScenePanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SchematicScenePanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int SchematicScenePanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
struct qt_meta_stringdata_FunctionViewerPanel_t {
    QByteArrayData data[9];
    char stringdata0[86];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionViewerPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionViewerPanel_t qt_meta_stringdata_FunctionViewerPanel = {
    {
QT_MOC_LITERAL(0, 0, 19), // "FunctionViewerPanel"
QT_MOC_LITERAL(1, 20, 9), // "onIoCurve"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 4), // "type"
QT_MOC_LITERAL(4, 36, 13), // "TDoubleParam*"
QT_MOC_LITERAL(5, 50, 5), // "curve"
QT_MOC_LITERAL(6, 56, 11), // "std::string"
QT_MOC_LITERAL(7, 68, 4), // "name"
QT_MOC_LITERAL(8, 73, 12) // "onEditObject"

    },
    "FunctionViewerPanel\0onIoCurve\0\0type\0"
    "TDoubleParam*\0curve\0std::string\0name\0"
    "onEditObject"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionViewerPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    3,   24,    2, 0x0a /* Public */,
       8,    0,   31,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, 0x80000000 | 4, 0x80000000 | 6,    3,    5,    7,
    QMetaType::Void,

       0        // eod
};

void FunctionViewerPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionViewerPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onIoCurve((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< TDoubleParam*(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 1: _t->onEditObject(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionViewerPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_FunctionViewerPanel.data,
    qt_meta_data_FunctionViewerPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionViewerPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionViewerPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionViewerPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int FunctionViewerPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionPanel_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionPanel_t qt_meta_stringdata_ToolOptionPanel = {
    {
QT_MOC_LITERAL(0, 0, 15) // "ToolOptionPanel"

    },
    "ToolOptionPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ToolOptionPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionPanel.data,
    qt_meta_data_ToolOptionPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int ToolOptionPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_CommandBarPanel_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CommandBarPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CommandBarPanel_t qt_meta_stringdata_CommandBarPanel = {
    {
QT_MOC_LITERAL(0, 0, 15) // "CommandBarPanel"

    },
    "CommandBarPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CommandBarPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void CommandBarPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CommandBarPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_CommandBarPanel.data,
    qt_meta_data_CommandBarPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CommandBarPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CommandBarPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CommandBarPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int CommandBarPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FlipbookBasePanel_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FlipbookBasePanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FlipbookBasePanel_t qt_meta_stringdata_FlipbookBasePanel = {
    {
QT_MOC_LITERAL(0, 0, 17) // "FlipbookBasePanel"

    },
    "FlipbookBasePanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FlipbookBasePanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FlipbookBasePanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FlipbookBasePanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_FlipbookBasePanel.data,
    qt_meta_data_FlipbookBasePanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FlipbookBasePanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FlipbookBasePanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FlipbookBasePanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int FlipbookBasePanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FlipbookPanel_t {
    QByteArrayData data[3];
    char stringdata0[39];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FlipbookPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FlipbookPanel_t qt_meta_stringdata_FlipbookPanel = {
    {
QT_MOC_LITERAL(0, 0, 13), // "FlipbookPanel"
QT_MOC_LITERAL(1, 14, 23), // "onMinimizeButtonToggled"
QT_MOC_LITERAL(2, 38, 0) // ""

    },
    "FlipbookPanel\0onMinimizeButtonToggled\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FlipbookPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void FlipbookPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FlipbookPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onMinimizeButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FlipbookPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<FlipbookBasePanel::staticMetaObject>(),
    qt_meta_stringdata_FlipbookPanel.data,
    qt_meta_data_FlipbookPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FlipbookPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FlipbookPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FlipbookPanel.stringdata0))
        return static_cast<void*>(this);
    return FlipbookBasePanel::qt_metacast(_clname);
}

int FlipbookPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FlipbookBasePanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ComboViewerPanelContainer_t {
    QByteArrayData data[1];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ComboViewerPanelContainer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ComboViewerPanelContainer_t qt_meta_stringdata_ComboViewerPanelContainer = {
    {
QT_MOC_LITERAL(0, 0, 25) // "ComboViewerPanelContainer"

    },
    "ComboViewerPanelContainer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ComboViewerPanelContainer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ComboViewerPanelContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ComboViewerPanelContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<StyleShortcutSwitchablePanel::staticMetaObject>(),
    qt_meta_stringdata_ComboViewerPanelContainer.data,
    qt_meta_data_ComboViewerPanelContainer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ComboViewerPanelContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComboViewerPanelContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ComboViewerPanelContainer.stringdata0))
        return static_cast<void*>(this);
    return StyleShortcutSwitchablePanel::qt_metacast(_clname);
}

int ComboViewerPanelContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyleShortcutSwitchablePanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SceneViewerPanelContainer_t {
    QByteArrayData data[1];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SceneViewerPanelContainer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SceneViewerPanelContainer_t qt_meta_stringdata_SceneViewerPanelContainer = {
    {
QT_MOC_LITERAL(0, 0, 25) // "SceneViewerPanelContainer"

    },
    "SceneViewerPanelContainer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SceneViewerPanelContainer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SceneViewerPanelContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SceneViewerPanelContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<StyleShortcutSwitchablePanel::staticMetaObject>(),
    qt_meta_stringdata_SceneViewerPanelContainer.data,
    qt_meta_data_SceneViewerPanelContainer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SceneViewerPanelContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SceneViewerPanelContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SceneViewerPanelContainer.stringdata0))
        return static_cast<void*>(this);
    return StyleShortcutSwitchablePanel::qt_metacast(_clname);
}

int SceneViewerPanelContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyleShortcutSwitchablePanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_FxSettingsPanel_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSettingsPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSettingsPanel_t qt_meta_stringdata_FxSettingsPanel = {
    {
QT_MOC_LITERAL(0, 0, 15) // "FxSettingsPanel"

    },
    "FxSettingsPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSettingsPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FxSettingsPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FxSettingsPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_FxSettingsPanel.data,
    qt_meta_data_FxSettingsPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSettingsPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSettingsPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSettingsPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int FxSettingsPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_VectorGuidedDrawingPanel_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_VectorGuidedDrawingPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_VectorGuidedDrawingPanel_t qt_meta_stringdata_VectorGuidedDrawingPanel = {
    {
QT_MOC_LITERAL(0, 0, 24) // "VectorGuidedDrawingPanel"

    },
    "VectorGuidedDrawingPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_VectorGuidedDrawingPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void VectorGuidedDrawingPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject VectorGuidedDrawingPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_VectorGuidedDrawingPanel.data,
    qt_meta_data_VectorGuidedDrawingPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *VectorGuidedDrawingPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VectorGuidedDrawingPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_VectorGuidedDrawingPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int VectorGuidedDrawingPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_LocatorPanel_t {
    QByteArrayData data[1];
    char stringdata0[13];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LocatorPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LocatorPanel_t qt_meta_stringdata_LocatorPanel = {
    {
QT_MOC_LITERAL(0, 0, 12) // "LocatorPanel"

    },
    "LocatorPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LocatorPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void LocatorPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject LocatorPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_LocatorPanel.data,
    qt_meta_data_LocatorPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LocatorPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LocatorPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LocatorPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int LocatorPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
