/****************************************************************************
** Meta object code from reading C++ file 'autolipsyncpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/autolipsyncpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'autolipsyncpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AutoLipSyncPopup_t {
    QByteArrayData data[25];
    char stringdata0[342];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AutoLipSyncPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AutoLipSyncPopup_t qt_meta_stringdata_AutoLipSyncPopup = {
    {
QT_MOC_LITERAL(0, 0, 16), // "AutoLipSyncPopup"
QT_MOC_LITERAL(1, 17, 13), // "onApplyButton"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 9), // "playSound"
QT_MOC_LITERAL(4, 42, 12), // "stopAllSound"
QT_MOC_LITERAL(5, 55, 12), // "setAudioFile"
QT_MOC_LITERAL(6, 68, 15), // "imageNavClicked"
QT_MOC_LITERAL(7, 84, 2), // "id"
QT_MOC_LITERAL(8, 87, 19), // "onStartValueChanged"
QT_MOC_LITERAL(9, 107, 14), // "onLevelChanged"
QT_MOC_LITERAL(10, 122, 5), // "index"
QT_MOC_LITERAL(11, 128, 19), // "onMediaStateChanged"
QT_MOC_LITERAL(12, 148, 19), // "QMediaPlayer::State"
QT_MOC_LITERAL(13, 168, 5), // "state"
QT_MOC_LITERAL(14, 174, 14), // "onAudioTimeout"
QT_MOC_LITERAL(15, 189, 13), // "onOutputReady"
QT_MOC_LITERAL(16, 203, 17), // "onRhubarbFinished"
QT_MOC_LITERAL(17, 221, 8), // "exitCode"
QT_MOC_LITERAL(18, 230, 20), // "QProcess::ExitStatus"
QT_MOC_LITERAL(19, 251, 10), // "exitStatus"
QT_MOC_LITERAL(20, 262, 14), // "onRhubarbError"
QT_MOC_LITERAL(21, 277, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(22, 300, 5), // "error"
QT_MOC_LITERAL(23, 306, 19), // "onAnalysisCancelled"
QT_MOC_LITERAL(24, 326, 15) // "onIconGenerated"

    },
    "AutoLipSyncPopup\0onApplyButton\0\0"
    "playSound\0stopAllSound\0setAudioFile\0"
    "imageNavClicked\0id\0onStartValueChanged\0"
    "onLevelChanged\0index\0onMediaStateChanged\0"
    "QMediaPlayer::State\0state\0onAudioTimeout\0"
    "onOutputReady\0onRhubarbFinished\0"
    "exitCode\0QProcess::ExitStatus\0exitStatus\0"
    "onRhubarbError\0QProcess::ProcessError\0"
    "error\0onAnalysisCancelled\0onIconGenerated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AutoLipSyncPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x0a /* Public */,
       3,    0,   85,    2, 0x0a /* Public */,
       4,    0,   86,    2, 0x0a /* Public */,
       5,    0,   87,    2, 0x0a /* Public */,
       6,    1,   88,    2, 0x0a /* Public */,
       8,    0,   91,    2, 0x0a /* Public */,
       9,    1,   92,    2, 0x0a /* Public */,
      11,    1,   95,    2, 0x0a /* Public */,
      14,    0,   98,    2, 0x0a /* Public */,
      15,    0,   99,    2, 0x0a /* Public */,
      16,    2,  100,    2, 0x0a /* Public */,
      20,    1,  105,    2, 0x0a /* Public */,
      23,    0,  108,    2, 0x0a /* Public */,
      24,    0,  109,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 18,   17,   19,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void AutoLipSyncPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AutoLipSyncPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onApplyButton(); break;
        case 1: _t->playSound(); break;
        case 2: _t->stopAllSound(); break;
        case 3: { bool _r = _t->setAudioFile();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->imageNavClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onStartValueChanged(); break;
        case 6: _t->onLevelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onMediaStateChanged((*reinterpret_cast< QMediaPlayer::State(*)>(_a[1]))); break;
        case 8: _t->onAudioTimeout(); break;
        case 9: _t->onOutputReady(); break;
        case 10: _t->onRhubarbFinished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 11: _t->onRhubarbError((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 12: _t->onAnalysisCancelled(); break;
        case 13: _t->onIconGenerated(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AutoLipSyncPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_AutoLipSyncPopup.data,
    qt_meta_data_AutoLipSyncPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AutoLipSyncPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AutoLipSyncPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AutoLipSyncPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int AutoLipSyncPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
