/****************************************************************************
** Meta object code from reading C++ file 'audiorecordingpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/audiorecordingpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'audiorecordingpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AudioRecordingPopup_t {
    QByteArrayData data[20];
    char stringdata0[363];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AudioRecordingPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AudioRecordingPopup_t qt_meta_stringdata_AudioRecordingPopup = {
    {
QT_MOC_LITERAL(0, 0, 19), // "AudioRecordingPopup"
QT_MOC_LITERAL(1, 20, 21), // "onRecordButtonPressed"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 20), // "updateRecordDuration"
QT_MOC_LITERAL(4, 64, 8), // "duration"
QT_MOC_LITERAL(5, 73, 22), // "updatePlaybackDuration"
QT_MOC_LITERAL(6, 96, 19), // "onPlayButtonPressed"
QT_MOC_LITERAL(7, 116, 19), // "onSaveButtonPressed"
QT_MOC_LITERAL(8, 136, 29), // "onPauseRecordingButtonPressed"
QT_MOC_LITERAL(9, 166, 28), // "onPausePlaybackButtonPressed"
QT_MOC_LITERAL(10, 195, 18), // "onPlayStateChanged"
QT_MOC_LITERAL(11, 214, 7), // "playing"
QT_MOC_LITERAL(12, 222, 21), // "onPlayXSheetCBChanged"
QT_MOC_LITERAL(13, 244, 6), // "status"
QT_MOC_LITERAL(14, 251, 19), // "onMediaStateChanged"
QT_MOC_LITERAL(15, 271, 19), // "QMediaPlayer::State"
QT_MOC_LITERAL(16, 291, 5), // "state"
QT_MOC_LITERAL(17, 297, 20), // "onInputDeviceChanged"
QT_MOC_LITERAL(18, 318, 22), // "onRefreshButtonPressed"
QT_MOC_LITERAL(19, 341, 21) // "onAudioSettingChanged"

    },
    "AudioRecordingPopup\0onRecordButtonPressed\0"
    "\0updateRecordDuration\0duration\0"
    "updatePlaybackDuration\0onPlayButtonPressed\0"
    "onSaveButtonPressed\0onPauseRecordingButtonPressed\0"
    "onPausePlaybackButtonPressed\0"
    "onPlayStateChanged\0playing\0"
    "onPlayXSheetCBChanged\0status\0"
    "onMediaStateChanged\0QMediaPlayer::State\0"
    "state\0onInputDeviceChanged\0"
    "onRefreshButtonPressed\0onAudioSettingChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AudioRecordingPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x08 /* Private */,
       3,    1,   80,    2, 0x08 /* Private */,
       5,    1,   83,    2, 0x08 /* Private */,
       6,    0,   86,    2, 0x08 /* Private */,
       7,    0,   87,    2, 0x08 /* Private */,
       8,    0,   88,    2, 0x08 /* Private */,
       9,    0,   89,    2, 0x08 /* Private */,
      10,    1,   90,    2, 0x08 /* Private */,
      12,    1,   93,    2, 0x08 /* Private */,
      14,    1,   96,    2, 0x08 /* Private */,
      17,    0,   99,    2, 0x08 /* Private */,
      18,    0,  100,    2, 0x08 /* Private */,
      19,    0,  101,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong,    4,
    QMetaType::Void, QMetaType::LongLong,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void AudioRecordingPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AudioRecordingPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onRecordButtonPressed(); break;
        case 1: _t->updateRecordDuration((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 2: _t->updatePlaybackDuration((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        case 3: _t->onPlayButtonPressed(); break;
        case 4: _t->onSaveButtonPressed(); break;
        case 5: _t->onPauseRecordingButtonPressed(); break;
        case 6: _t->onPausePlaybackButtonPressed(); break;
        case 7: _t->onPlayStateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->onPlayXSheetCBChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->onMediaStateChanged((*reinterpret_cast< QMediaPlayer::State(*)>(_a[1]))); break;
        case 10: _t->onInputDeviceChanged(); break;
        case 11: _t->onRefreshButtonPressed(); break;
        case 12: _t->onAudioSettingChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AudioRecordingPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_AudioRecordingPopup.data,
    qt_meta_data_AudioRecordingPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AudioRecordingPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AudioRecordingPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AudioRecordingPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int AudioRecordingPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}
struct qt_meta_stringdata_AudioWriterWAV_t {
    QByteArrayData data[4];
    char stringdata0[32];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AudioWriterWAV_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AudioWriterWAV_t qt_meta_stringdata_AudioWriterWAV = {
    {
QT_MOC_LITERAL(0, 0, 14), // "AudioWriterWAV"
QT_MOC_LITERAL(1, 15, 6), // "update"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 8) // "duration"

    },
    "AudioWriterWAV\0update\0\0duration"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AudioWriterWAV[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::LongLong,    3,

       0        // eod
};

void AudioWriterWAV::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AudioWriterWAV *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->update((*reinterpret_cast< qint64(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AudioWriterWAV::*)(qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AudioWriterWAV::update)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AudioWriterWAV::staticMetaObject = { {
    QMetaObject::SuperData::link<QIODevice::staticMetaObject>(),
    qt_meta_stringdata_AudioWriterWAV.data,
    qt_meta_data_AudioWriterWAV,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AudioWriterWAV::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AudioWriterWAV::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AudioWriterWAV.stringdata0))
        return static_cast<void*>(this);
    return QIODevice::qt_metacast(_clname);
}

int AudioWriterWAV::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QIODevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void AudioWriterWAV::update(qint64 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_AudioLevelsDisplay_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AudioLevelsDisplay_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AudioLevelsDisplay_t qt_meta_stringdata_AudioLevelsDisplay = {
    {
QT_MOC_LITERAL(0, 0, 18) // "AudioLevelsDisplay"

    },
    "AudioLevelsDisplay"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AudioLevelsDisplay[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void AudioLevelsDisplay::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject AudioLevelsDisplay::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_AudioLevelsDisplay.data,
    qt_meta_data_AudioLevelsDisplay,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AudioLevelsDisplay::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AudioLevelsDisplay::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AudioLevelsDisplay.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int AudioLevelsDisplay::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
