/****************************************************************************
** Meta object code from reading C++ file 'flipbook.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/flipbook.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'flipbook.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SaveImagesPopup_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SaveImagesPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SaveImagesPopup_t qt_meta_stringdata_SaveImagesPopup = {
    {
QT_MOC_LITERAL(0, 0, 15) // "SaveImagesPopup"

    },
    "SaveImagesPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SaveImagesPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SaveImagesPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SaveImagesPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_SaveImagesPopup.data,
    qt_meta_data_SaveImagesPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SaveImagesPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SaveImagesPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SaveImagesPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int SaveImagesPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_LoadImagesPopup_t {
    QByteArrayData data[5];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadImagesPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadImagesPopup_t qt_meta_stringdata_LoadImagesPopup = {
    {
QT_MOC_LITERAL(0, 0, 15), // "LoadImagesPopup"
QT_MOC_LITERAL(1, 16, 17), // "onFilePathClicked"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 9), // "TFilePath"
QT_MOC_LITERAL(4, 45, 17) // "onEditingFinished"

    },
    "LoadImagesPopup\0onFilePathClicked\0\0"
    "TFilePath\0onEditingFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadImagesPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x0a /* Public */,
       4,    0,   27,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,

       0        // eod
};

void LoadImagesPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LoadImagesPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onFilePathClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 1: _t->onEditingFinished(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LoadImagesPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_LoadImagesPopup.data,
    qt_meta_data_LoadImagesPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadImagesPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadImagesPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadImagesPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int LoadImagesPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_FlipBook_t {
    QByteArrayData data[25];
    char stringdata0[288];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FlipBook_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FlipBook_t qt_meta_stringdata_FlipBook = {
    {
QT_MOC_LITERAL(0, 0, 8), // "FlipBook"
QT_MOC_LITERAL(1, 9, 13), // "closeFlipBook"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 9), // "FlipBook*"
QT_MOC_LITERAL(4, 34, 9), // "unfreezed"
QT_MOC_LITERAL(5, 44, 11), // "imageLoaded"
QT_MOC_LITERAL(6, 56, 8), // "QString&"
QT_MOC_LITERAL(7, 65, 13), // "onDoubleClick"
QT_MOC_LITERAL(8, 79, 12), // "QMouseEvent*"
QT_MOC_LITERAL(9, 92, 2), // "me"
QT_MOC_LITERAL(10, 95, 15), // "onButtonPressed"
QT_MOC_LITERAL(11, 111, 20), // "FlipConsole::EGadget"
QT_MOC_LITERAL(12, 132, 6), // "button"
QT_MOC_LITERAL(13, 139, 20), // "onCloseButtonPressed"
QT_MOC_LITERAL(14, 160, 9), // "saveImage"
QT_MOC_LITERAL(15, 170, 6), // "freeze"
QT_MOC_LITERAL(16, 177, 2), // "on"
QT_MOC_LITERAL(17, 180, 10), // "saveImages"
QT_MOC_LITERAL(18, 191, 10), // "loadImages"
QT_MOC_LITERAL(19, 202, 15), // "performFxUpdate"
QT_MOC_LITERAL(20, 218, 10), // "regenerate"
QT_MOC_LITERAL(21, 229, 15), // "regenerateFrame"
QT_MOC_LITERAL(22, 245, 12), // "clonePreview"
QT_MOC_LITERAL(23, 258, 13), // "freezePreview"
QT_MOC_LITERAL(24, 272, 15) // "unfreezePreview"

    },
    "FlipBook\0closeFlipBook\0\0FlipBook*\0"
    "unfreezed\0imageLoaded\0QString&\0"
    "onDoubleClick\0QMouseEvent*\0me\0"
    "onButtonPressed\0FlipConsole::EGadget\0"
    "button\0onCloseButtonPressed\0saveImage\0"
    "freeze\0on\0saveImages\0loadImages\0"
    "performFxUpdate\0regenerate\0regenerateFrame\0"
    "clonePreview\0freezePreview\0unfreezePreview"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FlipBook[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x06 /* Public */,
       4,    1,   97,    2, 0x06 /* Public */,
       5,    1,  100,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  103,    2, 0x09 /* Protected */,
      10,    1,  106,    2, 0x09 /* Protected */,
      13,    0,  109,    2, 0x09 /* Protected */,
      14,    0,  110,    2, 0x09 /* Protected */,
      15,    1,  111,    2, 0x09 /* Protected */,
      17,    0,  114,    2, 0x0a /* Public */,
      18,    0,  115,    2, 0x0a /* Public */,
      19,    0,  116,    2, 0x0a /* Public */,
      20,    0,  117,    2, 0x0a /* Public */,
      21,    0,  118,    2, 0x0a /* Public */,
      22,    0,  119,    2, 0x0a /* Public */,
      23,    0,  120,    2, 0x0a /* Public */,
      24,    0,  121,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 6,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void FlipBook::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FlipBook *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closeFlipBook((*reinterpret_cast< FlipBook*(*)>(_a[1]))); break;
        case 1: _t->unfreezed((*reinterpret_cast< FlipBook*(*)>(_a[1]))); break;
        case 2: _t->imageLoaded((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->onDoubleClick((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 4: _t->onButtonPressed((*reinterpret_cast< FlipConsole::EGadget(*)>(_a[1]))); break;
        case 5: _t->onCloseButtonPressed(); break;
        case 6: _t->saveImage(); break;
        case 7: _t->freeze((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->saveImages(); break;
        case 9: _t->loadImages(); break;
        case 10: _t->performFxUpdate(); break;
        case 11: _t->regenerate(); break;
        case 12: _t->regenerateFrame(); break;
        case 13: _t->clonePreview(); break;
        case 14: _t->freezePreview(); break;
        case 15: _t->unfreezePreview(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FlipBook* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< FlipBook* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FlipBook::*)(FlipBook * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipBook::closeFlipBook)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FlipBook::*)(FlipBook * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipBook::unfreezed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FlipBook::*)(QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipBook::imageLoaded)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FlipBook::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_FlipBook.data,
    qt_meta_data_FlipBook,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FlipBook::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FlipBook::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FlipBook.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSoundOutputDeviceListener"))
        return static_cast< TSoundOutputDeviceListener*>(this);
    if (!strcmp(_clname, "FlipConsoleOwner"))
        return static_cast< FlipConsoleOwner*>(this);
    return QWidget::qt_metacast(_clname);
}

int FlipBook::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void FlipBook::closeFlipBook(FlipBook * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FlipBook::unfreezed(FlipBook * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FlipBook::imageLoaded(QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
