/****************************************************************************
** Meta object code from reading C++ file 'tthread.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../opentoonz/toonz/sources/include/tthread.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TThread__Runnable_t {
    QByteArrayData data[14];
    char stringdata0[149];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TThread__Runnable_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TThread__Runnable_t qt_meta_stringdata_TThread__Runnable = {
    {
QT_MOC_LITERAL(0, 0, 17), // "TThread::Runnable"
QT_MOC_LITERAL(1, 18, 7), // "started"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 18), // "TThread::RunnableP"
QT_MOC_LITERAL(4, 46, 6), // "sender"
QT_MOC_LITERAL(5, 53, 8), // "finished"
QT_MOC_LITERAL(6, 62, 9), // "exception"
QT_MOC_LITERAL(7, 72, 8), // "canceled"
QT_MOC_LITERAL(8, 81, 10), // "terminated"
QT_MOC_LITERAL(9, 92, 9), // "onStarted"
QT_MOC_LITERAL(10, 102, 10), // "onFinished"
QT_MOC_LITERAL(11, 113, 11), // "onException"
QT_MOC_LITERAL(12, 125, 10), // "onCanceled"
QT_MOC_LITERAL(13, 136, 12) // "onTerminated"

    },
    "TThread::Runnable\0started\0\0"
    "TThread::RunnableP\0sender\0finished\0"
    "exception\0canceled\0terminated\0onStarted\0"
    "onFinished\0onException\0onCanceled\0"
    "onTerminated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TThread__Runnable[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,
       5,    1,   67,    2, 0x06 /* Public */,
       6,    1,   70,    2, 0x06 /* Public */,
       7,    1,   73,    2, 0x06 /* Public */,
       8,    1,   76,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    1,   79,    2, 0x0a /* Public */,
      10,    1,   82,    2, 0x0a /* Public */,
      11,    1,   85,    2, 0x0a /* Public */,
      12,    1,   88,    2, 0x0a /* Public */,
      13,    1,   91,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void TThread::Runnable::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Runnable *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->started((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 1: _t->finished((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 2: _t->exception((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 3: _t->canceled((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 4: _t->terminated((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 5: _t->onStarted((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 6: _t->onFinished((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 7: _t->onException((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 8: _t->onCanceled((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        case 9: _t->onTerminated((*reinterpret_cast< TThread::RunnableP(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Runnable::*)(TThread::RunnableP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Runnable::started)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Runnable::*)(TThread::RunnableP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Runnable::finished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Runnable::*)(TThread::RunnableP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Runnable::exception)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Runnable::*)(TThread::RunnableP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Runnable::canceled)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Runnable::*)(TThread::RunnableP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Runnable::terminated)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TThread::Runnable::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TThread__Runnable.data,
    qt_meta_data_TThread__Runnable,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TThread::Runnable::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TThread::Runnable::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TThread__Runnable.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSmartObject"))
        return static_cast< TSmartObject*>(this);
    return QObject::qt_metacast(_clname);
}

int TThread::Runnable::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void TThread::Runnable::started(TThread::RunnableP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TThread::Runnable::finished(TThread::RunnableP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TThread::Runnable::exception(TThread::RunnableP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TThread::Runnable::canceled(TThread::RunnableP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TThread::Runnable::terminated(TThread::RunnableP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
