/****************************************************************************
** Meta object code from reading C++ file 'dvtextedit.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/dvtextedit.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dvtextedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__DvTextEditButton_t {
    QByteArrayData data[3];
    char stringdata0[33];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__DvTextEditButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__DvTextEditButton_t qt_meta_stringdata_DVGui__DvTextEditButton = {
    {
QT_MOC_LITERAL(0, 0, 23), // "DVGui::DvTextEditButton"
QT_MOC_LITERAL(1, 24, 7), // "clicked"
QT_MOC_LITERAL(2, 32, 0) // ""

    },
    "DVGui::DvTextEditButton\0clicked\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__DvTextEditButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void DVGui::DvTextEditButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvTextEditButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DvTextEditButton::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvTextEditButton::clicked)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::DvTextEditButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__DvTextEditButton.data,
    qt_meta_data_DVGui__DvTextEditButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::DvTextEditButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::DvTextEditButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__DvTextEditButton.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::DvTextEditButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DVGui::DvTextEditButton::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_DVGui__DvTextEdit_t {
    QByteArrayData data[21];
    char stringdata0[277];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__DvTextEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__DvTextEdit_t qt_meta_stringdata_DVGui__DvTextEdit = {
    {
QT_MOC_LITERAL(0, 0, 17), // "DVGui::DvTextEdit"
QT_MOC_LITERAL(1, 18, 7), // "focusIn"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 26), // "onCurrentCharFormatChanged"
QT_MOC_LITERAL(4, 54, 15), // "QTextCharFormat"
QT_MOC_LITERAL(5, 70, 6), // "format"
QT_MOC_LITERAL(6, 77, 23), // "onCursorPositionChanged"
QT_MOC_LITERAL(7, 101, 18), // "onSelectionChanged"
QT_MOC_LITERAL(8, 120, 24), // "onShowMiniToolBarClicked"
QT_MOC_LITERAL(9, 145, 13), // "setTextFamily"
QT_MOC_LITERAL(10, 159, 12), // "setTextColor"
QT_MOC_LITERAL(11, 172, 8), // "TPixel32"
QT_MOC_LITERAL(12, 181, 5), // "color"
QT_MOC_LITERAL(13, 187, 10), // "isDragging"
QT_MOC_LITERAL(14, 198, 11), // "setTextBold"
QT_MOC_LITERAL(15, 210, 13), // "setTextItalic"
QT_MOC_LITERAL(16, 224, 16), // "setTextUnderline"
QT_MOC_LITERAL(17, 241, 11), // "setTextSize"
QT_MOC_LITERAL(18, 253, 1), // "p"
QT_MOC_LITERAL(19, 255, 12), // "setTextAlign"
QT_MOC_LITERAL(20, 268, 8) // "QAction*"

    },
    "DVGui::DvTextEdit\0focusIn\0\0"
    "onCurrentCharFormatChanged\0QTextCharFormat\0"
    "format\0onCursorPositionChanged\0"
    "onSelectionChanged\0onShowMiniToolBarClicked\0"
    "setTextFamily\0setTextColor\0TPixel32\0"
    "color\0isDragging\0setTextBold\0setTextItalic\0"
    "setTextUnderline\0setTextSize\0p\0"
    "setTextAlign\0QAction*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__DvTextEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   75,    2, 0x09 /* Protected */,
       6,    0,   78,    2, 0x09 /* Protected */,
       7,    0,   79,    2, 0x09 /* Protected */,
       8,    0,   80,    2, 0x09 /* Protected */,
       9,    1,   81,    2, 0x09 /* Protected */,
      10,    2,   84,    2, 0x09 /* Protected */,
      14,    0,   89,    2, 0x09 /* Protected */,
      15,    0,   90,    2, 0x09 /* Protected */,
      16,    0,   91,    2, 0x09 /* Protected */,
      17,    1,   92,    2, 0x09 /* Protected */,
      19,    1,   95,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 11, QMetaType::Bool,   12,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   18,
    QMetaType::Void, 0x80000000 | 20,    2,

       0        // eod
};

void DVGui::DvTextEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvTextEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->focusIn(); break;
        case 1: _t->onCurrentCharFormatChanged((*reinterpret_cast< const QTextCharFormat(*)>(_a[1]))); break;
        case 2: _t->onCursorPositionChanged(); break;
        case 3: _t->onSelectionChanged(); break;
        case 4: _t->onShowMiniToolBarClicked(); break;
        case 5: _t->setTextFamily((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->setTextColor((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 7: _t->setTextBold(); break;
        case 8: _t->setTextItalic(); break;
        case 9: _t->setTextUnderline(); break;
        case 10: _t->setTextSize((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->setTextAlign((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DvTextEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvTextEdit::focusIn)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::DvTextEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QTextEdit::staticMetaObject>(),
    qt_meta_stringdata_DVGui__DvTextEdit.data,
    qt_meta_data_DVGui__DvTextEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::DvTextEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::DvTextEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__DvTextEdit.stringdata0))
        return static_cast<void*>(this);
    return QTextEdit::qt_metacast(_clname);
}

int DVGui::DvTextEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void DVGui::DvTextEdit::focusIn()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
