/****************************************************************************
** Meta object code from reading C++ file 'scriptbinding_scene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/scriptbinding_scene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scriptbinding_scene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TScriptBinding__Scene_t {
    QByteArrayData data[25];
    char stringdata0[202];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TScriptBinding__Scene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TScriptBinding__Scene_t qt_meta_stringdata_TScriptBinding__Scene = {
    {
QT_MOC_LITERAL(0, 0, 21), // "TScriptBinding::Scene"
QT_MOC_LITERAL(1, 22, 8), // "toString"
QT_MOC_LITERAL(2, 31, 12), // "QScriptValue"
QT_MOC_LITERAL(3, 44, 0), // ""
QT_MOC_LITERAL(4, 45, 4), // "load"
QT_MOC_LITERAL(5, 50, 5), // "fpArg"
QT_MOC_LITERAL(6, 56, 4), // "save"
QT_MOC_LITERAL(7, 61, 7), // "setCell"
QT_MOC_LITERAL(8, 69, 3), // "row"
QT_MOC_LITERAL(9, 73, 3), // "col"
QT_MOC_LITERAL(10, 77, 5), // "level"
QT_MOC_LITERAL(11, 83, 3), // "fid"
QT_MOC_LITERAL(12, 87, 4), // "cell"
QT_MOC_LITERAL(13, 92, 7), // "getCell"
QT_MOC_LITERAL(14, 100, 12), // "insertColumn"
QT_MOC_LITERAL(15, 113, 12), // "deleteColumn"
QT_MOC_LITERAL(16, 126, 9), // "getLevels"
QT_MOC_LITERAL(17, 136, 8), // "getLevel"
QT_MOC_LITERAL(18, 145, 4), // "name"
QT_MOC_LITERAL(19, 150, 8), // "newLevel"
QT_MOC_LITERAL(20, 159, 4), // "type"
QT_MOC_LITERAL(21, 164, 9), // "loadLevel"
QT_MOC_LITERAL(22, 174, 4), // "path"
QT_MOC_LITERAL(23, 179, 10), // "frameCount"
QT_MOC_LITERAL(24, 190, 11) // "columnCount"

    },
    "TScriptBinding::Scene\0toString\0"
    "QScriptValue\0\0load\0fpArg\0save\0setCell\0"
    "row\0col\0level\0fid\0cell\0getCell\0"
    "insertColumn\0deleteColumn\0getLevels\0"
    "getLevel\0name\0newLevel\0type\0loadLevel\0"
    "path\0frameCount\0columnCount"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TScriptBinding__Scene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       2,  122, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    0,   74,    3, 0x02 /* Public */,
       4,    1,   75,    3, 0x02 /* Public */,
       6,    1,   78,    3, 0x02 /* Public */,
       7,    4,   81,    3, 0x02 /* Public */,
       7,    3,   90,    3, 0x02 /* Public */,
      13,    2,   97,    3, 0x02 /* Public */,
      14,    1,  102,    3, 0x02 /* Public */,
      15,    1,  105,    3, 0x02 /* Public */,
      16,    0,  108,    3, 0x02 /* Public */,
      17,    1,  109,    3, 0x02 /* Public */,
      19,    2,  112,    3, 0x02 /* Public */,
      21,    2,  117,    3, 0x02 /* Public */,

 // methods: parameters
    0x80000000 | 2,
    0x80000000 | 2, 0x80000000 | 2,    5,
    0x80000000 | 2, 0x80000000 | 2,    5,
    0x80000000 | 2, QMetaType::Int, QMetaType::Int, 0x80000000 | 2, 0x80000000 | 2,    8,    9,   10,   11,
    0x80000000 | 2, QMetaType::Int, QMetaType::Int, 0x80000000 | 2,    8,    9,   12,
    0x80000000 | 2, QMetaType::Int, QMetaType::Int,    8,    9,
    0x80000000 | 2, QMetaType::Int,    9,
    0x80000000 | 2, QMetaType::Int,    9,
    0x80000000 | 2,
    0x80000000 | 2, QMetaType::QString,   18,
    0x80000000 | 2, QMetaType::QString, QMetaType::QString,   20,   18,
    0x80000000 | 2, QMetaType::QString, 0x80000000 | 2,   18,   22,

 // properties: name, type, flags
      23, QMetaType::Int, 0x00095001,
      24, QMetaType::Int, 0x00095001,

       0        // eod
};

void TScriptBinding::Scene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Scene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QScriptValue _r = _t->toString();
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QScriptValue _r = _t->load((*reinterpret_cast< const QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 2: { QScriptValue _r = _t->save((*reinterpret_cast< const QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 3: { QScriptValue _r = _t->setCell((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QScriptValue(*)>(_a[3])),(*reinterpret_cast< const QScriptValue(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 4: { QScriptValue _r = _t->setCell((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QScriptValue(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 5: { QScriptValue _r = _t->getCell((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 6: { QScriptValue _r = _t->insertColumn((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 7: { QScriptValue _r = _t->deleteColumn((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 8: { QScriptValue _r = _t->getLevels();
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 9: { QScriptValue _r = _t->getLevel((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 10: { QScriptValue _r = _t->newLevel((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 11: { QScriptValue _r = _t->loadLevel((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QScriptValue(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Scene *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->getFrameCount(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->getColumnCount(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TScriptBinding::Scene::staticMetaObject = { {
    QMetaObject::SuperData::link<Wrapper::staticMetaObject>(),
    qt_meta_stringdata_TScriptBinding__Scene.data,
    qt_meta_data_TScriptBinding__Scene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TScriptBinding::Scene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TScriptBinding::Scene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TScriptBinding__Scene.stringdata0))
        return static_cast<void*>(this);
    return Wrapper::qt_metacast(_clname);
}

int TScriptBinding::Scene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Wrapper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
