/****************************************************************************
** Meta object code from reading C++ file 'doublefield.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/doublefield.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'doublefield.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__DoubleValueLineEdit_t {
    QByteArrayData data[3];
    char stringdata0[41];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__DoubleValueLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__DoubleValueLineEdit_t qt_meta_stringdata_DVGui__DoubleValueLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 26), // "DVGui::DoubleValueLineEdit"
QT_MOC_LITERAL(1, 27, 12), // "valueChanged"
QT_MOC_LITERAL(2, 40, 0) // ""

    },
    "DVGui::DoubleValueLineEdit\0valueChanged\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__DoubleValueLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void DVGui::DoubleValueLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DoubleValueLineEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DoubleValueLineEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoubleValueLineEdit::valueChanged)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::DoubleValueLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<LineEdit::staticMetaObject>(),
    qt_meta_stringdata_DVGui__DoubleValueLineEdit.data,
    qt_meta_data_DVGui__DoubleValueLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::DoubleValueLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::DoubleValueLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__DoubleValueLineEdit.stringdata0))
        return static_cast<void*>(this);
    return LineEdit::qt_metacast(_clname);
}

int DVGui::DoubleValueLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DVGui::DoubleValueLineEdit::valueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_DVGui__DoubleValueField_t {
    QByteArrayData data[10];
    char stringdata0[150];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__DoubleValueField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__DoubleValueField_t qt_meta_stringdata_DVGui__DoubleValueField = {
    {
QT_MOC_LITERAL(0, 0, 23), // "DVGui::DoubleValueField"
QT_MOC_LITERAL(1, 24, 12), // "valueChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 17), // "valueEditedByHand"
QT_MOC_LITERAL(4, 56, 15), // "onSliderChanged"
QT_MOC_LITERAL(5, 72, 5), // "value"
QT_MOC_LITERAL(6, 78, 16), // "onSliderReleased"
QT_MOC_LITERAL(7, 95, 22), // "onLineEditValueChanged"
QT_MOC_LITERAL(8, 118, 20), // "onRollerValueChanged"
QT_MOC_LITERAL(9, 139, 10) // "isDragging"

    },
    "DVGui::DoubleValueField\0valueChanged\0"
    "\0valueEditedByHand\0onSliderChanged\0"
    "value\0onSliderReleased\0onLineEditValueChanged\0"
    "onRollerValueChanged\0isDragging"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__DoubleValueField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       3,    0,   47,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   48,    2, 0x09 /* Protected */,
       6,    0,   51,    2, 0x09 /* Protected */,
       7,    0,   52,    2, 0x09 /* Protected */,
       8,    1,   53,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    9,

       0        // eod
};

void DVGui::DoubleValueField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DoubleValueField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->valueEditedByHand(); break;
        case 2: _t->onSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onSliderReleased(); break;
        case 4: _t->onLineEditValueChanged(); break;
        case 5: _t->onRollerValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DoubleValueField::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoubleValueField::valueChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DoubleValueField::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DoubleValueField::valueEditedByHand)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::DoubleValueField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__DoubleValueField.data,
    qt_meta_data_DVGui__DoubleValueField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::DoubleValueField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::DoubleValueField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__DoubleValueField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::DoubleValueField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DVGui::DoubleValueField::valueChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::DoubleValueField::valueEditedByHand()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_DVGui__DoubleLineEdit_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__DoubleLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__DoubleLineEdit_t qt_meta_stringdata_DVGui__DoubleLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 21) // "DVGui::DoubleLineEdit"

    },
    "DVGui::DoubleLineEdit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__DoubleLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void DVGui::DoubleLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::DoubleLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<DoubleValueLineEdit::staticMetaObject>(),
    qt_meta_stringdata_DVGui__DoubleLineEdit.data,
    qt_meta_data_DVGui__DoubleLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::DoubleLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::DoubleLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__DoubleLineEdit.stringdata0))
        return static_cast<void*>(this);
    return DoubleValueLineEdit::qt_metacast(_clname);
}

int DVGui::DoubleLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DoubleValueLineEdit::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_DVGui__MeasuredDoubleLineEdit_t {
    QByteArrayData data[9];
    char stringdata0[137];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__MeasuredDoubleLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__MeasuredDoubleLineEdit_t qt_meta_stringdata_DVGui__MeasuredDoubleLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 29), // "DVGui::MeasuredDoubleLineEdit"
QT_MOC_LITERAL(1, 30, 17), // "onEditingFinished"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 13), // "onTextChanged"
QT_MOC_LITERAL(4, 63, 16), // "receiveMouseMove"
QT_MOC_LITERAL(5, 80, 12), // "QMouseEvent*"
QT_MOC_LITERAL(6, 93, 5), // "event"
QT_MOC_LITERAL(7, 99, 17), // "receiveMousePress"
QT_MOC_LITERAL(8, 117, 19) // "receiveMouseRelease"

    },
    "DVGui::MeasuredDoubleLineEdit\0"
    "onEditingFinished\0\0onTextChanged\0"
    "receiveMouseMove\0QMouseEvent*\0event\0"
    "receiveMousePress\0receiveMouseRelease"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__MeasuredDoubleLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x09 /* Protected */,
       3,    1,   40,    2, 0x09 /* Protected */,
       4,    1,   43,    2, 0x09 /* Protected */,
       7,    1,   46,    2, 0x09 /* Protected */,
       8,    1,   49,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 5,    6,

       0        // eod
};

void DVGui::MeasuredDoubleLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MeasuredDoubleLineEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onEditingFinished(); break;
        case 1: _t->onTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->receiveMouseMove((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 3: _t->receiveMousePress((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 4: _t->receiveMouseRelease((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::MeasuredDoubleLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<DoubleValueLineEdit::staticMetaObject>(),
    qt_meta_stringdata_DVGui__MeasuredDoubleLineEdit.data,
    qt_meta_data_DVGui__MeasuredDoubleLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::MeasuredDoubleLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::MeasuredDoubleLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__MeasuredDoubleLineEdit.stringdata0))
        return static_cast<void*>(this);
    return DoubleValueLineEdit::qt_metacast(_clname);
}

int DVGui::MeasuredDoubleLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DoubleValueLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__MeasuredDoubleField_t {
    QByteArrayData data[1];
    char stringdata0[27];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__MeasuredDoubleField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__MeasuredDoubleField_t qt_meta_stringdata_DVGui__MeasuredDoubleField = {
    {
QT_MOC_LITERAL(0, 0, 26) // "DVGui::MeasuredDoubleField"

    },
    "DVGui::MeasuredDoubleField"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__MeasuredDoubleField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void DVGui::MeasuredDoubleField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::MeasuredDoubleField::staticMetaObject = { {
    QMetaObject::SuperData::link<DoubleValueField::staticMetaObject>(),
    qt_meta_stringdata_DVGui__MeasuredDoubleField.data,
    qt_meta_data_DVGui__MeasuredDoubleField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::MeasuredDoubleField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::MeasuredDoubleField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__MeasuredDoubleField.stringdata0))
        return static_cast<void*>(this);
    return DoubleValueField::qt_metacast(_clname);
}

int DVGui::MeasuredDoubleField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DoubleValueField::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
