/****************************************************************************
** Meta object code from reading C++ file 'castviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/castviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'castviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CastTreeViewer_t {
    QByteArrayData data[19];
    char stringdata0[224];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CastTreeViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CastTreeViewer_t qt_meta_stringdata_CastTreeViewer = {
    {
QT_MOC_LITERAL(0, 0, 14), // "CastTreeViewer"
QT_MOC_LITERAL(1, 15, 17), // "itemMovedToFolder"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 13), // "onItemChanged"
QT_MOC_LITERAL(4, 48, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(5, 65, 4), // "item"
QT_MOC_LITERAL(6, 70, 6), // "column"
QT_MOC_LITERAL(7, 77, 15), // "onFolderChanged"
QT_MOC_LITERAL(8, 93, 7), // "current"
QT_MOC_LITERAL(9, 101, 8), // "previous"
QT_MOC_LITERAL(10, 110, 18), // "onSceneNameChanged"
QT_MOC_LITERAL(11, 129, 17), // "onCastFolderAdded"
QT_MOC_LITERAL(12, 147, 9), // "TFilePath"
QT_MOC_LITERAL(13, 157, 4), // "path"
QT_MOC_LITERAL(14, 162, 15), // "rebuildCastTree"
QT_MOC_LITERAL(15, 178, 8), // "folderUp"
QT_MOC_LITERAL(16, 187, 9), // "newFolder"
QT_MOC_LITERAL(17, 197, 12), // "deleteFolder"
QT_MOC_LITERAL(18, 210, 13) // "resizeToConts"

    },
    "CastTreeViewer\0itemMovedToFolder\0\0"
    "onItemChanged\0QTreeWidgetItem*\0item\0"
    "column\0onFolderChanged\0current\0previous\0"
    "onSceneNameChanged\0onCastFolderAdded\0"
    "TFilePath\0path\0rebuildCastTree\0folderUp\0"
    "newFolder\0deleteFolder\0resizeToConts"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CastTreeViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    2,   65,    2, 0x0a /* Public */,
       7,    2,   70,    2, 0x0a /* Public */,
      10,    0,   75,    2, 0x0a /* Public */,
      11,    1,   76,    2, 0x0a /* Public */,
      14,    0,   79,    2, 0x0a /* Public */,
      15,    0,   80,    2, 0x0a /* Public */,
      16,    0,   81,    2, 0x0a /* Public */,
      17,    0,   82,    2, 0x0a /* Public */,
      18,    0,   83,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int,    5,    6,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4,    8,    9,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CastTreeViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CastTreeViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemMovedToFolder(); break;
        case 1: _t->onItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->onFolderChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 3: _t->onSceneNameChanged(); break;
        case 4: _t->onCastFolderAdded((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 5: _t->rebuildCastTree(); break;
        case 6: _t->folderUp(); break;
        case 7: _t->newFolder(); break;
        case 8: _t->deleteFolder(); break;
        case 9: _t->resizeToConts(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CastTreeViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CastTreeViewer::itemMovedToFolder)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CastTreeViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeWidget::staticMetaObject>(),
    qt_meta_stringdata_CastTreeViewer.data,
    qt_meta_data_CastTreeViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CastTreeViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CastTreeViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CastTreeViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSelection"))
        return static_cast< TSelection*>(this);
    return QTreeWidget::qt_metacast(_clname);
}

int CastTreeViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void CastTreeViewer::itemMovedToFolder()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_CastBrowser_t {
    QByteArrayData data[7];
    char stringdata0[69];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CastBrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CastBrowser_t qt_meta_stringdata_CastBrowser = {
    {
QT_MOC_LITERAL(0, 0, 11), // "CastBrowser"
QT_MOC_LITERAL(1, 12, 13), // "folderChanged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(4, 44, 7), // "current"
QT_MOC_LITERAL(5, 52, 8), // "previous"
QT_MOC_LITERAL(6, 61, 7) // "refresh"

    },
    "CastBrowser\0folderChanged\0\0QTreeWidgetItem*\0"
    "current\0previous\0refresh"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CastBrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       6,    0,   29,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void,

       0        // eod
};

void CastBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CastBrowser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->folderChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 1: _t->refresh(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CastBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_meta_stringdata_CastBrowser.data,
    qt_meta_data_CastBrowser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CastBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CastBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CastBrowser.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DvItemListModel"))
        return static_cast< DvItemListModel*>(this);
    return QSplitter::qt_metacast(_clname);
}

int CastBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
