/****************************************************************************
** Meta object code from reading C++ file 'svnlockframerangedialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/svnlockframerangedialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'svnlockframerangedialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SVNLockFrameRangeDialog_t {
    QByteArrayData data[12];
    char stringdata0[176];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNLockFrameRangeDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNLockFrameRangeDialog_t qt_meta_stringdata_SVNLockFrameRangeDialog = {
    {
QT_MOC_LITERAL(0, 0, 23), // "SVNLockFrameRangeDialog"
QT_MOC_LITERAL(1, 24, 4), // "done"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 7), // "onError"
QT_MOC_LITERAL(4, 38, 13), // "onPropGetDone"
QT_MOC_LITERAL(5, 52, 13), // "onPropSetDone"
QT_MOC_LITERAL(6, 66, 19), // "onLockButtonClicked"
QT_MOC_LITERAL(7, 86, 21), // "onCancelButtonClicked"
QT_MOC_LITERAL(8, 108, 10), // "onLockDone"
QT_MOC_LITERAL(9, 119, 6), // "finish"
QT_MOC_LITERAL(10, 126, 25), // "onFromLineEditTextChanged"
QT_MOC_LITERAL(11, 152, 23) // "onToLineEditTextChanged"

    },
    "SVNLockFrameRangeDialog\0done\0\0onError\0"
    "onPropGetDone\0onPropSetDone\0"
    "onLockButtonClicked\0onCancelButtonClicked\0"
    "onLockDone\0finish\0onFromLineEditTextChanged\0"
    "onToLineEditTextChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNLockFrameRangeDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   67,    2, 0x09 /* Protected */,
       4,    1,   70,    2, 0x09 /* Protected */,
       5,    0,   73,    2, 0x09 /* Protected */,
       6,    0,   74,    2, 0x09 /* Protected */,
       7,    0,   75,    2, 0x09 /* Protected */,
       8,    0,   76,    2, 0x09 /* Protected */,
       9,    0,   77,    2, 0x09 /* Protected */,
      10,    0,   78,    2, 0x09 /* Protected */,
      11,    0,   79,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SVNLockFrameRangeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNLockFrameRangeDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onPropGetDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onPropSetDone(); break;
        case 4: _t->onLockButtonClicked(); break;
        case 5: _t->onCancelButtonClicked(); break;
        case 6: _t->onLockDone(); break;
        case 7: _t->finish(); break;
        case 8: _t->onFromLineEditTextChanged(); break;
        case 9: _t->onToLineEditTextChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SVNLockFrameRangeDialog::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SVNLockFrameRangeDialog::done)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNLockFrameRangeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNLockFrameRangeDialog.data,
    qt_meta_data_SVNLockFrameRangeDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNLockFrameRangeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNLockFrameRangeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNLockFrameRangeDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNLockFrameRangeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void SVNLockFrameRangeDialog::done(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SVNLockMultiFrameRangeDialog_t {
    QByteArrayData data[9];
    char stringdata0[142];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNLockMultiFrameRangeDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNLockMultiFrameRangeDialog_t qt_meta_stringdata_SVNLockMultiFrameRangeDialog = {
    {
QT_MOC_LITERAL(0, 0, 28), // "SVNLockMultiFrameRangeDialog"
QT_MOC_LITERAL(1, 29, 4), // "done"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 7), // "onError"
QT_MOC_LITERAL(4, 43, 19), // "onLockButtonClicked"
QT_MOC_LITERAL(5, 63, 17), // "onStatusRetrieved"
QT_MOC_LITERAL(6, 81, 25), // "onFromLineEditTextChanged"
QT_MOC_LITERAL(7, 107, 23), // "onToLineEditTextChanged"
QT_MOC_LITERAL(8, 131, 10) // "onLockDone"

    },
    "SVNLockMultiFrameRangeDialog\0done\0\0"
    "onError\0onLockButtonClicked\0"
    "onStatusRetrieved\0onFromLineEditTextChanged\0"
    "onToLineEditTextChanged\0onLockDone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNLockMultiFrameRangeDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   52,    2, 0x09 /* Protected */,
       4,    0,   55,    2, 0x09 /* Protected */,
       5,    1,   56,    2, 0x09 /* Protected */,
       6,    0,   59,    2, 0x09 /* Protected */,
       7,    0,   60,    2, 0x09 /* Protected */,
       8,    0,   61,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SVNLockMultiFrameRangeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNLockMultiFrameRangeDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onLockButtonClicked(); break;
        case 3: _t->onStatusRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->onFromLineEditTextChanged(); break;
        case 5: _t->onToLineEditTextChanged(); break;
        case 6: _t->onLockDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SVNLockMultiFrameRangeDialog::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SVNLockMultiFrameRangeDialog::done)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNLockMultiFrameRangeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNLockMultiFrameRangeDialog.data,
    qt_meta_data_SVNLockMultiFrameRangeDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNLockMultiFrameRangeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNLockMultiFrameRangeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNLockMultiFrameRangeDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNLockMultiFrameRangeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void SVNLockMultiFrameRangeDialog::done(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SVNUnlockFrameRangeDialog_t {
    QByteArrayData data[10];
    char stringdata0[127];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNUnlockFrameRangeDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNUnlockFrameRangeDialog_t qt_meta_stringdata_SVNUnlockFrameRangeDialog = {
    {
QT_MOC_LITERAL(0, 0, 25), // "SVNUnlockFrameRangeDialog"
QT_MOC_LITERAL(1, 26, 4), // "done"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 7), // "onError"
QT_MOC_LITERAL(4, 40, 13), // "onPropGetDone"
QT_MOC_LITERAL(5, 54, 13), // "onPropSetDone"
QT_MOC_LITERAL(6, 68, 21), // "onUnlockButtonClicked"
QT_MOC_LITERAL(7, 90, 12), // "onCommitDone"
QT_MOC_LITERAL(8, 103, 12), // "onUpdateDone"
QT_MOC_LITERAL(9, 116, 10) // "onLockDone"

    },
    "SVNUnlockFrameRangeDialog\0done\0\0onError\0"
    "onPropGetDone\0onPropSetDone\0"
    "onUnlockButtonClicked\0onCommitDone\0"
    "onUpdateDone\0onLockDone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNUnlockFrameRangeDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   57,    2, 0x09 /* Protected */,
       4,    1,   60,    2, 0x09 /* Protected */,
       5,    0,   63,    2, 0x09 /* Protected */,
       6,    0,   64,    2, 0x09 /* Protected */,
       7,    0,   65,    2, 0x09 /* Protected */,
       8,    0,   66,    2, 0x09 /* Protected */,
       9,    0,   67,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SVNUnlockFrameRangeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNUnlockFrameRangeDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onPropGetDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onPropSetDone(); break;
        case 4: _t->onUnlockButtonClicked(); break;
        case 5: _t->onCommitDone(); break;
        case 6: _t->onUpdateDone(); break;
        case 7: _t->onLockDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SVNUnlockFrameRangeDialog::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SVNUnlockFrameRangeDialog::done)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNUnlockFrameRangeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNUnlockFrameRangeDialog.data,
    qt_meta_data_SVNUnlockFrameRangeDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNUnlockFrameRangeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNUnlockFrameRangeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNUnlockFrameRangeDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNUnlockFrameRangeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void SVNUnlockFrameRangeDialog::done(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SVNUnlockMultiFrameRangeDialog_t {
    QByteArrayData data[7];
    char stringdata0[98];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNUnlockMultiFrameRangeDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNUnlockMultiFrameRangeDialog_t qt_meta_stringdata_SVNUnlockMultiFrameRangeDialog = {
    {
QT_MOC_LITERAL(0, 0, 30), // "SVNUnlockMultiFrameRangeDialog"
QT_MOC_LITERAL(1, 31, 4), // "done"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 7), // "onError"
QT_MOC_LITERAL(4, 45, 17), // "onStatusRetrieved"
QT_MOC_LITERAL(5, 63, 21), // "onUnlockButtonClicked"
QT_MOC_LITERAL(6, 85, 12) // "onUnlockDone"

    },
    "SVNUnlockMultiFrameRangeDialog\0done\0"
    "\0onError\0onStatusRetrieved\0"
    "onUnlockButtonClicked\0onUnlockDone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNUnlockMultiFrameRangeDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   42,    2, 0x09 /* Protected */,
       4,    1,   45,    2, 0x09 /* Protected */,
       5,    0,   48,    2, 0x09 /* Protected */,
       6,    0,   49,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SVNUnlockMultiFrameRangeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNUnlockMultiFrameRangeDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onStatusRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onUnlockButtonClicked(); break;
        case 4: _t->onUnlockDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SVNUnlockMultiFrameRangeDialog::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SVNUnlockMultiFrameRangeDialog::done)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNUnlockMultiFrameRangeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNUnlockMultiFrameRangeDialog.data,
    qt_meta_data_SVNUnlockMultiFrameRangeDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNUnlockMultiFrameRangeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNUnlockMultiFrameRangeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNUnlockMultiFrameRangeDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNUnlockMultiFrameRangeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void SVNUnlockMultiFrameRangeDialog::done(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SVNFrameRangeLockInfoDialog_t {
    QByteArrayData data[4];
    char stringdata0[51];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNFrameRangeLockInfoDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNFrameRangeLockInfoDialog_t qt_meta_stringdata_SVNFrameRangeLockInfoDialog = {
    {
QT_MOC_LITERAL(0, 0, 27), // "SVNFrameRangeLockInfoDialog"
QT_MOC_LITERAL(1, 28, 7), // "onError"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 13) // "onPropGetDone"

    },
    "SVNFrameRangeLockInfoDialog\0onError\0"
    "\0onPropGetDone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNFrameRangeLockInfoDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x09 /* Protected */,
       3,    1,   27,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void SVNFrameRangeLockInfoDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNFrameRangeLockInfoDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->onPropGetDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNFrameRangeLockInfoDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNFrameRangeLockInfoDialog.data,
    qt_meta_data_SVNFrameRangeLockInfoDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNFrameRangeLockInfoDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNFrameRangeLockInfoDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNFrameRangeLockInfoDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNFrameRangeLockInfoDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_SVNMultiFrameRangeLockInfoDialog_t {
    QByteArrayData data[4];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNMultiFrameRangeLockInfoDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNMultiFrameRangeLockInfoDialog_t qt_meta_stringdata_SVNMultiFrameRangeLockInfoDialog = {
    {
QT_MOC_LITERAL(0, 0, 32), // "SVNMultiFrameRangeLockInfoDialog"
QT_MOC_LITERAL(1, 33, 7), // "onError"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 17) // "onStatusRetrieved"

    },
    "SVNMultiFrameRangeLockInfoDialog\0"
    "onError\0\0onStatusRetrieved"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNMultiFrameRangeLockInfoDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x09 /* Protected */,
       3,    1,   27,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void SVNMultiFrameRangeLockInfoDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNMultiFrameRangeLockInfoDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->onStatusRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNMultiFrameRangeLockInfoDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNMultiFrameRangeLockInfoDialog.data,
    qt_meta_data_SVNMultiFrameRangeLockInfoDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNMultiFrameRangeLockInfoDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNMultiFrameRangeLockInfoDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNMultiFrameRangeLockInfoDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNMultiFrameRangeLockInfoDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
