/****************************************************************************
** Meta object code from reading C++ file 'keyframenavigator.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/keyframenavigator.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'keyframenavigator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_KeyframeNavigator_t {
    QByteArrayData data[15];
    char stringdata0[181];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KeyframeNavigator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KeyframeNavigator_t qt_meta_stringdata_KeyframeNavigator = {
    {
QT_MOC_LITERAL(0, 0, 17), // "KeyframeNavigator"
QT_MOC_LITERAL(1, 18, 15), // "setCurrentFrame"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 5), // "frame"
QT_MOC_LITERAL(4, 41, 14), // "setFrameHandle"
QT_MOC_LITERAL(5, 56, 13), // "TFrameHandle*"
QT_MOC_LITERAL(6, 70, 11), // "frameHandle"
QT_MOC_LITERAL(7, 82, 6), // "update"
QT_MOC_LITERAL(8, 89, 14), // "onNextKeyframe"
QT_MOC_LITERAL(9, 104, 8), // "QWidget*"
QT_MOC_LITERAL(10, 113, 5), // "panel"
QT_MOC_LITERAL(11, 119, 14), // "onPrevKeyframe"
QT_MOC_LITERAL(12, 134, 12), // "toggleKeyAct"
QT_MOC_LITERAL(13, 147, 16), // "toggleNextKeyAct"
QT_MOC_LITERAL(14, 164, 16) // "togglePrevKeyAct"

    },
    "KeyframeNavigator\0setCurrentFrame\0\0"
    "frame\0setFrameHandle\0TFrameHandle*\0"
    "frameHandle\0update\0onNextKeyframe\0"
    "QWidget*\0panel\0onPrevKeyframe\0"
    "toggleKeyAct\0toggleNextKeyAct\0"
    "togglePrevKeyAct"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KeyframeNavigator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x0a /* Public */,
       4,    1,   57,    2, 0x0a /* Public */,
       7,    0,   60,    2, 0x0a /* Public */,
       8,    1,   61,    2, 0x0a /* Public */,
      11,    1,   64,    2, 0x0a /* Public */,
      12,    0,   67,    2, 0x09 /* Protected */,
      13,    0,   68,    2, 0x09 /* Protected */,
      14,    0,   69,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void KeyframeNavigator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<KeyframeNavigator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setCurrentFrame((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setFrameHandle((*reinterpret_cast< TFrameHandle*(*)>(_a[1]))); break;
        case 2: _t->update(); break;
        case 3: _t->onNextKeyframe((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 4: _t->onPrevKeyframe((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 5: _t->toggleKeyAct(); break;
        case 6: _t->toggleNextKeyAct(); break;
        case 7: _t->togglePrevKeyAct(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TFrameHandle* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject KeyframeNavigator::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolBar::staticMetaObject>(),
    qt_meta_stringdata_KeyframeNavigator.data,
    qt_meta_data_KeyframeNavigator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *KeyframeNavigator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KeyframeNavigator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_KeyframeNavigator.stringdata0))
        return static_cast<void*>(this);
    return QToolBar::qt_metacast(_clname);
}

int KeyframeNavigator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
struct qt_meta_stringdata_ViewerKeyframeNavigator_t {
    QByteArrayData data[8];
    char stringdata0[113];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ViewerKeyframeNavigator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ViewerKeyframeNavigator_t qt_meta_stringdata_ViewerKeyframeNavigator = {
    {
QT_MOC_LITERAL(0, 0, 23), // "ViewerKeyframeNavigator"
QT_MOC_LITERAL(1, 24, 15), // "setObjectHandle"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 14), // "TObjectHandle*"
QT_MOC_LITERAL(4, 56, 12), // "objectHandle"
QT_MOC_LITERAL(5, 69, 15), // "setXsheetHandle"
QT_MOC_LITERAL(6, 85, 14), // "TXsheetHandle*"
QT_MOC_LITERAL(7, 100, 12) // "xsheetHandle"

    },
    "ViewerKeyframeNavigator\0setObjectHandle\0"
    "\0TObjectHandle*\0objectHandle\0"
    "setXsheetHandle\0TXsheetHandle*\0"
    "xsheetHandle"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ViewerKeyframeNavigator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x0a /* Public */,
       5,    1,   27,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,

       0        // eod
};

void ViewerKeyframeNavigator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ViewerKeyframeNavigator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setObjectHandle((*reinterpret_cast< TObjectHandle*(*)>(_a[1]))); break;
        case 1: _t->setXsheetHandle((*reinterpret_cast< TXsheetHandle*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TObjectHandle* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TXsheetHandle* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ViewerKeyframeNavigator::staticMetaObject = { {
    QMetaObject::SuperData::link<KeyframeNavigator::staticMetaObject>(),
    qt_meta_stringdata_ViewerKeyframeNavigator.data,
    qt_meta_data_ViewerKeyframeNavigator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ViewerKeyframeNavigator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ViewerKeyframeNavigator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ViewerKeyframeNavigator.stringdata0))
        return static_cast<void*>(this);
    return KeyframeNavigator::qt_metacast(_clname);
}

int ViewerKeyframeNavigator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KeyframeNavigator::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_PaletteKeyframeNavigator_t {
    QByteArrayData data[5];
    char stringdata0[73];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PaletteKeyframeNavigator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PaletteKeyframeNavigator_t qt_meta_stringdata_PaletteKeyframeNavigator = {
    {
QT_MOC_LITERAL(0, 0, 24), // "PaletteKeyframeNavigator"
QT_MOC_LITERAL(1, 25, 16), // "setPaletteHandle"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 15), // "TPaletteHandle*"
QT_MOC_LITERAL(4, 59, 13) // "paletteHandle"

    },
    "PaletteKeyframeNavigator\0setPaletteHandle\0"
    "\0TPaletteHandle*\0paletteHandle"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PaletteKeyframeNavigator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void PaletteKeyframeNavigator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PaletteKeyframeNavigator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setPaletteHandle((*reinterpret_cast< TPaletteHandle*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TPaletteHandle* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PaletteKeyframeNavigator::staticMetaObject = { {
    QMetaObject::SuperData::link<KeyframeNavigator::staticMetaObject>(),
    qt_meta_stringdata_PaletteKeyframeNavigator.data,
    qt_meta_data_PaletteKeyframeNavigator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PaletteKeyframeNavigator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaletteKeyframeNavigator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PaletteKeyframeNavigator.stringdata0))
        return static_cast<void*>(this);
    return KeyframeNavigator::qt_metacast(_clname);
}

int PaletteKeyframeNavigator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KeyframeNavigator::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_FxKeyframeNavigator_t {
    QByteArrayData data[5];
    char stringdata0[53];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxKeyframeNavigator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxKeyframeNavigator_t qt_meta_stringdata_FxKeyframeNavigator = {
    {
QT_MOC_LITERAL(0, 0, 19), // "FxKeyframeNavigator"
QT_MOC_LITERAL(1, 20, 11), // "setFxHandle"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 10), // "TFxHandle*"
QT_MOC_LITERAL(4, 44, 8) // "fxHandle"

    },
    "FxKeyframeNavigator\0setFxHandle\0\0"
    "TFxHandle*\0fxHandle"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxKeyframeNavigator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void FxKeyframeNavigator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxKeyframeNavigator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setFxHandle((*reinterpret_cast< TFxHandle*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TFxHandle* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxKeyframeNavigator::staticMetaObject = { {
    QMetaObject::SuperData::link<KeyframeNavigator::staticMetaObject>(),
    qt_meta_stringdata_FxKeyframeNavigator.data,
    qt_meta_data_FxKeyframeNavigator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxKeyframeNavigator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxKeyframeNavigator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxKeyframeNavigator.stringdata0))
        return static_cast<void*>(this);
    return KeyframeNavigator::qt_metacast(_clname);
}

int FxKeyframeNavigator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KeyframeNavigator::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
