#pragma once

#ifndef FXPARAMEDITORPOPUP_H
#define FXPARAMEDITORPOPUP_H

#include "toonzqt/dvdialog.h"

//=============================================================================
// FxParamEditorPopup
//-----------------------------------------------------------------------------

class FxParamEditorPopup final : public QDialog {
  Q_OBJECT

public:
  FxParamEditorPopup();
};

#endif  // FXPARAMEDITORPOPUP_H
