/****************************************************************************
** Meta object code from reading C++ file 'functiontoolbar.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/functiontoolbar.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'functiontoolbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FunctionToolbar_t {
    QByteArrayData data[16];
    char stringdata0[203];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionToolbar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionToolbar_t qt_meta_stringdata_FunctionToolbar = {
    {
QT_MOC_LITERAL(0, 0, 15), // "FunctionToolbar"
QT_MOC_LITERAL(1, 16, 22), // "numericalColumnToggled"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 8), // "setCurve"
QT_MOC_LITERAL(4, 49, 13), // "TDoubleParam*"
QT_MOC_LITERAL(5, 63, 5), // "curve"
QT_MOC_LITERAL(6, 69, 8), // "setFrame"
QT_MOC_LITERAL(7, 78, 5), // "frame"
QT_MOC_LITERAL(8, 84, 14), // "onNextKeyframe"
QT_MOC_LITERAL(9, 99, 8), // "QWidget*"
QT_MOC_LITERAL(10, 108, 5), // "panel"
QT_MOC_LITERAL(11, 114, 14), // "onPrevKeyframe"
QT_MOC_LITERAL(12, 129, 19), // "onValueFieldChanged"
QT_MOC_LITERAL(13, 149, 15), // "onFrameSwitched"
QT_MOC_LITERAL(14, 165, 18), // "onNavFrameSwitched"
QT_MOC_LITERAL(15, 184, 18) // "onSelectionChanged"

    },
    "FunctionToolbar\0numericalColumnToggled\0"
    "\0setCurve\0TDoubleParam*\0curve\0setFrame\0"
    "frame\0onNextKeyframe\0QWidget*\0panel\0"
    "onPrevKeyframe\0onValueFieldChanged\0"
    "onFrameSwitched\0onNavFrameSwitched\0"
    "onSelectionChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionToolbar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   60,    2, 0x0a /* Public */,
       6,    1,   63,    2, 0x0a /* Public */,
       8,    1,   66,    2, 0x0a /* Public */,
      11,    1,   69,    2, 0x0a /* Public */,
      12,    0,   72,    2, 0x08 /* Private */,
      13,    0,   73,    2, 0x08 /* Private */,
      14,    0,   74,    2, 0x08 /* Private */,
      15,    0,   75,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, QMetaType::Double,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void FunctionToolbar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionToolbar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->numericalColumnToggled(); break;
        case 1: _t->setCurve((*reinterpret_cast< TDoubleParam*(*)>(_a[1]))); break;
        case 2: _t->setFrame((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 3: _t->onNextKeyframe((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 4: _t->onPrevKeyframe((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        case 5: _t->onValueFieldChanged(); break;
        case 6: _t->onFrameSwitched(); break;
        case 7: _t->onNavFrameSwitched(); break;
        case 8: _t->onSelectionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FunctionToolbar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionToolbar::numericalColumnToggled)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionToolbar::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::ToolBar::staticMetaObject>(),
    qt_meta_stringdata_FunctionToolbar.data,
    qt_meta_data_FunctionToolbar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionToolbar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionToolbar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionToolbar.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TParamObserver"))
        return static_cast< TParamObserver*>(this);
    return DVGui::ToolBar::qt_metacast(_clname);
}

int FunctionToolbar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::ToolBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void FunctionToolbar::numericalColumnToggled()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
