#pragma once

#ifndef __TTWAIN_STATE_PD_H__
#define __TTWAIN_STATE_PD_H__

#ifdef __cplusplus
extern "C" {
#endif

int TTWAIN_LoadSourceManagerPD(void);
int TTWAIN_UnloadSourceManagerPD(void);
int TTWAIN_UnloadSourceManagerPD(void);
void TTWAIN_EmptyMessageQueuePD(void);

#ifdef __cplusplus
}
#endif

#endif
