/****************************************************************************
** Meta object code from reading C++ file 'exportxsheetpdf.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/exportxsheetpdf.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'exportxsheetpdf.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_XsheetPdfPreviewPane_t {
    QByteArrayData data[1];
    char stringdata0[21];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetPdfPreviewPane_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetPdfPreviewPane_t qt_meta_stringdata_XsheetPdfPreviewPane = {
    {
QT_MOC_LITERAL(0, 0, 20) // "XsheetPdfPreviewPane"

    },
    "XsheetPdfPreviewPane"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetPdfPreviewPane[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void XsheetPdfPreviewPane::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject XsheetPdfPreviewPane::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetPdfPreviewPane.data,
    qt_meta_data_XsheetPdfPreviewPane,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetPdfPreviewPane::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetPdfPreviewPane::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetPdfPreviewPane.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int XsheetPdfPreviewPane::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_XsheetPdfPreviewArea_t {
    QByteArrayData data[3];
    char stringdata0[34];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetPdfPreviewArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetPdfPreviewArea_t qt_meta_stringdata_XsheetPdfPreviewArea = {
    {
QT_MOC_LITERAL(0, 0, 20), // "XsheetPdfPreviewArea"
QT_MOC_LITERAL(1, 21, 11), // "fitToWindow"
QT_MOC_LITERAL(2, 33, 0) // ""

    },
    "XsheetPdfPreviewArea\0fitToWindow\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetPdfPreviewArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void XsheetPdfPreviewArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<XsheetPdfPreviewArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fitToWindow(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject XsheetPdfPreviewArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_XsheetPdfPreviewArea.data,
    qt_meta_data_XsheetPdfPreviewArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetPdfPreviewArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetPdfPreviewArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetPdfPreviewArea.stringdata0))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int XsheetPdfPreviewArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ExportXsheetPdfPopup_t {
    QByteArrayData data[14];
    char stringdata0[183];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportXsheetPdfPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportXsheetPdfPopup_t qt_meta_stringdata_ExportXsheetPdfPopup = {
    {
QT_MOC_LITERAL(0, 0, 20), // "ExportXsheetPdfPopup"
QT_MOC_LITERAL(1, 21, 8), // "onExport"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 11), // "onExportPNG"
QT_MOC_LITERAL(4, 43, 11), // "onExportCSV"
QT_MOC_LITERAL(5, 55, 12), // "initTemplate"
QT_MOC_LITERAL(6, 68, 7), // "setInfo"
QT_MOC_LITERAL(7, 76, 13), // "updatePreview"
QT_MOC_LITERAL(8, 90, 17), // "onLogoModeToggled"
QT_MOC_LITERAL(9, 108, 20), // "onLogoImgPathChanged"
QT_MOC_LITERAL(10, 129, 6), // "onPrev"
QT_MOC_LITERAL(11, 136, 6), // "onNext"
QT_MOC_LITERAL(12, 143, 22), // "onTickIdComboActivated"
QT_MOC_LITERAL(13, 166, 16) // "onDurationEdited"

    },
    "ExportXsheetPdfPopup\0onExport\0\0"
    "onExportPNG\0onExportCSV\0initTemplate\0"
    "setInfo\0updatePreview\0onLogoModeToggled\0"
    "onLogoImgPathChanged\0onPrev\0onNext\0"
    "onTickIdComboActivated\0onDurationEdited"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportXsheetPdfPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x09 /* Protected */,
       3,    0,   75,    2, 0x09 /* Protected */,
       4,    0,   76,    2, 0x09 /* Protected */,
       5,    0,   77,    2, 0x09 /* Protected */,
       6,    0,   78,    2, 0x09 /* Protected */,
       7,    0,   79,    2, 0x09 /* Protected */,
       8,    0,   80,    2, 0x09 /* Protected */,
       9,    0,   81,    2, 0x09 /* Protected */,
      10,    0,   82,    2, 0x09 /* Protected */,
      11,    0,   83,    2, 0x09 /* Protected */,
      12,    0,   84,    2, 0x09 /* Protected */,
      13,    0,   85,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ExportXsheetPdfPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExportXsheetPdfPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onExport(); break;
        case 1: _t->onExportPNG(); break;
        case 2: _t->onExportCSV(); break;
        case 3: _t->initTemplate(); break;
        case 4: _t->setInfo(); break;
        case 5: _t->updatePreview(); break;
        case 6: _t->onLogoModeToggled(); break;
        case 7: _t->onLogoImgPathChanged(); break;
        case 8: _t->onPrev(); break;
        case 9: _t->onNext(); break;
        case 10: _t->onTickIdComboActivated(); break;
        case 11: _t->onDurationEdited(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ExportXsheetPdfPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_ExportXsheetPdfPopup.data,
    qt_meta_data_ExportXsheetPdfPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportXsheetPdfPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportXsheetPdfPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportXsheetPdfPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int ExportXsheetPdfPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
