/****************************************************************************
** Meta object code from reading C++ file 'xsheetviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/xsheetviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xsheetviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_XsheetScrollArea_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetScrollArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetScrollArea_t qt_meta_stringdata_XsheetScrollArea = {
    {
QT_MOC_LITERAL(0, 0, 16) // "XsheetScrollArea"

    },
    "XsheetScrollArea"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetScrollArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void XsheetScrollArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject XsheetScrollArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_XsheetScrollArea.data,
    qt_meta_data_XsheetScrollArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetScrollArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetScrollArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetScrollArea.stringdata0))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int XsheetScrollArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_XsheetViewer_t {
    QByteArrayData data[158];
    char stringdata0[3308];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetViewer_t qt_meta_stringdata_XsheetViewer = {
    {
QT_MOC_LITERAL(0, 0, 12), // "XsheetViewer"
QT_MOC_LITERAL(1, 13, 18), // "orientationChanged"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 18), // "const Orientation*"
QT_MOC_LITERAL(4, 52, 14), // "newOrientation"
QT_MOC_LITERAL(5, 67, 16), // "positionSections"
QT_MOC_LITERAL(6, 84, 15), // "onSceneSwitched"
QT_MOC_LITERAL(7, 100, 15), // "onXsheetChanged"
QT_MOC_LITERAL(8, 116, 22), // "onCurrentFrameSwitched"
QT_MOC_LITERAL(9, 139, 22), // "onPlayingStatusChanged"
QT_MOC_LITERAL(10, 162, 23), // "onCurrentColumnSwitched"
QT_MOC_LITERAL(11, 186, 19), // "onSelectionSwitched"
QT_MOC_LITERAL(12, 206, 11), // "TSelection*"
QT_MOC_LITERAL(13, 218, 12), // "oldSelection"
QT_MOC_LITERAL(14, 231, 12), // "newSelection"
QT_MOC_LITERAL(15, 244, 18), // "onSelectionChanged"
QT_MOC_LITERAL(16, 263, 9), // "selection"
QT_MOC_LITERAL(17, 273, 13), // "updateAllAree"
QT_MOC_LITERAL(18, 287, 10), // "isDragging"
QT_MOC_LITERAL(19, 298, 16), // "updateColumnArea"
QT_MOC_LITERAL(20, 315, 20), // "updateCellColumnAree"
QT_MOC_LITERAL(21, 336, 17), // "updateCellRowAree"
QT_MOC_LITERAL(22, 354, 14), // "onScrubStopped"
QT_MOC_LITERAL(23, 369, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(24, 389, 8), // "prefName"
QT_MOC_LITERAL(25, 398, 17), // "changeWindowTitle"
QT_MOC_LITERAL(26, 416, 16), // "resetXsheetNotes"
QT_MOC_LITERAL(27, 433, 20), // "onOrientationChanged"
QT_MOC_LITERAL(28, 454, 23), // "onPrepareToScrollOffset"
QT_MOC_LITERAL(29, 478, 6), // "offset"
QT_MOC_LITERAL(30, 485, 18), // "onZoomScrollAdjust"
QT_MOC_LITERAL(31, 504, 8), // "QPointF&"
QT_MOC_LITERAL(32, 513, 6), // "toZoom"
QT_MOC_LITERAL(33, 520, 18), // "setFrameZoomFactor"
QT_MOC_LITERAL(34, 539, 1), // "f"
QT_MOC_LITERAL(35, 541, 18), // "getFrameZoomFactor"
QT_MOC_LITERAL(36, 560, 22), // "getFrameZoomAdjustment"
QT_MOC_LITERAL(37, 583, 11), // "zoomOnFrame"
QT_MOC_LITERAL(38, 595, 5), // "frame"
QT_MOC_LITERAL(39, 601, 6), // "factor"
QT_MOC_LITERAL(40, 608, 17), // "LightLightBGColor"
QT_MOC_LITERAL(41, 626, 12), // "LightBGColor"
QT_MOC_LITERAL(42, 639, 7), // "BGColor"
QT_MOC_LITERAL(43, 647, 11), // "DarkBGColor"
QT_MOC_LITERAL(44, 659, 14), // "LightLineColor"
QT_MOC_LITERAL(45, 674, 13), // "DarkLineColor"
QT_MOC_LITERAL(46, 688, 19), // "ColumnIconLineColor"
QT_MOC_LITERAL(47, 708, 21), // "TimelineIconLineColor"
QT_MOC_LITERAL(48, 730, 17), // "CurrentRowBgColor"
QT_MOC_LITERAL(49, 748, 15), // "MarkerLineColor"
QT_MOC_LITERAL(50, 764, 18), // "SecMarkerLineColor"
QT_MOC_LITERAL(51, 783, 23), // "SelectedMarkerLineColor"
QT_MOC_LITERAL(52, 807, 17), // "VerticalLineColor"
QT_MOC_LITERAL(53, 825, 21), // "VerticalLineHeadColor"
QT_MOC_LITERAL(54, 847, 9), // "TextColor"
QT_MOC_LITERAL(55, 857, 14), // "ErrorTextColor"
QT_MOC_LITERAL(56, 872, 21), // "CurrentFrameTextColor"
QT_MOC_LITERAL(57, 894, 17), // "SelectedTextColor"
QT_MOC_LITERAL(58, 912, 14), // "FrameTextColor"
QT_MOC_LITERAL(59, 927, 17), // "KeyframeLineColor"
QT_MOC_LITERAL(60, 945, 21), // "PreviewFrameTextColor"
QT_MOC_LITERAL(61, 967, 20), // "OnionSkinAreaBgColor"
QT_MOC_LITERAL(62, 988, 25), // "FrameRangeMarkerLineColor"
QT_MOC_LITERAL(63, 1014, 25), // "CurrentTimeIndicatorColor"
QT_MOC_LITERAL(64, 1040, 15), // "ColumnTextColor"
QT_MOC_LITERAL(65, 1056, 24), // "HighlightColumnTextColor"
QT_MOC_LITERAL(66, 1081, 20), // "EmptyColumnHeadColor"
QT_MOC_LITERAL(67, 1102, 14), // "EmptyCellColor"
QT_MOC_LITERAL(68, 1117, 19), // "NotEmptyColumnColor"
QT_MOC_LITERAL(69, 1137, 22), // "SelectedEmptyCellColor"
QT_MOC_LITERAL(70, 1160, 13), // "LevelEndColor"
QT_MOC_LITERAL(71, 1174, 14), // "CellFocusColor"
QT_MOC_LITERAL(72, 1189, 14), // "PlayRangeColor"
QT_MOC_LITERAL(73, 1204, 16), // "LevelColumnColor"
QT_MOC_LITERAL(74, 1221, 22), // "LevelColumnBorderColor"
QT_MOC_LITERAL(75, 1244, 24), // "SelectedLevelColumnColor"
QT_MOC_LITERAL(76, 1269, 17), // "VectorColumnColor"
QT_MOC_LITERAL(77, 1287, 23), // "VectorColumnBorderColor"
QT_MOC_LITERAL(78, 1311, 25), // "SelectedVectorColumnColor"
QT_MOC_LITERAL(79, 1337, 16), // "ChildColumnColor"
QT_MOC_LITERAL(80, 1354, 22), // "ChildColumnBorderColor"
QT_MOC_LITERAL(81, 1377, 24), // "SelectedChildColumnColor"
QT_MOC_LITERAL(82, 1402, 20), // "FullcolorColumnColor"
QT_MOC_LITERAL(83, 1423, 26), // "FullcolorColumnBorderColor"
QT_MOC_LITERAL(84, 1450, 28), // "SelectedFullcolorColumnColor"
QT_MOC_LITERAL(85, 1479, 13), // "FxColumnColor"
QT_MOC_LITERAL(86, 1493, 19), // "FxColumnBorderColor"
QT_MOC_LITERAL(87, 1513, 21), // "SelectedFxColumnColor"
QT_MOC_LITERAL(88, 1535, 20), // "ReferenceColumnColor"
QT_MOC_LITERAL(89, 1556, 26), // "ReferenceColumnBorderColor"
QT_MOC_LITERAL(90, 1583, 28), // "SelectedReferenceColumnColor"
QT_MOC_LITERAL(91, 1612, 18), // "PaletteColumnColor"
QT_MOC_LITERAL(92, 1631, 24), // "PaletteColumnBorderColor"
QT_MOC_LITERAL(93, 1656, 26), // "SelectedPaletteColumnColor"
QT_MOC_LITERAL(94, 1683, 15), // "MeshColumnColor"
QT_MOC_LITERAL(95, 1699, 21), // "MeshColumnBorderColor"
QT_MOC_LITERAL(96, 1721, 23), // "SelectedMeshColumnColor"
QT_MOC_LITERAL(97, 1745, 10), // "TableColor"
QT_MOC_LITERAL(98, 1756, 8), // "PegColor"
QT_MOC_LITERAL(99, 1765, 20), // "SoundTextColumnColor"
QT_MOC_LITERAL(100, 1786, 26), // "SoundTextColumnBorderColor"
QT_MOC_LITERAL(101, 1813, 28), // "SelectedSoundTextColumnColor"
QT_MOC_LITERAL(102, 1842, 16), // "SoundColumnColor"
QT_MOC_LITERAL(103, 1859, 22), // "SoundColumnBorderColor"
QT_MOC_LITERAL(104, 1882, 24), // "SelectedSoundColumnColor"
QT_MOC_LITERAL(105, 1907, 18), // "SoundColumnHlColor"
QT_MOC_LITERAL(106, 1926, 21), // "SoundColumnTrackColor"
QT_MOC_LITERAL(107, 1948, 15), // "MetaColumnColor"
QT_MOC_LITERAL(108, 1964, 21), // "MetaColumnBorderColor"
QT_MOC_LITERAL(109, 1986, 23), // "SelectedMetaColumnColor"
QT_MOC_LITERAL(110, 2010, 20), // "ColumnHeadPastelizer"
QT_MOC_LITERAL(111, 2031, 18), // "SelectedColumnHead"
QT_MOC_LITERAL(112, 2050, 19), // "FoldedColumnBGColor"
QT_MOC_LITERAL(113, 2070, 21), // "FoldedColumnLineColor"
QT_MOC_LITERAL(114, 2092, 23), // "XsheetColumnNameBgColor"
QT_MOC_LITERAL(115, 2116, 27), // "XsheetDragBarHighlightColor"
QT_MOC_LITERAL(116, 2144, 17), // "ActiveCameraColor"
QT_MOC_LITERAL(117, 2162, 25), // "SelectedActiveCameraColor"
QT_MOC_LITERAL(118, 2188, 16), // "OtherCameraColor"
QT_MOC_LITERAL(119, 2205, 24), // "SelectedOtherCameraColor"
QT_MOC_LITERAL(120, 2230, 28), // "XsheetPreviewButtonBgOnColor"
QT_MOC_LITERAL(121, 2259, 26), // "XsheetPreviewButtonOnImage"
QT_MOC_LITERAL(122, 2286, 29), // "XsheetPreviewButtonBgOffColor"
QT_MOC_LITERAL(123, 2316, 27), // "XsheetPreviewButtonOffImage"
QT_MOC_LITERAL(124, 2344, 30), // "XsheetUnifiedButtonTranspImage"
QT_MOC_LITERAL(125, 2375, 29), // "XsheetCamstandButtonBgOnColor"
QT_MOC_LITERAL(126, 2405, 27), // "XsheetCamstandButtonOnImage"
QT_MOC_LITERAL(127, 2433, 31), // "XsheetCamstandButtonTranspImage"
QT_MOC_LITERAL(128, 2465, 30), // "XsheetCamstandButtonBgOffColor"
QT_MOC_LITERAL(129, 2496, 28), // "XsheetCamstandButtonOffImage"
QT_MOC_LITERAL(130, 2525, 25), // "XsheetLockButtonBgOnColor"
QT_MOC_LITERAL(131, 2551, 23), // "XsheetLockButtonOnImage"
QT_MOC_LITERAL(132, 2575, 26), // "XsheetLockButtonBgOffColor"
QT_MOC_LITERAL(133, 2602, 24), // "XsheetLockButtonOffImage"
QT_MOC_LITERAL(134, 2627, 25), // "XsheetConfigButtonBgColor"
QT_MOC_LITERAL(135, 2653, 23), // "XsheetConfigButtonImage"
QT_MOC_LITERAL(136, 2677, 30), // "TimelinePreviewButtonBgOnColor"
QT_MOC_LITERAL(137, 2708, 28), // "TimelinePreviewButtonOnImage"
QT_MOC_LITERAL(138, 2737, 31), // "TimelinePreviewButtonBgOffColor"
QT_MOC_LITERAL(139, 2769, 29), // "TimelinePreviewButtonOffImage"
QT_MOC_LITERAL(140, 2799, 32), // "TimelineUnifiedButtonTranspImage"
QT_MOC_LITERAL(141, 2832, 31), // "TimelineCamstandButtonBgOnColor"
QT_MOC_LITERAL(142, 2864, 29), // "TimelineCamstandButtonOnImage"
QT_MOC_LITERAL(143, 2894, 33), // "TimelineCamstandButtonTranspI..."
QT_MOC_LITERAL(144, 2928, 32), // "TimelineCamstandButtonBgOffColor"
QT_MOC_LITERAL(145, 2961, 30), // "TimelineCamstandButtonOffImage"
QT_MOC_LITERAL(146, 2992, 27), // "TimelineLockButtonBgOnColor"
QT_MOC_LITERAL(147, 3020, 25), // "TimelineLockButtonOnImage"
QT_MOC_LITERAL(148, 3046, 28), // "TimelineLockButtonBgOffColor"
QT_MOC_LITERAL(149, 3075, 26), // "TimelineLockButtonOffImage"
QT_MOC_LITERAL(150, 3102, 27), // "TimelineConfigButtonBgColor"
QT_MOC_LITERAL(151, 3130, 25), // "TimelineConfigButtonImage"
QT_MOC_LITERAL(152, 3156, 23), // "LayerHeaderPreviewImage"
QT_MOC_LITERAL(153, 3180, 27), // "LayerHeaderPreviewOverImage"
QT_MOC_LITERAL(154, 3208, 24), // "LayerHeaderCamstandImage"
QT_MOC_LITERAL(155, 3233, 28), // "LayerHeaderCamstandOverImage"
QT_MOC_LITERAL(156, 3262, 20), // "LayerHeaderLockImage"
QT_MOC_LITERAL(157, 3283, 24) // "LayerHeaderLockOverImage"

    },
    "XsheetViewer\0orientationChanged\0\0"
    "const Orientation*\0newOrientation\0"
    "positionSections\0onSceneSwitched\0"
    "onXsheetChanged\0onCurrentFrameSwitched\0"
    "onPlayingStatusChanged\0onCurrentColumnSwitched\0"
    "onSelectionSwitched\0TSelection*\0"
    "oldSelection\0newSelection\0onSelectionChanged\0"
    "selection\0updateAllAree\0isDragging\0"
    "updateColumnArea\0updateCellColumnAree\0"
    "updateCellRowAree\0onScrubStopped\0"
    "onPreferenceChanged\0prefName\0"
    "changeWindowTitle\0resetXsheetNotes\0"
    "onOrientationChanged\0onPrepareToScrollOffset\0"
    "offset\0onZoomScrollAdjust\0QPointF&\0"
    "toZoom\0setFrameZoomFactor\0f\0"
    "getFrameZoomFactor\0getFrameZoomAdjustment\0"
    "zoomOnFrame\0frame\0factor\0LightLightBGColor\0"
    "LightBGColor\0BGColor\0DarkBGColor\0"
    "LightLineColor\0DarkLineColor\0"
    "ColumnIconLineColor\0TimelineIconLineColor\0"
    "CurrentRowBgColor\0MarkerLineColor\0"
    "SecMarkerLineColor\0SelectedMarkerLineColor\0"
    "VerticalLineColor\0VerticalLineHeadColor\0"
    "TextColor\0ErrorTextColor\0CurrentFrameTextColor\0"
    "SelectedTextColor\0FrameTextColor\0"
    "KeyframeLineColor\0PreviewFrameTextColor\0"
    "OnionSkinAreaBgColor\0FrameRangeMarkerLineColor\0"
    "CurrentTimeIndicatorColor\0ColumnTextColor\0"
    "HighlightColumnTextColor\0EmptyColumnHeadColor\0"
    "EmptyCellColor\0NotEmptyColumnColor\0"
    "SelectedEmptyCellColor\0LevelEndColor\0"
    "CellFocusColor\0PlayRangeColor\0"
    "LevelColumnColor\0LevelColumnBorderColor\0"
    "SelectedLevelColumnColor\0VectorColumnColor\0"
    "VectorColumnBorderColor\0"
    "SelectedVectorColumnColor\0ChildColumnColor\0"
    "ChildColumnBorderColor\0SelectedChildColumnColor\0"
    "FullcolorColumnColor\0FullcolorColumnBorderColor\0"
    "SelectedFullcolorColumnColor\0FxColumnColor\0"
    "FxColumnBorderColor\0SelectedFxColumnColor\0"
    "ReferenceColumnColor\0ReferenceColumnBorderColor\0"
    "SelectedReferenceColumnColor\0"
    "PaletteColumnColor\0PaletteColumnBorderColor\0"
    "SelectedPaletteColumnColor\0MeshColumnColor\0"
    "MeshColumnBorderColor\0SelectedMeshColumnColor\0"
    "TableColor\0PegColor\0SoundTextColumnColor\0"
    "SoundTextColumnBorderColor\0"
    "SelectedSoundTextColumnColor\0"
    "SoundColumnColor\0SoundColumnBorderColor\0"
    "SelectedSoundColumnColor\0SoundColumnHlColor\0"
    "SoundColumnTrackColor\0MetaColumnColor\0"
    "MetaColumnBorderColor\0SelectedMetaColumnColor\0"
    "ColumnHeadPastelizer\0SelectedColumnHead\0"
    "FoldedColumnBGColor\0FoldedColumnLineColor\0"
    "XsheetColumnNameBgColor\0"
    "XsheetDragBarHighlightColor\0"
    "ActiveCameraColor\0SelectedActiveCameraColor\0"
    "OtherCameraColor\0SelectedOtherCameraColor\0"
    "XsheetPreviewButtonBgOnColor\0"
    "XsheetPreviewButtonOnImage\0"
    "XsheetPreviewButtonBgOffColor\0"
    "XsheetPreviewButtonOffImage\0"
    "XsheetUnifiedButtonTranspImage\0"
    "XsheetCamstandButtonBgOnColor\0"
    "XsheetCamstandButtonOnImage\0"
    "XsheetCamstandButtonTranspImage\0"
    "XsheetCamstandButtonBgOffColor\0"
    "XsheetCamstandButtonOffImage\0"
    "XsheetLockButtonBgOnColor\0"
    "XsheetLockButtonOnImage\0"
    "XsheetLockButtonBgOffColor\0"
    "XsheetLockButtonOffImage\0"
    "XsheetConfigButtonBgColor\0"
    "XsheetConfigButtonImage\0"
    "TimelinePreviewButtonBgOnColor\0"
    "TimelinePreviewButtonOnImage\0"
    "TimelinePreviewButtonBgOffColor\0"
    "TimelinePreviewButtonOffImage\0"
    "TimelineUnifiedButtonTranspImage\0"
    "TimelineCamstandButtonBgOnColor\0"
    "TimelineCamstandButtonOnImage\0"
    "TimelineCamstandButtonTranspImage\0"
    "TimelineCamstandButtonBgOffColor\0"
    "TimelineCamstandButtonOffImage\0"
    "TimelineLockButtonBgOnColor\0"
    "TimelineLockButtonOnImage\0"
    "TimelineLockButtonBgOffColor\0"
    "TimelineLockButtonOffImage\0"
    "TimelineConfigButtonBgColor\0"
    "TimelineConfigButtonImage\0"
    "LayerHeaderPreviewImage\0"
    "LayerHeaderPreviewOverImage\0"
    "LayerHeaderCamstandImage\0"
    "LayerHeaderCamstandOverImage\0"
    "LayerHeaderLockImage\0LayerHeaderLockOverImage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
     118,  190, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  139,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  142,    2, 0x0a /* Public */,
       6,    0,  143,    2, 0x0a /* Public */,
       7,    0,  144,    2, 0x0a /* Public */,
       8,    0,  145,    2, 0x0a /* Public */,
       9,    0,  146,    2, 0x0a /* Public */,
      10,    0,  147,    2, 0x0a /* Public */,
      11,    2,  148,    2, 0x0a /* Public */,
      15,    1,  153,    2, 0x0a /* Public */,
      17,    1,  156,    2, 0x0a /* Public */,
      17,    0,  159,    2, 0x2a /* Public | MethodCloned */,
      19,    0,  160,    2, 0x0a /* Public */,
      20,    0,  161,    2, 0x0a /* Public */,
      21,    0,  162,    2, 0x0a /* Public */,
      22,    0,  163,    2, 0x0a /* Public */,
      23,    1,  164,    2, 0x0a /* Public */,
      25,    0,  167,    2, 0x0a /* Public */,
      26,    0,  168,    2, 0x0a /* Public */,
      27,    1,  169,    2, 0x0a /* Public */,
      28,    1,  172,    2, 0x0a /* Public */,
      30,    2,  175,    2, 0x0a /* Public */,
      33,    1,  180,    2, 0x0a /* Public */,
      35,    0,  183,    2, 0x0a /* Public */,
      36,    0,  184,    2, 0x0a /* Public */,
      37,    2,  185,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12, 0x80000000 | 12,   13,   14,
    QMetaType::Void, 0x80000000 | 12,   16,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QPointF,   29,
    QMetaType::Void, 0x80000000 | 31, QMetaType::Bool,   29,   32,
    QMetaType::Void, QMetaType::Int,   34,
    QMetaType::Int,
    QMetaType::QPoint,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   38,   39,

 // properties: name, type, flags
      40, QMetaType::QColor, 0x00095103,
      41, QMetaType::QColor, 0x00095103,
      42, QMetaType::QColor, 0x00095103,
      43, QMetaType::QColor, 0x00095103,
      44, QMetaType::QColor, 0x00095103,
      45, QMetaType::QColor, 0x00095103,
      46, QMetaType::QColor, 0x00095103,
      47, QMetaType::QColor, 0x00095103,
      48, QMetaType::QColor, 0x00095103,
      49, QMetaType::QColor, 0x00095103,
      50, QMetaType::QColor, 0x00095103,
      51, QMetaType::QColor, 0x00095103,
      52, QMetaType::QColor, 0x00095103,
      53, QMetaType::QColor, 0x00095103,
      54, QMetaType::QColor, 0x00095103,
      55, QMetaType::QColor, 0x00095103,
      56, QMetaType::QColor, 0x00095103,
      57, QMetaType::QColor, 0x00095103,
      58, QMetaType::QColor, 0x00095103,
      59, QMetaType::QColor, 0x00095103,
      60, QMetaType::QColor, 0x00095103,
      61, QMetaType::QColor, 0x00095103,
      62, QMetaType::QColor, 0x00095103,
      63, QMetaType::QColor, 0x00095103,
      64, QMetaType::QColor, 0x00095103,
      65, QMetaType::QColor, 0x00095103,
      66, QMetaType::QColor, 0x00095103,
      67, QMetaType::QColor, 0x00095103,
      68, QMetaType::QColor, 0x00095103,
      69, QMetaType::QColor, 0x00095103,
      70, QMetaType::QColor, 0x00095103,
      71, QMetaType::QColor, 0x00095103,
      72, QMetaType::QColor, 0x00095103,
      73, QMetaType::QColor, 0x00095103,
      74, QMetaType::QColor, 0x00095103,
      75, QMetaType::QColor, 0x00095103,
      76, QMetaType::QColor, 0x00095103,
      77, QMetaType::QColor, 0x00095103,
      78, QMetaType::QColor, 0x00095103,
      79, QMetaType::QColor, 0x00095103,
      80, QMetaType::QColor, 0x00095103,
      81, QMetaType::QColor, 0x00095103,
      82, QMetaType::QColor, 0x00095103,
      83, QMetaType::QColor, 0x00095103,
      84, QMetaType::QColor, 0x00095103,
      85, QMetaType::QColor, 0x00095103,
      86, QMetaType::QColor, 0x00095103,
      87, QMetaType::QColor, 0x00095103,
      88, QMetaType::QColor, 0x00095103,
      89, QMetaType::QColor, 0x00095103,
      90, QMetaType::QColor, 0x00095103,
      91, QMetaType::QColor, 0x00095103,
      92, QMetaType::QColor, 0x00095103,
      93, QMetaType::QColor, 0x00095103,
      94, QMetaType::QColor, 0x00095103,
      95, QMetaType::QColor, 0x00095103,
      96, QMetaType::QColor, 0x00095103,
      97, QMetaType::QColor, 0x00095103,
      98, QMetaType::QColor, 0x00095103,
      99, QMetaType::QColor, 0x00095103,
     100, QMetaType::QColor, 0x00095103,
     101, QMetaType::QColor, 0x00095103,
     102, QMetaType::QColor, 0x00095003,
     103, QMetaType::QColor, 0x00095003,
     104, QMetaType::QColor, 0x00095003,
     105, QMetaType::QColor, 0x00095003,
     106, QMetaType::QColor, 0x00095003,
     107, QMetaType::QColor, 0x00095103,
     108, QMetaType::QColor, 0x00095103,
     109, QMetaType::QColor, 0x00095103,
     110, QMetaType::QColor, 0x00095103,
     111, QMetaType::QColor, 0x00095103,
     112, QMetaType::QColor, 0x00095103,
     113, QMetaType::QColor, 0x00095103,
     114, QMetaType::QColor, 0x00095103,
     115, QMetaType::QColor, 0x00095103,
     116, QMetaType::QColor, 0x00095103,
     117, QMetaType::QColor, 0x00095103,
     118, QMetaType::QColor, 0x00095103,
     119, QMetaType::QColor, 0x00095103,
     120, QMetaType::QColor, 0x00095103,
     121, QMetaType::QString, 0x00095103,
     122, QMetaType::QColor, 0x00095103,
     123, QMetaType::QString, 0x00095103,
     124, QMetaType::QString, 0x00095103,
     125, QMetaType::QColor, 0x00095103,
     126, QMetaType::QString, 0x00095103,
     127, QMetaType::QString, 0x00095103,
     128, QMetaType::QColor, 0x00095103,
     129, QMetaType::QString, 0x00095103,
     130, QMetaType::QColor, 0x00095103,
     131, QMetaType::QString, 0x00095103,
     132, QMetaType::QColor, 0x00095103,
     133, QMetaType::QString, 0x00095103,
     134, QMetaType::QColor, 0x00095103,
     135, QMetaType::QString, 0x00095103,
     136, QMetaType::QColor, 0x00095103,
     137, QMetaType::QString, 0x00095103,
     138, QMetaType::QColor, 0x00095103,
     139, QMetaType::QString, 0x00095103,
     140, QMetaType::QString, 0x00095103,
     141, QMetaType::QColor, 0x00095103,
     142, QMetaType::QString, 0x00095103,
     143, QMetaType::QString, 0x00095103,
     144, QMetaType::QColor, 0x00095103,
     145, QMetaType::QString, 0x00095103,
     146, QMetaType::QColor, 0x00095103,
     147, QMetaType::QString, 0x00095103,
     148, QMetaType::QColor, 0x00095103,
     149, QMetaType::QString, 0x00095103,
     150, QMetaType::QColor, 0x00095103,
     151, QMetaType::QString, 0x00095103,
     152, QMetaType::QString, 0x00095103,
     153, QMetaType::QString, 0x00095103,
     154, QMetaType::QString, 0x00095103,
     155, QMetaType::QString, 0x00095103,
     156, QMetaType::QString, 0x00095103,
     157, QMetaType::QString, 0x00095103,

       0        // eod
};

void XsheetViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<XsheetViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->orientationChanged((*reinterpret_cast< const Orientation*(*)>(_a[1]))); break;
        case 1: _t->positionSections(); break;
        case 2: _t->onSceneSwitched(); break;
        case 3: _t->onXsheetChanged(); break;
        case 4: _t->onCurrentFrameSwitched(); break;
        case 5: _t->onPlayingStatusChanged(); break;
        case 6: _t->onCurrentColumnSwitched(); break;
        case 7: _t->onSelectionSwitched((*reinterpret_cast< TSelection*(*)>(_a[1])),(*reinterpret_cast< TSelection*(*)>(_a[2]))); break;
        case 8: _t->onSelectionChanged((*reinterpret_cast< TSelection*(*)>(_a[1]))); break;
        case 9: _t->updateAllAree((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->updateAllAree(); break;
        case 11: _t->updateColumnArea(); break;
        case 12: _t->updateCellColumnAree(); break;
        case 13: _t->updateCellRowAree(); break;
        case 14: _t->onScrubStopped(); break;
        case 15: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->changeWindowTitle(); break;
        case 17: _t->resetXsheetNotes(); break;
        case 18: _t->onOrientationChanged((*reinterpret_cast< const Orientation*(*)>(_a[1]))); break;
        case 19: _t->onPrepareToScrollOffset((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 20: _t->onZoomScrollAdjust((*reinterpret_cast< QPointF(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 21: _t->setFrameZoomFactor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: { int _r = _t->getFrameZoomFactor();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 23: { QPoint _r = _t->getFrameZoomAdjustment();
            if (_a[0]) *reinterpret_cast< QPoint*>(_a[0]) = std::move(_r); }  break;
        case 24: _t->zoomOnFrame((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (XsheetViewer::*)(const Orientation * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&XsheetViewer::orientationChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<XsheetViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getLightLightBGColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getLightBGColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getBGColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getDarkBGColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getLightLineColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getDarkLineColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getColumnIconLineColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->getTimelineIconLineColor(); break;
        case 8: *reinterpret_cast< QColor*>(_v) = _t->getCurrentRowBgColor(); break;
        case 9: *reinterpret_cast< QColor*>(_v) = _t->getMarkerLineColor(); break;
        case 10: *reinterpret_cast< QColor*>(_v) = _t->getSecMarkerLineColor(); break;
        case 11: *reinterpret_cast< QColor*>(_v) = _t->getSelectedMarkerLineColor(); break;
        case 12: *reinterpret_cast< QColor*>(_v) = _t->getVerticalLineColor(); break;
        case 13: *reinterpret_cast< QColor*>(_v) = _t->getVerticalLineHeadColor(); break;
        case 14: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 15: *reinterpret_cast< QColor*>(_v) = _t->getErrorTextColor(); break;
        case 16: *reinterpret_cast< QColor*>(_v) = _t->getCurrentFrameTextColor(); break;
        case 17: *reinterpret_cast< QColor*>(_v) = _t->getSelectedTextColor(); break;
        case 18: *reinterpret_cast< QColor*>(_v) = _t->getFrameTextColor(); break;
        case 19: *reinterpret_cast< QColor*>(_v) = _t->getKeyframeLineColor(); break;
        case 20: *reinterpret_cast< QColor*>(_v) = _t->getPreviewFrameTextColor(); break;
        case 21: *reinterpret_cast< QColor*>(_v) = _t->getOnionSkinAreaBgColor(); break;
        case 22: *reinterpret_cast< QColor*>(_v) = _t->getFrameRangeMarkerLineColor(); break;
        case 23: *reinterpret_cast< QColor*>(_v) = _t->getCurrentTimeIndicatorColor(); break;
        case 24: *reinterpret_cast< QColor*>(_v) = _t->getColumnTextColor(); break;
        case 25: *reinterpret_cast< QColor*>(_v) = _t->getHighlightColumnTextColor(); break;
        case 26: *reinterpret_cast< QColor*>(_v) = _t->getEmptyColumnHeadColor(); break;
        case 27: *reinterpret_cast< QColor*>(_v) = _t->getEmptyCellColor(); break;
        case 28: *reinterpret_cast< QColor*>(_v) = _t->getNotEmptyColumnColor(); break;
        case 29: *reinterpret_cast< QColor*>(_v) = _t->getSelectedEmptyCellColor(); break;
        case 30: *reinterpret_cast< QColor*>(_v) = _t->getLevelEndColor(); break;
        case 31: *reinterpret_cast< QColor*>(_v) = _t->getCellFocusColor(); break;
        case 32: *reinterpret_cast< QColor*>(_v) = _t->getPlayRangeColor(); break;
        case 33: *reinterpret_cast< QColor*>(_v) = _t->getLevelColumnColor(); break;
        case 34: *reinterpret_cast< QColor*>(_v) = _t->getLevelColumnBorderColor(); break;
        case 35: *reinterpret_cast< QColor*>(_v) = _t->getSelectedLevelColumnColor(); break;
        case 36: *reinterpret_cast< QColor*>(_v) = _t->getVectorColumnColor(); break;
        case 37: *reinterpret_cast< QColor*>(_v) = _t->getVectorColumnBorderColor(); break;
        case 38: *reinterpret_cast< QColor*>(_v) = _t->getSelectedVectorColumnColor(); break;
        case 39: *reinterpret_cast< QColor*>(_v) = _t->getChildColumnColor(); break;
        case 40: *reinterpret_cast< QColor*>(_v) = _t->getChildColumnBorderColor(); break;
        case 41: *reinterpret_cast< QColor*>(_v) = _t->getSelectedChildColumnColor(); break;
        case 42: *reinterpret_cast< QColor*>(_v) = _t->getFullcolorColumnColor(); break;
        case 43: *reinterpret_cast< QColor*>(_v) = _t->getFullcolorColumnBorderColor(); break;
        case 44: *reinterpret_cast< QColor*>(_v) = _t->getSelectedFullcolorColumnColor(); break;
        case 45: *reinterpret_cast< QColor*>(_v) = _t->getFxColumnColor(); break;
        case 46: *reinterpret_cast< QColor*>(_v) = _t->getFxColumnBorderColor(); break;
        case 47: *reinterpret_cast< QColor*>(_v) = _t->getSelectedFxColumnColor(); break;
        case 48: *reinterpret_cast< QColor*>(_v) = _t->getReferenceColumnColor(); break;
        case 49: *reinterpret_cast< QColor*>(_v) = _t->getReferenceColumnBorderColor(); break;
        case 50: *reinterpret_cast< QColor*>(_v) = _t->getSelectedReferenceColumnColor(); break;
        case 51: *reinterpret_cast< QColor*>(_v) = _t->getPaletteColumnColor(); break;
        case 52: *reinterpret_cast< QColor*>(_v) = _t->getPaletteColumnBorderColor(); break;
        case 53: *reinterpret_cast< QColor*>(_v) = _t->getSelectedPaletteColumnColor(); break;
        case 54: *reinterpret_cast< QColor*>(_v) = _t->getMeshColumnColor(); break;
        case 55: *reinterpret_cast< QColor*>(_v) = _t->getMeshColumnBorderColor(); break;
        case 56: *reinterpret_cast< QColor*>(_v) = _t->getSelectedMeshColumnColor(); break;
        case 57: *reinterpret_cast< QColor*>(_v) = _t->getTableColor(); break;
        case 58: *reinterpret_cast< QColor*>(_v) = _t->getPegColor(); break;
        case 59: *reinterpret_cast< QColor*>(_v) = _t->getSoundTextColumnColor(); break;
        case 60: *reinterpret_cast< QColor*>(_v) = _t->getSoundTextColumnBorderColor(); break;
        case 61: *reinterpret_cast< QColor*>(_v) = _t->getSelectedSoundTextColumnColor(); break;
        case 62: *reinterpret_cast< QColor*>(_v) = _t->m_soundColumnColor; break;
        case 63: *reinterpret_cast< QColor*>(_v) = _t->m_soundColumnBorderColor; break;
        case 64: *reinterpret_cast< QColor*>(_v) = _t->m_selectedSoundColumnColor; break;
        case 65: *reinterpret_cast< QColor*>(_v) = _t->m_soundColumnHlColor; break;
        case 66: *reinterpret_cast< QColor*>(_v) = _t->m_soundColumnTrackColor; break;
        case 67: *reinterpret_cast< QColor*>(_v) = _t->getMetaColumnColor(); break;
        case 68: *reinterpret_cast< QColor*>(_v) = _t->getMetaColumnBorderColor(); break;
        case 69: *reinterpret_cast< QColor*>(_v) = _t->getSelectedMetaColumnColor(); break;
        case 70: *reinterpret_cast< QColor*>(_v) = _t->getColumnHeadPastelizer(); break;
        case 71: *reinterpret_cast< QColor*>(_v) = _t->getSelectedColumnHead(); break;
        case 72: *reinterpret_cast< QColor*>(_v) = _t->getFoldedColumnBGColor(); break;
        case 73: *reinterpret_cast< QColor*>(_v) = _t->getFoldedColumnLineColor(); break;
        case 74: *reinterpret_cast< QColor*>(_v) = _t->getXsheetColumnNameBgColor(); break;
        case 75: *reinterpret_cast< QColor*>(_v) = _t->getXsheetDragBarHighlightColor(); break;
        case 76: *reinterpret_cast< QColor*>(_v) = _t->getActiveCameraColor(); break;
        case 77: *reinterpret_cast< QColor*>(_v) = _t->getSelectedActiveCameraColor(); break;
        case 78: *reinterpret_cast< QColor*>(_v) = _t->getOtherCameraColor(); break;
        case 79: *reinterpret_cast< QColor*>(_v) = _t->getSelectedOtherCameraColor(); break;
        case 80: *reinterpret_cast< QColor*>(_v) = _t->getXsheetPreviewButtonBgOnColor(); break;
        case 81: *reinterpret_cast< QString*>(_v) = _t->getXsheetPreviewButtonOnImage(); break;
        case 82: *reinterpret_cast< QColor*>(_v) = _t->getXsheetPreviewButtonBgOffColor(); break;
        case 83: *reinterpret_cast< QString*>(_v) = _t->getXsheetPreviewButtonOffImage(); break;
        case 84: *reinterpret_cast< QString*>(_v) = _t->getXsheetUnifiedButtonTranspImage(); break;
        case 85: *reinterpret_cast< QColor*>(_v) = _t->getXsheetCamstandButtonBgOnColor(); break;
        case 86: *reinterpret_cast< QString*>(_v) = _t->getXsheetCamstandButtonOnImage(); break;
        case 87: *reinterpret_cast< QString*>(_v) = _t->getXsheetCamstandButtonTranspImage(); break;
        case 88: *reinterpret_cast< QColor*>(_v) = _t->getXsheetCamstandButtonBgOffColor(); break;
        case 89: *reinterpret_cast< QString*>(_v) = _t->getXsheetCamstandButtonOffImage(); break;
        case 90: *reinterpret_cast< QColor*>(_v) = _t->getXsheetLockButtonBgOnColor(); break;
        case 91: *reinterpret_cast< QString*>(_v) = _t->getXsheetLockButtonOnImage(); break;
        case 92: *reinterpret_cast< QColor*>(_v) = _t->getXsheetLockButtonBgOffColor(); break;
        case 93: *reinterpret_cast< QString*>(_v) = _t->getXsheetLockButtonOffImage(); break;
        case 94: *reinterpret_cast< QColor*>(_v) = _t->getXsheetConfigButtonBgColor(); break;
        case 95: *reinterpret_cast< QString*>(_v) = _t->getXsheetConfigButtonImage(); break;
        case 96: *reinterpret_cast< QColor*>(_v) = _t->getTimelinePreviewButtonBgOnColor(); break;
        case 97: *reinterpret_cast< QString*>(_v) = _t->getTimelinePreviewButtonOnImage(); break;
        case 98: *reinterpret_cast< QColor*>(_v) = _t->getTimelinePreviewButtonBgOffColor(); break;
        case 99: *reinterpret_cast< QString*>(_v) = _t->getTimelinePreviewButtonOffImage(); break;
        case 100: *reinterpret_cast< QString*>(_v) = _t->getTimelineUnifiedButtonTranspImage(); break;
        case 101: *reinterpret_cast< QColor*>(_v) = _t->getTimelineCamstandButtonBgOnColor(); break;
        case 102: *reinterpret_cast< QString*>(_v) = _t->getTimelineCamstandButtonOnImage(); break;
        case 103: *reinterpret_cast< QString*>(_v) = _t->getTimelineCamstandButtonTranspImage(); break;
        case 104: *reinterpret_cast< QColor*>(_v) = _t->getTimelineCamstandButtonBgOffColor(); break;
        case 105: *reinterpret_cast< QString*>(_v) = _t->getTimelineCamstandButtonOffImage(); break;
        case 106: *reinterpret_cast< QColor*>(_v) = _t->getTimelineLockButtonBgOnColor(); break;
        case 107: *reinterpret_cast< QString*>(_v) = _t->getTimelineLockButtonOnImage(); break;
        case 108: *reinterpret_cast< QColor*>(_v) = _t->getTimelineLockButtonBgOffColor(); break;
        case 109: *reinterpret_cast< QString*>(_v) = _t->getTimelineLockButtonOffImage(); break;
        case 110: *reinterpret_cast< QColor*>(_v) = _t->getTimelineConfigButtonBgColor(); break;
        case 111: *reinterpret_cast< QString*>(_v) = _t->getTimelineConfigButtonImage(); break;
        case 112: *reinterpret_cast< QString*>(_v) = _t->getLayerHeaderPreviewImage(); break;
        case 113: *reinterpret_cast< QString*>(_v) = _t->getLayerHeaderPreviewOverImage(); break;
        case 114: *reinterpret_cast< QString*>(_v) = _t->getLayerHeaderCamstandImage(); break;
        case 115: *reinterpret_cast< QString*>(_v) = _t->getLayerHeaderCamstandOverImage(); break;
        case 116: *reinterpret_cast< QString*>(_v) = _t->getLayerHeaderLockImage(); break;
        case 117: *reinterpret_cast< QString*>(_v) = _t->getLayerHeaderLockOverImage(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<XsheetViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLightLightBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setLightBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setDarkBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setLightLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setDarkLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setColumnIconLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setTimelineIconLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 8: _t->setCurrentRowBgColor(*reinterpret_cast< QColor*>(_v)); break;
        case 9: _t->setMarkerLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 10: _t->setSecMarkerLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 11: _t->setSelectedMarkerLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 12: _t->setVerticalLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 13: _t->setVerticalLineHeadColor(*reinterpret_cast< QColor*>(_v)); break;
        case 14: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 15: _t->setErrorTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 16: _t->setCurrentFrameTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 17: _t->setSelectedTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 18: _t->setFrameTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 19: _t->setKeyframeLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 20: _t->setPreviewFrameTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 21: _t->setOnionSkinAreaBgColor(*reinterpret_cast< QColor*>(_v)); break;
        case 22: _t->setFrameRangeMarkerLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 23: _t->setCurrentTimeIndicatorColor(*reinterpret_cast< QColor*>(_v)); break;
        case 24: _t->setColumnTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 25: _t->setHighlightColumnTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 26: _t->setEmptyColumnHeadColor(*reinterpret_cast< QColor*>(_v)); break;
        case 27: _t->setEmptyCellColor(*reinterpret_cast< QColor*>(_v)); break;
        case 28: _t->setNotEmptyColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 29: _t->setSelectedEmptyCellColor(*reinterpret_cast< QColor*>(_v)); break;
        case 30: _t->setLevelEndColor(*reinterpret_cast< QColor*>(_v)); break;
        case 31: _t->setCellFocusColor(*reinterpret_cast< QColor*>(_v)); break;
        case 32: _t->setPlayRangeColor(*reinterpret_cast< QColor*>(_v)); break;
        case 33: _t->setLevelColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 34: _t->setLevelColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 35: _t->setSelectedLevelColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 36: _t->setVectorColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 37: _t->setVectorColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 38: _t->setSelectedVectorColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 39: _t->setChildColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 40: _t->setChildColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 41: _t->setSelectedChildColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 42: _t->setFullcolorColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 43: _t->setFullcolorColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 44: _t->setSelectedFullcolorColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 45: _t->setFxColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 46: _t->setFxColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 47: _t->setSelectedFxColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 48: _t->setReferenceColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 49: _t->setReferenceColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 50: _t->setSelectedReferenceColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 51: _t->setPaletteColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 52: _t->setPaletteColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 53: _t->setSelectedPaletteColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 54: _t->setMeshColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 55: _t->setMeshColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 56: _t->setSelectedMeshColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 57: _t->setTableColor(*reinterpret_cast< QColor*>(_v)); break;
        case 58: _t->setPegColor(*reinterpret_cast< QColor*>(_v)); break;
        case 59: _t->setSoundTextColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 60: _t->setSoundTextColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 61: _t->setSelectedSoundTextColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 62:
            if (_t->m_soundColumnColor != *reinterpret_cast< QColor*>(_v)) {
                _t->m_soundColumnColor = *reinterpret_cast< QColor*>(_v);
            }
            break;
        case 63:
            if (_t->m_soundColumnBorderColor != *reinterpret_cast< QColor*>(_v)) {
                _t->m_soundColumnBorderColor = *reinterpret_cast< QColor*>(_v);
            }
            break;
        case 64:
            if (_t->m_selectedSoundColumnColor != *reinterpret_cast< QColor*>(_v)) {
                _t->m_selectedSoundColumnColor = *reinterpret_cast< QColor*>(_v);
            }
            break;
        case 65:
            if (_t->m_soundColumnHlColor != *reinterpret_cast< QColor*>(_v)) {
                _t->m_soundColumnHlColor = *reinterpret_cast< QColor*>(_v);
            }
            break;
        case 66:
            if (_t->m_soundColumnTrackColor != *reinterpret_cast< QColor*>(_v)) {
                _t->m_soundColumnTrackColor = *reinterpret_cast< QColor*>(_v);
            }
            break;
        case 67: _t->setMetaColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 68: _t->setMetaColumnBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 69: _t->setSelectedMetaColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 70: _t->setColumnHeadPastelizer(*reinterpret_cast< QColor*>(_v)); break;
        case 71: _t->setSelectedColumnHead(*reinterpret_cast< QColor*>(_v)); break;
        case 72: _t->setFoldedColumnBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 73: _t->setFoldedColumnLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 74: _t->setXsheetColumnNameBgColor(*reinterpret_cast< QColor*>(_v)); break;
        case 75: _t->setXsheetDragBarHighlightColor(*reinterpret_cast< QColor*>(_v)); break;
        case 76: _t->setActiveCameraColor(*reinterpret_cast< QColor*>(_v)); break;
        case 77: _t->setSelectedActiveCameraColor(*reinterpret_cast< QColor*>(_v)); break;
        case 78: _t->setOtherCameraColor(*reinterpret_cast< QColor*>(_v)); break;
        case 79: _t->setSelectedOtherCameraColor(*reinterpret_cast< QColor*>(_v)); break;
        case 80: _t->setXsheetPreviewButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 81: _t->setXsheetPreviewButtonOnImage(*reinterpret_cast< QString*>(_v)); break;
        case 82: _t->setXsheetPreviewButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 83: _t->setXsheetPreviewButtonOffImage(*reinterpret_cast< QString*>(_v)); break;
        case 84: _t->setXsheetUnifiedButtonTranspImage(*reinterpret_cast< QString*>(_v)); break;
        case 85: _t->setXsheetCamstandButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 86: _t->setXsheetCamstandButtonOnImage(*reinterpret_cast< QString*>(_v)); break;
        case 87: _t->setXsheetCamstandButtonTranspImage(*reinterpret_cast< QString*>(_v)); break;
        case 88: _t->setXsheetCamstandButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 89: _t->setXsheetCamstandButtonOffImage(*reinterpret_cast< QString*>(_v)); break;
        case 90: _t->setXsheetLockButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 91: _t->setXsheetLockButtonOnImage(*reinterpret_cast< QString*>(_v)); break;
        case 92: _t->setXsheetLockButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 93: _t->setXsheetLockButtonOffImage(*reinterpret_cast< QString*>(_v)); break;
        case 94: _t->setXsheetConfigButtonBgColor(*reinterpret_cast< QColor*>(_v)); break;
        case 95: _t->setXsheetConfigButtonImage(*reinterpret_cast< QString*>(_v)); break;
        case 96: _t->setTimelinePreviewButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 97: _t->setTimelinePreviewButtonOnImage(*reinterpret_cast< QString*>(_v)); break;
        case 98: _t->setTimelinePreviewButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 99: _t->setTimelinePreviewButtonOffImage(*reinterpret_cast< QString*>(_v)); break;
        case 100: _t->setTimelineUnifiedButtonTranspImage(*reinterpret_cast< QString*>(_v)); break;
        case 101: _t->setTimelineCamstandButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 102: _t->setTimelineCamstandButtonOnImage(*reinterpret_cast< QString*>(_v)); break;
        case 103: _t->setTimelineCamstandButtonTranspImage(*reinterpret_cast< QString*>(_v)); break;
        case 104: _t->setTimelineCamstandButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 105: _t->setTimelineCamstandButtonOffImage(*reinterpret_cast< QString*>(_v)); break;
        case 106: _t->setTimelineLockButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 107: _t->setTimelineLockButtonOnImage(*reinterpret_cast< QString*>(_v)); break;
        case 108: _t->setTimelineLockButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 109: _t->setTimelineLockButtonOffImage(*reinterpret_cast< QString*>(_v)); break;
        case 110: _t->setTimelineConfigButtonBgColor(*reinterpret_cast< QColor*>(_v)); break;
        case 111: _t->setTimelineConfigButtonImage(*reinterpret_cast< QString*>(_v)); break;
        case 112: _t->setLayerHeaderPreviewImage(*reinterpret_cast< QString*>(_v)); break;
        case 113: _t->setLayerHeaderPreviewOverImage(*reinterpret_cast< QString*>(_v)); break;
        case 114: _t->setLayerHeaderCamstandImage(*reinterpret_cast< QString*>(_v)); break;
        case 115: _t->setLayerHeaderCamstandOverImage(*reinterpret_cast< QString*>(_v)); break;
        case 116: _t->setLayerHeaderLockImage(*reinterpret_cast< QString*>(_v)); break;
        case 117: _t->setLayerHeaderLockOverImage(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject XsheetViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_XsheetViewer.data,
    qt_meta_data_XsheetViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SaveLoadQSettings"))
        return static_cast< SaveLoadQSettings*>(this);
    return QFrame::qt_metacast(_clname);
}

int XsheetViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 25;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 118;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 118;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 118;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 118;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 118;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 118;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void XsheetViewer::orientationChanged(const Orientation * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
