/****************************************************************************
** Meta object code from reading C++ file 'scriptbinding_level.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/scriptbinding_level.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scriptbinding_level.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TScriptBinding__Level_t {
    QByteArrayData data[18];
    char stringdata0[146];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TScriptBinding__Level_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TScriptBinding__Level_t qt_meta_stringdata_TScriptBinding__Level = {
    {
QT_MOC_LITERAL(0, 0, 21), // "TScriptBinding::Level"
QT_MOC_LITERAL(1, 22, 8), // "toString"
QT_MOC_LITERAL(2, 31, 12), // "QScriptValue"
QT_MOC_LITERAL(3, 44, 0), // ""
QT_MOC_LITERAL(4, 45, 8), // "getFrame"
QT_MOC_LITERAL(5, 54, 3), // "fid"
QT_MOC_LITERAL(6, 58, 15), // "getFrameByIndex"
QT_MOC_LITERAL(7, 74, 5), // "index"
QT_MOC_LITERAL(8, 80, 8), // "setFrame"
QT_MOC_LITERAL(9, 89, 5), // "image"
QT_MOC_LITERAL(10, 95, 11), // "getFrameIds"
QT_MOC_LITERAL(11, 107, 4), // "load"
QT_MOC_LITERAL(12, 112, 2), // "fp"
QT_MOC_LITERAL(13, 115, 4), // "save"
QT_MOC_LITERAL(14, 120, 4), // "type"
QT_MOC_LITERAL(15, 125, 10), // "frameCount"
QT_MOC_LITERAL(16, 136, 4), // "name"
QT_MOC_LITERAL(17, 141, 4) // "path"

    },
    "TScriptBinding::Level\0toString\0"
    "QScriptValue\0\0getFrame\0fid\0getFrameByIndex\0"
    "index\0setFrame\0image\0getFrameIds\0load\0"
    "fp\0save\0type\0frameCount\0name\0path"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TScriptBinding__Level[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       4,   68, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    0,   49,    3, 0x02 /* Public */,
       4,    1,   50,    3, 0x02 /* Public */,
       6,    1,   53,    3, 0x02 /* Public */,
       8,    2,   56,    3, 0x02 /* Public */,
      10,    0,   61,    3, 0x02 /* Public */,
      11,    1,   62,    3, 0x02 /* Public */,
      13,    1,   65,    3, 0x02 /* Public */,

 // methods: parameters
    0x80000000 | 2,
    0x80000000 | 2, 0x80000000 | 2,    5,
    0x80000000 | 2, 0x80000000 | 2,    7,
    0x80000000 | 2, 0x80000000 | 2, 0x80000000 | 2,    5,    9,
    0x80000000 | 2,
    0x80000000 | 2, 0x80000000 | 2,   12,
    0x80000000 | 2, 0x80000000 | 2,   12,

 // properties: name, type, flags
      14, QMetaType::QString, 0x00095001,
      15, QMetaType::Int, 0x00095001,
      16, QMetaType::QString, 0x00095103,
      17, 0x80000000 | 2, 0x0009510b,

       0        // eod
};

void TScriptBinding::Level::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Level *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QScriptValue _r = _t->toString();
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QScriptValue _r = _t->getFrame((*reinterpret_cast< const QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 2: { QScriptValue _r = _t->getFrameByIndex((*reinterpret_cast< const QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 3: { QScriptValue _r = _t->setFrame((*reinterpret_cast< const QScriptValue(*)>(_a[1])),(*reinterpret_cast< const QScriptValue(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 4: { QScriptValue _r = _t->getFrameIds();
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 5: { QScriptValue _r = _t->load((*reinterpret_cast< const QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 6: { QScriptValue _r = _t->save((*reinterpret_cast< const QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<Level *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->getType(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->getFrameCount(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->getName(); break;
        case 3: *reinterpret_cast< QScriptValue*>(_v) = _t->getPath(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<Level *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setName(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setPath(*reinterpret_cast< QScriptValue*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TScriptBinding::Level::staticMetaObject = { {
    QMetaObject::SuperData::link<Wrapper::staticMetaObject>(),
    qt_meta_stringdata_TScriptBinding__Level.data,
    qt_meta_data_TScriptBinding__Level,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TScriptBinding::Level::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TScriptBinding::Level::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TScriptBinding__Level.stringdata0))
        return static_cast<void*>(this);
    return Wrapper::qt_metacast(_clname);
}

int TScriptBinding::Level::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Wrapper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
