/****************************************************************************
** Meta object code from reading C++ file 'scriptbinding_image_builder.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/scriptbinding_image_builder.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scriptbinding_image_builder.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TScriptBinding__Transform_t {
    QByteArrayData data[13];
    char stringdata0[92];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TScriptBinding__Transform_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TScriptBinding__Transform_t qt_meta_stringdata_TScriptBinding__Transform = {
    {
QT_MOC_LITERAL(0, 0, 25), // "TScriptBinding::Transform"
QT_MOC_LITERAL(1, 26, 8), // "toString"
QT_MOC_LITERAL(2, 35, 12), // "QScriptValue"
QT_MOC_LITERAL(3, 48, 0), // ""
QT_MOC_LITERAL(4, 49, 9), // "translate"
QT_MOC_LITERAL(5, 59, 1), // "x"
QT_MOC_LITERAL(6, 61, 1), // "y"
QT_MOC_LITERAL(7, 63, 6), // "rotate"
QT_MOC_LITERAL(8, 70, 7), // "degrees"
QT_MOC_LITERAL(9, 78, 5), // "scale"
QT_MOC_LITERAL(10, 84, 1), // "s"
QT_MOC_LITERAL(11, 86, 2), // "sx"
QT_MOC_LITERAL(12, 89, 2) // "sy"

    },
    "TScriptBinding::Transform\0toString\0"
    "QScriptValue\0\0translate\0x\0y\0rotate\0"
    "degrees\0scale\0s\0sx\0sy"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TScriptBinding__Transform[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    0,   39,    3, 0x02 /* Public */,
       4,    2,   40,    3, 0x02 /* Public */,
       7,    1,   45,    3, 0x02 /* Public */,
       9,    1,   48,    3, 0x02 /* Public */,
       9,    2,   51,    3, 0x02 /* Public */,

 // methods: parameters
    0x80000000 | 2,
    0x80000000 | 2, QMetaType::Double, QMetaType::Double,    5,    6,
    0x80000000 | 2, QMetaType::Double,    8,
    0x80000000 | 2, QMetaType::Double,   10,
    0x80000000 | 2, QMetaType::Double, QMetaType::Double,   11,   12,

       0        // eod
};

void TScriptBinding::Transform::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Transform *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QScriptValue _r = _t->toString();
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QScriptValue _r = _t->translate((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 2: { QScriptValue _r = _t->rotate((*reinterpret_cast< double(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 3: { QScriptValue _r = _t->scale((*reinterpret_cast< double(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 4: { QScriptValue _r = _t->scale((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TScriptBinding::Transform::staticMetaObject = { {
    QMetaObject::SuperData::link<Wrapper::staticMetaObject>(),
    qt_meta_stringdata_TScriptBinding__Transform.data,
    qt_meta_data_TScriptBinding__Transform,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TScriptBinding::Transform::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TScriptBinding::Transform::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TScriptBinding__Transform.stringdata0))
        return static_cast<void*>(this);
    return Wrapper::qt_metacast(_clname);
}

int TScriptBinding::Transform::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Wrapper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
struct qt_meta_stringdata_TScriptBinding__ImageBuilder_t {
    QByteArrayData data[11];
    char stringdata0[102];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TScriptBinding__ImageBuilder_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TScriptBinding__ImageBuilder_t qt_meta_stringdata_TScriptBinding__ImageBuilder = {
    {
QT_MOC_LITERAL(0, 0, 28), // "TScriptBinding::ImageBuilder"
QT_MOC_LITERAL(1, 29, 8), // "toString"
QT_MOC_LITERAL(2, 38, 12), // "QScriptValue"
QT_MOC_LITERAL(3, 51, 0), // ""
QT_MOC_LITERAL(4, 52, 3), // "add"
QT_MOC_LITERAL(5, 56, 3), // "img"
QT_MOC_LITERAL(6, 60, 14), // "transformation"
QT_MOC_LITERAL(7, 75, 5), // "clear"
QT_MOC_LITERAL(8, 81, 4), // "fill"
QT_MOC_LITERAL(9, 86, 9), // "colorName"
QT_MOC_LITERAL(10, 96, 5) // "image"

    },
    "TScriptBinding::ImageBuilder\0toString\0"
    "QScriptValue\0\0add\0img\0transformation\0"
    "clear\0fill\0colorName\0image"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TScriptBinding__ImageBuilder[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       1,   52, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // methods: name, argc, parameters, tag, flags
       1,    0,   39,    3, 0x02 /* Public */,
       4,    1,   40,    3, 0x02 /* Public */,
       4,    2,   43,    3, 0x02 /* Public */,
       7,    0,   48,    3, 0x02 /* Public */,
       8,    1,   49,    3, 0x02 /* Public */,

 // methods: parameters
    0x80000000 | 2,
    0x80000000 | 2, 0x80000000 | 2,    5,
    0x80000000 | 2, 0x80000000 | 2, 0x80000000 | 2,    5,    6,
    QMetaType::Void,
    0x80000000 | 2, QMetaType::QString,    9,

 // properties: name, type, flags
      10, 0x80000000 | 2, 0x00095009,

       0        // eod
};

void TScriptBinding::ImageBuilder::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ImageBuilder *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QScriptValue _r = _t->toString();
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QScriptValue _r = _t->add((*reinterpret_cast< QScriptValue(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 2: { QScriptValue _r = _t->add((*reinterpret_cast< QScriptValue(*)>(_a[1])),(*reinterpret_cast< QScriptValue(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->clear(); break;
        case 4: { QScriptValue _r = _t->fill((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QScriptValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<ImageBuilder *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QScriptValue*>(_v) = _t->getImage(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TScriptBinding::ImageBuilder::staticMetaObject = { {
    QMetaObject::SuperData::link<Wrapper::staticMetaObject>(),
    qt_meta_stringdata_TScriptBinding__ImageBuilder.data,
    qt_meta_data_TScriptBinding__ImageBuilder,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TScriptBinding::ImageBuilder::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TScriptBinding::ImageBuilder::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TScriptBinding__ImageBuilder.stringdata0))
        return static_cast<void*>(this);
    return Wrapper::qt_metacast(_clname);
}

int TScriptBinding::ImageBuilder::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Wrapper::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
