/****************************************************************************
** Meta object code from reading C++ file 'cleanupsettingspane.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/cleanupsettingspane.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cleanupsettingspane.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CleanupSaveInField_t {
    QByteArrayData data[3];
    char stringdata0[36];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CleanupSaveInField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CleanupSaveInField_t qt_meta_stringdata_CleanupSaveInField = {
    {
QT_MOC_LITERAL(0, 0, 18), // "CleanupSaveInField"
QT_MOC_LITERAL(1, 19, 15), // "browseDirectory"
QT_MOC_LITERAL(2, 35, 0) // ""

    },
    "CleanupSaveInField\0browseDirectory\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CleanupSaveInField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void CleanupSaveInField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CleanupSaveInField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->browseDirectory(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CleanupSaveInField::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::FileField::staticMetaObject>(),
    qt_meta_stringdata_CleanupSaveInField.data,
    qt_meta_data_CleanupSaveInField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CleanupSaveInField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CleanupSaveInField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CleanupSaveInField.stringdata0))
        return static_cast<void*>(this);
    return DVGui::FileField::qt_metacast(_clname);
}

int CleanupSaveInField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::FileField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_CleanupSettingsPane_t {
    QByteArrayData data[21];
    char stringdata0[307];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CleanupSettingsPane_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CleanupSettingsPane_t qt_meta_stringdata_CleanupSettingsPane = {
    {
QT_MOC_LITERAL(0, 0, 19), // "CleanupSettingsPane"
QT_MOC_LITERAL(1, 20, 9), // "updateGui"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 19), // "postProcessPreviews"
QT_MOC_LITERAL(4, 51, 18), // "CleanupParameters*"
QT_MOC_LITERAL(5, 70, 6), // "params"
QT_MOC_LITERAL(6, 77, 9), // "oldParams"
QT_MOC_LITERAL(7, 87, 15), // "updateImageInfo"
QT_MOC_LITERAL(8, 103, 19), // "getLastSelectedPath"
QT_MOC_LITERAL(9, 123, 9), // "TFilePath"
QT_MOC_LITERAL(10, 133, 15), // "onImageSwitched"
QT_MOC_LITERAL(11, 149, 20), // "onPreviewDataChanged"
QT_MOC_LITERAL(12, 170, 11), // "postProcess"
QT_MOC_LITERAL(13, 182, 11), // "onClnLoaded"
QT_MOC_LITERAL(14, 194, 17), // "onRestoreSettings"
QT_MOC_LITERAL(15, 212, 23), // "onGenericSettingsChange"
QT_MOC_LITERAL(16, 236, 12), // "onPathChange"
QT_MOC_LITERAL(17, 249, 17), // "onSharpnessChange"
QT_MOC_LITERAL(18, 267, 8), // "dragging"
QT_MOC_LITERAL(19, 276, 15), // "onLevelSwitched"
QT_MOC_LITERAL(20, 292, 14) // "onSaveSettings"

    },
    "CleanupSettingsPane\0updateGui\0\0"
    "postProcessPreviews\0CleanupParameters*\0"
    "params\0oldParams\0updateImageInfo\0"
    "getLastSelectedPath\0TFilePath\0"
    "onImageSwitched\0onPreviewDataChanged\0"
    "postProcess\0onClnLoaded\0onRestoreSettings\0"
    "onGenericSettingsChange\0onPathChange\0"
    "onSharpnessChange\0dragging\0onLevelSwitched\0"
    "onSaveSettings"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CleanupSettingsPane[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x0a /* Public */,
       1,    2,   87,    2, 0x0a /* Public */,
       7,    0,   92,    2, 0x0a /* Public */,
       8,    0,   93,    2, 0x0a /* Public */,
      10,    0,   94,    2, 0x08 /* Private */,
      11,    0,   95,    2, 0x08 /* Private */,
      12,    0,   96,    2, 0x08 /* Private */,
      13,    0,   97,    2, 0x08 /* Private */,
      14,    0,   98,    2, 0x08 /* Private */,
      15,    0,   99,    2, 0x08 /* Private */,
      16,    0,  100,    2, 0x08 /* Private */,
      17,    1,  101,    2, 0x08 /* Private */,
      19,    0,  104,    2, 0x08 /* Private */,
      20,    0,  105,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4,    5,    6,
    QMetaType::Void,
    0x80000000 | 9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CleanupSettingsPane::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CleanupSettingsPane *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateGui((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->updateGui((*reinterpret_cast< CleanupParameters*(*)>(_a[1])),(*reinterpret_cast< CleanupParameters*(*)>(_a[2]))); break;
        case 2: _t->updateImageInfo(); break;
        case 3: { TFilePath _r = _t->getLastSelectedPath();
            if (_a[0]) *reinterpret_cast< TFilePath*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->onImageSwitched(); break;
        case 5: _t->onPreviewDataChanged(); break;
        case 6: _t->postProcess(); break;
        case 7: _t->onClnLoaded(); break;
        case 8: _t->onRestoreSettings(); break;
        case 9: _t->onGenericSettingsChange(); break;
        case 10: _t->onPathChange(); break;
        case 11: _t->onSharpnessChange((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->onLevelSwitched(); break;
        case 13: _t->onSaveSettings(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CleanupSettingsPane::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_CleanupSettingsPane.data,
    qt_meta_data_CleanupSettingsPane,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CleanupSettingsPane::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CleanupSettingsPane::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CleanupSettingsPane.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int CleanupSettingsPane::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
