/****************************************************************************
** Meta object code from reading C++ file 'imageutils.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/imageutils.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'imageutils.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ImageUtils__FrameTaskNotifier_t {
    QByteArrayData data[8];
    char stringdata0[99];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ImageUtils__FrameTaskNotifier_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ImageUtils__FrameTaskNotifier_t qt_meta_stringdata_ImageUtils__FrameTaskNotifier = {
    {
QT_MOC_LITERAL(0, 0, 29), // "ImageUtils::FrameTaskNotifier"
QT_MOC_LITERAL(1, 30, 14), // "frameCompleted"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 14), // "levelCompleted"
QT_MOC_LITERAL(4, 61, 9), // "TFilePath"
QT_MOC_LITERAL(5, 71, 8), // "fullPath"
QT_MOC_LITERAL(6, 80, 5), // "error"
QT_MOC_LITERAL(7, 86, 12) // "onCancelTask"

    },
    "ImageUtils::FrameTaskNotifier\0"
    "frameCompleted\0\0levelCompleted\0TFilePath\0"
    "fullPath\0error\0onCancelTask"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ImageUtils__FrameTaskNotifier[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       3,    1,   37,    2, 0x06 /* Public */,
       6,    0,   40,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   41,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void ImageUtils::FrameTaskNotifier::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FrameTaskNotifier *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->frameCompleted((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->levelCompleted((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 2: _t->error(); break;
        case 3: _t->onCancelTask(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FrameTaskNotifier::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FrameTaskNotifier::frameCompleted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FrameTaskNotifier::*)(const TFilePath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FrameTaskNotifier::levelCompleted)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FrameTaskNotifier::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FrameTaskNotifier::error)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ImageUtils::FrameTaskNotifier::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ImageUtils__FrameTaskNotifier.data,
    qt_meta_data_ImageUtils__FrameTaskNotifier,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ImageUtils::FrameTaskNotifier::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImageUtils::FrameTaskNotifier::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ImageUtils__FrameTaskNotifier.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ImageUtils::FrameTaskNotifier::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void ImageUtils::FrameTaskNotifier::frameCompleted(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ImageUtils::FrameTaskNotifier::levelCompleted(const TFilePath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ImageUtils::FrameTaskNotifier::error()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_ImageUtils__FullScreenWidget_t {
    QByteArrayData data[8];
    char stringdata0[88];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ImageUtils__FullScreenWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ImageUtils__FullScreenWidget_t qt_meta_stringdata_ImageUtils__FullScreenWidget = {
    {
QT_MOC_LITERAL(0, 0, 28), // "ImageUtils::FullScreenWidget"
QT_MOC_LITERAL(1, 29, 16), // "toggleFullScreen"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 4), // "quit"
QT_MOC_LITERAL(4, 52, 14), // "opacityChanged"
QT_MOC_LITERAL(5, 67, 5), // "value"
QT_MOC_LITERAL(6, 73, 5), // "bool&"
QT_MOC_LITERAL(7, 79, 8) // "hideMain"

    },
    "ImageUtils::FullScreenWidget\0"
    "toggleFullScreen\0\0quit\0opacityChanged\0"
    "value\0bool&\0hideMain"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ImageUtils__FullScreenWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x0a /* Public */,
       1,    0,   32,    2, 0x2a /* Public | MethodCloned */,
       4,    2,   33,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Bool, QMetaType::Bool,    3,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 6,    5,    7,

       0        // eod
};

void ImageUtils::FullScreenWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FullScreenWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->toggleFullScreen((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->toggleFullScreen();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->opacityChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ImageUtils::FullScreenWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ImageUtils__FullScreenWidget.data,
    qt_meta_data_ImageUtils__FullScreenWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ImageUtils::FullScreenWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImageUtils::FullScreenWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ImageUtils__FullScreenWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ImageUtils::FullScreenWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
