/****************************************************************************
** Meta object code from reading C++ file 'canon.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../opentoonz/toonz/sources/stopmotion/canon.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canon.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Canon_t {
    QByteArrayData data[26];
    char stringdata0[538];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Canon_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Canon_t qt_meta_stringdata_Canon = {
    {
QT_MOC_LITERAL(0, 0, 5), // "Canon"
QT_MOC_LITERAL(1, 6, 22), // "apertureOptionsChanged"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 17), // "isoOptionsChanged"
QT_MOC_LITERAL(4, 48, 26), // "shutterSpeedOptionsChanged"
QT_MOC_LITERAL(5, 75, 22), // "exposureOptionsChanged"
QT_MOC_LITERAL(6, 98, 26), // "whiteBalanceOptionsChanged"
QT_MOC_LITERAL(7, 125, 23), // "colorTemperatureChanged"
QT_MOC_LITERAL(8, 149, 26), // "imageQualityOptionsChanged"
QT_MOC_LITERAL(9, 176, 26), // "pictureStyleOptionsChanged"
QT_MOC_LITERAL(10, 203, 21), // "apertureChangedSignal"
QT_MOC_LITERAL(11, 225, 16), // "isoChangedSignal"
QT_MOC_LITERAL(12, 242, 25), // "shutterSpeedChangedSignal"
QT_MOC_LITERAL(13, 268, 21), // "exposureChangedSignal"
QT_MOC_LITERAL(14, 290, 25), // "whiteBalanceChangedSignal"
QT_MOC_LITERAL(15, 316, 29), // "colorTemperatureChangedSignal"
QT_MOC_LITERAL(16, 346, 25), // "imageQualityChangedSignal"
QT_MOC_LITERAL(17, 372, 25), // "pictureStyleChangedSignal"
QT_MOC_LITERAL(18, 398, 11), // "modeChanged"
QT_MOC_LITERAL(19, 410, 17), // "focusCheckToggled"
QT_MOC_LITERAL(20, 428, 21), // "pickFocusCheckToggled"
QT_MOC_LITERAL(21, 450, 25), // "scaleFullSizeImagesSignal"
QT_MOC_LITERAL(22, 476, 18), // "newCanonImageReady"
QT_MOC_LITERAL(23, 495, 18), // "canonCameraChanged"
QT_MOC_LITERAL(24, 514, 12), // "onImageReady"
QT_MOC_LITERAL(25, 527, 10) // "onFinished"

    },
    "Canon\0apertureOptionsChanged\0\0"
    "isoOptionsChanged\0shutterSpeedOptionsChanged\0"
    "exposureOptionsChanged\0"
    "whiteBalanceOptionsChanged\0"
    "colorTemperatureChanged\0"
    "imageQualityOptionsChanged\0"
    "pictureStyleOptionsChanged\0"
    "apertureChangedSignal\0isoChangedSignal\0"
    "shutterSpeedChangedSignal\0"
    "exposureChangedSignal\0whiteBalanceChangedSignal\0"
    "colorTemperatureChangedSignal\0"
    "imageQualityChangedSignal\0"
    "pictureStyleChangedSignal\0modeChanged\0"
    "focusCheckToggled\0pickFocusCheckToggled\0"
    "scaleFullSizeImagesSignal\0newCanonImageReady\0"
    "canonCameraChanged\0onImageReady\0"
    "onFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Canon[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      22,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  134,    2, 0x06 /* Public */,
       3,    0,  135,    2, 0x06 /* Public */,
       4,    0,  136,    2, 0x06 /* Public */,
       5,    0,  137,    2, 0x06 /* Public */,
       6,    0,  138,    2, 0x06 /* Public */,
       7,    0,  139,    2, 0x06 /* Public */,
       8,    0,  140,    2, 0x06 /* Public */,
       9,    0,  141,    2, 0x06 /* Public */,
      10,    1,  142,    2, 0x06 /* Public */,
      11,    1,  145,    2, 0x06 /* Public */,
      12,    1,  148,    2, 0x06 /* Public */,
      13,    1,  151,    2, 0x06 /* Public */,
      14,    1,  154,    2, 0x06 /* Public */,
      15,    1,  157,    2, 0x06 /* Public */,
      16,    1,  160,    2, 0x06 /* Public */,
      17,    1,  163,    2, 0x06 /* Public */,
      18,    0,  166,    2, 0x06 /* Public */,
      19,    1,  167,    2, 0x06 /* Public */,
      20,    1,  170,    2, 0x06 /* Public */,
      21,    1,  173,    2, 0x06 /* Public */,
      22,    0,  176,    2, 0x06 /* Public */,
      23,    1,  177,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      24,    1,  180,    2, 0x0a /* Public */,
      25,    0,  183,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void Canon::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Canon *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->apertureOptionsChanged(); break;
        case 1: _t->isoOptionsChanged(); break;
        case 2: _t->shutterSpeedOptionsChanged(); break;
        case 3: _t->exposureOptionsChanged(); break;
        case 4: _t->whiteBalanceOptionsChanged(); break;
        case 5: _t->colorTemperatureChanged(); break;
        case 6: _t->imageQualityOptionsChanged(); break;
        case 7: _t->pictureStyleOptionsChanged(); break;
        case 8: _t->apertureChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 9: _t->isoChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 10: _t->shutterSpeedChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 11: _t->exposureChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 12: _t->whiteBalanceChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 13: _t->colorTemperatureChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 14: _t->imageQualityChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 15: _t->pictureStyleChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 16: _t->modeChanged(); break;
        case 17: _t->focusCheckToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->pickFocusCheckToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->scaleFullSizeImagesSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->newCanonImageReady(); break;
        case 21: _t->canonCameraChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 22: _t->onImageReady((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 23: _t->onFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::apertureOptionsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::isoOptionsChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::shutterSpeedOptionsChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::exposureOptionsChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::whiteBalanceOptionsChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::colorTemperatureChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::imageQualityOptionsChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::pictureStyleOptionsChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::apertureChangedSignal)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::isoChangedSignal)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::shutterSpeedChangedSignal)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::exposureChangedSignal)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::whiteBalanceChangedSignal)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::colorTemperatureChangedSignal)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::imageQualityChangedSignal)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::pictureStyleChangedSignal)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::modeChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (Canon::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::focusCheckToggled)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (Canon::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::pickFocusCheckToggled)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (Canon::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::scaleFullSizeImagesSignal)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (Canon::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::newCanonImageReady)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (Canon::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Canon::canonCameraChanged)) {
                *result = 21;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Canon::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Canon.data,
    qt_meta_data_Canon,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Canon::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Canon::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Canon.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Canon::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void Canon::apertureOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Canon::isoOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Canon::shutterSpeedOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Canon::exposureOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Canon::whiteBalanceOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Canon::colorTemperatureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Canon::imageQualityOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Canon::pictureStyleOptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Canon::apertureChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void Canon::isoChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void Canon::shutterSpeedChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void Canon::exposureChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void Canon::whiteBalanceChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void Canon::colorTemperatureChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void Canon::imageQualityChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void Canon::pictureStyleChangedSignal(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void Canon::modeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void Canon::focusCheckToggled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void Canon::pickFocusCheckToggled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void Canon::scaleFullSizeImagesSignal(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void Canon::newCanonImageReady()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void Canon::canonCameraChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
