/****************************************************************************
** Meta object code from reading C++ file 'exportlevelpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/exportlevelpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'exportlevelpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ExportLevelPopup_t {
    QByteArrayData data[9];
    char stringdata0[113];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportLevelPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportLevelPopup_t qt_meta_stringdata_ExportLevelPopup = {
    {
QT_MOC_LITERAL(0, 0, 16), // "ExportLevelPopup"
QT_MOC_LITERAL(1, 17, 17), // "updateOnSelection"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 16), // "onOptionsClicked"
QT_MOC_LITERAL(4, 53, 7), // "onRetas"
QT_MOC_LITERAL(5, 61, 10), // "initFolder"
QT_MOC_LITERAL(6, 72, 15), // "onformatChanged"
QT_MOC_LITERAL(7, 88, 10), // "checkAlpha"
QT_MOC_LITERAL(8, 99, 13) // "updatePreview"

    },
    "ExportLevelPopup\0updateOnSelection\0\0"
    "onOptionsClicked\0onRetas\0initFolder\0"
    "onformatChanged\0checkAlpha\0updatePreview"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportLevelPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x08 /* Private */,
       3,    0,   50,    2, 0x08 /* Private */,
       4,    1,   51,    2, 0x08 /* Private */,
       5,    0,   54,    2, 0x08 /* Private */,
       6,    1,   55,    2, 0x08 /* Private */,
       7,    0,   58,    2, 0x08 /* Private */,
       8,    0,   59,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ExportLevelPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExportLevelPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateOnSelection(); break;
        case 1: _t->onOptionsClicked(); break;
        case 2: _t->onRetas((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->initFolder(); break;
        case 4: _t->onformatChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->checkAlpha(); break;
        case 6: _t->updatePreview(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ExportLevelPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<FileBrowserPopup::staticMetaObject>(),
    qt_meta_stringdata_ExportLevelPopup.data,
    qt_meta_data_ExportLevelPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportLevelPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportLevelPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportLevelPopup.stringdata0))
        return static_cast<void*>(this);
    return FileBrowserPopup::qt_metacast(_clname);
}

int ExportLevelPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FileBrowserPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
struct qt_meta_stringdata_ExportLevelPopup__ExportOptions_t {
    QByteArrayData data[7];
    char stringdata0[111];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportLevelPopup__ExportOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportLevelPopup__ExportOptions_t qt_meta_stringdata_ExportLevelPopup__ExportOptions = {
    {
QT_MOC_LITERAL(0, 0, 31), // "ExportLevelPopup::ExportOptions"
QT_MOC_LITERAL(1, 32, 14), // "optionsChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 10), // "updateXRes"
QT_MOC_LITERAL(4, 59, 10), // "updateYRes"
QT_MOC_LITERAL(5, 70, 8), // "scaleRes"
QT_MOC_LITERAL(6, 79, 31) // "onThicknessTransformModeChanged"

    },
    "ExportLevelPopup::ExportOptions\0"
    "optionsChanged\0\0updateXRes\0updateYRes\0"
    "scaleRes\0onThicknessTransformModeChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportLevelPopup__ExportOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   40,    2, 0x08 /* Private */,
       4,    0,   41,    2, 0x08 /* Private */,
       5,    0,   42,    2, 0x08 /* Private */,
       6,    0,   43,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ExportLevelPopup::ExportOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExportOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->optionsChanged(); break;
        case 1: _t->updateXRes(); break;
        case 2: _t->updateYRes(); break;
        case 3: _t->scaleRes(); break;
        case 4: _t->onThicknessTransformModeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ExportOptions::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ExportOptions::optionsChanged)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ExportLevelPopup::ExportOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_ExportLevelPopup__ExportOptions.data,
    qt_meta_data_ExportLevelPopup__ExportOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportLevelPopup::ExportOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportLevelPopup::ExportOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportLevelPopup__ExportOptions.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int ExportLevelPopup::ExportOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void ExportLevelPopup::ExportOptions::optionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
