/****************************************************************************
** Meta object code from reading C++ file 'filebrowser.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/filebrowser.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filebrowser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FrameCountReader_t {
    QByteArrayData data[3];
    char stringdata0[39];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FrameCountReader_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FrameCountReader_t qt_meta_stringdata_FrameCountReader = {
    {
QT_MOC_LITERAL(0, 0, 16), // "FrameCountReader"
QT_MOC_LITERAL(1, 17, 20), // "calculatedFrameCount"
QT_MOC_LITERAL(2, 38, 0) // ""

    },
    "FrameCountReader\0calculatedFrameCount\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FrameCountReader[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void FrameCountReader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FrameCountReader *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->calculatedFrameCount(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FrameCountReader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FrameCountReader::calculatedFrameCount)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FrameCountReader::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FrameCountReader.data,
    qt_meta_data_FrameCountReader,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FrameCountReader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FrameCountReader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FrameCountReader.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FrameCountReader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void FrameCountReader::calculatedFrameCount()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_FileBrowser_t {
    QByteArrayData data[54];
    char stringdata0[922];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FileBrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FileBrowser_t qt_meta_stringdata_FileBrowser = {
    {
QT_MOC_LITERAL(0, 0, 11), // "FileBrowser"
QT_MOC_LITERAL(1, 12, 15), // "filePathClicked"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 9), // "TFilePath"
QT_MOC_LITERAL(4, 39, 21), // "filePathDoubleClicked"
QT_MOC_LITERAL(5, 61, 17), // "filePathsSelected"
QT_MOC_LITERAL(6, 79, 19), // "std::set<TFilePath>"
QT_MOC_LITERAL(7, 99, 33), // "std::list<std::vector<TFrameI..."
QT_MOC_LITERAL(8, 133, 17), // "treeFolderChanged"
QT_MOC_LITERAL(9, 151, 14), // "historyChanged"
QT_MOC_LITERAL(10, 166, 19), // "onTreeFolderChanged"
QT_MOC_LITERAL(11, 186, 7), // "refresh"
QT_MOC_LITERAL(12, 194, 12), // "changeFolder"
QT_MOC_LITERAL(13, 207, 11), // "QModelIndex"
QT_MOC_LITERAL(14, 219, 5), // "index"
QT_MOC_LITERAL(15, 225, 13), // "onDataChanged"
QT_MOC_LITERAL(16, 239, 7), // "topLeft"
QT_MOC_LITERAL(17, 247, 11), // "bottomRight"
QT_MOC_LITERAL(18, 259, 13), // "loadResources"
QT_MOC_LITERAL(19, 273, 13), // "onClickedItem"
QT_MOC_LITERAL(20, 287, 19), // "onDoubleClickedItem"
QT_MOC_LITERAL(21, 307, 15), // "onSelectedItems"
QT_MOC_LITERAL(22, 323, 13), // "std::set<int>"
QT_MOC_LITERAL(23, 337, 7), // "indexes"
QT_MOC_LITERAL(24, 345, 8), // "folderUp"
QT_MOC_LITERAL(25, 354, 9), // "newFolder"
QT_MOC_LITERAL(26, 364, 18), // "onBackButtonPushed"
QT_MOC_LITERAL(27, 383, 17), // "onFwdButtonPushed"
QT_MOC_LITERAL(28, 401, 14), // "onFolderEdited"
QT_MOC_LITERAL(29, 416, 18), // "storeFolderHistory"
QT_MOC_LITERAL(30, 435, 12), // "clearHistory"
QT_MOC_LITERAL(31, 448, 18), // "renameAsToonzLevel"
QT_MOC_LITERAL(32, 467, 12), // "renameFolder"
QT_MOC_LITERAL(33, 480, 27), // "updateAndEditVersionControl"
QT_MOC_LITERAL(34, 508, 18), // "editVersionControl"
QT_MOC_LITERAL(35, 527, 20), // "unlockVersionControl"
QT_MOC_LITERAL(36, 548, 28), // "editFrameRangeVersionControl"
QT_MOC_LITERAL(37, 577, 30), // "unlockFrameRangeVersionControl"
QT_MOC_LITERAL(38, 608, 27), // "putFrameRangeVersionControl"
QT_MOC_LITERAL(39, 636, 30), // "revertFrameRangeVersionControl"
QT_MOC_LITERAL(40, 667, 19), // "showLockInformation"
QT_MOC_LITERAL(41, 687, 22), // "showFrameRangeLockInfo"
QT_MOC_LITERAL(42, 710, 17), // "putVersionControl"
QT_MOC_LITERAL(43, 728, 20), // "revertVersionControl"
QT_MOC_LITERAL(44, 749, 20), // "deleteVersionControl"
QT_MOC_LITERAL(45, 770, 17), // "getVersionControl"
QT_MOC_LITERAL(46, 788, 25), // "getRevisionVersionControl"
QT_MOC_LITERAL(47, 814, 18), // "getRevisionHistory"
QT_MOC_LITERAL(48, 833, 27), // "onVersionControlCommandDone"
QT_MOC_LITERAL(49, 861, 5), // "files"
QT_MOC_LITERAL(50, 867, 19), // "onFileSystemChanged"
QT_MOC_LITERAL(51, 887, 10), // "folderPath"
QT_MOC_LITERAL(52, 898, 14), // "tryToOpenScene"
QT_MOC_LITERAL(53, 913, 8) // "filePath"

    },
    "FileBrowser\0filePathClicked\0\0TFilePath\0"
    "filePathDoubleClicked\0filePathsSelected\0"
    "std::set<TFilePath>\0"
    "std::list<std::vector<TFrameId> >\0"
    "treeFolderChanged\0historyChanged\0"
    "onTreeFolderChanged\0refresh\0changeFolder\0"
    "QModelIndex\0index\0onDataChanged\0topLeft\0"
    "bottomRight\0loadResources\0onClickedItem\0"
    "onDoubleClickedItem\0onSelectedItems\0"
    "std::set<int>\0indexes\0folderUp\0newFolder\0"
    "onBackButtonPushed\0onFwdButtonPushed\0"
    "onFolderEdited\0storeFolderHistory\0"
    "clearHistory\0renameAsToonzLevel\0"
    "renameFolder\0updateAndEditVersionControl\0"
    "editVersionControl\0unlockVersionControl\0"
    "editFrameRangeVersionControl\0"
    "unlockFrameRangeVersionControl\0"
    "putFrameRangeVersionControl\0"
    "revertFrameRangeVersionControl\0"
    "showLockInformation\0showFrameRangeLockInfo\0"
    "putVersionControl\0revertVersionControl\0"
    "deleteVersionControl\0getVersionControl\0"
    "getRevisionVersionControl\0getRevisionHistory\0"
    "onVersionControlCommandDone\0files\0"
    "onFileSystemChanged\0folderPath\0"
    "tryToOpenScene\0filePath"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FileBrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      40,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  214,    2, 0x06 /* Public */,
       4,    1,  217,    2, 0x06 /* Public */,
       5,    2,  220,    2, 0x06 /* Public */,
       8,    1,  225,    2, 0x06 /* Public */,
       9,    2,  228,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,  233,    2, 0x0a /* Public */,
      11,    0,  234,    2, 0x09 /* Protected */,
      12,    1,  235,    2, 0x09 /* Protected */,
      15,    2,  238,    2, 0x09 /* Protected */,
      18,    0,  243,    2, 0x09 /* Protected */,
      19,    1,  244,    2, 0x09 /* Protected */,
      20,    1,  247,    2, 0x09 /* Protected */,
      21,    1,  250,    2, 0x09 /* Protected */,
      24,    0,  253,    2, 0x09 /* Protected */,
      25,    0,  254,    2, 0x09 /* Protected */,
      26,    0,  255,    2, 0x09 /* Protected */,
      27,    0,  256,    2, 0x09 /* Protected */,
      28,    0,  257,    2, 0x09 /* Protected */,
      29,    0,  258,    2, 0x09 /* Protected */,
      30,    0,  259,    2, 0x09 /* Protected */,
      31,    0,  260,    2, 0x09 /* Protected */,
      32,    0,  261,    2, 0x09 /* Protected */,
      33,    0,  262,    2, 0x09 /* Protected */,
      34,    0,  263,    2, 0x09 /* Protected */,
      35,    0,  264,    2, 0x09 /* Protected */,
      36,    0,  265,    2, 0x09 /* Protected */,
      37,    0,  266,    2, 0x09 /* Protected */,
      38,    0,  267,    2, 0x09 /* Protected */,
      39,    0,  268,    2, 0x09 /* Protected */,
      40,    0,  269,    2, 0x09 /* Protected */,
      41,    0,  270,    2, 0x09 /* Protected */,
      42,    0,  271,    2, 0x09 /* Protected */,
      43,    0,  272,    2, 0x09 /* Protected */,
      44,    0,  273,    2, 0x09 /* Protected */,
      45,    0,  274,    2, 0x09 /* Protected */,
      46,    0,  275,    2, 0x09 /* Protected */,
      47,    0,  276,    2, 0x09 /* Protected */,
      48,    1,  277,    2, 0x09 /* Protected */,
      50,    1,  280,    2, 0x09 /* Protected */,
      52,    1,  283,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 7,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 13,   16,   17,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,   49,
    QMetaType::Void, QMetaType::QString,   51,
    QMetaType::Void, 0x80000000 | 3,   53,

       0        // eod
};

void FileBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FileBrowser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->filePathClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 1: _t->filePathDoubleClicked((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 2: _t->filePathsSelected((*reinterpret_cast< const std::set<TFilePath>(*)>(_a[1])),(*reinterpret_cast< const std::list<std::vector<TFrameId> >(*)>(_a[2]))); break;
        case 3: _t->treeFolderChanged((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 4: _t->historyChanged((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->onTreeFolderChanged(); break;
        case 6: _t->refresh(); break;
        case 7: _t->changeFolder((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 8: _t->onDataChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 9: _t->loadResources(); break;
        case 10: _t->onClickedItem((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->onDoubleClickedItem((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->onSelectedItems((*reinterpret_cast< const std::set<int>(*)>(_a[1]))); break;
        case 13: _t->folderUp(); break;
        case 14: _t->newFolder(); break;
        case 15: _t->onBackButtonPushed(); break;
        case 16: _t->onFwdButtonPushed(); break;
        case 17: _t->onFolderEdited(); break;
        case 18: _t->storeFolderHistory(); break;
        case 19: _t->clearHistory(); break;
        case 20: _t->renameAsToonzLevel(); break;
        case 21: _t->renameFolder(); break;
        case 22: _t->updateAndEditVersionControl(); break;
        case 23: _t->editVersionControl(); break;
        case 24: _t->unlockVersionControl(); break;
        case 25: _t->editFrameRangeVersionControl(); break;
        case 26: _t->unlockFrameRangeVersionControl(); break;
        case 27: _t->putFrameRangeVersionControl(); break;
        case 28: _t->revertFrameRangeVersionControl(); break;
        case 29: _t->showLockInformation(); break;
        case 30: _t->showFrameRangeLockInfo(); break;
        case 31: _t->putVersionControl(); break;
        case 32: _t->revertVersionControl(); break;
        case 33: _t->deleteVersionControl(); break;
        case 34: _t->getVersionControl(); break;
        case 35: _t->getRevisionVersionControl(); break;
        case 36: _t->getRevisionHistory(); break;
        case 37: _t->onVersionControlCommandDone((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 38: _t->onFileSystemChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 39: _t->tryToOpenScene((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FileBrowser::*)(const TFilePath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBrowser::filePathClicked)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FileBrowser::*)(const TFilePath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBrowser::filePathDoubleClicked)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FileBrowser::*)(const std::set<TFilePath> & , const std::list<std::vector<TFrameId>> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBrowser::filePathsSelected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FileBrowser::*)(const TFilePath & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBrowser::treeFolderChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FileBrowser::*)(bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FileBrowser::historyChanged)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FileBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_FileBrowser.data,
    qt_meta_data_FileBrowser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FileBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FileBrowser.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DvItemListModel"))
        return static_cast< DvItemListModel*>(this);
    return QFrame::qt_metacast(_clname);
}

int FileBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 40;
    }
    return _id;
}

// SIGNAL 0
void FileBrowser::filePathClicked(const TFilePath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FileBrowser::filePathDoubleClicked(const TFilePath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FileBrowser::filePathsSelected(const std::set<TFilePath> & _t1, const std::list<std::vector<TFrameId>> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void FileBrowser::treeFolderChanged(const TFilePath & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void FileBrowser::historyChanged(bool _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_RenameAsToonzPopup_t {
    QByteArrayData data[3];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RenameAsToonzPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RenameAsToonzPopup_t qt_meta_stringdata_RenameAsToonzPopup = {
    {
QT_MOC_LITERAL(0, 0, 18), // "RenameAsToonzPopup"
QT_MOC_LITERAL(1, 19, 4), // "onOk"
QT_MOC_LITERAL(2, 24, 0) // ""

    },
    "RenameAsToonzPopup\0onOk\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RenameAsToonzPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void RenameAsToonzPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RenameAsToonzPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onOk(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject RenameAsToonzPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_RenameAsToonzPopup.data,
    qt_meta_data_RenameAsToonzPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RenameAsToonzPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RenameAsToonzPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RenameAsToonzPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int RenameAsToonzPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
