/****************************************************************************
** Meta object code from reading C++ file 'xshnoteviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/xshnoteviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xshnoteviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_XsheetGUI__NotePopup_t {
    QByteArrayData data[16];
    char stringdata0[239];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__NotePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__NotePopup_t qt_meta_stringdata_XsheetGUI__NotePopup = {
    {
QT_MOC_LITERAL(0, 0, 20), // "XsheetGUI::NotePopup"
QT_MOC_LITERAL(1, 21, 16), // "onColor1Switched"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 8), // "TPixel32"
QT_MOC_LITERAL(4, 48, 10), // "isDragging"
QT_MOC_LITERAL(5, 59, 16), // "onColor2Switched"
QT_MOC_LITERAL(6, 76, 16), // "onColor3Switched"
QT_MOC_LITERAL(7, 93, 16), // "onColor4Switched"
QT_MOC_LITERAL(8, 110, 16), // "onColor5Switched"
QT_MOC_LITERAL(9, 127, 16), // "onColor6Switched"
QT_MOC_LITERAL(10, 144, 16), // "onColor7Switched"
QT_MOC_LITERAL(11, 161, 14), // "onColorChanged"
QT_MOC_LITERAL(12, 176, 11), // "onNoteAdded"
QT_MOC_LITERAL(13, 188, 15), // "onNoteDiscarded"
QT_MOC_LITERAL(14, 204, 17), // "onTextEditFocusIn"
QT_MOC_LITERAL(15, 222, 16) // "onXsheetSwitched"

    },
    "XsheetGUI::NotePopup\0onColor1Switched\0"
    "\0TPixel32\0isDragging\0onColor2Switched\0"
    "onColor3Switched\0onColor4Switched\0"
    "onColor5Switched\0onColor6Switched\0"
    "onColor7Switched\0onColorChanged\0"
    "onNoteAdded\0onNoteDiscarded\0"
    "onTextEditFocusIn\0onXsheetSwitched"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__NotePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   74,    2, 0x09 /* Protected */,
       5,    2,   79,    2, 0x09 /* Protected */,
       6,    2,   84,    2, 0x09 /* Protected */,
       7,    2,   89,    2, 0x09 /* Protected */,
       8,    2,   94,    2, 0x09 /* Protected */,
       9,    2,   99,    2, 0x09 /* Protected */,
      10,    2,  104,    2, 0x09 /* Protected */,
      11,    2,  109,    2, 0x09 /* Protected */,
      12,    0,  114,    2, 0x09 /* Protected */,
      13,    0,  115,    2, 0x09 /* Protected */,
      14,    0,  116,    2, 0x09 /* Protected */,
      15,    0,  117,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void XsheetGUI::NotePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NotePopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onColor1Switched((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onColor2Switched((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->onColor3Switched((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->onColor4Switched((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->onColor5Switched((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->onColor6Switched((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->onColor7Switched((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 7: _t->onColorChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 8: _t->onNoteAdded(); break;
        case 9: _t->onNoteDiscarded(); break;
        case 10: _t->onTextEditFocusIn(); break;
        case 11: _t->onXsheetSwitched(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::NotePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__NotePopup.data,
    qt_meta_data_XsheetGUI__NotePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::NotePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::NotePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__NotePopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int XsheetGUI::NotePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__NoteWidget_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__NoteWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__NoteWidget_t qt_meta_stringdata_XsheetGUI__NoteWidget = {
    {
QT_MOC_LITERAL(0, 0, 21) // "XsheetGUI::NoteWidget"

    },
    "XsheetGUI::NoteWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__NoteWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void XsheetGUI::NoteWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::NoteWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__NoteWidget.data,
    qt_meta_data_XsheetGUI__NoteWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::NoteWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::NoteWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__NoteWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int XsheetGUI::NoteWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_XsheetGUI__NoteArea_t {
    QByteArrayData data[12];
    char stringdata0[174];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_XsheetGUI__NoteArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_XsheetGUI__NoteArea_t qt_meta_stringdata_XsheetGUI__NoteArea = {
    {
QT_MOC_LITERAL(0, 0, 19), // "XsheetGUI::NoteArea"
QT_MOC_LITERAL(1, 20, 15), // "flipOrientation"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 13), // "toggleNewNote"
QT_MOC_LITERAL(4, 51, 8), // "nextNote"
QT_MOC_LITERAL(5, 60, 8), // "precNote"
QT_MOC_LITERAL(6, 69, 26), // "onFrameDisplayStyleChanged"
QT_MOC_LITERAL(7, 96, 2), // "id"
QT_MOC_LITERAL(8, 99, 26), // "onXsheetOrientationChanged"
QT_MOC_LITERAL(9, 126, 18), // "const Orientation*"
QT_MOC_LITERAL(10, 145, 11), // "orientation"
QT_MOC_LITERAL(11, 157, 16) // "onClickHamburger"

    },
    "XsheetGUI::NoteArea\0flipOrientation\0"
    "\0toggleNewNote\0nextNote\0precNote\0"
    "onFrameDisplayStyleChanged\0id\0"
    "onXsheetOrientationChanged\0"
    "const Orientation*\0orientation\0"
    "onClickHamburger"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_XsheetGUI__NoteArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x09 /* Protected */,
       3,    0,   50,    2, 0x09 /* Protected */,
       4,    0,   51,    2, 0x09 /* Protected */,
       5,    0,   52,    2, 0x09 /* Protected */,
       6,    1,   53,    2, 0x09 /* Protected */,
       8,    1,   56,    2, 0x09 /* Protected */,
      11,    0,   59,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,

       0        // eod
};

void XsheetGUI::NoteArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NoteArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->flipOrientation(); break;
        case 1: _t->toggleNewNote(); break;
        case 2: _t->nextNote(); break;
        case 3: _t->precNote(); break;
        case 4: _t->onFrameDisplayStyleChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onXsheetOrientationChanged((*reinterpret_cast< const Orientation*(*)>(_a[1]))); break;
        case 6: _t->onClickHamburger(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject XsheetGUI::NoteArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_XsheetGUI__NoteArea.data,
    qt_meta_data_XsheetGUI__NoteArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *XsheetGUI::NoteArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XsheetGUI::NoteArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_XsheetGUI__NoteArea.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int XsheetGUI::NoteArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
