/****************************************************************************
** Meta object code from reading C++ file 'paletteviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/paletteviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'paletteviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PaletteViewer_t {
    QByteArrayData data[33];
    char stringdata0[510];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PaletteViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PaletteViewer_t qt_meta_stringdata_PaletteViewer = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PaletteViewer"
QT_MOC_LITERAL(1, 14, 11), // "setPageView"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 16), // "currentIndexPage"
QT_MOC_LITERAL(4, 44, 10), // "addNewPage"
QT_MOC_LITERAL(5, 55, 11), // "addNewColor"
QT_MOC_LITERAL(6, 67, 10), // "deletePage"
QT_MOC_LITERAL(7, 78, 17), // "saveStudioPalette"
QT_MOC_LITERAL(8, 96, 20), // "onColorStyleSwitched"
QT_MOC_LITERAL(9, 117, 16), // "onPaletteChanged"
QT_MOC_LITERAL(10, 134, 17), // "onPaletteSwitched"
QT_MOC_LITERAL(11, 152, 15), // "onFrameSwitched"
QT_MOC_LITERAL(12, 168, 16), // "onTabTextChanged"
QT_MOC_LITERAL(13, 185, 8), // "tabIndex"
QT_MOC_LITERAL(14, 194, 10), // "onViewMode"
QT_MOC_LITERAL(15, 205, 8), // "QAction*"
QT_MOC_LITERAL(16, 214, 17), // "changeWindowTitle"
QT_MOC_LITERAL(17, 232, 8), // "movePage"
QT_MOC_LITERAL(18, 241, 8), // "srcIndex"
QT_MOC_LITERAL(19, 250, 8), // "dstIndex"
QT_MOC_LITERAL(20, 259, 13), // "startDragDrop"
QT_MOC_LITERAL(21, 273, 17), // "onNameDisplayMode"
QT_MOC_LITERAL(22, 291, 11), // "setIsLocked"
QT_MOC_LITERAL(23, 303, 4), // "lock"
QT_MOC_LITERAL(24, 308, 14), // "onSwitchToPage"
QT_MOC_LITERAL(25, 323, 9), // "pageIndex"
QT_MOC_LITERAL(26, 333, 27), // "onShowNewStyleButtonToggled"
QT_MOC_LITERAL(27, 361, 18), // "toggleToolbarOnTop"
QT_MOC_LITERAL(28, 380, 24), // "toggleKeyframeVisibility"
QT_MOC_LITERAL(29, 405, 28), // "toggleNewStylePageVisibility"
QT_MOC_LITERAL(30, 434, 28), // "togglePaletteGizmoVisibility"
QT_MOC_LITERAL(31, 463, 26), // "toggleNameEditorVisibility"
QT_MOC_LITERAL(32, 490, 19) // "toggleVariableWidth"

    },
    "PaletteViewer\0setPageView\0\0currentIndexPage\0"
    "addNewPage\0addNewColor\0deletePage\0"
    "saveStudioPalette\0onColorStyleSwitched\0"
    "onPaletteChanged\0onPaletteSwitched\0"
    "onFrameSwitched\0onTabTextChanged\0"
    "tabIndex\0onViewMode\0QAction*\0"
    "changeWindowTitle\0movePage\0srcIndex\0"
    "dstIndex\0startDragDrop\0onNameDisplayMode\0"
    "setIsLocked\0lock\0onSwitchToPage\0"
    "pageIndex\0onShowNewStyleButtonToggled\0"
    "toggleToolbarOnTop\0toggleKeyframeVisibility\0"
    "toggleNewStylePageVisibility\0"
    "togglePaletteGizmoVisibility\0"
    "toggleNameEditorVisibility\0"
    "toggleVariableWidth"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PaletteViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  134,    2, 0x09 /* Protected */,
       4,    0,  137,    2, 0x09 /* Protected */,
       5,    0,  138,    2, 0x09 /* Protected */,
       6,    0,  139,    2, 0x09 /* Protected */,
       7,    0,  140,    2, 0x09 /* Protected */,
       8,    0,  141,    2, 0x09 /* Protected */,
       9,    0,  142,    2, 0x09 /* Protected */,
      10,    0,  143,    2, 0x09 /* Protected */,
      11,    0,  144,    2, 0x09 /* Protected */,
      12,    1,  145,    2, 0x09 /* Protected */,
      14,    1,  148,    2, 0x09 /* Protected */,
      16,    0,  151,    2, 0x09 /* Protected */,
      17,    2,  152,    2, 0x09 /* Protected */,
      20,    0,  157,    2, 0x09 /* Protected */,
      21,    1,  158,    2, 0x09 /* Protected */,
      22,    1,  161,    2, 0x09 /* Protected */,
      24,    1,  164,    2, 0x09 /* Protected */,
      26,    0,  167,    2, 0x09 /* Protected */,
      27,    0,  168,    2, 0x09 /* Protected */,
      28,    1,  169,    2, 0x09 /* Protected */,
      29,    1,  172,    2, 0x09 /* Protected */,
      30,    1,  175,    2, 0x09 /* Protected */,
      31,    1,  178,    2, 0x09 /* Protected */,
      32,    1,  181,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   18,   19,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void PaletteViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PaletteViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setPageView((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->addNewPage(); break;
        case 2: _t->addNewColor(); break;
        case 3: _t->deletePage(); break;
        case 4: _t->saveStudioPalette(); break;
        case 5: _t->onColorStyleSwitched(); break;
        case 6: _t->onPaletteChanged(); break;
        case 7: _t->onPaletteSwitched(); break;
        case 8: _t->onFrameSwitched(); break;
        case 9: _t->onTabTextChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->onViewMode((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 11: _t->changeWindowTitle(); break;
        case 12: _t->movePage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 13: _t->startDragDrop(); break;
        case 14: _t->onNameDisplayMode((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 15: _t->setIsLocked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->onSwitchToPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->onShowNewStyleButtonToggled(); break;
        case 18: _t->toggleToolbarOnTop(); break;
        case 19: _t->toggleKeyframeVisibility((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->toggleNewStylePageVisibility((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->togglePaletteGizmoVisibility((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->toggleNameEditorVisibility((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->toggleVariableWidth((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PaletteViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_PaletteViewer.data,
    qt_meta_data_PaletteViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PaletteViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaletteViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PaletteViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SaveLoadQSettings"))
        return static_cast< SaveLoadQSettings*>(this);
    return QFrame::qt_metacast(_clname);
}

int PaletteViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
