/****************************************************************************
** Meta object code from reading C++ file 'convertpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/convertpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'convertpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ConvertPopup_t {
    QByteArrayData data[19];
    char stringdata0[253];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ConvertPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ConvertPopup_t qt_meta_stringdata_ConvertPopup = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ConvertPopup"
QT_MOC_LITERAL(1, 13, 5), // "apply"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 16), // "onOptionsClicked"
QT_MOC_LITERAL(4, 37, 17), // "onConvertFinished"
QT_MOC_LITERAL(5, 55, 17), // "onTlvModeSelected"
QT_MOC_LITERAL(6, 73, 7), // "tlvMode"
QT_MOC_LITERAL(7, 81, 16), // "onFormatSelected"
QT_MOC_LITERAL(8, 98, 6), // "format"
QT_MOC_LITERAL(9, 105, 19), // "onAntialiasSelected"
QT_MOC_LITERAL(10, 125, 5), // "index"
QT_MOC_LITERAL(11, 131, 15), // "onFileInChanged"
QT_MOC_LITERAL(12, 147, 14), // "onRangeChanged"
QT_MOC_LITERAL(13, 162, 16), // "onLevelConverted"
QT_MOC_LITERAL(14, 179, 9), // "TFilePath"
QT_MOC_LITERAL(15, 189, 8), // "fullPath"
QT_MOC_LITERAL(16, 198, 15), // "onFormatChanged"
QT_MOC_LITERAL(17, 214, 20), // "onPalettePathChanged"
QT_MOC_LITERAL(18, 235, 17) // "onDpiModeSelected"

    },
    "ConvertPopup\0apply\0\0onOptionsClicked\0"
    "onConvertFinished\0onTlvModeSelected\0"
    "tlvMode\0onFormatSelected\0format\0"
    "onAntialiasSelected\0index\0onFileInChanged\0"
    "onRangeChanged\0onLevelConverted\0"
    "TFilePath\0fullPath\0onFormatChanged\0"
    "onPalettePathChanged\0onDpiModeSelected"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ConvertPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x0a /* Public */,
       3,    0,   75,    2, 0x0a /* Public */,
       4,    0,   76,    2, 0x0a /* Public */,
       5,    1,   77,    2, 0x0a /* Public */,
       7,    1,   80,    2, 0x0a /* Public */,
       9,    1,   83,    2, 0x0a /* Public */,
      11,    0,   86,    2, 0x0a /* Public */,
      12,    0,   87,    2, 0x0a /* Public */,
      13,    1,   88,    2, 0x0a /* Public */,
      16,    1,   91,    2, 0x0a /* Public */,
      17,    0,   94,    2, 0x0a /* Public */,
      18,    1,   95,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   10,

       0        // eod
};

void ConvertPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ConvertPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->apply(); break;
        case 1: _t->onOptionsClicked(); break;
        case 2: _t->onConvertFinished(); break;
        case 3: _t->onTlvModeSelected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->onFormatSelected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->onAntialiasSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->onFileInChanged(); break;
        case 7: _t->onRangeChanged(); break;
        case 8: _t->onLevelConverted((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 9: _t->onFormatChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->onPalettePathChanged(); break;
        case 11: _t->onDpiModeSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ConvertPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_ConvertPopup.data,
    qt_meta_data_ConvertPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ConvertPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConvertPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ConvertPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int ConvertPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
