

#ifdef __cplusplus
extern "C" {
#endif

#include "ttwain_statePD.h"

int TTWAIN_LoadSourceManagerPD(void) { return 0; }
int TTWAIN_UnloadSourceManagerPD(void) { return 1; }

#ifdef __cplusplus
}
#endif
