/****************************************************************************
** Meta object code from reading C++ file 'tooloptionscontrols.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/tnztools/tooloptionscontrols.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tooloptionscontrols.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ToolOptionCheckbox_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionCheckbox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionCheckbox_t qt_meta_stringdata_ToolOptionCheckbox = {
    {
QT_MOC_LITERAL(0, 0, 18) // "ToolOptionCheckbox"

    },
    "ToolOptionCheckbox"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionCheckbox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ToolOptionCheckbox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionCheckbox::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::CheckBox::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionCheckbox.data,
    qt_meta_data_ToolOptionCheckbox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionCheckbox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionCheckbox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionCheckbox.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::CheckBox::qt_metacast(_clname);
}

int ToolOptionCheckbox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::CheckBox::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ToolOptionSlider_t {
    QByteArrayData data[4];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionSlider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionSlider_t qt_meta_stringdata_ToolOptionSlider = {
    {
QT_MOC_LITERAL(0, 0, 16), // "ToolOptionSlider"
QT_MOC_LITERAL(1, 17, 14), // "onValueChanged"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 10) // "isDragging"

    },
    "ToolOptionSlider\0onValueChanged\0\0"
    "isDragging"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionSlider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void ToolOptionSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionSlider *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::DoubleField::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionSlider.data,
    qt_meta_data_ToolOptionSlider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionSlider.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::DoubleField::qt_metacast(_clname);
}

int ToolOptionSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::DoubleField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionPairSlider_t {
    QByteArrayData data[4];
    char stringdata0[49];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionPairSlider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionPairSlider_t qt_meta_stringdata_ToolOptionPairSlider = {
    {
QT_MOC_LITERAL(0, 0, 20), // "ToolOptionPairSlider"
QT_MOC_LITERAL(1, 21, 15), // "onValuesChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 10) // "isDragging"

    },
    "ToolOptionPairSlider\0onValuesChanged\0"
    "\0isDragging"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionPairSlider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void ToolOptionPairSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionPairSlider *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValuesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionPairSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::DoublePairField::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionPairSlider.data,
    qt_meta_data_ToolOptionPairSlider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionPairSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionPairSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionPairSlider.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::DoublePairField::qt_metacast(_clname);
}

int ToolOptionPairSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::DoublePairField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionIntPairSlider_t {
    QByteArrayData data[4];
    char stringdata0[52];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionIntPairSlider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionIntPairSlider_t qt_meta_stringdata_ToolOptionIntPairSlider = {
    {
QT_MOC_LITERAL(0, 0, 23), // "ToolOptionIntPairSlider"
QT_MOC_LITERAL(1, 24, 15), // "onValuesChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 10) // "isDragging"

    },
    "ToolOptionIntPairSlider\0onValuesChanged\0"
    "\0isDragging"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionIntPairSlider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void ToolOptionIntPairSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionIntPairSlider *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValuesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionIntPairSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::IntPairField::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionIntPairSlider.data,
    qt_meta_data_ToolOptionIntPairSlider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionIntPairSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionIntPairSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionIntPairSlider.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::IntPairField::qt_metacast(_clname);
}

int ToolOptionIntPairSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::IntPairField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionIntSlider_t {
    QByteArrayData data[4];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionIntSlider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionIntSlider_t qt_meta_stringdata_ToolOptionIntSlider = {
    {
QT_MOC_LITERAL(0, 0, 19), // "ToolOptionIntSlider"
QT_MOC_LITERAL(1, 20, 14), // "onValueChanged"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 10) // "isDragging"

    },
    "ToolOptionIntSlider\0onValueChanged\0\0"
    "isDragging"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionIntSlider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void ToolOptionIntSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionIntSlider *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionIntSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::IntField::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionIntSlider.data,
    qt_meta_data_ToolOptionIntSlider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionIntSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionIntSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionIntSlider.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::IntField::qt_metacast(_clname);
}

int ToolOptionIntSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::IntField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionCombo_t {
    QByteArrayData data[6];
    char stringdata0[72];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionCombo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionCombo_t qt_meta_stringdata_ToolOptionCombo = {
    {
QT_MOC_LITERAL(0, 0, 15), // "ToolOptionCombo"
QT_MOC_LITERAL(1, 16, 18), // "reloadComboBoxList"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 11), // "std::string"
QT_MOC_LITERAL(4, 48, 11), // "loadEntries"
QT_MOC_LITERAL(5, 60, 11) // "onActivated"

    },
    "ToolOptionCombo\0reloadComboBoxList\0\0"
    "std::string\0loadEntries\0onActivated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionCombo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x0a /* Public */,
       4,    0,   32,    2, 0x0a /* Public */,
       5,    1,   33,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void ToolOptionCombo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionCombo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->reloadComboBoxList((*reinterpret_cast< std::string(*)>(_a[1]))); break;
        case 1: _t->loadEntries(); break;
        case 2: _t->onActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionCombo::staticMetaObject = { {
    QMetaObject::SuperData::link<QComboBox::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionCombo.data,
    qt_meta_data_ToolOptionCombo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionCombo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionCombo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionCombo.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return QComboBox::qt_metacast(_clname);
}

int ToolOptionCombo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionFontCombo_t {
    QByteArrayData data[3];
    char stringdata0[33];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionFontCombo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionFontCombo_t qt_meta_stringdata_ToolOptionFontCombo = {
    {
QT_MOC_LITERAL(0, 0, 19), // "ToolOptionFontCombo"
QT_MOC_LITERAL(1, 20, 11), // "onActivated"
QT_MOC_LITERAL(2, 32, 0) // ""

    },
    "ToolOptionFontCombo\0onActivated\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionFontCombo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void ToolOptionFontCombo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionFontCombo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionFontCombo::staticMetaObject = { {
    QMetaObject::SuperData::link<QFontComboBox::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionFontCombo.data,
    qt_meta_data_ToolOptionFontCombo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionFontCombo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionFontCombo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionFontCombo.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return QFontComboBox::qt_metacast(_clname);
}

int ToolOptionFontCombo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFontComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionPopupButton_t {
    QByteArrayData data[4];
    char stringdata0[53];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionPopupButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionPopupButton_t qt_meta_stringdata_ToolOptionPopupButton = {
    {
QT_MOC_LITERAL(0, 0, 21), // "ToolOptionPopupButton"
QT_MOC_LITERAL(1, 22, 11), // "onActivated"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 17) // "doSetCurrentIndex"

    },
    "ToolOptionPopupButton\0onActivated\0\0"
    "doSetCurrentIndex"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionPopupButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x0a /* Public */,
       3,    1,   27,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void ToolOptionPopupButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionPopupButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->doSetCurrentIndex((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionPopupButton::staticMetaObject = { {
    QMetaObject::SuperData::link<PopupButton::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionPopupButton.data,
    qt_meta_data_ToolOptionPopupButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionPopupButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionPopupButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionPopupButton.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return PopupButton::qt_metacast(_clname);
}

int ToolOptionPopupButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PopupButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionTextField_t {
    QByteArrayData data[3];
    char stringdata0[36];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionTextField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionTextField_t qt_meta_stringdata_ToolOptionTextField = {
    {
QT_MOC_LITERAL(0, 0, 19), // "ToolOptionTextField"
QT_MOC_LITERAL(1, 20, 14), // "onValueChanged"
QT_MOC_LITERAL(2, 35, 0) // ""

    },
    "ToolOptionTextField\0onValueChanged\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionTextField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void ToolOptionTextField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionTextField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValueChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionTextField::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::LineEdit::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionTextField.data,
    qt_meta_data_ToolOptionTextField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionTextField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionTextField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionTextField.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::LineEdit::qt_metacast(_clname);
}

int ToolOptionTextField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::LineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_StyleIndexFieldAndChip_t {
    QByteArrayData data[4];
    char stringdata0[51];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleIndexFieldAndChip_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleIndexFieldAndChip_t qt_meta_stringdata_StyleIndexFieldAndChip = {
    {
QT_MOC_LITERAL(0, 0, 22), // "StyleIndexFieldAndChip"
QT_MOC_LITERAL(1, 23, 14), // "onValueChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 11) // "updateColor"

    },
    "StyleIndexFieldAndChip\0onValueChanged\0"
    "\0updateColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleIndexFieldAndChip[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x0a /* Public */,
       3,    0,   27,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,

       0        // eod
};

void StyleIndexFieldAndChip::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StyleIndexFieldAndChip *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValueChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->updateColor(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleIndexFieldAndChip::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::StyleIndexLineEdit::staticMetaObject>(),
    qt_meta_stringdata_StyleIndexFieldAndChip.data,
    qt_meta_data_StyleIndexFieldAndChip,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleIndexFieldAndChip::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleIndexFieldAndChip::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleIndexFieldAndChip.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::StyleIndexLineEdit::qt_metacast(_clname);
}

int StyleIndexFieldAndChip::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::StyleIndexLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionParamRelayField_t {
    QByteArrayData data[3];
    char stringdata0[42];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionParamRelayField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionParamRelayField_t qt_meta_stringdata_ToolOptionParamRelayField = {
    {
QT_MOC_LITERAL(0, 0, 25), // "ToolOptionParamRelayField"
QT_MOC_LITERAL(1, 26, 14), // "onValueChanged"
QT_MOC_LITERAL(2, 41, 0) // ""

    },
    "ToolOptionParamRelayField\0onValueChanged\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionParamRelayField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void ToolOptionParamRelayField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionParamRelayField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValueChanged(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionParamRelayField::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::MeasuredDoubleLineEdit::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionParamRelayField.data,
    qt_meta_data_ToolOptionParamRelayField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionParamRelayField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionParamRelayField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionParamRelayField.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return DVGui::MeasuredDoubleLineEdit::qt_metacast(_clname);
}

int ToolOptionParamRelayField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::MeasuredDoubleLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_MeasuredValueField_t {
    QByteArrayData data[14];
    char stringdata0[190];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MeasuredValueField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MeasuredValueField_t qt_meta_stringdata_MeasuredValueField = {
    {
QT_MOC_LITERAL(0, 0, 18), // "MeasuredValueField"
QT_MOC_LITERAL(1, 19, 20), // "measuredValueChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(4, 57, 5), // "value"
QT_MOC_LITERAL(5, 63, 9), // "addToUndo"
QT_MOC_LITERAL(6, 73, 6), // "commit"
QT_MOC_LITERAL(7, 80, 13), // "onTextChanged"
QT_MOC_LITERAL(8, 94, 21), // "errorHighlightingTick"
QT_MOC_LITERAL(9, 116, 16), // "receiveMouseMove"
QT_MOC_LITERAL(10, 133, 12), // "QMouseEvent*"
QT_MOC_LITERAL(11, 146, 5), // "event"
QT_MOC_LITERAL(12, 152, 17), // "receiveMousePress"
QT_MOC_LITERAL(13, 170, 19) // "receiveMouseRelease"

    },
    "MeasuredValueField\0measuredValueChanged\0"
    "\0TMeasuredValue*\0value\0addToUndo\0"
    "commit\0onTextChanged\0errorHighlightingTick\0"
    "receiveMouseMove\0QMouseEvent*\0event\0"
    "receiveMousePress\0receiveMouseRelease"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MeasuredValueField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   54,    2, 0x06 /* Public */,
       1,    1,   59,    2, 0x26 /* Public | MethodCloned */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   62,    2, 0x09 /* Protected */,
       7,    1,   63,    2, 0x09 /* Protected */,
       8,    0,   66,    2, 0x09 /* Protected */,
       9,    1,   67,    2, 0x09 /* Protected */,
      12,    1,   70,    2, 0x09 /* Protected */,
      13,    1,   73,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10,   11,

       0        // eod
};

void MeasuredValueField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MeasuredValueField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->measuredValueChanged((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->measuredValueChanged((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        case 2: _t->commit(); break;
        case 3: _t->onTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->errorHighlightingTick(); break;
        case 5: _t->receiveMouseMove((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 6: _t->receiveMousePress((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 7: _t->receiveMouseRelease((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MeasuredValueField::*)(TMeasuredValue * , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MeasuredValueField::measuredValueChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MeasuredValueField::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::LineEdit::staticMetaObject>(),
    qt_meta_stringdata_MeasuredValueField.data,
    qt_meta_data_MeasuredValueField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MeasuredValueField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MeasuredValueField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MeasuredValueField.stringdata0))
        return static_cast<void*>(this);
    return DVGui::LineEdit::qt_metacast(_clname);
}

int MeasuredValueField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::LineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void MeasuredValueField::measuredValueChanged(TMeasuredValue * _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_PegbarChannelField_t {
    QByteArrayData data[8];
    char stringdata0[83];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PegbarChannelField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PegbarChannelField_t qt_meta_stringdata_PegbarChannelField = {
    {
QT_MOC_LITERAL(0, 0, 18), // "PegbarChannelField"
QT_MOC_LITERAL(1, 19, 18), // "onScaleTypeChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 4), // "type"
QT_MOC_LITERAL(4, 44, 8), // "onChange"
QT_MOC_LITERAL(5, 53, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(6, 69, 3), // "fld"
QT_MOC_LITERAL(7, 73, 9) // "addToUndo"

    },
    "PegbarChannelField\0onScaleTypeChanged\0"
    "\0type\0onChange\0TMeasuredValue*\0fld\0"
    "addToUndo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PegbarChannelField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x0a /* Public */,
       4,    2,   32,    2, 0x09 /* Protected */,
       4,    1,   37,    2, 0x29 /* Protected | MethodCloned */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,    6,    7,
    QMetaType::Void, 0x80000000 | 5,    6,

       0        // eod
};

void PegbarChannelField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PegbarChannelField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onScaleTypeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PegbarChannelField::staticMetaObject = { {
    QMetaObject::SuperData::link<MeasuredValueField::staticMetaObject>(),
    qt_meta_stringdata_PegbarChannelField.data,
    qt_meta_data_PegbarChannelField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PegbarChannelField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PegbarChannelField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PegbarChannelField.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return MeasuredValueField::qt_metacast(_clname);
}

int PegbarChannelField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MeasuredValueField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_PegbarCenterField_t {
    QByteArrayData data[6];
    char stringdata0[58];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PegbarCenterField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PegbarCenterField_t qt_meta_stringdata_PegbarCenterField = {
    {
QT_MOC_LITERAL(0, 0, 17), // "PegbarCenterField"
QT_MOC_LITERAL(1, 18, 8), // "onChange"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(4, 44, 3), // "fld"
QT_MOC_LITERAL(5, 48, 9) // "addToUndo"

    },
    "PegbarCenterField\0onChange\0\0TMeasuredValue*\0"
    "fld\0addToUndo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PegbarCenterField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       1,    1,   29,    2, 0x29 /* Protected | MethodCloned */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void PegbarCenterField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PegbarCenterField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PegbarCenterField::staticMetaObject = { {
    QMetaObject::SuperData::link<MeasuredValueField::staticMetaObject>(),
    qt_meta_stringdata_PegbarCenterField.data,
    qt_meta_data_PegbarCenterField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PegbarCenterField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PegbarCenterField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PegbarCenterField.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return MeasuredValueField::qt_metacast(_clname);
}

int PegbarCenterField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MeasuredValueField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_NoScaleField_t {
    QByteArrayData data[6];
    char stringdata0[53];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NoScaleField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NoScaleField_t qt_meta_stringdata_NoScaleField = {
    {
QT_MOC_LITERAL(0, 0, 12), // "NoScaleField"
QT_MOC_LITERAL(1, 13, 8), // "onChange"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(4, 39, 3), // "fld"
QT_MOC_LITERAL(5, 43, 9) // "addToUndo"

    },
    "NoScaleField\0onChange\0\0TMeasuredValue*\0"
    "fld\0addToUndo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NoScaleField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       1,    1,   29,    2, 0x29 /* Protected | MethodCloned */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void NoScaleField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NoScaleField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject NoScaleField::staticMetaObject = { {
    QMetaObject::SuperData::link<MeasuredValueField::staticMetaObject>(),
    qt_meta_stringdata_NoScaleField.data,
    qt_meta_data_NoScaleField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *NoScaleField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NoScaleField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_NoScaleField.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return MeasuredValueField::qt_metacast(_clname);
}

int NoScaleField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MeasuredValueField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_PropertyMenuButton_t {
    QByteArrayData data[6];
    char stringdata0[70];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertyMenuButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertyMenuButton_t qt_meta_stringdata_PropertyMenuButton = {
    {
QT_MOC_LITERAL(0, 0, 18), // "PropertyMenuButton"
QT_MOC_LITERAL(1, 19, 17), // "onPropertyChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 4), // "name"
QT_MOC_LITERAL(4, 43, 17), // "onActionTriggered"
QT_MOC_LITERAL(5, 61, 8) // "QAction*"

    },
    "PropertyMenuButton\0onPropertyChanged\0"
    "\0name\0onActionTriggered\0QAction*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertyMenuButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   27,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5,    2,

       0        // eod
};

void PropertyMenuButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyMenuButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onPropertyChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->onActionTriggered((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PropertyMenuButton::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PropertyMenuButton::onPropertyChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertyMenuButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolButton::staticMetaObject>(),
    qt_meta_stringdata_PropertyMenuButton.data,
    qt_meta_data_PropertyMenuButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertyMenuButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertyMenuButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertyMenuButton.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ToolOptionControl"))
        return static_cast< ToolOptionControl*>(this);
    return QToolButton::qt_metacast(_clname);
}

int PropertyMenuButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void PropertyMenuButton::onPropertyChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SelectionScaleField_t {
    QByteArrayData data[7];
    char stringdata0[72];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SelectionScaleField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SelectionScaleField_t qt_meta_stringdata_SelectionScaleField = {
    {
QT_MOC_LITERAL(0, 0, 19), // "SelectionScaleField"
QT_MOC_LITERAL(1, 20, 11), // "valueChange"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 9), // "addToUndo"
QT_MOC_LITERAL(4, 43, 8), // "onChange"
QT_MOC_LITERAL(5, 52, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(6, 68, 3) // "fld"

    },
    "SelectionScaleField\0valueChange\0\0"
    "addToUndo\0onChange\0TMeasuredValue*\0"
    "fld"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SelectionScaleField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    2,   32,    2, 0x09 /* Protected */,
       4,    1,   37,    2, 0x29 /* Protected | MethodCloned */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,    6,    3,
    QMetaType::Void, 0x80000000 | 5,    6,

       0        // eod
};

void SelectionScaleField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SelectionScaleField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChange((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SelectionScaleField::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SelectionScaleField::valueChange)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SelectionScaleField::staticMetaObject = { {
    QMetaObject::SuperData::link<MeasuredValueField::staticMetaObject>(),
    qt_meta_stringdata_SelectionScaleField.data,
    qt_meta_data_SelectionScaleField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SelectionScaleField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SelectionScaleField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SelectionScaleField.stringdata0))
        return static_cast<void*>(this);
    return MeasuredValueField::qt_metacast(_clname);
}

int SelectionScaleField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MeasuredValueField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void SelectionScaleField::valueChange(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SelectionRotationField_t {
    QByteArrayData data[6];
    char stringdata0[63];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SelectionRotationField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SelectionRotationField_t qt_meta_stringdata_SelectionRotationField = {
    {
QT_MOC_LITERAL(0, 0, 22), // "SelectionRotationField"
QT_MOC_LITERAL(1, 23, 8), // "onChange"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(4, 49, 3), // "fld"
QT_MOC_LITERAL(5, 53, 9) // "addToUndo"

    },
    "SelectionRotationField\0onChange\0\0"
    "TMeasuredValue*\0fld\0addToUndo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SelectionRotationField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       1,    1,   29,    2, 0x29 /* Protected | MethodCloned */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void SelectionRotationField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SelectionRotationField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SelectionRotationField::staticMetaObject = { {
    QMetaObject::SuperData::link<MeasuredValueField::staticMetaObject>(),
    qt_meta_stringdata_SelectionRotationField.data,
    qt_meta_data_SelectionRotationField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SelectionRotationField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SelectionRotationField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SelectionRotationField.stringdata0))
        return static_cast<void*>(this);
    return MeasuredValueField::qt_metacast(_clname);
}

int SelectionRotationField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MeasuredValueField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_SelectionMoveField_t {
    QByteArrayData data[6];
    char stringdata0[59];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SelectionMoveField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SelectionMoveField_t qt_meta_stringdata_SelectionMoveField = {
    {
QT_MOC_LITERAL(0, 0, 18), // "SelectionMoveField"
QT_MOC_LITERAL(1, 19, 8), // "onChange"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(4, 45, 3), // "fld"
QT_MOC_LITERAL(5, 49, 9) // "addToUndo"

    },
    "SelectionMoveField\0onChange\0\0"
    "TMeasuredValue*\0fld\0addToUndo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SelectionMoveField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       1,    1,   29,    2, 0x29 /* Protected | MethodCloned */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void SelectionMoveField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SelectionMoveField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SelectionMoveField::staticMetaObject = { {
    QMetaObject::SuperData::link<MeasuredValueField::staticMetaObject>(),
    qt_meta_stringdata_SelectionMoveField.data,
    qt_meta_data_SelectionMoveField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SelectionMoveField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SelectionMoveField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SelectionMoveField.stringdata0))
        return static_cast<void*>(this);
    return MeasuredValueField::qt_metacast(_clname);
}

int SelectionMoveField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MeasuredValueField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ThickChangeField_t {
    QByteArrayData data[6];
    char stringdata0[57];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ThickChangeField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ThickChangeField_t qt_meta_stringdata_ThickChangeField = {
    {
QT_MOC_LITERAL(0, 0, 16), // "ThickChangeField"
QT_MOC_LITERAL(1, 17, 8), // "onChange"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 15), // "TMeasuredValue*"
QT_MOC_LITERAL(4, 43, 3), // "fld"
QT_MOC_LITERAL(5, 47, 9) // "addToUndo"

    },
    "ThickChangeField\0onChange\0\0TMeasuredValue*\0"
    "fld\0addToUndo"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ThickChangeField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x09 /* Protected */,
       1,    1,   29,    2, 0x29 /* Protected | MethodCloned */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void ThickChangeField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ThickChangeField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onChange((*reinterpret_cast< TMeasuredValue*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ThickChangeField::staticMetaObject = { {
    QMetaObject::SuperData::link<MeasuredValueField::staticMetaObject>(),
    qt_meta_stringdata_ThickChangeField.data,
    qt_meta_data_ThickChangeField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ThickChangeField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ThickChangeField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ThickChangeField.stringdata0))
        return static_cast<void*>(this);
    return MeasuredValueField::qt_metacast(_clname);
}

int ThickChangeField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MeasuredValueField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ClickableLabel_t {
    QByteArrayData data[7];
    char stringdata0[75];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ClickableLabel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ClickableLabel_t qt_meta_stringdata_ClickableLabel = {
    {
QT_MOC_LITERAL(0, 0, 14), // "ClickableLabel"
QT_MOC_LITERAL(1, 15, 12), // "onMousePress"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 12), // "QMouseEvent*"
QT_MOC_LITERAL(4, 42, 5), // "event"
QT_MOC_LITERAL(5, 48, 11), // "onMouseMove"
QT_MOC_LITERAL(6, 60, 14) // "onMouseRelease"

    },
    "ClickableLabel\0onMousePress\0\0QMouseEvent*\0"
    "event\0onMouseMove\0onMouseRelease"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ClickableLabel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       5,    1,   32,    2, 0x06 /* Public */,
       6,    1,   35,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,

       0        // eod
};

void ClickableLabel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ClickableLabel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onMousePress((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 1: _t->onMouseMove((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 2: _t->onMouseRelease((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ClickableLabel::*)(QMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ClickableLabel::onMousePress)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ClickableLabel::*)(QMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ClickableLabel::onMouseMove)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ClickableLabel::*)(QMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ClickableLabel::onMouseRelease)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ClickableLabel::staticMetaObject = { {
    QMetaObject::SuperData::link<QLabel::staticMetaObject>(),
    qt_meta_stringdata_ClickableLabel.data,
    qt_meta_data_ClickableLabel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ClickableLabel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ClickableLabel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ClickableLabel.stringdata0))
        return static_cast<void*>(this);
    return QLabel::qt_metacast(_clname);
}

int ClickableLabel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLabel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void ClickableLabel::onMousePress(QMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ClickableLabel::onMouseMove(QMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ClickableLabel::onMouseRelease(QMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
