/****************************************************************************
** Meta object code from reading C++ file 'plastictool.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/tnztools/plastictool.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plastictool.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PlasticTool_t {
    QByteArrayData data[25];
    char stringdata0[419];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PlasticTool_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PlasticTool_t qt_meta_stringdata_PlasticTool = {
    {
QT_MOC_LITERAL(0, 0, 11), // "PlasticTool"
QT_MOC_LITERAL(1, 12, 18), // "skelIdsListChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 13), // "skelIdChanged"
QT_MOC_LITERAL(4, 46, 18), // "swapEdge_mesh_undo"
QT_MOC_LITERAL(5, 65, 22), // "collapseEdge_mesh_undo"
QT_MOC_LITERAL(6, 88, 19), // "splitEdge_mesh_undo"
QT_MOC_LITERAL(7, 108, 18), // "cutEdges_mesh_undo"
QT_MOC_LITERAL(8, 127, 25), // "deleteSelectedVertex_undo"
QT_MOC_LITERAL(9, 153, 11), // "setKey_undo"
QT_MOC_LITERAL(10, 165, 17), // "setGlobalKey_undo"
QT_MOC_LITERAL(11, 183, 15), // "setRestKey_undo"
QT_MOC_LITERAL(12, 199, 21), // "setGlobalRestKey_undo"
QT_MOC_LITERAL(13, 221, 12), // "copySkeleton"
QT_MOC_LITERAL(14, 234, 18), // "pasteSkeleton_undo"
QT_MOC_LITERAL(15, 253, 15), // "copyDeformation"
QT_MOC_LITERAL(16, 269, 21), // "pasteDeformation_undo"
QT_MOC_LITERAL(17, 291, 15), // "onFrameSwitched"
QT_MOC_LITERAL(18, 307, 16), // "onColumnSwitched"
QT_MOC_LITERAL(19, 324, 15), // "onXsheetChanged"
QT_MOC_LITERAL(20, 340, 17), // "onShowMeshToggled"
QT_MOC_LITERAL(21, 358, 2), // "on"
QT_MOC_LITERAL(22, 361, 15), // "onShowSOToggled"
QT_MOC_LITERAL(23, 377, 21), // "onShowRigidityToggled"
QT_MOC_LITERAL(24, 399, 19) // "onShowSkelOSToggled"

    },
    "PlasticTool\0skelIdsListChanged\0\0"
    "skelIdChanged\0swapEdge_mesh_undo\0"
    "collapseEdge_mesh_undo\0splitEdge_mesh_undo\0"
    "cutEdges_mesh_undo\0deleteSelectedVertex_undo\0"
    "setKey_undo\0setGlobalKey_undo\0"
    "setRestKey_undo\0setGlobalRestKey_undo\0"
    "copySkeleton\0pasteSkeleton_undo\0"
    "copyDeformation\0pasteDeformation_undo\0"
    "onFrameSwitched\0onColumnSwitched\0"
    "onXsheetChanged\0onShowMeshToggled\0on\0"
    "onShowSOToggled\0onShowRigidityToggled\0"
    "onShowSkelOSToggled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PlasticTool[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      22,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  124,    2, 0x06 /* Public */,
       3,    0,  125,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  126,    2, 0x0a /* Public */,
       5,    0,  127,    2, 0x0a /* Public */,
       6,    0,  128,    2, 0x0a /* Public */,
       7,    0,  129,    2, 0x0a /* Public */,
       8,    0,  130,    2, 0x0a /* Public */,
       9,    0,  131,    2, 0x0a /* Public */,
      10,    0,  132,    2, 0x0a /* Public */,
      11,    0,  133,    2, 0x0a /* Public */,
      12,    0,  134,    2, 0x0a /* Public */,
      13,    0,  135,    2, 0x0a /* Public */,
      14,    0,  136,    2, 0x0a /* Public */,
      15,    0,  137,    2, 0x0a /* Public */,
      16,    0,  138,    2, 0x0a /* Public */,
      17,    0,  139,    2, 0x08 /* Private */,
      18,    0,  140,    2, 0x08 /* Private */,
      19,    0,  141,    2, 0x08 /* Private */,
      20,    1,  142,    2, 0x08 /* Private */,
      22,    1,  145,    2, 0x08 /* Private */,
      23,    1,  148,    2, 0x08 /* Private */,
      24,    1,  151,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, QMetaType::Bool,   21,

       0        // eod
};

void PlasticTool::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlasticTool *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->skelIdsListChanged(); break;
        case 1: _t->skelIdChanged(); break;
        case 2: _t->swapEdge_mesh_undo(); break;
        case 3: _t->collapseEdge_mesh_undo(); break;
        case 4: _t->splitEdge_mesh_undo(); break;
        case 5: _t->cutEdges_mesh_undo(); break;
        case 6: _t->deleteSelectedVertex_undo(); break;
        case 7: _t->setKey_undo(); break;
        case 8: _t->setGlobalKey_undo(); break;
        case 9: _t->setRestKey_undo(); break;
        case 10: _t->setGlobalRestKey_undo(); break;
        case 11: _t->copySkeleton(); break;
        case 12: _t->pasteSkeleton_undo(); break;
        case 13: _t->copyDeformation(); break;
        case 14: _t->pasteDeformation_undo(); break;
        case 15: _t->onFrameSwitched(); break;
        case 16: _t->onColumnSwitched(); break;
        case 17: _t->onXsheetChanged(); break;
        case 18: _t->onShowMeshToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->onShowSOToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->onShowRigidityToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->onShowSkelOSToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlasticTool::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlasticTool::skelIdsListChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PlasticTool::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlasticTool::skelIdChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PlasticTool::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_PlasticTool.data,
    qt_meta_data_PlasticTool,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PlasticTool::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlasticTool::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PlasticTool.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TTool"))
        return static_cast< TTool*>(this);
    if (!strcmp(_clname, "TParamObserver"))
        return static_cast< TParamObserver*>(this);
    if (!strcmp(_clname, "TSelection::View"))
        return static_cast< TSelection::View*>(this);
    return QObject::qt_metacast(_clname);
}

int PlasticTool::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void PlasticTool::skelIdsListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PlasticTool::skelIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_PlasticToolOptionsBox_t {
    QByteArrayData data[7];
    char stringdata0[106];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PlasticToolOptionsBox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PlasticToolOptionsBox_t qt_meta_stringdata_PlasticToolOptionsBox = {
    {
QT_MOC_LITERAL(0, 0, 21), // "PlasticToolOptionsBox"
QT_MOC_LITERAL(1, 22, 20), // "onSkelIdsListChanged"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 15), // "onSkelIdChanged"
QT_MOC_LITERAL(4, 60, 14), // "onSkelIdEdited"
QT_MOC_LITERAL(5, 75, 13), // "onAddSkeleton"
QT_MOC_LITERAL(6, 89, 16) // "onRemoveSkeleton"

    },
    "PlasticToolOptionsBox\0onSkelIdsListChanged\0"
    "\0onSkelIdChanged\0onSkelIdEdited\0"
    "onAddSkeleton\0onRemoveSkeleton"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PlasticToolOptionsBox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x08 /* Private */,
       3,    0,   40,    2, 0x08 /* Private */,
       4,    0,   41,    2, 0x08 /* Private */,
       5,    0,   42,    2, 0x08 /* Private */,
       6,    0,   43,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PlasticToolOptionsBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlasticToolOptionsBox *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSkelIdsListChanged(); break;
        case 1: _t->onSkelIdChanged(); break;
        case 2: _t->onSkelIdEdited(); break;
        case 3: _t->onAddSkeleton(); break;
        case 4: _t->onRemoveSkeleton(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PlasticToolOptionsBox::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericToolOptionsBox::staticMetaObject>(),
    qt_meta_stringdata_PlasticToolOptionsBox.data,
    qt_meta_data_PlasticToolOptionsBox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PlasticToolOptionsBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlasticToolOptionsBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PlasticToolOptionsBox.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TProperty::Listener"))
        return static_cast< TProperty::Listener*>(this);
    return GenericToolOptionsBox::qt_metacast(_clname);
}

int PlasticToolOptionsBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericToolOptionsBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
