/****************************************************************************
** Meta object code from reading C++ file 'viewerpane.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/viewerpane.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'viewerpane.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BaseViewerPanel_t {
    QByteArrayData data[26];
    char stringdata0[423];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BaseViewerPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BaseViewerPanel_t qt_meta_stringdata_BaseViewerPanel = {
    {
QT_MOC_LITERAL(0, 0, 15), // "BaseViewerPanel"
QT_MOC_LITERAL(1, 16, 17), // "changeWindowTitle"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 16), // "updateFrameRange"
QT_MOC_LITERAL(4, 52, 14), // "onSceneChanged"
QT_MOC_LITERAL(5, 67, 18), // "onXshLevelSwitched"
QT_MOC_LITERAL(6, 86, 10), // "TXshLevel*"
QT_MOC_LITERAL(7, 97, 18), // "updateFrameMarkers"
QT_MOC_LITERAL(8, 116, 15), // "onButtonPressed"
QT_MOC_LITERAL(9, 132, 20), // "FlipConsole::EGadget"
QT_MOC_LITERAL(10, 153, 6), // "button"
QT_MOC_LITERAL(11, 160, 21), // "setFlipHButtonChecked"
QT_MOC_LITERAL(12, 182, 7), // "checked"
QT_MOC_LITERAL(13, 190, 21), // "setFlipVButtonChecked"
QT_MOC_LITERAL(14, 212, 17), // "enableFullPreview"
QT_MOC_LITERAL(15, 230, 7), // "enabled"
QT_MOC_LITERAL(16, 238, 22), // "enableSubCameraPreview"
QT_MOC_LITERAL(17, 261, 15), // "onFrameSwitched"
QT_MOC_LITERAL(18, 277, 15), // "onSceneSwitched"
QT_MOC_LITERAL(19, 293, 18), // "onFrameTypeChanged"
QT_MOC_LITERAL(20, 312, 22), // "onPlayingStatusChanged"
QT_MOC_LITERAL(21, 335, 7), // "playing"
QT_MOC_LITERAL(22, 343, 25), // "onShowHideActionTriggered"
QT_MOC_LITERAL(23, 369, 8), // "QAction*"
QT_MOC_LITERAL(24, 378, 22), // "onPreviewStatusChanged"
QT_MOC_LITERAL(25, 401, 21) // "onActiveViewerChanged"

    },
    "BaseViewerPanel\0changeWindowTitle\0\0"
    "updateFrameRange\0onSceneChanged\0"
    "onXshLevelSwitched\0TXshLevel*\0"
    "updateFrameMarkers\0onButtonPressed\0"
    "FlipConsole::EGadget\0button\0"
    "setFlipHButtonChecked\0checked\0"
    "setFlipVButtonChecked\0enableFullPreview\0"
    "enabled\0enableSubCameraPreview\0"
    "onFrameSwitched\0onSceneSwitched\0"
    "onFrameTypeChanged\0onPlayingStatusChanged\0"
    "playing\0onShowHideActionTriggered\0"
    "QAction*\0onPreviewStatusChanged\0"
    "onActiveViewerChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BaseViewerPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x0a /* Public */,
       3,    0,  100,    2, 0x0a /* Public */,
       4,    0,  101,    2, 0x0a /* Public */,
       5,    1,  102,    2, 0x0a /* Public */,
       7,    0,  105,    2, 0x0a /* Public */,
       8,    1,  106,    2, 0x0a /* Public */,
      11,    1,  109,    2, 0x0a /* Public */,
      13,    1,  112,    2, 0x0a /* Public */,
      14,    1,  115,    2, 0x0a /* Public */,
      16,    1,  118,    2, 0x0a /* Public */,
      17,    0,  121,    2, 0x09 /* Protected */,
      18,    0,  122,    2, 0x09 /* Protected */,
      19,    0,  123,    2, 0x09 /* Protected */,
      20,    1,  124,    2, 0x09 /* Protected */,
      22,    1,  127,    2, 0x09 /* Protected */,
      24,    0,  130,    2, 0x09 /* Protected */,
      25,    0,  131,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   21,
    QMetaType::Void, 0x80000000 | 23,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void BaseViewerPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BaseViewerPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changeWindowTitle(); break;
        case 1: _t->updateFrameRange(); break;
        case 2: _t->onSceneChanged(); break;
        case 3: _t->onXshLevelSwitched((*reinterpret_cast< TXshLevel*(*)>(_a[1]))); break;
        case 4: _t->updateFrameMarkers(); break;
        case 5: _t->onButtonPressed((*reinterpret_cast< FlipConsole::EGadget(*)>(_a[1]))); break;
        case 6: _t->setFlipHButtonChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->setFlipVButtonChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->enableFullPreview((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->enableSubCameraPreview((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->onFrameSwitched(); break;
        case 11: _t->onSceneSwitched(); break;
        case 12: _t->onFrameTypeChanged(); break;
        case 13: _t->onPlayingStatusChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->onShowHideActionTriggered((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 15: _t->onPreviewStatusChanged(); break;
        case 16: _t->onActiveViewerChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< TXshLevel* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BaseViewerPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_BaseViewerPanel.data,
    qt_meta_data_BaseViewerPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BaseViewerPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BaseViewerPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BaseViewerPanel.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FlipConsoleOwner"))
        return static_cast< FlipConsoleOwner*>(this);
    if (!strcmp(_clname, "SaveLoadQSettings"))
        return static_cast< SaveLoadQSettings*>(this);
    return QFrame::qt_metacast(_clname);
}

int BaseViewerPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}
struct qt_meta_stringdata_SceneViewerPanel_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SceneViewerPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SceneViewerPanel_t qt_meta_stringdata_SceneViewerPanel = {
    {
QT_MOC_LITERAL(0, 0, 16) // "SceneViewerPanel"

    },
    "SceneViewerPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SceneViewerPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SceneViewerPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SceneViewerPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseViewerPanel::staticMetaObject>(),
    qt_meta_stringdata_SceneViewerPanel.data,
    qt_meta_data_SceneViewerPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SceneViewerPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SceneViewerPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SceneViewerPanel.stringdata0))
        return static_cast<void*>(this);
    return BaseViewerPanel::qt_metacast(_clname);
}

int SceneViewerPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseViewerPanel::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
