/****************************************************************************
** Meta object code from reading C++ file 'startuppopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/startuppopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'startuppopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StartupPopup_t {
    QByteArrayData data[24];
    char stringdata0[403];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StartupPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StartupPopup_t qt_meta_stringdata_StartupPopup = {
    {
QT_MOC_LITERAL(0, 0, 12), // "StartupPopup"
QT_MOC_LITERAL(1, 13, 20), // "onRecentSceneClicked"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 5), // "index"
QT_MOC_LITERAL(4, 41, 22), // "onExistingSceneClicked"
QT_MOC_LITERAL(5, 64, 14), // "onCreateButton"
QT_MOC_LITERAL(6, 79, 20), // "onShowAtStartChanged"
QT_MOC_LITERAL(7, 100, 15), // "updateProjectCB"
QT_MOC_LITERAL(8, 116, 16), // "onProjectChanged"
QT_MOC_LITERAL(9, 133, 25), // "onNewProjectButtonPressed"
QT_MOC_LITERAL(10, 159, 26), // "onOpenProjectButtonPressed"
QT_MOC_LITERAL(11, 186, 29), // "onExploreProjectButtonPressed"
QT_MOC_LITERAL(12, 216, 24), // "onLoadSceneButtonPressed"
QT_MOC_LITERAL(13, 241, 14), // "onSceneChanged"
QT_MOC_LITERAL(14, 256, 16), // "updateResolution"
QT_MOC_LITERAL(15, 273, 10), // "updateSize"
QT_MOC_LITERAL(16, 284, 12), // "onDpiChanged"
QT_MOC_LITERAL(17, 297, 9), // "addPreset"
QT_MOC_LITERAL(18, 307, 12), // "removePreset"
QT_MOC_LITERAL(19, 320, 16), // "onPresetSelected"
QT_MOC_LITERAL(20, 337, 3), // "str"
QT_MOC_LITERAL(21, 341, 19), // "onCameraUnitChanged"
QT_MOC_LITERAL(22, 361, 19), // "onAutoSaveOnChanged"
QT_MOC_LITERAL(23, 381, 21) // "onAutoSaveTimeChanged"

    },
    "StartupPopup\0onRecentSceneClicked\0\0"
    "index\0onExistingSceneClicked\0"
    "onCreateButton\0onShowAtStartChanged\0"
    "updateProjectCB\0onProjectChanged\0"
    "onNewProjectButtonPressed\0"
    "onOpenProjectButtonPressed\0"
    "onExploreProjectButtonPressed\0"
    "onLoadSceneButtonPressed\0onSceneChanged\0"
    "updateResolution\0updateSize\0onDpiChanged\0"
    "addPreset\0removePreset\0onPresetSelected\0"
    "str\0onCameraUnitChanged\0onAutoSaveOnChanged\0"
    "onAutoSaveTimeChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StartupPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  114,    2, 0x0a /* Public */,
       4,    1,  117,    2, 0x0a /* Public */,
       5,    0,  120,    2, 0x0a /* Public */,
       6,    1,  121,    2, 0x0a /* Public */,
       7,    0,  124,    2, 0x0a /* Public */,
       8,    1,  125,    2, 0x0a /* Public */,
       9,    0,  128,    2, 0x0a /* Public */,
      10,    0,  129,    2, 0x0a /* Public */,
      11,    0,  130,    2, 0x0a /* Public */,
      12,    0,  131,    2, 0x0a /* Public */,
      13,    0,  132,    2, 0x0a /* Public */,
      14,    0,  133,    2, 0x0a /* Public */,
      15,    0,  134,    2, 0x0a /* Public */,
      16,    0,  135,    2, 0x0a /* Public */,
      17,    0,  136,    2, 0x0a /* Public */,
      18,    0,  137,    2, 0x0a /* Public */,
      19,    1,  138,    2, 0x0a /* Public */,
      21,    1,  141,    2, 0x0a /* Public */,
      22,    1,  144,    2, 0x0a /* Public */,
      23,    0,  147,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,

       0        // eod
};

void StartupPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StartupPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onRecentSceneClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onExistingSceneClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onCreateButton(); break;
        case 3: _t->onShowAtStartChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->updateProjectCB(); break;
        case 5: _t->onProjectChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->onNewProjectButtonPressed(); break;
        case 7: _t->onOpenProjectButtonPressed(); break;
        case 8: _t->onExploreProjectButtonPressed(); break;
        case 9: _t->onLoadSceneButtonPressed(); break;
        case 10: _t->onSceneChanged(); break;
        case 11: _t->updateResolution(); break;
        case 12: _t->updateSize(); break;
        case 13: _t->onDpiChanged(); break;
        case 14: _t->addPreset(); break;
        case 15: _t->removePreset(); break;
        case 16: _t->onPresetSelected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->onCameraUnitChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->onAutoSaveOnChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->onAutoSaveTimeChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StartupPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_StartupPopup.data,
    qt_meta_data_StartupPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StartupPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StartupPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StartupPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int StartupPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}
struct qt_meta_stringdata_StartupLabel_t {
    QByteArrayData data[4];
    char stringdata0[31];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StartupLabel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StartupLabel_t qt_meta_stringdata_StartupLabel = {
    {
QT_MOC_LITERAL(0, 0, 12), // "StartupLabel"
QT_MOC_LITERAL(1, 13, 10), // "wasClicked"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 5) // "index"

    },
    "StartupLabel\0wasClicked\0\0index"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StartupLabel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,

       0        // eod
};

void StartupLabel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StartupLabel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->wasClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StartupLabel::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StartupLabel::wasClicked)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StartupLabel::staticMetaObject = { {
    QMetaObject::SuperData::link<QLabel::staticMetaObject>(),
    qt_meta_stringdata_StartupLabel.data,
    qt_meta_data_StartupLabel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StartupLabel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StartupLabel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StartupLabel.stringdata0))
        return static_cast<void*>(this);
    return QLabel::qt_metacast(_clname);
}

int StartupLabel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLabel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void StartupLabel::wasClicked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StartupScenesList_t {
    QByteArrayData data[7];
    char stringdata0[73];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StartupScenesList_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StartupScenesList_t qt_meta_stringdata_StartupScenesList = {
    {
QT_MOC_LITERAL(0, 0, 17), // "StartupScenesList"
QT_MOC_LITERAL(1, 18, 11), // "itemClicked"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 5), // "index"
QT_MOC_LITERAL(4, 37, 13), // "onItemClicked"
QT_MOC_LITERAL(5, 51, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(6, 68, 4) // "item"

    },
    "StartupScenesList\0itemClicked\0\0index\0"
    "onItemClicked\0QListWidgetItem*\0item"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StartupScenesList[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   27,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5,    6,

       0        // eod
};

void StartupScenesList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StartupScenesList *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onItemClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StartupScenesList::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StartupScenesList::itemClicked)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StartupScenesList::staticMetaObject = { {
    QMetaObject::SuperData::link<QListWidget::staticMetaObject>(),
    qt_meta_stringdata_StartupScenesList.data,
    qt_meta_data_StartupScenesList,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StartupScenesList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StartupScenesList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StartupScenesList.stringdata0))
        return static_cast<void*>(this);
    return QListWidget::qt_metacast(_clname);
}

int StartupScenesList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StartupScenesList::itemClicked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
