/****************************************************************************
** Meta object code from reading C++ file 'boardsettingspopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/boardsettingspopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'boardsettingspopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BoardView_t {
    QByteArrayData data[1];
    char stringdata0[10];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BoardView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BoardView_t qt_meta_stringdata_BoardView = {
    {
QT_MOC_LITERAL(0, 0, 9) // "BoardView"

    },
    "BoardView"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BoardView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void BoardView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject BoardView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_BoardView.data,
    qt_meta_data_BoardView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BoardView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BoardView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BoardView.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int BoardView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ItemInfoView_t {
    QByteArrayData data[15];
    char stringdata0[253];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ItemInfoView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ItemInfoView_t qt_meta_stringdata_ItemInfoView = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ItemInfoView"
QT_MOC_LITERAL(1, 13, 19), // "itemPropertyChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 14), // "updateListView"
QT_MOC_LITERAL(4, 49, 12), // "onNameEdited"
QT_MOC_LITERAL(5, 62, 19), // "onMaxFontSizeEdited"
QT_MOC_LITERAL(6, 82, 20), // "onTypeComboActivated"
QT_MOC_LITERAL(7, 103, 17), // "onFreeTextChanged"
QT_MOC_LITERAL(8, 121, 16), // "onImgPathChanged"
QT_MOC_LITERAL(9, 138, 18), // "onFontComboChanged"
QT_MOC_LITERAL(10, 157, 19), // "onBoldButtonClicked"
QT_MOC_LITERAL(11, 177, 21), // "onItalicButtonClicked"
QT_MOC_LITERAL(12, 199, 18), // "onFontColorChanged"
QT_MOC_LITERAL(13, 218, 8), // "TPixel32"
QT_MOC_LITERAL(14, 227, 25) // "onImgARModeComboActivated"

    },
    "ItemInfoView\0itemPropertyChanged\0\0"
    "updateListView\0onNameEdited\0"
    "onMaxFontSizeEdited\0onTypeComboActivated\0"
    "onFreeTextChanged\0onImgPathChanged\0"
    "onFontComboChanged\0onBoldButtonClicked\0"
    "onItalicButtonClicked\0onFontColorChanged\0"
    "TPixel32\0onImgARModeComboActivated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ItemInfoView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   72,    2, 0x09 /* Protected */,
       5,    0,   73,    2, 0x09 /* Protected */,
       6,    1,   74,    2, 0x09 /* Protected */,
       7,    0,   77,    2, 0x09 /* Protected */,
       8,    0,   78,    2, 0x09 /* Protected */,
       9,    1,   79,    2, 0x09 /* Protected */,
      10,    1,   82,    2, 0x09 /* Protected */,
      11,    1,   85,    2, 0x09 /* Protected */,
      12,    2,   88,    2, 0x09 /* Protected */,
      14,    0,   93,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QFont,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Bool,    2,    2,
    QMetaType::Void,

       0        // eod
};

void ItemInfoView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ItemInfoView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemPropertyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onNameEdited(); break;
        case 2: _t->onMaxFontSizeEdited(); break;
        case 3: _t->onTypeComboActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->onFreeTextChanged(); break;
        case 5: _t->onImgPathChanged(); break;
        case 6: _t->onFontComboChanged((*reinterpret_cast< const QFont(*)>(_a[1]))); break;
        case 7: _t->onBoldButtonClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->onItalicButtonClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->onFontColorChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 10: _t->onImgARModeComboActivated(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ItemInfoView::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ItemInfoView::itemPropertyChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ItemInfoView::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_meta_stringdata_ItemInfoView.data,
    qt_meta_data_ItemInfoView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ItemInfoView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ItemInfoView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ItemInfoView.stringdata0))
        return static_cast<void*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int ItemInfoView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void ItemInfoView::itemPropertyChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_ItemListView_t {
    QByteArrayData data[9];
    char stringdata0[170];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ItemListView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ItemListView_t qt_meta_stringdata_ItemListView = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ItemListView"
QT_MOC_LITERAL(1, 13, 19), // "currentItemSwitched"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 18), // "itemAddedOrDeleted"
QT_MOC_LITERAL(4, 53, 21), // "onCurrentItemSwitched"
QT_MOC_LITERAL(5, 75, 22), // "onNewItemButtonClicked"
QT_MOC_LITERAL(6, 98, 25), // "onDeleteItemButtonClicked"
QT_MOC_LITERAL(7, 124, 21), // "onMoveUpButtonClicked"
QT_MOC_LITERAL(8, 146, 23) // "onMoveDownButtonClicked"

    },
    "ItemListView\0currentItemSwitched\0\0"
    "itemAddedOrDeleted\0onCurrentItemSwitched\0"
    "onNewItemButtonClicked\0onDeleteItemButtonClicked\0"
    "onMoveUpButtonClicked\0onMoveDownButtonClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ItemListView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       3,    0,   52,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   53,    2, 0x09 /* Protected */,
       5,    0,   56,    2, 0x09 /* Protected */,
       6,    0,   57,    2, 0x09 /* Protected */,
       7,    0,   58,    2, 0x09 /* Protected */,
       8,    0,   59,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ItemListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ItemListView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentItemSwitched((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->itemAddedOrDeleted(); break;
        case 2: _t->onCurrentItemSwitched((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onNewItemButtonClicked(); break;
        case 4: _t->onDeleteItemButtonClicked(); break;
        case 5: _t->onMoveUpButtonClicked(); break;
        case 6: _t->onMoveDownButtonClicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ItemListView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ItemListView::currentItemSwitched)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ItemListView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ItemListView::itemAddedOrDeleted)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ItemListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ItemListView.data,
    qt_meta_data_ItemListView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ItemListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ItemListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ItemListView.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ItemListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void ItemListView::currentItemSwitched(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ItemListView::itemAddedOrDeleted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_BoardSettingsPopup_t {
    QByteArrayData data[11];
    char stringdata0[185];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BoardSettingsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BoardSettingsPopup_t qt_meta_stringdata_BoardSettingsPopup = {
    {
QT_MOC_LITERAL(0, 0, 18), // "BoardSettingsPopup"
QT_MOC_LITERAL(1, 19, 21), // "onCurrentItemSwitched"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 20), // "onItemAddedOrDeleted"
QT_MOC_LITERAL(4, 63, 21), // "onItemPropertyChanged"
QT_MOC_LITERAL(5, 85, 14), // "updateListView"
QT_MOC_LITERAL(6, 100, 16), // "onDurationEdited"
QT_MOC_LITERAL(7, 117, 22), // "onFileNameSuffixEdited"
QT_MOC_LITERAL(8, 140, 12), // "onLoadPreset"
QT_MOC_LITERAL(9, 153, 12), // "onSavePreset"
QT_MOC_LITERAL(10, 166, 18) // "onExportBoardImage"

    },
    "BoardSettingsPopup\0onCurrentItemSwitched\0"
    "\0onItemAddedOrDeleted\0onItemPropertyChanged\0"
    "updateListView\0onDurationEdited\0"
    "onFileNameSuffixEdited\0onLoadPreset\0"
    "onSavePreset\0onExportBoardImage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BoardSettingsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x09 /* Protected */,
       3,    0,   57,    2, 0x09 /* Protected */,
       4,    1,   58,    2, 0x09 /* Protected */,
       6,    0,   61,    2, 0x09 /* Protected */,
       7,    0,   62,    2, 0x09 /* Protected */,
       8,    0,   63,    2, 0x09 /* Protected */,
       9,    0,   64,    2, 0x09 /* Protected */,
      10,    0,   65,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void BoardSettingsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BoardSettingsPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onCurrentItemSwitched((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->onItemAddedOrDeleted(); break;
        case 2: _t->onItemPropertyChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onDurationEdited(); break;
        case 4: _t->onFileNameSuffixEdited(); break;
        case 5: _t->onLoadPreset(); break;
        case 6: _t->onSavePreset(); break;
        case 7: _t->onExportBoardImage(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BoardSettingsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_BoardSettingsPopup.data,
    qt_meta_data_BoardSettingsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BoardSettingsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BoardSettingsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BoardSettingsPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int BoardSettingsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}
struct qt_meta_stringdata_SaveBoardPresetFilePopup_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SaveBoardPresetFilePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SaveBoardPresetFilePopup_t qt_meta_stringdata_SaveBoardPresetFilePopup = {
    {
QT_MOC_LITERAL(0, 0, 24) // "SaveBoardPresetFilePopup"

    },
    "SaveBoardPresetFilePopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SaveBoardPresetFilePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SaveBoardPresetFilePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SaveBoardPresetFilePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericSaveFilePopup::staticMetaObject>(),
    qt_meta_stringdata_SaveBoardPresetFilePopup.data,
    qt_meta_data_SaveBoardPresetFilePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SaveBoardPresetFilePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SaveBoardPresetFilePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SaveBoardPresetFilePopup.stringdata0))
        return static_cast<void*>(this);
    return GenericSaveFilePopup::qt_metacast(_clname);
}

int SaveBoardPresetFilePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericSaveFilePopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_LoadBoardPresetFilePopup_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadBoardPresetFilePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadBoardPresetFilePopup_t qt_meta_stringdata_LoadBoardPresetFilePopup = {
    {
QT_MOC_LITERAL(0, 0, 24) // "LoadBoardPresetFilePopup"

    },
    "LoadBoardPresetFilePopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadBoardPresetFilePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void LoadBoardPresetFilePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject LoadBoardPresetFilePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericLoadFilePopup::staticMetaObject>(),
    qt_meta_stringdata_LoadBoardPresetFilePopup.data,
    qt_meta_data_LoadBoardPresetFilePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadBoardPresetFilePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadBoardPresetFilePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadBoardPresetFilePopup.stringdata0))
        return static_cast<void*>(this);
    return GenericLoadFilePopup::qt_metacast(_clname);
}

int LoadBoardPresetFilePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericLoadFilePopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SaveBoardImagePopup_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SaveBoardImagePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SaveBoardImagePopup_t qt_meta_stringdata_SaveBoardImagePopup = {
    {
QT_MOC_LITERAL(0, 0, 19) // "SaveBoardImagePopup"

    },
    "SaveBoardImagePopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SaveBoardImagePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SaveBoardImagePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SaveBoardImagePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericSaveFilePopup::staticMetaObject>(),
    qt_meta_stringdata_SaveBoardImagePopup.data,
    qt_meta_data_SaveBoardImagePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SaveBoardImagePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SaveBoardImagePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SaveBoardImagePopup.stringdata0))
        return static_cast<void*>(this);
    return GenericSaveFilePopup::qt_metacast(_clname);
}

int SaveBoardImagePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericSaveFilePopup::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
