/****************************************************************************
** Meta object code from reading C++ file 'functionviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/functionviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'functionviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FunctionViewer_t {
    QByteArrayData data[32];
    char stringdata0[431];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionViewer_t qt_meta_stringdata_FunctionViewer = {
    {
QT_MOC_LITERAL(0, 0, 14), // "FunctionViewer"
QT_MOC_LITERAL(1, 15, 12), // "curveChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 7), // "curveIo"
QT_MOC_LITERAL(4, 37, 4), // "type"
QT_MOC_LITERAL(5, 42, 13), // "TDoubleParam*"
QT_MOC_LITERAL(6, 56, 5), // "curve"
QT_MOC_LITERAL(7, 62, 11), // "std::string"
QT_MOC_LITERAL(8, 74, 4), // "name"
QT_MOC_LITERAL(9, 79, 10), // "editObject"
QT_MOC_LITERAL(10, 90, 12), // "refreshModel"
QT_MOC_LITERAL(11, 103, 12), // "rebuildModel"
QT_MOC_LITERAL(12, 116, 15), // "onFrameSwitched"
QT_MOC_LITERAL(13, 132, 10), // "toggleMode"
QT_MOC_LITERAL(14, 143, 19), // "onValueFieldChanged"
QT_MOC_LITERAL(15, 163, 15), // "onXsheetChanged"
QT_MOC_LITERAL(16, 179, 21), // "onStageObjectSwitched"
QT_MOC_LITERAL(17, 201, 20), // "onStageObjectChanged"
QT_MOC_LITERAL(18, 222, 10), // "isDragging"
QT_MOC_LITERAL(19, 233, 12), // "onFxSwitched"
QT_MOC_LITERAL(20, 246, 14), // "onCurveChanged"
QT_MOC_LITERAL(21, 261, 15), // "onCurveSelected"
QT_MOC_LITERAL(22, 277, 18), // "onSelectionChanged"
QT_MOC_LITERAL(23, 296, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(24, 316, 21), // "doSwitchCurrentObject"
QT_MOC_LITERAL(25, 338, 13), // "TStageObject*"
QT_MOC_LITERAL(26, 352, 3), // "obj"
QT_MOC_LITERAL(27, 356, 17), // "doSwitchCurrentFx"
QT_MOC_LITERAL(28, 374, 4), // "TFx*"
QT_MOC_LITERAL(29, 379, 2), // "fx"
QT_MOC_LITERAL(30, 382, 22), // "isExpressionPageActive"
QT_MOC_LITERAL(31, 405, 25) // "propagateExternalSetFrame"

    },
    "FunctionViewer\0curveChanged\0\0curveIo\0"
    "type\0TDoubleParam*\0curve\0std::string\0"
    "name\0editObject\0refreshModel\0rebuildModel\0"
    "onFrameSwitched\0toggleMode\0"
    "onValueFieldChanged\0onXsheetChanged\0"
    "onStageObjectSwitched\0onStageObjectChanged\0"
    "isDragging\0onFxSwitched\0onCurveChanged\0"
    "onCurveSelected\0onSelectionChanged\0"
    "onPreferenceChanged\0doSwitchCurrentObject\0"
    "TStageObject*\0obj\0doSwitchCurrentFx\0"
    "TFx*\0fx\0isExpressionPageActive\0"
    "propagateExternalSetFrame"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x06 /* Public */,
       3,    3,  120,    2, 0x06 /* Public */,
       9,    0,  127,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,  128,    2, 0x0a /* Public */,
      11,    0,  129,    2, 0x0a /* Public */,
      12,    0,  130,    2, 0x0a /* Public */,
      13,    0,  131,    2, 0x0a /* Public */,
      14,    0,  132,    2, 0x0a /* Public */,
      15,    0,  133,    2, 0x0a /* Public */,
      16,    0,  134,    2, 0x0a /* Public */,
      17,    1,  135,    2, 0x0a /* Public */,
      17,    0,  138,    2, 0x2a /* Public | MethodCloned */,
      19,    0,  139,    2, 0x0a /* Public */,
      20,    1,  140,    2, 0x0a /* Public */,
      21,    1,  143,    2, 0x0a /* Public */,
      22,    0,  146,    2, 0x0a /* Public */,
      23,    1,  147,    2, 0x0a /* Public */,
      24,    1,  150,    2, 0x0a /* Public */,
      27,    1,  153,    2, 0x0a /* Public */,
      30,    0,  156,    2, 0x0a /* Public */,
      31,    0,  157,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 5, 0x80000000 | 7,    4,    6,    8,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 25,   26,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Bool,
    QMetaType::Void,

       0        // eod
};

void FunctionViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->curveChanged(); break;
        case 1: _t->curveIo((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< TDoubleParam*(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 2: _t->editObject(); break;
        case 3: _t->refreshModel(); break;
        case 4: _t->rebuildModel(); break;
        case 5: _t->onFrameSwitched(); break;
        case 6: _t->toggleMode(); break;
        case 7: _t->onValueFieldChanged(); break;
        case 8: _t->onXsheetChanged(); break;
        case 9: _t->onStageObjectSwitched(); break;
        case 10: _t->onStageObjectChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->onStageObjectChanged(); break;
        case 12: _t->onFxSwitched(); break;
        case 13: _t->onCurveChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->onCurveSelected((*reinterpret_cast< TDoubleParam*(*)>(_a[1]))); break;
        case 15: _t->onSelectionChanged(); break;
        case 16: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->doSwitchCurrentObject((*reinterpret_cast< TStageObject*(*)>(_a[1]))); break;
        case 18: _t->doSwitchCurrentFx((*reinterpret_cast< TFx*(*)>(_a[1]))); break;
        case 19: { bool _r = _t->isExpressionPageActive();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->propagateExternalSetFrame(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FunctionViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionViewer::curveChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FunctionViewer::*)(int , TDoubleParam * , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionViewer::curveIo)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FunctionViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionViewer::editObject)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_meta_stringdata_FunctionViewer.data,
    qt_meta_data_FunctionViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SaveLoadQSettings"))
        return static_cast< SaveLoadQSettings*>(this);
    return QSplitter::qt_metacast(_clname);
}

int FunctionViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void FunctionViewer::curveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FunctionViewer::curveIo(int _t1, TDoubleParam * _t2, const std::string & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FunctionViewer::editObject()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
