/****************************************************************************
** Meta object code from reading C++ file 'brushpresetpanel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/brushpresetpanel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'brushpresetpanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BrushPresetItem_t {
    QByteArrayData data[8];
    char stringdata0[88];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BrushPresetItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BrushPresetItem_t qt_meta_stringdata_BrushPresetItem = {
    {
QT_MOC_LITERAL(0, 0, 15), // "BrushPresetItem"
QT_MOC_LITERAL(1, 16, 14), // "presetSelected"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 10), // "presetName"
QT_MOC_LITERAL(4, 43, 8), // "toolType"
QT_MOC_LITERAL(5, 52, 15), // "presetReordered"
QT_MOC_LITERAL(6, 68, 10), // "fromPreset"
QT_MOC_LITERAL(7, 79, 8) // "toPreset"

    },
    "BrushPresetItem\0presetSelected\0\0"
    "presetName\0toolType\0presetReordered\0"
    "fromPreset\0toPreset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BrushPresetItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   24,    2, 0x06 /* Public */,
       5,    2,   29,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    6,    7,

       0        // eod
};

void BrushPresetItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BrushPresetItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->presetSelected((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->presetReordered((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BrushPresetItem::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BrushPresetItem::presetSelected)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (BrushPresetItem::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BrushPresetItem::presetReordered)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BrushPresetItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolButton::staticMetaObject>(),
    qt_meta_stringdata_BrushPresetItem.data,
    qt_meta_data_BrushPresetItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BrushPresetItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BrushPresetItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BrushPresetItem.stringdata0))
        return static_cast<void*>(this);
    return QToolButton::qt_metacast(_clname);
}

int BrushPresetItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void BrushPresetItem::presetSelected(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void BrushPresetItem::presetReordered(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_BrushPresetPanel_t {
    QByteArrayData data[17];
    char stringdata0[244];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BrushPresetPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BrushPresetPanel_t qt_meta_stringdata_BrushPresetPanel = {
    {
QT_MOC_LITERAL(0, 0, 16), // "BrushPresetPanel"
QT_MOC_LITERAL(1, 17, 14), // "onToolSwitched"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 13), // "onToolChanged"
QT_MOC_LITERAL(4, 47, 19), // "onPresetItemClicked"
QT_MOC_LITERAL(5, 67, 10), // "presetName"
QT_MOC_LITERAL(6, 78, 8), // "toolType"
QT_MOC_LITERAL(7, 87, 18), // "onAddPresetClicked"
QT_MOC_LITERAL(8, 106, 21), // "onRemovePresetClicked"
QT_MOC_LITERAL(9, 128, 16), // "onRefreshClicked"
QT_MOC_LITERAL(10, 145, 25), // "onToolComboBoxListChanged"
QT_MOC_LITERAL(11, 171, 11), // "std::string"
QT_MOC_LITERAL(12, 183, 2), // "id"
QT_MOC_LITERAL(13, 186, 17), // "onViewModeChanged"
QT_MOC_LITERAL(14, 204, 8), // "ViewMode"
QT_MOC_LITERAL(15, 213, 4), // "mode"
QT_MOC_LITERAL(16, 218, 25) // "onShowHideActionTriggered"

    },
    "BrushPresetPanel\0onToolSwitched\0\0"
    "onToolChanged\0onPresetItemClicked\0"
    "presetName\0toolType\0onAddPresetClicked\0"
    "onRemovePresetClicked\0onRefreshClicked\0"
    "onToolComboBoxListChanged\0std::string\0"
    "id\0onViewModeChanged\0ViewMode\0mode\0"
    "onShowHideActionTriggered"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BrushPresetPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x08 /* Private */,
       3,    0,   60,    2, 0x08 /* Private */,
       4,    2,   61,    2, 0x08 /* Private */,
       7,    0,   66,    2, 0x08 /* Private */,
       8,    0,   67,    2, 0x08 /* Private */,
       9,    0,   68,    2, 0x08 /* Private */,
      10,    1,   69,    2, 0x08 /* Private */,
      13,    1,   72,    2, 0x08 /* Private */,
      16,    0,   75,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,

       0        // eod
};

void BrushPresetPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BrushPresetPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onToolSwitched(); break;
        case 1: _t->onToolChanged(); break;
        case 2: _t->onPresetItemClicked((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->onAddPresetClicked(); break;
        case 4: _t->onRemovePresetClicked(); break;
        case 5: _t->onRefreshClicked(); break;
        case 6: _t->onToolComboBoxListChanged((*reinterpret_cast< std::string(*)>(_a[1]))); break;
        case 7: _t->onViewModeChanged((*reinterpret_cast< ViewMode(*)>(_a[1]))); break;
        case 8: _t->onShowHideActionTriggered(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BrushPresetPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_BrushPresetPanel.data,
    qt_meta_data_BrushPresetPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BrushPresetPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BrushPresetPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BrushPresetPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int BrushPresetPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
