/****************************************************************************
** Meta object code from reading C++ file 'flipconsole.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/flipconsole.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'flipconsole.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PlaybackExecutor_t {
    QByteArrayData data[7];
    char stringdata0[67];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PlaybackExecutor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PlaybackExecutor_t qt_meta_stringdata_PlaybackExecutor = {
    {
QT_MOC_LITERAL(0, 0, 16), // "PlaybackExecutor"
QT_MOC_LITERAL(1, 17, 9), // "nextFrame"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 3), // "fps"
QT_MOC_LITERAL(4, 32, 14), // "QElapsedTimer*"
QT_MOC_LITERAL(5, 47, 5), // "timer"
QT_MOC_LITERAL(6, 53, 13) // "targetInstant"

    },
    "PlaybackExecutor\0nextFrame\0\0fps\0"
    "QElapsedTimer*\0timer\0targetInstant"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PlaybackExecutor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    3,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, 0x80000000 | 4, QMetaType::LongLong,    3,    5,    6,

       0        // eod
};

void PlaybackExecutor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlaybackExecutor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->nextFrame((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QElapsedTimer*(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlaybackExecutor::*)(int , QElapsedTimer * , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlaybackExecutor::nextFrame)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PlaybackExecutor::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_PlaybackExecutor.data,
    qt_meta_data_PlaybackExecutor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PlaybackExecutor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaybackExecutor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PlaybackExecutor.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int PlaybackExecutor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void PlaybackExecutor::nextFrame(int _t1, QElapsedTimer * _t2, qint64 _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_FlipSlider_t {
    QByteArrayData data[17];
    char stringdata0[243];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FlipSlider_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FlipSlider_t qt_meta_stringdata_FlipSlider = {
    {
QT_MOC_LITERAL(0, 0, 10), // "FlipSlider"
QT_MOC_LITERAL(1, 11, 18), // "flipSliderReleased"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 17), // "flipSliderPressed"
QT_MOC_LITERAL(4, 49, 8), // "PBHeight"
QT_MOC_LITERAL(5, 58, 9), // "PBOverlay"
QT_MOC_LITERAL(6, 68, 8), // "PBMarker"
QT_MOC_LITERAL(7, 77, 17), // "PBColorMarginLeft"
QT_MOC_LITERAL(8, 95, 16), // "PBColorMarginTop"
QT_MOC_LITERAL(9, 112, 18), // "PBColorMarginRight"
QT_MOC_LITERAL(10, 131, 19), // "PBColorMarginBottom"
QT_MOC_LITERAL(11, 151, 18), // "PBMarkerMarginLeft"
QT_MOC_LITERAL(12, 170, 19), // "PBMarkerMarginRight"
QT_MOC_LITERAL(13, 190, 9), // "baseColor"
QT_MOC_LITERAL(14, 200, 15), // "notStartedColor"
QT_MOC_LITERAL(15, 216, 12), // "startedColor"
QT_MOC_LITERAL(16, 229, 13) // "finishedColor"

    },
    "FlipSlider\0flipSliderReleased\0\0"
    "flipSliderPressed\0PBHeight\0PBOverlay\0"
    "PBMarker\0PBColorMarginLeft\0PBColorMarginTop\0"
    "PBColorMarginRight\0PBColorMarginBottom\0"
    "PBMarkerMarginLeft\0PBMarkerMarginRight\0"
    "baseColor\0notStartedColor\0startedColor\0"
    "finishedColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FlipSlider[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
      13,   26, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,
       3,    0,   25,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       4, QMetaType::Int, 0x00095103,
       5, QMetaType::QImage, 0x00095103,
       6, QMetaType::QImage, 0x00095103,
       7, QMetaType::Int, 0x00095103,
       8, QMetaType::Int, 0x00095103,
       9, QMetaType::Int, 0x00095103,
      10, QMetaType::Int, 0x00095103,
      11, QMetaType::Int, 0x00095103,
      12, QMetaType::Int, 0x00095103,
      13, QMetaType::QColor, 0x00095103,
      14, QMetaType::QColor, 0x00095103,
      15, QMetaType::QColor, 0x00095103,
      16, QMetaType::QColor, 0x00095103,

       0        // eod
};

void FlipSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FlipSlider *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->flipSliderReleased(); break;
        case 1: _t->flipSliderPressed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FlipSlider::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipSlider::flipSliderReleased)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FlipSlider::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipSlider::flipSliderPressed)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<FlipSlider *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->getPBHeight(); break;
        case 1: *reinterpret_cast< QImage*>(_v) = _t->getPBOverlay(); break;
        case 2: *reinterpret_cast< QImage*>(_v) = _t->getPBMarker(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->getPBColorMarginLeft(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->getPBColorMarginTop(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->getPBColorMarginRight(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->getPBColorMarginBottom(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->getPBMarkerMarginLeft(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->getPBMarkerMarginRight(); break;
        case 9: *reinterpret_cast< QColor*>(_v) = _t->getBaseColor(); break;
        case 10: *reinterpret_cast< QColor*>(_v) = _t->getNotStartedColor(); break;
        case 11: *reinterpret_cast< QColor*>(_v) = _t->getStartedColor(); break;
        case 12: *reinterpret_cast< QColor*>(_v) = _t->getFinishedColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<FlipSlider *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPBHeight(*reinterpret_cast< int*>(_v)); break;
        case 1: _t->setPBOverlay(*reinterpret_cast< QImage*>(_v)); break;
        case 2: _t->setPBMarker(*reinterpret_cast< QImage*>(_v)); break;
        case 3: _t->setPBColorMarginLeft(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setPBColorMarginTop(*reinterpret_cast< int*>(_v)); break;
        case 5: _t->setPBColorMarginRight(*reinterpret_cast< int*>(_v)); break;
        case 6: _t->setPBColorMarginBottom(*reinterpret_cast< int*>(_v)); break;
        case 7: _t->setPBMarkerMarginLeft(*reinterpret_cast< int*>(_v)); break;
        case 8: _t->setPBMarkerMarginRight(*reinterpret_cast< int*>(_v)); break;
        case 9: _t->setBaseColor(*reinterpret_cast< QColor*>(_v)); break;
        case 10: _t->setNotStartedColor(*reinterpret_cast< QColor*>(_v)); break;
        case 11: _t->setStartedColor(*reinterpret_cast< QColor*>(_v)); break;
        case 12: _t->setFinishedColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FlipSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractSlider::staticMetaObject>(),
    qt_meta_stringdata_FlipSlider.data,
    qt_meta_data_FlipSlider,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FlipSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FlipSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FlipSlider.stringdata0))
        return static_cast<void*>(this);
    return QAbstractSlider::qt_metacast(_clname);
}

int FlipSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractSlider::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 13;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 13;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void FlipSlider::flipSliderReleased()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FlipSlider::flipSliderPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_FlipConsole_t {
    QByteArrayData data[31];
    char stringdata0[388];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FlipConsole_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FlipConsole_t qt_meta_stringdata_FlipConsole = {
    {
QT_MOC_LITERAL(0, 0, 11), // "FlipConsole"
QT_MOC_LITERAL(1, 12, 13), // "buttonPressed"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 20), // "FlipConsole::EGadget"
QT_MOC_LITERAL(4, 48, 6), // "button"
QT_MOC_LITERAL(5, 55, 16), // "playStateChanged"
QT_MOC_LITERAL(6, 72, 9), // "isPlaying"
QT_MOC_LITERAL(7, 82, 14), // "sliderReleased"
QT_MOC_LITERAL(8, 97, 17), // "OnSetCurrentFrame"
QT_MOC_LITERAL(9, 115, 20), // "OnFrameSliderRelease"
QT_MOC_LITERAL(10, 136, 18), // "OnFrameSliderPress"
QT_MOC_LITERAL(11, 155, 13), // "setCurrentFPS"
QT_MOC_LITERAL(12, 169, 8), // "dragging"
QT_MOC_LITERAL(13, 178, 15), // "onButtonPressed"
QT_MOC_LITERAL(14, 194, 8), // "QAction*"
QT_MOC_LITERAL(15, 203, 6), // "action"
QT_MOC_LITERAL(16, 210, 21), // "incrementCurrentFrame"
QT_MOC_LITERAL(17, 232, 5), // "delta"
QT_MOC_LITERAL(18, 238, 11), // "onNextFrame"
QT_MOC_LITERAL(19, 250, 3), // "fps"
QT_MOC_LITERAL(20, 254, 14), // "QElapsedTimer*"
QT_MOC_LITERAL(21, 269, 5), // "timer"
QT_MOC_LITERAL(22, 275, 6), // "target"
QT_MOC_LITERAL(23, 282, 24), // "onCustomizeButtonPressed"
QT_MOC_LITERAL(24, 307, 10), // "drawBlanks"
QT_MOC_LITERAL(25, 318, 4), // "from"
QT_MOC_LITERAL(26, 323, 2), // "to"
QT_MOC_LITERAL(27, 326, 15), // "onSliderRelease"
QT_MOC_LITERAL(28, 342, 11), // "onFPSEdited"
QT_MOC_LITERAL(29, 354, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(30, 374, 13) // "FpsFieldColor"

    },
    "FlipConsole\0buttonPressed\0\0"
    "FlipConsole::EGadget\0button\0"
    "playStateChanged\0isPlaying\0sliderReleased\0"
    "OnSetCurrentFrame\0OnFrameSliderRelease\0"
    "OnFrameSliderPress\0setCurrentFPS\0"
    "dragging\0onButtonPressed\0QAction*\0"
    "action\0incrementCurrentFrame\0delta\0"
    "onNextFrame\0fps\0QElapsedTimer*\0timer\0"
    "target\0onCustomizeButtonPressed\0"
    "drawBlanks\0from\0to\0onSliderRelease\0"
    "onFPSEdited\0onPreferenceChanged\0"
    "FpsFieldColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FlipConsole[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       1,  156, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x06 /* Public */,
       5,    1,  107,    2, 0x06 /* Public */,
       7,    0,  110,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,  111,    2, 0x09 /* Protected */,
       9,    0,  112,    2, 0x09 /* Protected */,
      10,    0,  113,    2, 0x09 /* Protected */,
       8,    1,  114,    2, 0x09 /* Protected */,
      11,    1,  117,    2, 0x09 /* Protected */,
      11,    1,  120,    2, 0x09 /* Protected */,
      13,    1,  123,    2, 0x09 /* Protected */,
      13,    1,  126,    2, 0x09 /* Protected */,
      16,    1,  129,    2, 0x09 /* Protected */,
      18,    3,  132,    2, 0x09 /* Protected */,
      23,    1,  139,    2, 0x09 /* Protected */,
      24,    4,  142,    2, 0x09 /* Protected */,
      27,    0,  151,    2, 0x09 /* Protected */,
      28,    0,  152,    2, 0x09 /* Protected */,
      29,    1,  153,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 20, QMetaType::LongLong,   19,   21,   22,
    QMetaType::Void, 0x80000000 | 14,    2,
    QMetaType::Bool, QMetaType::Int, QMetaType::Int, 0x80000000 | 20, QMetaType::LongLong,   25,   26,   21,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // properties: name, type, flags
      30, QMetaType::QColor, 0x00095103,

       0        // eod
};

void FlipConsole::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FlipConsole *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->buttonPressed((*reinterpret_cast< FlipConsole::EGadget(*)>(_a[1]))); break;
        case 1: _t->playStateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->sliderReleased(); break;
        case 3: _t->OnSetCurrentFrame(); break;
        case 4: _t->OnFrameSliderRelease(); break;
        case 5: _t->OnFrameSliderPress(); break;
        case 6: _t->OnSetCurrentFrame((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->setCurrentFPS((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->setCurrentFPS((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->onButtonPressed((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 10: _t->onButtonPressed((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->incrementCurrentFrame((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->onNextFrame((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QElapsedTimer*(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 13: _t->onCustomizeButtonPressed((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 14: { bool _r = _t->drawBlanks((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< QElapsedTimer*(*)>(_a[3])),(*reinterpret_cast< qint64(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->onSliderRelease(); break;
        case 16: _t->onFPSEdited(); break;
        case 17: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FlipConsole::*)(FlipConsole::EGadget );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipConsole::buttonPressed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FlipConsole::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipConsole::playStateChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FlipConsole::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FlipConsole::sliderReleased)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<FlipConsole *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getFpsFieldColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<FlipConsole *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFpsFieldColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject FlipConsole::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_FlipConsole.data,
    qt_meta_data_FlipConsole,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FlipConsole::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FlipConsole::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FlipConsole.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int FlipConsole::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void FlipConsole::buttonPressed(FlipConsole::EGadget _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FlipConsole::playStateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FlipConsole::sliderReleased()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
