/****************************************************************************
** Meta object code from reading C++ file 'pane.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/pane.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pane.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TPanelTitleBarButton_t {
    QByteArrayData data[10];
    char stringdata0[106];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPanelTitleBarButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPanelTitleBarButton_t qt_meta_stringdata_TPanelTitleBarButton = {
    {
QT_MOC_LITERAL(0, 0, 20), // "TPanelTitleBarButton"
QT_MOC_LITERAL(1, 21, 7), // "toggled"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 7), // "pressed"
QT_MOC_LITERAL(4, 38, 10), // "setPressed"
QT_MOC_LITERAL(5, 49, 8), // "OffColor"
QT_MOC_LITERAL(6, 58, 9), // "OverColor"
QT_MOC_LITERAL(7, 68, 12), // "PressedColor"
QT_MOC_LITERAL(8, 81, 11), // "FreezeColor"
QT_MOC_LITERAL(9, 93, 12) // "PreviewColor"

    },
    "TPanelTitleBarButton\0toggled\0\0pressed\0"
    "setPressed\0OffColor\0OverColor\0"
    "PressedColor\0FreezeColor\0PreviewColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPanelTitleBarButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       5,   30, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   27,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // properties: name, type, flags
       5, QMetaType::QColor, 0x00095103,
       6, QMetaType::QColor, 0x00095103,
       7, QMetaType::QColor, 0x00095103,
       8, QMetaType::QColor, 0x00095103,
       9, QMetaType::QColor, 0x00095103,

       0        // eod
};

void TPanelTitleBarButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TPanelTitleBarButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->setPressed((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TPanelTitleBarButton::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPanelTitleBarButton::toggled)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<TPanelTitleBarButton *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getOffColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getOverColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getPressedColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getFreezeColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getPreviewColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<TPanelTitleBarButton *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setOverColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setPressedColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setFreezeColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setPreviewColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TPanelTitleBarButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TPanelTitleBarButton.data,
    qt_meta_data_TPanelTitleBarButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPanelTitleBarButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPanelTitleBarButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPanelTitleBarButton.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int TPanelTitleBarButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void TPanelTitleBarButton::toggled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_TPanelTitleBarButtonForBindToRoom_t {
    QByteArrayData data[1];
    char stringdata0[34];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPanelTitleBarButtonForBindToRoom_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPanelTitleBarButtonForBindToRoom_t qt_meta_stringdata_TPanelTitleBarButtonForBindToRoom = {
    {
QT_MOC_LITERAL(0, 0, 33) // "TPanelTitleBarButtonForBindTo..."

    },
    "TPanelTitleBarButtonForBindToRoom"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPanelTitleBarButtonForBindToRoom[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void TPanelTitleBarButtonForBindToRoom::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TPanelTitleBarButtonForBindToRoom::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanelTitleBarButton::staticMetaObject>(),
    qt_meta_stringdata_TPanelTitleBarButtonForBindToRoom.data,
    qt_meta_data_TPanelTitleBarButtonForBindToRoom,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPanelTitleBarButtonForBindToRoom::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPanelTitleBarButtonForBindToRoom::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPanelTitleBarButtonForBindToRoom.stringdata0))
        return static_cast<void*>(this);
    return TPanelTitleBarButton::qt_metacast(_clname);
}

int TPanelTitleBarButtonForBindToRoom::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanelTitleBarButton::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_TPanelTitleBarButtonForSafeArea_t {
    QByteArrayData data[3];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPanelTitleBarButtonForSafeArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPanelTitleBarButtonForSafeArea_t qt_meta_stringdata_TPanelTitleBarButtonForSafeArea = {
    {
QT_MOC_LITERAL(0, 0, 31), // "TPanelTitleBarButtonForSafeArea"
QT_MOC_LITERAL(1, 32, 13), // "onSetSafeArea"
QT_MOC_LITERAL(2, 46, 0) // ""

    },
    "TPanelTitleBarButtonForSafeArea\0"
    "onSetSafeArea\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPanelTitleBarButtonForSafeArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void TPanelTitleBarButtonForSafeArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TPanelTitleBarButtonForSafeArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSetSafeArea(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TPanelTitleBarButtonForSafeArea::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanelTitleBarButton::staticMetaObject>(),
    qt_meta_stringdata_TPanelTitleBarButtonForSafeArea.data,
    qt_meta_data_TPanelTitleBarButtonForSafeArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPanelTitleBarButtonForSafeArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPanelTitleBarButtonForSafeArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPanelTitleBarButtonForSafeArea.stringdata0))
        return static_cast<void*>(this);
    return TPanelTitleBarButton::qt_metacast(_clname);
}

int TPanelTitleBarButtonForSafeArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanelTitleBarButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_TPanelTitleBarButtonForPreview_t {
    QByteArrayData data[3];
    char stringdata0[53];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPanelTitleBarButtonForPreview_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPanelTitleBarButtonForPreview_t qt_meta_stringdata_TPanelTitleBarButtonForPreview = {
    {
QT_MOC_LITERAL(0, 0, 30), // "TPanelTitleBarButtonForPreview"
QT_MOC_LITERAL(1, 31, 20), // "onSetPreviewBehavior"
QT_MOC_LITERAL(2, 52, 0) // ""

    },
    "TPanelTitleBarButtonForPreview\0"
    "onSetPreviewBehavior\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPanelTitleBarButtonForPreview[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void TPanelTitleBarButtonForPreview::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TPanelTitleBarButtonForPreview *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSetPreviewBehavior(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TPanelTitleBarButtonForPreview::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanelTitleBarButton::staticMetaObject>(),
    qt_meta_stringdata_TPanelTitleBarButtonForPreview.data,
    qt_meta_data_TPanelTitleBarButtonForPreview,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPanelTitleBarButtonForPreview::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPanelTitleBarButtonForPreview::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPanelTitleBarButtonForPreview.stringdata0))
        return static_cast<void*>(this);
    return TPanelTitleBarButton::qt_metacast(_clname);
}

int TPanelTitleBarButtonForPreview::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanelTitleBarButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_TPanelTitleBarButtonSet_t {
    QByteArrayData data[4];
    char stringdata0[37];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPanelTitleBarButtonSet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPanelTitleBarButtonSet_t qt_meta_stringdata_TPanelTitleBarButtonSet = {
    {
QT_MOC_LITERAL(0, 0, 23), // "TPanelTitleBarButtonSet"
QT_MOC_LITERAL(1, 24, 8), // "selected"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 2) // "id"

    },
    "TPanelTitleBarButtonSet\0selected\0\0id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPanelTitleBarButtonSet[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,

       0        // eod
};

void TPanelTitleBarButtonSet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TPanelTitleBarButtonSet *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selected((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TPanelTitleBarButtonSet::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPanelTitleBarButtonSet::selected)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TPanelTitleBarButtonSet::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TPanelTitleBarButtonSet.data,
    qt_meta_data_TPanelTitleBarButtonSet,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPanelTitleBarButtonSet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPanelTitleBarButtonSet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPanelTitleBarButtonSet.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TPanelTitleBarButtonSet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void TPanelTitleBarButtonSet::selected(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_TPanelTitleBar_t {
    QByteArrayData data[13];
    char stringdata0[180];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPanelTitleBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPanelTitleBar_t qt_meta_stringdata_TPanelTitleBar = {
    {
QT_MOC_LITERAL(0, 0, 14), // "TPanelTitleBar"
QT_MOC_LITERAL(1, 15, 18), // "closeButtonPressed"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 11), // "doubleClick"
QT_MOC_LITERAL(4, 47, 12), // "QMouseEvent*"
QT_MOC_LITERAL(5, 60, 2), // "me"
QT_MOC_LITERAL(6, 63, 12), // "BorderPixmap"
QT_MOC_LITERAL(7, 76, 18), // "ActiveBorderPixmap"
QT_MOC_LITERAL(8, 95, 17), // "FloatBorderPixmap"
QT_MOC_LITERAL(9, 113, 23), // "FloatActiveBorderPixmap"
QT_MOC_LITERAL(10, 137, 10), // "TitleColor"
QT_MOC_LITERAL(11, 148, 16), // "ActiveTitleColor"
QT_MOC_LITERAL(12, 165, 14) // "CloseOverColor"

    },
    "TPanelTitleBar\0closeButtonPressed\0\0"
    "doubleClick\0QMouseEvent*\0me\0BorderPixmap\0"
    "ActiveBorderPixmap\0FloatBorderPixmap\0"
    "FloatActiveBorderPixmap\0TitleColor\0"
    "ActiveTitleColor\0CloseOverColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPanelTitleBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       7,   28, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,
       3,    1,   25,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,

 // properties: name, type, flags
       6, QMetaType::QPixmap, 0x00095103,
       7, QMetaType::QPixmap, 0x00095103,
       8, QMetaType::QPixmap, 0x00095103,
       9, QMetaType::QPixmap, 0x00095103,
      10, QMetaType::QColor, 0x00095103,
      11, QMetaType::QColor, 0x00095103,
      12, QMetaType::QColor, 0x00095103,

       0        // eod
};

void TPanelTitleBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TPanelTitleBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closeButtonPressed(); break;
        case 1: _t->doubleClick((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TPanelTitleBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPanelTitleBar::closeButtonPressed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TPanelTitleBar::*)(QMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPanelTitleBar::doubleClick)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<TPanelTitleBar *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QPixmap*>(_v) = _t->getBorderPixmap(); break;
        case 1: *reinterpret_cast< QPixmap*>(_v) = _t->getActiveBorderPixmap(); break;
        case 2: *reinterpret_cast< QPixmap*>(_v) = _t->getFloatBorderPixmap(); break;
        case 3: *reinterpret_cast< QPixmap*>(_v) = _t->getFloatActiveBorderPixmap(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getTitleColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getActiveTitleColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getCloseOverColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<TPanelTitleBar *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBorderPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 1: _t->setActiveBorderPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 2: _t->setFloatBorderPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 3: _t->setFloatActiveBorderPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 4: _t->setTitleColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setActiveTitleColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setCloseOverColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TPanelTitleBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_TPanelTitleBar.data,
    qt_meta_data_TPanelTitleBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPanelTitleBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPanelTitleBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPanelTitleBar.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int TPanelTitleBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void TPanelTitleBar::closeButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TPanelTitleBar::doubleClick(QMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_TPanel_t {
    QByteArrayData data[12];
    char stringdata0[160];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TPanel_t qt_meta_stringdata_TPanel = {
    {
QT_MOC_LITERAL(0, 0, 6), // "TPanel"
QT_MOC_LITERAL(1, 7, 11), // "doubleClick"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 12), // "QMouseEvent*"
QT_MOC_LITERAL(4, 33, 2), // "me"
QT_MOC_LITERAL(5, 36, 18), // "closeButtonPressed"
QT_MOC_LITERAL(6, 55, 20), // "onCloseButtonPressed"
QT_MOC_LITERAL(7, 76, 28), // "onCustomContextMenuRequested"
QT_MOC_LITERAL(8, 105, 3), // "pos"
QT_MOC_LITERAL(9, 109, 18), // "widgetFocusOnEnter"
QT_MOC_LITERAL(10, 128, 23), // "widgetClearFocusOnLeave"
QT_MOC_LITERAL(11, 152, 7) // "BGColor"

    },
    "TPanel\0doubleClick\0\0QMouseEvent*\0me\0"
    "closeButtonPressed\0onCloseButtonPressed\0"
    "onCustomContextMenuRequested\0pos\0"
    "widgetFocusOnEnter\0widgetClearFocusOnLeave\0"
    "BGColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       1,   54, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       5,    0,   47,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   48,    2, 0x09 /* Protected */,
       7,    1,   49,    2, 0x09 /* Protected */,
       9,    0,   52,    2, 0x09 /* Protected */,
      10,    0,   53,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    8,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      11, QMetaType::QColor, 0x00095103,

       0        // eod
};

void TPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->doubleClick((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        case 1: _t->closeButtonPressed(); break;
        case 2: _t->onCloseButtonPressed(); break;
        case 3: _t->onCustomContextMenuRequested((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 4: _t->widgetFocusOnEnter(); break;
        case 5: _t->widgetClearFocusOnLeave(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TPanel::*)(QMouseEvent * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPanel::doubleClick)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TPanel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TPanel::closeButtonPressed)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<TPanel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getBGColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<TPanel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBGColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject TPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TDockWidget::staticMetaObject>(),
    qt_meta_stringdata_TPanel.data,
    qt_meta_data_TPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TPanel.stringdata0))
        return static_cast<void*>(this);
    return TDockWidget::qt_metacast(_clname);
}

int TPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TDockWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void TPanel::doubleClick(QMouseEvent * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TPanel::closeButtonPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
