/****************************************************************************
** Meta object code from reading C++ file 'stageschematicscene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/stageschematicscene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'stageschematicscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StageSchematicScene_t {
    QByteArrayData data[41];
    char stringdata0[625];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StageSchematicScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StageSchematicScene_t qt_meta_stringdata_StageSchematicScene = {
    {
QT_MOC_LITERAL(0, 0, 19), // "StageSchematicScene"
QT_MOC_LITERAL(1, 20, 10), // "editObject"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 10), // "doCollapse"
QT_MOC_LITERAL(4, 43, 21), // "QList<TStageObjectId>"
QT_MOC_LITERAL(5, 65, 14), // "doExplodeChild"
QT_MOC_LITERAL(6, 80, 19), // "onResetCenterPublic"
QT_MOC_LITERAL(7, 100, 17), // "onEditGroupPublic"
QT_MOC_LITERAL(8, 118, 22), // "onCameraActivatePublic"
QT_MOC_LITERAL(9, 141, 20), // "onRemoveSplinePublic"
QT_MOC_LITERAL(10, 162, 18), // "onSaveSplinePublic"
QT_MOC_LITERAL(11, 181, 18), // "onLoadSplinePublic"
QT_MOC_LITERAL(12, 200, 19), // "onPathToggledPublic"
QT_MOC_LITERAL(13, 220, 5), // "state"
QT_MOC_LITERAL(14, 226, 17), // "onCpToggledPublic"
QT_MOC_LITERAL(15, 244, 7), // "toggled"
QT_MOC_LITERAL(16, 252, 19), // "onSelectionSwitched"
QT_MOC_LITERAL(17, 272, 11), // "TSelection*"
QT_MOC_LITERAL(18, 284, 6), // "oldSel"
QT_MOC_LITERAL(19, 291, 6), // "newSel"
QT_MOC_LITERAL(20, 298, 13), // "onPegbarAdded"
QT_MOC_LITERAL(21, 312, 13), // "onSplineAdded"
QT_MOC_LITERAL(22, 326, 13), // "onCameraAdded"
QT_MOC_LITERAL(23, 340, 13), // "onResetCenter"
QT_MOC_LITERAL(24, 354, 16), // "onCameraActivate"
QT_MOC_LITERAL(25, 371, 14), // "onRemoveSpline"
QT_MOC_LITERAL(26, 386, 12), // "onSaveSpline"
QT_MOC_LITERAL(27, 399, 12), // "onLoadSpline"
QT_MOC_LITERAL(28, 412, 13), // "onPathToggled"
QT_MOC_LITERAL(29, 426, 11), // "onCpToggled"
QT_MOC_LITERAL(30, 438, 15), // "onXsheetChanged"
QT_MOC_LITERAL(31, 454, 14), // "onSceneChanged"
QT_MOC_LITERAL(32, 469, 22), // "onCurrentObjectChanged"
QT_MOC_LITERAL(33, 492, 14), // "TStageObjectId"
QT_MOC_LITERAL(34, 507, 22), // "onCurrentColumnChanged"
QT_MOC_LITERAL(35, 530, 18), // "onSelectionChanged"
QT_MOC_LITERAL(36, 549, 10), // "onCollapse"
QT_MOC_LITERAL(37, 560, 11), // "onEditGroup"
QT_MOC_LITERAL(38, 572, 17), // "onNodeChangedSize"
QT_MOC_LITERAL(39, 590, 23), // "onSwitchPortModeToggled"
QT_MOC_LITERAL(40, 614, 10) // "withLetter"

    },
    "StageSchematicScene\0editObject\0\0"
    "doCollapse\0QList<TStageObjectId>\0"
    "doExplodeChild\0onResetCenterPublic\0"
    "onEditGroupPublic\0onCameraActivatePublic\0"
    "onRemoveSplinePublic\0onSaveSplinePublic\0"
    "onLoadSplinePublic\0onPathToggledPublic\0"
    "state\0onCpToggledPublic\0toggled\0"
    "onSelectionSwitched\0TSelection*\0oldSel\0"
    "newSel\0onPegbarAdded\0onSplineAdded\0"
    "onCameraAdded\0onResetCenter\0"
    "onCameraActivate\0onRemoveSpline\0"
    "onSaveSpline\0onLoadSpline\0onPathToggled\0"
    "onCpToggled\0onXsheetChanged\0onSceneChanged\0"
    "onCurrentObjectChanged\0TStageObjectId\0"
    "onCurrentColumnChanged\0onSelectionChanged\0"
    "onCollapse\0onEditGroup\0onNodeChangedSize\0"
    "onSwitchPortModeToggled\0withLetter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StageSchematicScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      31,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  169,    2, 0x06 /* Public */,
       3,    1,  170,    2, 0x06 /* Public */,
       5,    1,  173,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,  176,    2, 0x0a /* Public */,
       7,    0,  177,    2, 0x0a /* Public */,
       8,    0,  178,    2, 0x0a /* Public */,
       9,    0,  179,    2, 0x0a /* Public */,
      10,    0,  180,    2, 0x0a /* Public */,
      11,    0,  181,    2, 0x0a /* Public */,
      12,    1,  182,    2, 0x0a /* Public */,
      14,    1,  185,    2, 0x0a /* Public */,
      16,    2,  188,    2, 0x09 /* Protected */,
      20,    0,  193,    2, 0x09 /* Protected */,
      21,    0,  194,    2, 0x09 /* Protected */,
      22,    0,  195,    2, 0x09 /* Protected */,
      23,    0,  196,    2, 0x09 /* Protected */,
      24,    0,  197,    2, 0x09 /* Protected */,
      25,    0,  198,    2, 0x09 /* Protected */,
      26,    0,  199,    2, 0x09 /* Protected */,
      27,    0,  200,    2, 0x09 /* Protected */,
      28,    1,  201,    2, 0x09 /* Protected */,
      29,    1,  204,    2, 0x09 /* Protected */,
      30,    0,  207,    2, 0x09 /* Protected */,
      31,    0,  208,    2, 0x09 /* Protected */,
      32,    2,  209,    2, 0x09 /* Protected */,
      34,    1,  214,    2, 0x09 /* Protected */,
      35,    0,  217,    2, 0x09 /* Protected */,
      36,    1,  218,    2, 0x09 /* Protected */,
      37,    0,  221,    2, 0x09 /* Protected */,
      38,    0,  222,    2, 0x09 /* Protected */,
      39,    1,  223,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void, 0x80000000 | 4,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void, 0x80000000 | 17, 0x80000000 | 17,   18,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 33, QMetaType::Bool,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   40,

       0        // eod
};

void StageSchematicScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StageSchematicScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editObject(); break;
        case 1: _t->doCollapse((*reinterpret_cast< QList<TStageObjectId>(*)>(_a[1]))); break;
        case 2: _t->doExplodeChild((*reinterpret_cast< QList<TStageObjectId>(*)>(_a[1]))); break;
        case 3: _t->onResetCenterPublic(); break;
        case 4: _t->onEditGroupPublic(); break;
        case 5: _t->onCameraActivatePublic(); break;
        case 6: _t->onRemoveSplinePublic(); break;
        case 7: _t->onSaveSplinePublic(); break;
        case 8: _t->onLoadSplinePublic(); break;
        case 9: _t->onPathToggledPublic((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->onCpToggledPublic((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->onSelectionSwitched((*reinterpret_cast< TSelection*(*)>(_a[1])),(*reinterpret_cast< TSelection*(*)>(_a[2]))); break;
        case 12: _t->onPegbarAdded(); break;
        case 13: _t->onSplineAdded(); break;
        case 14: _t->onCameraAdded(); break;
        case 15: _t->onResetCenter(); break;
        case 16: _t->onCameraActivate(); break;
        case 17: _t->onRemoveSpline(); break;
        case 18: _t->onSaveSpline(); break;
        case 19: _t->onLoadSpline(); break;
        case 20: _t->onPathToggled((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->onCpToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->onXsheetChanged(); break;
        case 23: _t->onSceneChanged(); break;
        case 24: _t->onCurrentObjectChanged((*reinterpret_cast< const TStageObjectId(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 25: _t->onCurrentColumnChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->onSelectionChanged(); break;
        case 27: _t->onCollapse((*reinterpret_cast< QList<TStageObjectId>(*)>(_a[1]))); break;
        case 28: _t->onEditGroup(); break;
        case 29: _t->onNodeChangedSize(); break;
        case 30: _t->onSwitchPortModeToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StageSchematicScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicScene::editObject)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (StageSchematicScene::*)(QList<TStageObjectId> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicScene::doCollapse)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (StageSchematicScene::*)(QList<TStageObjectId> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StageSchematicScene::doExplodeChild)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StageSchematicScene::staticMetaObject = { {
    QMetaObject::SuperData::link<SchematicScene::staticMetaObject>(),
    qt_meta_stringdata_StageSchematicScene.data,
    qt_meta_data_StageSchematicScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StageSchematicScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StageSchematicScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StageSchematicScene.stringdata0))
        return static_cast<void*>(this);
    return SchematicScene::qt_metacast(_clname);
}

int StageSchematicScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchematicScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 31;
    }
    return _id;
}

// SIGNAL 0
void StageSchematicScene::editObject()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void StageSchematicScene::doCollapse(QList<TStageObjectId> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void StageSchematicScene::doExplodeChild(QList<TStageObjectId> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
