/****************************************************************************
** Meta object code from reading C++ file 'toolpropertiespanel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/toolpropertiespanel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'toolpropertiespanel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ToolPropertyButton_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolPropertyButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolPropertyButton_t qt_meta_stringdata_ToolPropertyButton = {
    {
QT_MOC_LITERAL(0, 0, 18) // "ToolPropertyButton"

    },
    "ToolPropertyButton"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolPropertyButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ToolPropertyButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolPropertyButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolButton::staticMetaObject>(),
    qt_meta_stringdata_ToolPropertyButton.data,
    qt_meta_data_ToolPropertyButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolPropertyButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolPropertyButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolPropertyButton.stringdata0))
        return static_cast<void*>(this);
    return QToolButton::qt_metacast(_clname);
}

int ToolPropertyButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolButton::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ToolPropertiesPanel_t {
    QByteArrayData data[19];
    char stringdata0[284];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolPropertiesPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolPropertiesPanel_t qt_meta_stringdata_ToolPropertiesPanel = {
    {
QT_MOC_LITERAL(0, 0, 19), // "ToolPropertiesPanel"
QT_MOC_LITERAL(1, 20, 14), // "onToolSwitched"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 13), // "onToolChanged"
QT_MOC_LITERAL(4, 50, 13), // "onSizeChanged"
QT_MOC_LITERAL(5, 64, 5), // "value"
QT_MOC_LITERAL(6, 70, 17), // "onHardnessChanged"
QT_MOC_LITERAL(7, 88, 16), // "onOpacityChanged"
QT_MOC_LITERAL(8, 105, 18), // "onLockAlphaChanged"
QT_MOC_LITERAL(9, 124, 7), // "checked"
QT_MOC_LITERAL(10, 132, 19), // "onPencilModeChanged"
QT_MOC_LITERAL(11, 152, 18), // "onDrawOrderChanged"
QT_MOC_LITERAL(12, 171, 5), // "index"
QT_MOC_LITERAL(13, 177, 12), // "onCapChanged"
QT_MOC_LITERAL(14, 190, 13), // "onJoinChanged"
QT_MOC_LITERAL(15, 204, 15), // "onSmoothChanged"
QT_MOC_LITERAL(16, 220, 19), // "onAssistantsChanged"
QT_MOC_LITERAL(17, 240, 17), // "onPressureChanged"
QT_MOC_LITERAL(18, 258, 25) // "onShowHideActionTriggered"

    },
    "ToolPropertiesPanel\0onToolSwitched\0\0"
    "onToolChanged\0onSizeChanged\0value\0"
    "onHardnessChanged\0onOpacityChanged\0"
    "onLockAlphaChanged\0checked\0"
    "onPencilModeChanged\0onDrawOrderChanged\0"
    "index\0onCapChanged\0onJoinChanged\0"
    "onSmoothChanged\0onAssistantsChanged\0"
    "onPressureChanged\0onShowHideActionTriggered"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolPropertiesPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x08 /* Private */,
       3,    0,   85,    2, 0x08 /* Private */,
       4,    1,   86,    2, 0x08 /* Private */,
       6,    1,   89,    2, 0x08 /* Private */,
       7,    1,   92,    2, 0x08 /* Private */,
       8,    1,   95,    2, 0x08 /* Private */,
      10,    1,   98,    2, 0x08 /* Private */,
      11,    1,  101,    2, 0x08 /* Private */,
      13,    1,  104,    2, 0x08 /* Private */,
      14,    1,  107,    2, 0x08 /* Private */,
      15,    1,  110,    2, 0x08 /* Private */,
      16,    1,  113,    2, 0x08 /* Private */,
      17,    1,  116,    2, 0x08 /* Private */,
      18,    0,  119,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,

       0        // eod
};

void ToolPropertiesPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolPropertiesPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onToolSwitched(); break;
        case 1: _t->onToolChanged(); break;
        case 2: _t->onSizeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onHardnessChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->onOpacityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onLockAlphaChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->onPencilModeChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->onDrawOrderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->onCapChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->onJoinChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->onSmoothChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->onAssistantsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->onPressureChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->onShowHideActionTriggered(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolPropertiesPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<TPanel::staticMetaObject>(),
    qt_meta_stringdata_ToolPropertiesPanel.data,
    qt_meta_data_ToolPropertiesPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolPropertiesPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolPropertiesPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolPropertiesPanel.stringdata0))
        return static_cast<void*>(this);
    return TPanel::qt_metacast(_clname);
}

int ToolPropertiesPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
