/****************************************************************************
** Meta object code from reading C++ file 'exportalllevelspopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/exportalllevelspopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'exportalllevelspopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ExportAllLevelsPopup_t {
    QByteArrayData data[15];
    char stringdata0[165];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportAllLevelsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportAllLevelsPopup_t qt_meta_stringdata_ExportAllLevelsPopup = {
    {
QT_MOC_LITERAL(0, 0, 20), // "ExportAllLevelsPopup"
QT_MOC_LITERAL(1, 21, 27), // "collectSelectedSimpleLevels"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 9), // "showEvent"
QT_MOC_LITERAL(4, 60, 11), // "QShowEvent*"
QT_MOC_LITERAL(5, 72, 2), // "se"
QT_MOC_LITERAL(6, 75, 9), // "hideEvent"
QT_MOC_LITERAL(7, 85, 11), // "QHideEvent*"
QT_MOC_LITERAL(8, 97, 2), // "he"
QT_MOC_LITERAL(9, 100, 17), // "updateOnSelection"
QT_MOC_LITERAL(10, 118, 11), // "onExportAll"
QT_MOC_LITERAL(11, 130, 7), // "toggled"
QT_MOC_LITERAL(12, 138, 4), // "skip"
QT_MOC_LITERAL(13, 143, 16), // "getLevelTypeName"
QT_MOC_LITERAL(14, 160, 4) // "type"

    },
    "ExportAllLevelsPopup\0collectSelectedSimpleLevels\0"
    "\0showEvent\0QShowEvent*\0se\0hideEvent\0"
    "QHideEvent*\0he\0updateOnSelection\0"
    "onExportAll\0toggled\0skip\0getLevelTypeName\0"
    "type"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportAllLevelsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x08 /* Private */,
       3,    1,   50,    2, 0x08 /* Private */,
       6,    1,   53,    2, 0x08 /* Private */,
       9,    0,   56,    2, 0x08 /* Private */,
      10,    1,   57,    2, 0x08 /* Private */,
      12,    0,   60,    2, 0x08 /* Private */,
      13,    1,   61,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void,
    QMetaType::QString, QMetaType::Int,   14,

       0        // eod
};

void ExportAllLevelsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ExportAllLevelsPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->collectSelectedSimpleLevels(); break;
        case 1: _t->showEvent((*reinterpret_cast< QShowEvent*(*)>(_a[1]))); break;
        case 2: _t->hideEvent((*reinterpret_cast< QHideEvent*(*)>(_a[1]))); break;
        case 3: _t->updateOnSelection(); break;
        case 4: _t->onExportAll((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->skip(); break;
        case 6: { QString _r = _t->getLevelTypeName((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ExportAllLevelsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<ExportLevelPopup::staticMetaObject>(),
    qt_meta_stringdata_ExportAllLevelsPopup.data,
    qt_meta_data_ExportAllLevelsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportAllLevelsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportAllLevelsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportAllLevelsPopup.stringdata0))
        return static_cast<void*>(this);
    return ExportLevelPopup::qt_metacast(_clname);
}

int ExportAllLevelsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ExportLevelPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
