/****************************************************************************
** Meta object code from reading C++ file 'camerasettingswidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/camerasettingswidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'camerasettingswidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CameraSettingsWidget_t {
    QByteArrayData data[18];
    char stringdata0[232];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CameraSettingsWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CameraSettingsWidget_t qt_meta_stringdata_CameraSettingsWidget = {
    {
QT_MOC_LITERAL(0, 0, 20), // "CameraSettingsWidget"
QT_MOC_LITERAL(1, 21, 7), // "changed"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 17), // "levelSettingsUsed"
QT_MOC_LITERAL(4, 48, 11), // "onLxChanged"
QT_MOC_LITERAL(5, 60, 11), // "onLyChanged"
QT_MOC_LITERAL(6, 72, 11), // "onArChanged"
QT_MOC_LITERAL(7, 84, 13), // "onXResChanged"
QT_MOC_LITERAL(8, 98, 13), // "onYResChanged"
QT_MOC_LITERAL(9, 112, 13), // "onXDpiChanged"
QT_MOC_LITERAL(10, 126, 13), // "onYDpiChanged"
QT_MOC_LITERAL(11, 140, 12), // "onFspChanged"
QT_MOC_LITERAL(12, 153, 7), // "checked"
QT_MOC_LITERAL(13, 161, 13), // "onPrevToggled"
QT_MOC_LITERAL(14, 175, 16), // "onPresetSelected"
QT_MOC_LITERAL(15, 192, 9), // "addPreset"
QT_MOC_LITERAL(16, 202, 12), // "removePreset"
QT_MOC_LITERAL(17, 215, 16) // "useLevelSettings"

    },
    "CameraSettingsWidget\0changed\0\0"
    "levelSettingsUsed\0onLxChanged\0onLyChanged\0"
    "onArChanged\0onXResChanged\0onYResChanged\0"
    "onXDpiChanged\0onYDpiChanged\0onFspChanged\0"
    "checked\0onPrevToggled\0onPresetSelected\0"
    "addPreset\0removePreset\0useLevelSettings"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CameraSettingsWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x06 /* Public */,
       3,    0,   90,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   91,    2, 0x09 /* Protected */,
       5,    0,   92,    2, 0x09 /* Protected */,
       6,    0,   93,    2, 0x09 /* Protected */,
       7,    0,   94,    2, 0x09 /* Protected */,
       8,    0,   95,    2, 0x09 /* Protected */,
       9,    0,   96,    2, 0x09 /* Protected */,
      10,    0,   97,    2, 0x09 /* Protected */,
      11,    1,   98,    2, 0x09 /* Protected */,
      13,    1,  101,    2, 0x09 /* Protected */,
      14,    1,  104,    2, 0x09 /* Protected */,
      15,    0,  107,    2, 0x09 /* Protected */,
      16,    0,  108,    2, 0x09 /* Protected */,
      17,    0,  109,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::Bool,   12,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CameraSettingsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CameraSettingsWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->levelSettingsUsed(); break;
        case 2: _t->onLxChanged(); break;
        case 3: _t->onLyChanged(); break;
        case 4: _t->onArChanged(); break;
        case 5: _t->onXResChanged(); break;
        case 6: _t->onYResChanged(); break;
        case 7: _t->onXDpiChanged(); break;
        case 8: _t->onYDpiChanged(); break;
        case 9: _t->onFspChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->onPrevToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->onPresetSelected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 12: _t->addPreset(); break;
        case 13: _t->removePreset(); break;
        case 14: _t->useLevelSettings(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CameraSettingsWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraSettingsWidget::changed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CameraSettingsWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CameraSettingsWidget::levelSettingsUsed)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CameraSettingsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_CameraSettingsWidget.data,
    qt_meta_data_CameraSettingsWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CameraSettingsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CameraSettingsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CameraSettingsWidget.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int CameraSettingsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void CameraSettingsWidget::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CameraSettingsWidget::levelSettingsUsed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
