/****************************************************************************
** Meta object code from reading C++ file 'previewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/previewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'previewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Previewer_t {
    QByteArrayData data[19];
    char stringdata0[251];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Previewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Previewer_t qt_meta_stringdata_Previewer = {
    {
QT_MOC_LITERAL(0, 0, 9), // "Previewer"
QT_MOC_LITERAL(1, 10, 14), // "activedChanged"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 12), // "startedFrame"
QT_MOC_LITERAL(4, 39, 23), // "TRenderPort::RenderData"
QT_MOC_LITERAL(5, 63, 10), // "renderData"
QT_MOC_LITERAL(6, 74, 13), // "renderedFrame"
QT_MOC_LITERAL(7, 88, 11), // "failedFrame"
QT_MOC_LITERAL(8, 100, 9), // "saveFrame"
QT_MOC_LITERAL(9, 110, 18), // "saveRenderedFrames"
QT_MOC_LITERAL(10, 129, 6), // "update"
QT_MOC_LITERAL(11, 136, 10), // "updateView"
QT_MOC_LITERAL(12, 147, 14), // "onLevelChanged"
QT_MOC_LITERAL(13, 162, 11), // "onFxChanged"
QT_MOC_LITERAL(14, 174, 15), // "onXsheetChanged"
QT_MOC_LITERAL(15, 190, 15), // "onObjectChanged"
QT_MOC_LITERAL(16, 206, 14), // "onStartedFrame"
QT_MOC_LITERAL(17, 221, 15), // "onRenderedFrame"
QT_MOC_LITERAL(18, 237, 13) // "onFailedFrame"

    },
    "Previewer\0activedChanged\0\0startedFrame\0"
    "TRenderPort::RenderData\0renderData\0"
    "renderedFrame\0failedFrame\0saveFrame\0"
    "saveRenderedFrames\0update\0updateView\0"
    "onLevelChanged\0onFxChanged\0onXsheetChanged\0"
    "onObjectChanged\0onStartedFrame\0"
    "onRenderedFrame\0onFailedFrame"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Previewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x06 /* Public */,
       3,    1,   90,    2, 0x06 /* Public */,
       6,    1,   93,    2, 0x06 /* Public */,
       7,    1,   96,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,   99,    2, 0x0a /* Public */,
       9,    0,  100,    2, 0x0a /* Public */,
      10,    0,  101,    2, 0x0a /* Public */,
      11,    0,  102,    2, 0x0a /* Public */,
      12,    0,  103,    2, 0x0a /* Public */,
      13,    0,  104,    2, 0x0a /* Public */,
      14,    0,  105,    2, 0x0a /* Public */,
      15,    0,  106,    2, 0x0a /* Public */,
      16,    1,  107,    2, 0x09 /* Protected */,
      17,    1,  110,    2, 0x09 /* Protected */,
      18,    1,  113,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 4,    5,

       0        // eod
};

void Previewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Previewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activedChanged(); break;
        case 1: _t->startedFrame((*reinterpret_cast< TRenderPort::RenderData(*)>(_a[1]))); break;
        case 2: _t->renderedFrame((*reinterpret_cast< TRenderPort::RenderData(*)>(_a[1]))); break;
        case 3: _t->failedFrame((*reinterpret_cast< TRenderPort::RenderData(*)>(_a[1]))); break;
        case 4: _t->saveFrame(); break;
        case 5: _t->saveRenderedFrames(); break;
        case 6: _t->update(); break;
        case 7: _t->updateView(); break;
        case 8: _t->onLevelChanged(); break;
        case 9: _t->onFxChanged(); break;
        case 10: _t->onXsheetChanged(); break;
        case 11: _t->onObjectChanged(); break;
        case 12: _t->onStartedFrame((*reinterpret_cast< TRenderPort::RenderData(*)>(_a[1]))); break;
        case 13: _t->onRenderedFrame((*reinterpret_cast< TRenderPort::RenderData(*)>(_a[1]))); break;
        case 14: _t->onFailedFrame((*reinterpret_cast< TRenderPort::RenderData(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Previewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Previewer::activedChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Previewer::*)(TRenderPort::RenderData );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Previewer::startedFrame)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Previewer::*)(TRenderPort::RenderData );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Previewer::renderedFrame)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Previewer::*)(TRenderPort::RenderData );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Previewer::failedFrame)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Previewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Previewer.data,
    qt_meta_data_Previewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Previewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Previewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Previewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TFxObserver"))
        return static_cast< TFxObserver*>(this);
    return QObject::qt_metacast(_clname);
}

int Previewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Previewer::activedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Previewer::startedFrame(TRenderPort::RenderData _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Previewer::renderedFrame(TRenderPort::RenderData _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Previewer::failedFrame(TRenderPort::RenderData _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
