/****************************************************************************
** Meta object code from reading C++ file 'fxsettings.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/fxsettings.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fxsettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ParamsPage_t {
    QByteArrayData data[4];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ParamsPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ParamsPage_t qt_meta_stringdata_ParamsPage = {
    {
QT_MOC_LITERAL(0, 0, 10), // "ParamsPage"
QT_MOC_LITERAL(1, 11, 24), // "preferredPageSizeChanged"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 9) // "TextColor"

    },
    "ParamsPage\0preferredPageSizeChanged\0"
    "\0TextColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ParamsPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       1,   20, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // properties: name, type, flags
       3, QMetaType::QColor, 0x00095103,

       0        // eod
};

void ParamsPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ParamsPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->preferredPageSizeChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ParamsPage::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ParamsPage::preferredPageSizeChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<ParamsPage *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<ParamsPage *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ParamsPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_ParamsPage.data,
    qt_meta_data_ParamsPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ParamsPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ParamsPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ParamsPage.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int ParamsPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void ParamsPage::preferredPageSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_ParamsPageSet_t {
    QByteArrayData data[6];
    char stringdata0[71];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ParamsPageSet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ParamsPageSet_t qt_meta_stringdata_ParamsPageSet = {
    {
QT_MOC_LITERAL(0, 0, 13), // "ParamsPageSet"
QT_MOC_LITERAL(1, 14, 7), // "setPage"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 12), // "openHelpFile"
QT_MOC_LITERAL(4, 36, 11), // "openHelpUrl"
QT_MOC_LITERAL(5, 48, 22) // "recomputePreferredSize"

    },
    "ParamsPageSet\0setPage\0\0openHelpFile\0"
    "openHelpUrl\0recomputePreferredSize"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ParamsPageSet[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x09 /* Protected */,
       3,    0,   37,    2, 0x09 /* Protected */,
       4,    0,   38,    2, 0x09 /* Protected */,
       5,    0,   39,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ParamsPageSet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ParamsPageSet *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->openHelpFile(); break;
        case 2: _t->openHelpUrl(); break;
        case 3: _t->recomputePreferredSize(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ParamsPageSet::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ParamsPageSet.data,
    qt_meta_data_ParamsPageSet,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ParamsPageSet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ParamsPageSet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ParamsPageSet.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ParamsPageSet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_ParamViewer_t {
    QByteArrayData data[7];
    char stringdata0[117];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ParamViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ParamViewer_t qt_meta_stringdata_ParamViewer = {
    {
QT_MOC_LITERAL(0, 0, 11), // "ParamViewer"
QT_MOC_LITERAL(1, 12, 21), // "currentFxParamChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 20), // "actualFxParamChanged"
QT_MOC_LITERAL(4, 56, 15), // "paramKeyChanged"
QT_MOC_LITERAL(5, 72, 20), // "preferredSizeChanged"
QT_MOC_LITERAL(6, 93, 23) // "showSwatchButtonToggled"

    },
    "ParamViewer\0currentFxParamChanged\0\0"
    "actualFxParamChanged\0paramKeyChanged\0"
    "preferredSizeChanged\0showSwatchButtonToggled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ParamViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,
       3,    0,   40,    2, 0x06 /* Public */,
       4,    0,   41,    2, 0x06 /* Public */,
       5,    1,   42,    2, 0x06 /* Public */,
       6,    1,   45,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QSize,    2,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void ParamViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ParamViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentFxParamChanged(); break;
        case 1: _t->actualFxParamChanged(); break;
        case 2: _t->paramKeyChanged(); break;
        case 3: _t->preferredSizeChanged((*reinterpret_cast< QSize(*)>(_a[1]))); break;
        case 4: _t->showSwatchButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ParamViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ParamViewer::currentFxParamChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ParamViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ParamViewer::actualFxParamChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ParamViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ParamViewer::paramKeyChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ParamViewer::*)(QSize );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ParamViewer::preferredSizeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ParamViewer::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ParamViewer::showSwatchButtonToggled)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ParamViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_ParamViewer.data,
    qt_meta_data_ParamViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ParamViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ParamViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ParamViewer.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int ParamViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void ParamViewer::currentFxParamChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ParamViewer::actualFxParamChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ParamViewer::paramKeyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ParamViewer::preferredSizeChanged(QSize _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ParamViewer::showSwatchButtonToggled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_FxSettings_t {
    QByteArrayData data[19];
    char stringdata0[252];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSettings_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSettings_t qt_meta_stringdata_FxSettings = {
    {
QT_MOC_LITERAL(0, 0, 10), // "FxSettings"
QT_MOC_LITERAL(1, 11, 15), // "setCurrentFrame"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 12), // "setCurrentFx"
QT_MOC_LITERAL(4, 41, 15), // "setCurrentScene"
QT_MOC_LITERAL(5, 57, 18), // "notifySceneChanged"
QT_MOC_LITERAL(6, 76, 12), // "updateViewer"
QT_MOC_LITERAL(7, 89, 17), // "updateParamViewer"
QT_MOC_LITERAL(8, 107, 14), // "onPointChanged"
QT_MOC_LITERAL(9, 122, 5), // "index"
QT_MOC_LITERAL(10, 128, 7), // "TPointD"
QT_MOC_LITERAL(11, 136, 1), // "p"
QT_MOC_LITERAL(12, 138, 17), // "onViewModeChanged"
QT_MOC_LITERAL(13, 156, 8), // "QAction*"
QT_MOC_LITERAL(14, 165, 10), // "setWhiteBg"
QT_MOC_LITERAL(15, 176, 10), // "setBlackBg"
QT_MOC_LITERAL(16, 187, 15), // "setCheckboardBg"
QT_MOC_LITERAL(17, 203, 22), // "onPreferredSizeChanged"
QT_MOC_LITERAL(18, 226, 25) // "onShowSwatchButtonToggled"

    },
    "FxSettings\0setCurrentFrame\0\0setCurrentFx\0"
    "setCurrentScene\0notifySceneChanged\0"
    "updateViewer\0updateParamViewer\0"
    "onPointChanged\0index\0TPointD\0p\0"
    "onViewModeChanged\0QAction*\0setWhiteBg\0"
    "setBlackBg\0setCheckboardBg\0"
    "onPreferredSizeChanged\0onShowSwatchButtonToggled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSettings[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x0a /* Public */,
       3,    0,   80,    2, 0x0a /* Public */,
       4,    0,   81,    2, 0x0a /* Public */,
       5,    0,   82,    2, 0x0a /* Public */,
       6,    0,   83,    2, 0x09 /* Protected */,
       7,    0,   84,    2, 0x09 /* Protected */,
       8,    2,   85,    2, 0x09 /* Protected */,
      12,    1,   90,    2, 0x09 /* Protected */,
      14,    0,   93,    2, 0x09 /* Protected */,
      15,    0,   94,    2, 0x09 /* Protected */,
      16,    0,   95,    2, 0x09 /* Protected */,
      17,    1,   96,    2, 0x09 /* Protected */,
      18,    1,   99,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 10,    9,   11,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QSize,    2,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void FxSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSettings *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setCurrentFrame(); break;
        case 1: _t->setCurrentFx(); break;
        case 2: _t->setCurrentScene(); break;
        case 3: _t->notifySceneChanged(); break;
        case 4: _t->updateViewer(); break;
        case 5: _t->updateParamViewer(); break;
        case 6: _t->onPointChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const TPointD(*)>(_a[2]))); break;
        case 7: _t->onViewModeChanged((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 8: _t->setWhiteBg(); break;
        case 9: _t->setBlackBg(); break;
        case 10: _t->setCheckboardBg(); break;
        case 11: _t->onPreferredSizeChanged((*reinterpret_cast< QSize(*)>(_a[1]))); break;
        case 12: _t->onShowSwatchButtonToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_meta_stringdata_FxSettings.data,
    qt_meta_data_FxSettings,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSettings.stringdata0))
        return static_cast<void*>(this);
    return QSplitter::qt_metacast(_clname);
}

int FxSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
