/****************************************************************************
** Meta object code from reading C++ file 'functionsheet.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/functionsheet.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'functionsheet.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FunctionSheetButtonArea_t {
    QByteArrayData data[3];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionSheetButtonArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionSheetButtonArea_t qt_meta_stringdata_FunctionSheetButtonArea = {
    {
QT_MOC_LITERAL(0, 0, 23), // "FunctionSheetButtonArea"
QT_MOC_LITERAL(1, 24, 18), // "syncSizeBtnToggled"
QT_MOC_LITERAL(2, 43, 0) // ""

    },
    "FunctionSheetButtonArea\0syncSizeBtnToggled\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionSheetButtonArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void FunctionSheetButtonArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionSheetButtonArea *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->syncSizeBtnToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FunctionSheetButtonArea::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FunctionSheetButtonArea::syncSizeBtnToggled)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionSheetButtonArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_FunctionSheetButtonArea.data,
    qt_meta_data_FunctionSheetButtonArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionSheetButtonArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionSheetButtonArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionSheetButtonArea.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int FunctionSheetButtonArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void FunctionSheetButtonArea::syncSizeBtnToggled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_FunctionSheetCellViewer_t {
    QByteArrayData data[6];
    char stringdata0[95];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionSheetCellViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionSheetCellViewer_t qt_meta_stringdata_FunctionSheetCellViewer = {
    {
QT_MOC_LITERAL(0, 0, 23), // "FunctionSheetCellViewer"
QT_MOC_LITERAL(1, 24, 27), // "onCellEditorEditingFinished"
QT_MOC_LITERAL(2, 52, 0), // ""
QT_MOC_LITERAL(3, 53, 22), // "onMouseMovedInLineEdit"
QT_MOC_LITERAL(4, 76, 12), // "QMouseEvent*"
QT_MOC_LITERAL(5, 89, 5) // "event"

    },
    "FunctionSheetCellViewer\0"
    "onCellEditorEditingFinished\0\0"
    "onMouseMovedInLineEdit\0QMouseEvent*\0"
    "event"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionSheetCellViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x08 /* Private */,
       3,    1,   25,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,

       0        // eod
};

void FunctionSheetCellViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionSheetCellViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onCellEditorEditingFinished(); break;
        case 1: _t->onMouseMovedInLineEdit((*reinterpret_cast< QMouseEvent*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionSheetCellViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<Spreadsheet::CellPanel::staticMetaObject>(),
    qt_meta_stringdata_FunctionSheetCellViewer.data,
    qt_meta_data_FunctionSheetCellViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionSheetCellViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionSheetCellViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionSheetCellViewer.stringdata0))
        return static_cast<void*>(this);
    return Spreadsheet::CellPanel::qt_metacast(_clname);
}

int FunctionSheetCellViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Spreadsheet::CellPanel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_FunctionSheet_t {
    QByteArrayData data[8];
    char stringdata0[133];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FunctionSheet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FunctionSheet_t qt_meta_stringdata_FunctionSheet = {
    {
QT_MOC_LITERAL(0, 0, 13), // "FunctionSheet"
QT_MOC_LITERAL(1, 14, 9), // "updateAll"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 15), // "onFrameSwitched"
QT_MOC_LITERAL(4, 41, 23), // "onCurrentChannelChanged"
QT_MOC_LITERAL(5, 65, 27), // "FunctionTreeModel::Channel*"
QT_MOC_LITERAL(6, 93, 20), // "onSyncSizeBtnToggled"
QT_MOC_LITERAL(7, 114, 18) // "onZoomScaleChanged"

    },
    "FunctionSheet\0updateAll\0\0onFrameSwitched\0"
    "onCurrentChannelChanged\0"
    "FunctionTreeModel::Channel*\0"
    "onSyncSizeBtnToggled\0onZoomScaleChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FunctionSheet[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x0a /* Public */,
       3,    0,   40,    2, 0x0a /* Public */,
       4,    1,   41,    2, 0x0a /* Public */,
       6,    1,   44,    2, 0x0a /* Public */,
       7,    0,   47,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void FunctionSheet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FunctionSheet *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateAll(); break;
        case 1: _t->onFrameSwitched(); break;
        case 2: _t->onCurrentChannelChanged((*reinterpret_cast< FunctionTreeModel::Channel*(*)>(_a[1]))); break;
        case 3: _t->onSyncSizeBtnToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->onZoomScaleChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FunctionSheet::staticMetaObject = { {
    QMetaObject::SuperData::link<SpreadsheetViewer::staticMetaObject>(),
    qt_meta_stringdata_FunctionSheet.data,
    qt_meta_data_FunctionSheet,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FunctionSheet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FunctionSheet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FunctionSheet.stringdata0))
        return static_cast<void*>(this);
    return SpreadsheetViewer::qt_metacast(_clname);
}

int FunctionSheet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SpreadsheetViewer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
