/****************************************************************************
** Meta object code from reading C++ file 'schematicviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/schematicviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'schematicviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SchematicScene_t {
    QByteArrayData data[4];
    char stringdata0[48];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SchematicScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SchematicScene_t qt_meta_stringdata_SchematicScene = {
    {
QT_MOC_LITERAL(0, 0, 14), // "SchematicScene"
QT_MOC_LITERAL(1, 15, 19), // "onSelectionSwitched"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 11) // "TSelection*"

    },
    "SchematicScene\0onSelectionSwitched\0\0"
    "TSelection*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SchematicScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    2,    2,

       0        // eod
};

void SchematicScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SchematicScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSelectionSwitched((*reinterpret_cast< TSelection*(*)>(_a[1])),(*reinterpret_cast< TSelection*(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SchematicScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_meta_stringdata_SchematicScene.data,
    qt_meta_data_SchematicScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SchematicScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SchematicScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SchematicScene.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int SchematicScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_SchematicSceneViewer_t {
    QByteArrayData data[6];
    char stringdata0[75];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SchematicSceneViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SchematicSceneViewer_t qt_meta_stringdata_SchematicSceneViewer = {
    {
QT_MOC_LITERAL(0, 0, 20), // "SchematicSceneViewer"
QT_MOC_LITERAL(1, 21, 15), // "centerOnCurrent"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 12), // "reorderScene"
QT_MOC_LITERAL(4, 51, 14), // "normalizeScene"
QT_MOC_LITERAL(5, 66, 8) // "fitScene"

    },
    "SchematicSceneViewer\0centerOnCurrent\0"
    "\0reorderScene\0normalizeScene\0fitScene"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SchematicSceneViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x09 /* Protected */,
       3,    0,   35,    2, 0x09 /* Protected */,
       4,    0,   36,    2, 0x0a /* Public */,
       5,    0,   37,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SchematicSceneViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SchematicSceneViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->centerOnCurrent(); break;
        case 1: _t->reorderScene(); break;
        case 2: _t->normalizeScene(); break;
        case 3: _t->fitScene(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SchematicSceneViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsView::staticMetaObject>(),
    qt_meta_stringdata_SchematicSceneViewer.data,
    qt_meta_data_SchematicSceneViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SchematicSceneViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SchematicSceneViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SchematicSceneViewer.stringdata0))
        return static_cast<void*>(this);
    return QGraphicsView::qt_metacast(_clname);
}

int SchematicSceneViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_SchematicViewer_t {
    QByteArrayData data[66];
    char stringdata0[1188];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SchematicViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SchematicViewer_t qt_meta_stringdata_SchematicViewer = {
    {
QT_MOC_LITERAL(0, 0, 15), // "SchematicViewer"
QT_MOC_LITERAL(1, 16, 11), // "showPreview"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 4), // "TFxP"
QT_MOC_LITERAL(4, 34, 10), // "doCollapse"
QT_MOC_LITERAL(5, 45, 11), // "QList<TFxP>"
QT_MOC_LITERAL(6, 57, 21), // "QList<TStageObjectId>"
QT_MOC_LITERAL(7, 79, 14), // "doExplodeChild"
QT_MOC_LITERAL(8, 94, 10), // "editObject"
QT_MOC_LITERAL(9, 105, 11), // "doDeleteFxs"
QT_MOC_LITERAL(10, 117, 18), // "const FxSelection*"
QT_MOC_LITERAL(11, 136, 20), // "doDeleteStageObjects"
QT_MOC_LITERAL(12, 157, 27), // "const StageObjectSelection*"
QT_MOC_LITERAL(13, 185, 12), // "columnPasted"
QT_MOC_LITERAL(14, 198, 18), // "QList<TXshColumnP>"
QT_MOC_LITERAL(15, 217, 15), // "updateSchematic"
QT_MOC_LITERAL(16, 233, 14), // "onSceneChanged"
QT_MOC_LITERAL(17, 248, 15), // "onSceneSwitched"
QT_MOC_LITERAL(18, 264, 12), // "updateScenes"
QT_MOC_LITERAL(19, 277, 14), // "changeNodeSize"
QT_MOC_LITERAL(20, 292, 17), // "selectModeEnabled"
QT_MOC_LITERAL(21, 310, 15), // "zoomModeEnabled"
QT_MOC_LITERAL(22, 326, 15), // "handModeEnabled"
QT_MOC_LITERAL(23, 342, 9), // "deleteFxs"
QT_MOC_LITERAL(24, 352, 18), // "deleteStageObjects"
QT_MOC_LITERAL(25, 371, 9), // "TextColor"
QT_MOC_LITERAL(26, 381, 20), // "PassThroughTextColor"
QT_MOC_LITERAL(27, 402, 17), // "VerticalLineColor"
QT_MOC_LITERAL(28, 420, 9), // "LinkColor"
QT_MOC_LITERAL(29, 430, 17), // "SelectedLinkColor"
QT_MOC_LITERAL(30, 448, 19), // "SelectedBorderColor"
QT_MOC_LITERAL(31, 468, 19), // "MotionPathLinkColor"
QT_MOC_LITERAL(32, 488, 27), // "MotionPathSelectedLinkColor"
QT_MOC_LITERAL(33, 516, 16), // "LevelColumnColor"
QT_MOC_LITERAL(34, 533, 17), // "VectorColumnColor"
QT_MOC_LITERAL(35, 551, 16), // "ChildColumnColor"
QT_MOC_LITERAL(36, 568, 20), // "FullcolorColumnColor"
QT_MOC_LITERAL(37, 589, 13), // "FxColumnColor"
QT_MOC_LITERAL(38, 603, 18), // "PaletteColumnColor"
QT_MOC_LITERAL(39, 622, 15), // "MeshColumnColor"
QT_MOC_LITERAL(40, 638, 15), // "MetaColumnColor"
QT_MOC_LITERAL(41, 654, 20), // "ReferenceColumnColor"
QT_MOC_LITERAL(42, 675, 10), // "TableColor"
QT_MOC_LITERAL(43, 686, 17), // "ActiveCameraColor"
QT_MOC_LITERAL(44, 704, 16), // "OtherCameraColor"
QT_MOC_LITERAL(45, 721, 10), // "GroupColor"
QT_MOC_LITERAL(46, 732, 8), // "PegColor"
QT_MOC_LITERAL(47, 741, 11), // "SplineColor"
QT_MOC_LITERAL(48, 753, 17), // "ActiveOutputColor"
QT_MOC_LITERAL(49, 771, 16), // "OtherOutputColor"
QT_MOC_LITERAL(50, 788, 11), // "XsheetColor"
QT_MOC_LITERAL(51, 800, 16), // "PassThroughColor"
QT_MOC_LITERAL(52, 817, 13), // "NormalFxColor"
QT_MOC_LITERAL(53, 831, 12), // "MacroFxColor"
QT_MOC_LITERAL(54, 844, 18), // "ImageAdjustFxColor"
QT_MOC_LITERAL(55, 863, 20), // "LayerBlendingFxColor"
QT_MOC_LITERAL(56, 884, 12), // "MatteFxColor"
QT_MOC_LITERAL(57, 897, 31), // "SchematicPreviewButtonBgOnColor"
QT_MOC_LITERAL(58, 929, 29), // "SchematicPreviewButtonOnImage"
QT_MOC_LITERAL(59, 959, 32), // "SchematicPreviewButtonBgOffColor"
QT_MOC_LITERAL(60, 992, 30), // "SchematicPreviewButtonOffImage"
QT_MOC_LITERAL(61, 1023, 32), // "SchematicCamstandButtonBgOnColor"
QT_MOC_LITERAL(62, 1056, 30), // "SchematicCamstandButtonOnImage"
QT_MOC_LITERAL(63, 1087, 34), // "SchematicCamstandButtonTransp..."
QT_MOC_LITERAL(64, 1122, 33), // "SchematicCamstandButtonBgOffC..."
QT_MOC_LITERAL(65, 1156, 31) // "SchematicCamstandButtonOffImage"

    },
    "SchematicViewer\0showPreview\0\0TFxP\0"
    "doCollapse\0QList<TFxP>\0QList<TStageObjectId>\0"
    "doExplodeChild\0editObject\0doDeleteFxs\0"
    "const FxSelection*\0doDeleteStageObjects\0"
    "const StageObjectSelection*\0columnPasted\0"
    "QList<TXshColumnP>\0updateSchematic\0"
    "onSceneChanged\0onSceneSwitched\0"
    "updateScenes\0changeNodeSize\0"
    "selectModeEnabled\0zoomModeEnabled\0"
    "handModeEnabled\0deleteFxs\0deleteStageObjects\0"
    "TextColor\0PassThroughTextColor\0"
    "VerticalLineColor\0LinkColor\0"
    "SelectedLinkColor\0SelectedBorderColor\0"
    "MotionPathLinkColor\0MotionPathSelectedLinkColor\0"
    "LevelColumnColor\0VectorColumnColor\0"
    "ChildColumnColor\0FullcolorColumnColor\0"
    "FxColumnColor\0PaletteColumnColor\0"
    "MeshColumnColor\0MetaColumnColor\0"
    "ReferenceColumnColor\0TableColor\0"
    "ActiveCameraColor\0OtherCameraColor\0"
    "GroupColor\0PegColor\0SplineColor\0"
    "ActiveOutputColor\0OtherOutputColor\0"
    "XsheetColor\0PassThroughColor\0NormalFxColor\0"
    "MacroFxColor\0ImageAdjustFxColor\0"
    "LayerBlendingFxColor\0MatteFxColor\0"
    "SchematicPreviewButtonBgOnColor\0"
    "SchematicPreviewButtonOnImage\0"
    "SchematicPreviewButtonBgOffColor\0"
    "SchematicPreviewButtonOffImage\0"
    "SchematicCamstandButtonBgOnColor\0"
    "SchematicCamstandButtonOnImage\0"
    "SchematicCamstandButtonTranspImage\0"
    "SchematicCamstandButtonBgOffColor\0"
    "SchematicCamstandButtonOffImage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SchematicViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
      41,  144, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  109,    2, 0x06 /* Public */,
       4,    1,  112,    2, 0x06 /* Public */,
       4,    1,  115,    2, 0x06 /* Public */,
       7,    1,  118,    2, 0x06 /* Public */,
       7,    1,  121,    2, 0x06 /* Public */,
       8,    0,  124,    2, 0x06 /* Public */,
       9,    1,  125,    2, 0x06 /* Public */,
      11,    1,  128,    2, 0x06 /* Public */,
      13,    1,  131,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,  134,    2, 0x0a /* Public */,
      16,    0,  135,    2, 0x09 /* Protected */,
      17,    0,  136,    2, 0x09 /* Protected */,
      18,    0,  137,    2, 0x09 /* Protected */,
      19,    0,  138,    2, 0x09 /* Protected */,
      20,    0,  139,    2, 0x09 /* Protected */,
      21,    0,  140,    2, 0x09 /* Protected */,
      22,    0,  141,    2, 0x09 /* Protected */,
      23,    0,  142,    2, 0x09 /* Protected */,
      24,    0,  143,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, 0x80000000 | 5,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,    2,
    QMetaType::Void, 0x80000000 | 12,    2,
    QMetaType::Void, 0x80000000 | 14,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      25, QMetaType::QColor, 0x00095103,
      26, QMetaType::QColor, 0x00095103,
      27, QMetaType::QColor, 0x00095103,
      28, QMetaType::QColor, 0x00095103,
      29, QMetaType::QColor, 0x00095103,
      30, QMetaType::QColor, 0x00095103,
      31, QMetaType::QColor, 0x00095103,
      32, QMetaType::QColor, 0x00095103,
      33, QMetaType::QColor, 0x00095103,
      34, QMetaType::QColor, 0x00095103,
      35, QMetaType::QColor, 0x00095103,
      36, QMetaType::QColor, 0x00095103,
      37, QMetaType::QColor, 0x00095103,
      38, QMetaType::QColor, 0x00095103,
      39, QMetaType::QColor, 0x00095103,
      40, QMetaType::QColor, 0x00095103,
      41, QMetaType::QColor, 0x00095003,
      42, QMetaType::QColor, 0x00095103,
      43, QMetaType::QColor, 0x00095103,
      44, QMetaType::QColor, 0x00095103,
      45, QMetaType::QColor, 0x00095103,
      46, QMetaType::QColor, 0x00095103,
      47, QMetaType::QColor, 0x00095103,
      48, QMetaType::QColor, 0x00095103,
      49, QMetaType::QColor, 0x00095103,
      50, QMetaType::QColor, 0x00095103,
      51, QMetaType::QColor, 0x00095103,
      52, QMetaType::QColor, 0x00095103,
      53, QMetaType::QColor, 0x00095103,
      54, QMetaType::QColor, 0x00095103,
      55, QMetaType::QColor, 0x00095103,
      56, QMetaType::QColor, 0x00095103,
      57, QMetaType::QColor, 0x00095103,
      58, QMetaType::QIcon, 0x00095103,
      59, QMetaType::QColor, 0x00095103,
      60, QMetaType::QIcon, 0x00095103,
      61, QMetaType::QColor, 0x00095103,
      62, QMetaType::QIcon, 0x00095103,
      63, QMetaType::QIcon, 0x00095103,
      64, QMetaType::QColor, 0x00095103,
      65, QMetaType::QIcon, 0x00095103,

       0        // eod
};

void SchematicViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SchematicViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->showPreview((*reinterpret_cast< TFxP(*)>(_a[1]))); break;
        case 1: _t->doCollapse((*reinterpret_cast< const QList<TFxP>(*)>(_a[1]))); break;
        case 2: _t->doCollapse((*reinterpret_cast< QList<TStageObjectId>(*)>(_a[1]))); break;
        case 3: _t->doExplodeChild((*reinterpret_cast< const QList<TFxP>(*)>(_a[1]))); break;
        case 4: _t->doExplodeChild((*reinterpret_cast< QList<TStageObjectId>(*)>(_a[1]))); break;
        case 5: _t->editObject(); break;
        case 6: _t->doDeleteFxs((*reinterpret_cast< const FxSelection*(*)>(_a[1]))); break;
        case 7: _t->doDeleteStageObjects((*reinterpret_cast< const StageObjectSelection*(*)>(_a[1]))); break;
        case 8: _t->columnPasted((*reinterpret_cast< const QList<TXshColumnP>(*)>(_a[1]))); break;
        case 9: _t->updateSchematic(); break;
        case 10: _t->onSceneChanged(); break;
        case 11: _t->onSceneSwitched(); break;
        case 12: _t->updateScenes(); break;
        case 13: _t->changeNodeSize(); break;
        case 14: _t->selectModeEnabled(); break;
        case 15: _t->zoomModeEnabled(); break;
        case 16: _t->handModeEnabled(); break;
        case 17: _t->deleteFxs(); break;
        case 18: _t->deleteStageObjects(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SchematicViewer::*)(TFxP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::showPreview)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)(const QList<TFxP> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::doCollapse)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)(QList<TStageObjectId> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::doCollapse)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)(const QList<TFxP> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::doExplodeChild)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)(QList<TStageObjectId> );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::doExplodeChild)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::editObject)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)(const FxSelection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::doDeleteFxs)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)(const StageObjectSelection * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::doDeleteStageObjects)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (SchematicViewer::*)(const QList<TXshColumnP> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SchematicViewer::columnPasted)) {
                *result = 8;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<SchematicViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getPassThroughTextColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getVerticalLineColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getLinkColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getSelectedLinkColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getSelectedBorderColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getMotionPathLinkColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->getMotionPathSelectedLinkColor(); break;
        case 8: *reinterpret_cast< QColor*>(_v) = _t->getLevelColumnColor(); break;
        case 9: *reinterpret_cast< QColor*>(_v) = _t->getVectorColumnColor(); break;
        case 10: *reinterpret_cast< QColor*>(_v) = _t->getChildColumnColor(); break;
        case 11: *reinterpret_cast< QColor*>(_v) = _t->getFullcolorColumnColor(); break;
        case 12: *reinterpret_cast< QColor*>(_v) = _t->getFxColumnColor(); break;
        case 13: *reinterpret_cast< QColor*>(_v) = _t->getPaletteColumnColor(); break;
        case 14: *reinterpret_cast< QColor*>(_v) = _t->getMeshColumnColor(); break;
        case 15: *reinterpret_cast< QColor*>(_v) = _t->getMetaColumnColor(); break;
        case 16: *reinterpret_cast< QColor*>(_v) = _t->m_referenceColumnColor; break;
        case 17: *reinterpret_cast< QColor*>(_v) = _t->getTableColor(); break;
        case 18: *reinterpret_cast< QColor*>(_v) = _t->getActiveCameraColor(); break;
        case 19: *reinterpret_cast< QColor*>(_v) = _t->getOtherCameraColor(); break;
        case 20: *reinterpret_cast< QColor*>(_v) = _t->getGroupColor(); break;
        case 21: *reinterpret_cast< QColor*>(_v) = _t->getPegColor(); break;
        case 22: *reinterpret_cast< QColor*>(_v) = _t->getSplineColor(); break;
        case 23: *reinterpret_cast< QColor*>(_v) = _t->getActiveOutputColor(); break;
        case 24: *reinterpret_cast< QColor*>(_v) = _t->getOtherOutputColor(); break;
        case 25: *reinterpret_cast< QColor*>(_v) = _t->getXsheetColor(); break;
        case 26: *reinterpret_cast< QColor*>(_v) = _t->getPassThroughColor(); break;
        case 27: *reinterpret_cast< QColor*>(_v) = _t->getNormalFxColor(); break;
        case 28: *reinterpret_cast< QColor*>(_v) = _t->getMacroFxColor(); break;
        case 29: *reinterpret_cast< QColor*>(_v) = _t->getImageAdjustFxColor(); break;
        case 30: *reinterpret_cast< QColor*>(_v) = _t->getLayerBlendingFxColor(); break;
        case 31: *reinterpret_cast< QColor*>(_v) = _t->getMatteFxColor(); break;
        case 32: *reinterpret_cast< QColor*>(_v) = _t->getSchematicPreviewButtonBgOnColor(); break;
        case 33: *reinterpret_cast< QIcon*>(_v) = _t->getSchematicPreviewButtonOnImage(); break;
        case 34: *reinterpret_cast< QColor*>(_v) = _t->getSchematicPreviewButtonBgOffColor(); break;
        case 35: *reinterpret_cast< QIcon*>(_v) = _t->getSchematicPreviewButtonOffImage(); break;
        case 36: *reinterpret_cast< QColor*>(_v) = _t->getSchematicCamstandButtonBgOnColor(); break;
        case 37: *reinterpret_cast< QIcon*>(_v) = _t->getSchematicCamstandButtonOnImage(); break;
        case 38: *reinterpret_cast< QIcon*>(_v) = _t->getSchematicCamstandButtonTranspImage(); break;
        case 39: *reinterpret_cast< QColor*>(_v) = _t->getSchematicCamstandButtonBgOffColor(); break;
        case 40: *reinterpret_cast< QIcon*>(_v) = _t->getSchematicCamstandButtonOffImage(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<SchematicViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setPassThroughTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setVerticalLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setLinkColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setSelectedLinkColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setSelectedBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setMotionPathLinkColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setMotionPathSelectedLinkColor(*reinterpret_cast< QColor*>(_v)); break;
        case 8: _t->setLevelColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 9: _t->setVectorColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 10: _t->setChildColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 11: _t->setFullcolorColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 12: _t->setFxColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 13: _t->setPaletteColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 14: _t->setMeshColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 15: _t->setMetaColumnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 16:
            if (_t->m_referenceColumnColor != *reinterpret_cast< QColor*>(_v)) {
                _t->m_referenceColumnColor = *reinterpret_cast< QColor*>(_v);
            }
            break;
        case 17: _t->setTableColor(*reinterpret_cast< QColor*>(_v)); break;
        case 18: _t->setActiveCameraColor(*reinterpret_cast< QColor*>(_v)); break;
        case 19: _t->setOtherCameraColor(*reinterpret_cast< QColor*>(_v)); break;
        case 20: _t->setGroupColor(*reinterpret_cast< QColor*>(_v)); break;
        case 21: _t->setPegColor(*reinterpret_cast< QColor*>(_v)); break;
        case 22: _t->setSplineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 23: _t->setActiveOutputColor(*reinterpret_cast< QColor*>(_v)); break;
        case 24: _t->setOtherOutputColor(*reinterpret_cast< QColor*>(_v)); break;
        case 25: _t->setXsheetColor(*reinterpret_cast< QColor*>(_v)); break;
        case 26: _t->setPassThroughColor(*reinterpret_cast< QColor*>(_v)); break;
        case 27: _t->setNormalFxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 28: _t->setMacroFxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 29: _t->setImageAdjustFxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 30: _t->setLayerBlendingFxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 31: _t->setMatteFxColor(*reinterpret_cast< QColor*>(_v)); break;
        case 32: _t->setSchematicPreviewButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 33: _t->setSchematicPreviewButtonOnImage(*reinterpret_cast< QIcon*>(_v)); break;
        case 34: _t->setSchematicPreviewButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 35: _t->setSchematicPreviewButtonOffImage(*reinterpret_cast< QIcon*>(_v)); break;
        case 36: _t->setSchematicCamstandButtonBgOnColor(*reinterpret_cast< QColor*>(_v)); break;
        case 37: _t->setSchematicCamstandButtonOnImage(*reinterpret_cast< QIcon*>(_v)); break;
        case 38: _t->setSchematicCamstandButtonTranspImage(*reinterpret_cast< QIcon*>(_v)); break;
        case 39: _t->setSchematicCamstandButtonBgOffColor(*reinterpret_cast< QColor*>(_v)); break;
        case 40: _t->setSchematicCamstandButtonOffImage(*reinterpret_cast< QIcon*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject SchematicViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SchematicViewer.data,
    qt_meta_data_SchematicViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SchematicViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SchematicViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SchematicViewer.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SchematicViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 41;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 41;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void SchematicViewer::showPreview(TFxP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void SchematicViewer::doCollapse(const QList<TFxP> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void SchematicViewer::doCollapse(QList<TStageObjectId> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void SchematicViewer::doExplodeChild(const QList<TFxP> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void SchematicViewer::doExplodeChild(QList<TStageObjectId> _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void SchematicViewer::editObject()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void SchematicViewer::doDeleteFxs(const FxSelection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void SchematicViewer::doDeleteStageObjects(const StageObjectSelection * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void SchematicViewer::columnPasted(const QList<TXshColumnP> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
