/****************************************************************************
** Meta object code from reading C++ file 'stopmotion.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../opentoonz/toonz/sources/stopmotion/stopmotion.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'stopmotion.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StopMotion_t {
    QByteArrayData data[41];
    char stringdata0[741];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StopMotion_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StopMotion_t qt_meta_stringdata_StopMotion = {
    {
QT_MOC_LITERAL(0, 0, 10), // "StopMotion"
QT_MOC_LITERAL(1, 11, 13), // "cameraChanged"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 17), // "newCameraSelected"
QT_MOC_LITERAL(4, 44, 24), // "webcamResolutionsChanged"
QT_MOC_LITERAL(5, 69, 27), // "newWebcamResolutionSelected"
QT_MOC_LITERAL(6, 97, 16), // "updateCameraList"
QT_MOC_LITERAL(7, 114, 21), // "newLiveViewImageReady"
QT_MOC_LITERAL(8, 136, 15), // "liveViewStopped"
QT_MOC_LITERAL(9, 152, 13), // "newImageReady"
QT_MOC_LITERAL(10, 166, 15), // "liveViewChanged"
QT_MOC_LITERAL(11, 182, 25), // "liveViewOnAllFramesSignal"
QT_MOC_LITERAL(12, 208, 13), // "newDimensions"
QT_MOC_LITERAL(13, 222, 30), // "alwaysUseLiveViewImagesToggled"
QT_MOC_LITERAL(14, 253, 15), // "filePathChanged"
QT_MOC_LITERAL(15, 269, 16), // "levelNameChanged"
QT_MOC_LITERAL(16, 286, 15), // "fileTypeChanged"
QT_MOC_LITERAL(17, 302, 18), // "frameNumberChanged"
QT_MOC_LITERAL(18, 321, 20), // "frameInfoTextChanged"
QT_MOC_LITERAL(19, 342, 24), // "xSheetFrameNumberChanged"
QT_MOC_LITERAL(20, 367, 14), // "optionsChanged"
QT_MOC_LITERAL(21, 382, 18), // "subsamplingChanged"
QT_MOC_LITERAL(22, 401, 14), // "opacityChanged"
QT_MOC_LITERAL(23, 416, 19), // "placeOnXSheetSignal"
QT_MOC_LITERAL(24, 436, 15), // "useNumpadSignal"
QT_MOC_LITERAL(25, 452, 23), // "drawBeneathLevelsSignal"
QT_MOC_LITERAL(26, 476, 23), // "reviewTimeChangedSignal"
QT_MOC_LITERAL(27, 500, 17), // "playCaptureSignal"
QT_MOC_LITERAL(28, 518, 15), // "intervalToggled"
QT_MOC_LITERAL(29, 534, 15), // "intervalStarted"
QT_MOC_LITERAL(30, 550, 15), // "intervalStopped"
QT_MOC_LITERAL(31, 566, 21), // "intervalAmountChanged"
QT_MOC_LITERAL(32, 588, 9), // "onTimeout"
QT_MOC_LITERAL(33, 598, 15), // "onReviewTimeout"
QT_MOC_LITERAL(34, 614, 29), // "onIntervalCaptureTimerTimeout"
QT_MOC_LITERAL(35, 644, 22), // "captureWebcamOnTimeout"
QT_MOC_LITERAL(36, 667, 6), // "update"
QT_MOC_LITERAL(37, 674, 11), // "importImage"
QT_MOC_LITERAL(38, 686, 15), // "onSceneSwitched"
QT_MOC_LITERAL(39, 702, 17), // "onPlaybackChanged"
QT_MOC_LITERAL(40, 720, 20) // "onCanonCameraChanged"

    },
    "StopMotion\0cameraChanged\0\0newCameraSelected\0"
    "webcamResolutionsChanged\0"
    "newWebcamResolutionSelected\0"
    "updateCameraList\0newLiveViewImageReady\0"
    "liveViewStopped\0newImageReady\0"
    "liveViewChanged\0liveViewOnAllFramesSignal\0"
    "newDimensions\0alwaysUseLiveViewImagesToggled\0"
    "filePathChanged\0levelNameChanged\0"
    "fileTypeChanged\0frameNumberChanged\0"
    "frameInfoTextChanged\0xSheetFrameNumberChanged\0"
    "optionsChanged\0subsamplingChanged\0"
    "opacityChanged\0placeOnXSheetSignal\0"
    "useNumpadSignal\0drawBeneathLevelsSignal\0"
    "reviewTimeChangedSignal\0playCaptureSignal\0"
    "intervalToggled\0intervalStarted\0"
    "intervalStopped\0intervalAmountChanged\0"
    "onTimeout\0onReviewTimeout\0"
    "onIntervalCaptureTimerTimeout\0"
    "captureWebcamOnTimeout\0update\0importImage\0"
    "onSceneSwitched\0onPlaybackChanged\0"
    "onCanonCameraChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StopMotion[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      39,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      30,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  209,    2, 0x06 /* Public */,
       3,    2,  212,    2, 0x06 /* Public */,
       4,    0,  217,    2, 0x06 /* Public */,
       5,    1,  218,    2, 0x06 /* Public */,
       6,    1,  221,    2, 0x06 /* Public */,
       7,    0,  224,    2, 0x06 /* Public */,
       8,    0,  225,    2, 0x06 /* Public */,
       9,    0,  226,    2, 0x06 /* Public */,
      10,    1,  227,    2, 0x06 /* Public */,
      11,    1,  230,    2, 0x06 /* Public */,
      12,    0,  233,    2, 0x06 /* Public */,
      13,    1,  234,    2, 0x06 /* Public */,
      14,    1,  237,    2, 0x06 /* Public */,
      15,    1,  240,    2, 0x06 /* Public */,
      16,    1,  243,    2, 0x06 /* Public */,
      17,    1,  246,    2, 0x06 /* Public */,
      18,    1,  249,    2, 0x06 /* Public */,
      19,    1,  252,    2, 0x06 /* Public */,
      20,    0,  255,    2, 0x06 /* Public */,
      21,    1,  256,    2, 0x06 /* Public */,
      22,    1,  259,    2, 0x06 /* Public */,
      23,    1,  262,    2, 0x06 /* Public */,
      24,    1,  265,    2, 0x06 /* Public */,
      25,    1,  268,    2, 0x06 /* Public */,
      26,    1,  271,    2, 0x06 /* Public */,
      27,    1,  274,    2, 0x06 /* Public */,
      28,    1,  277,    2, 0x06 /* Public */,
      29,    0,  280,    2, 0x06 /* Public */,
      30,    0,  281,    2, 0x06 /* Public */,
      31,    1,  282,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      32,    0,  285,    2, 0x0a /* Public */,
      33,    0,  286,    2, 0x0a /* Public */,
      34,    0,  287,    2, 0x0a /* Public */,
      35,    0,  288,    2, 0x0a /* Public */,
      36,    0,  289,    2, 0x0a /* Public */,
      37,    0,  290,    2, 0x0a /* Public */,
      38,    0,  291,    2, 0x0a /* Public */,
      39,    0,  292,    2, 0x0a /* Public */,
      40,    1,  293,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void StopMotion::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StopMotion *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cameraChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->newCameraSelected((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->webcamResolutionsChanged(); break;
        case 3: _t->newWebcamResolutionSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->updateCameraList((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->newLiveViewImageReady(); break;
        case 6: _t->liveViewStopped(); break;
        case 7: _t->newImageReady(); break;
        case 8: _t->liveViewChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->liveViewOnAllFramesSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->newDimensions(); break;
        case 11: _t->alwaysUseLiveViewImagesToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->filePathChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 13: _t->levelNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 14: _t->fileTypeChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 15: _t->frameNumberChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->frameInfoTextChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 17: _t->xSheetFrameNumberChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->optionsChanged(); break;
        case 19: _t->subsamplingChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->opacityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->placeOnXSheetSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->useNumpadSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->drawBeneathLevelsSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->reviewTimeChangedSignal((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->playCaptureSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 26: _t->intervalToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 27: _t->intervalStarted(); break;
        case 28: _t->intervalStopped(); break;
        case 29: _t->intervalAmountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 30: _t->onTimeout(); break;
        case 31: _t->onReviewTimeout(); break;
        case 32: _t->onIntervalCaptureTimerTimeout(); break;
        case 33: _t->captureWebcamOnTimeout(); break;
        case 34: _t->update(); break;
        case 35: { bool _r = _t->importImage();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->onSceneSwitched(); break;
        case 37: _t->onPlaybackChanged(); break;
        case 38: _t->onCanonCameraChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StopMotion::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::cameraChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::newCameraSelected)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::webcamResolutionsChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::newWebcamResolutionSelected)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::updateCameraList)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::newLiveViewImageReady)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::liveViewStopped)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::newImageReady)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::liveViewChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::liveViewOnAllFramesSignal)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::newDimensions)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::alwaysUseLiveViewImagesToggled)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::filePathChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::levelNameChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::fileTypeChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::frameNumberChanged)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::frameInfoTextChanged)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::xSheetFrameNumberChanged)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::optionsChanged)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::subsamplingChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::opacityChanged)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::placeOnXSheetSignal)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::useNumpadSignal)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::drawBeneathLevelsSignal)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::reviewTimeChangedSignal)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::playCaptureSignal)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::intervalToggled)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::intervalStarted)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::intervalStopped)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (StopMotion::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StopMotion::intervalAmountChanged)) {
                *result = 29;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StopMotion::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_StopMotion.data,
    qt_meta_data_StopMotion,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StopMotion::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StopMotion::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StopMotion.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int StopMotion::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 39;
    }
    return _id;
}

// SIGNAL 0
void StopMotion::cameraChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void StopMotion::newCameraSelected(int _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void StopMotion::webcamResolutionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void StopMotion::newWebcamResolutionSelected(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void StopMotion::updateCameraList(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void StopMotion::newLiveViewImageReady()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void StopMotion::liveViewStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void StopMotion::newImageReady()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void StopMotion::liveViewChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void StopMotion::liveViewOnAllFramesSignal(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void StopMotion::newDimensions()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void StopMotion::alwaysUseLiveViewImagesToggled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void StopMotion::filePathChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void StopMotion::levelNameChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void StopMotion::fileTypeChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void StopMotion::frameNumberChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void StopMotion::frameInfoTextChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void StopMotion::xSheetFrameNumberChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 17, _a);
}

// SIGNAL 18
void StopMotion::optionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void StopMotion::subsamplingChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void StopMotion::opacityChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void StopMotion::placeOnXSheetSignal(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void StopMotion::useNumpadSignal(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 22, _a);
}

// SIGNAL 23
void StopMotion::drawBeneathLevelsSignal(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void StopMotion::reviewTimeChangedSignal(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 24, _a);
}

// SIGNAL 25
void StopMotion::playCaptureSignal(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}

// SIGNAL 26
void StopMotion::intervalToggled(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 26, _a);
}

// SIGNAL 27
void StopMotion::intervalStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void StopMotion::intervalStopped()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void StopMotion::intervalAmountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 29, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
