/****************************************************************************
** Meta object code from reading C++ file 'filmstrip.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/filmstrip.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'filmstrip.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FilmstripFrames_t {
    QByteArrayData data[19];
    char stringdata0[305];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FilmstripFrames_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FilmstripFrames_t qt_meta_stringdata_FilmstripFrames = {
    {
QT_MOC_LITERAL(0, 0, 15), // "FilmstripFrames"
QT_MOC_LITERAL(1, 16, 24), // "orientationToggledSignal"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 21), // "comboBoxToggledSignal"
QT_MOC_LITERAL(4, 64, 22), // "navigatorToggledSignal"
QT_MOC_LITERAL(5, 87, 19), // "levelSelectedSignal"
QT_MOC_LITERAL(6, 107, 14), // "onLevelChanged"
QT_MOC_LITERAL(7, 122, 15), // "onLevelSwitched"
QT_MOC_LITERAL(8, 138, 10), // "TXshLevel*"
QT_MOC_LITERAL(9, 149, 15), // "onFrameSwitched"
QT_MOC_LITERAL(10, 165, 9), // "getViewer"
QT_MOC_LITERAL(11, 175, 18), // "orientationToggled"
QT_MOC_LITERAL(12, 194, 15), // "comboBoxToggled"
QT_MOC_LITERAL(13, 210, 16), // "navigatorToggled"
QT_MOC_LITERAL(14, 227, 13), // "levelSelected"
QT_MOC_LITERAL(15, 241, 26), // "onViewerAboutToBeDestroyed"
QT_MOC_LITERAL(16, 268, 7), // "BGColor"
QT_MOC_LITERAL(17, 276, 14), // "LightLineColor"
QT_MOC_LITERAL(18, 291, 13) // "DarkLineColor"

    },
    "FilmstripFrames\0orientationToggledSignal\0"
    "\0comboBoxToggledSignal\0navigatorToggledSignal\0"
    "levelSelectedSignal\0onLevelChanged\0"
    "onLevelSwitched\0TXshLevel*\0onFrameSwitched\0"
    "getViewer\0orientationToggled\0"
    "comboBoxToggled\0navigatorToggled\0"
    "levelSelected\0onViewerAboutToBeDestroyed\0"
    "BGColor\0LightLineColor\0DarkLineColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FilmstripFrames[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       3,  106, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       3,    0,   82,    2, 0x06 /* Public */,
       4,    0,   83,    2, 0x06 /* Public */,
       5,    1,   84,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,   87,    2, 0x09 /* Protected */,
       7,    1,   88,    2, 0x09 /* Protected */,
       9,    0,   91,    2, 0x09 /* Protected */,
      10,    0,   92,    2, 0x09 /* Protected */,
      11,    1,   93,    2, 0x09 /* Protected */,
      12,    1,   96,    2, 0x09 /* Protected */,
      13,    1,   99,    2, 0x09 /* Protected */,
      14,    1,  102,    2, 0x09 /* Protected */,
      15,    0,  105,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

 // properties: name, type, flags
      16, QMetaType::QColor, 0x00095103,
      17, QMetaType::QColor, 0x00095103,
      18, QMetaType::QColor, 0x00095103,

       0        // eod
};

void FilmstripFrames::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FilmstripFrames *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->orientationToggledSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->comboBoxToggledSignal(); break;
        case 2: _t->navigatorToggledSignal(); break;
        case 3: _t->levelSelectedSignal((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->onLevelChanged(); break;
        case 5: _t->onLevelSwitched((*reinterpret_cast< TXshLevel*(*)>(_a[1]))); break;
        case 6: _t->onFrameSwitched(); break;
        case 7: _t->getViewer(); break;
        case 8: _t->orientationToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->comboBoxToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->navigatorToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->levelSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->onViewerAboutToBeDestroyed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FilmstripFrames::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilmstripFrames::orientationToggledSignal)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FilmstripFrames::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilmstripFrames::comboBoxToggledSignal)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FilmstripFrames::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilmstripFrames::navigatorToggledSignal)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FilmstripFrames::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FilmstripFrames::levelSelectedSignal)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<FilmstripFrames *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getBGColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getLightLineColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getDarkLineColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<FilmstripFrames *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setLightLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setDarkLineColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject FilmstripFrames::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_FilmstripFrames.data,
    qt_meta_data_FilmstripFrames,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FilmstripFrames::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FilmstripFrames::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FilmstripFrames.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSelection::View"))
        return static_cast< TSelection::View*>(this);
    return QFrame::qt_metacast(_clname);
}

int FilmstripFrames::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void FilmstripFrames::orientationToggledSignal(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FilmstripFrames::comboBoxToggledSignal()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FilmstripFrames::navigatorToggledSignal()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void FilmstripFrames::levelSelectedSignal(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_Filmstrip_t {
    QByteArrayData data[14];
    char stringdata0[204];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Filmstrip_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Filmstrip_t qt_meta_stringdata_Filmstrip = {
    {
QT_MOC_LITERAL(0, 0, 9), // "Filmstrip"
QT_MOC_LITERAL(1, 10, 15), // "onLevelSwitched"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 10), // "TXshLevel*"
QT_MOC_LITERAL(4, 38, 8), // "oldLevel"
QT_MOC_LITERAL(5, 47, 13), // "onSliderMoved"
QT_MOC_LITERAL(6, 61, 14), // "onLevelChanged"
QT_MOC_LITERAL(7, 76, 27), // "updateChooseLevelComboItems"
QT_MOC_LITERAL(8, 104, 25), // "onChooseLevelComboChanged"
QT_MOC_LITERAL(9, 130, 5), // "index"
QT_MOC_LITERAL(10, 136, 15), // "onFrameSwitched"
QT_MOC_LITERAL(11, 152, 18), // "orientationToggled"
QT_MOC_LITERAL(12, 171, 15), // "comboBoxToggled"
QT_MOC_LITERAL(13, 187, 16) // "navigatorToggled"

    },
    "Filmstrip\0onLevelSwitched\0\0TXshLevel*\0"
    "oldLevel\0onSliderMoved\0onLevelChanged\0"
    "updateChooseLevelComboItems\0"
    "onChooseLevelComboChanged\0index\0"
    "onFrameSwitched\0orientationToggled\0"
    "comboBoxToggled\0navigatorToggled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Filmstrip[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       5,    1,   62,    2, 0x0a /* Public */,
       6,    0,   65,    2, 0x0a /* Public */,
       7,    0,   66,    2, 0x0a /* Public */,
       8,    1,   67,    2, 0x0a /* Public */,
      10,    0,   70,    2, 0x0a /* Public */,
      11,    1,   71,    2, 0x0a /* Public */,
      12,    0,   74,    2, 0x0a /* Public */,
      13,    0,   75,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Filmstrip::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Filmstrip *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onLevelSwitched((*reinterpret_cast< TXshLevel*(*)>(_a[1]))); break;
        case 1: _t->onSliderMoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onLevelChanged(); break;
        case 3: _t->updateChooseLevelComboItems(); break;
        case 4: _t->onChooseLevelComboChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onFrameSwitched(); break;
        case 6: _t->orientationToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->comboBoxToggled(); break;
        case 8: _t->navigatorToggled(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Filmstrip::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Filmstrip.data,
    qt_meta_data_Filmstrip,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Filmstrip::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Filmstrip::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Filmstrip.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SaveLoadQSettings"))
        return static_cast< SaveLoadQSettings*>(this);
    return QWidget::qt_metacast(_clname);
}

int Filmstrip::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
struct qt_meta_stringdata_InbetweenDialog_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_InbetweenDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_InbetweenDialog_t qt_meta_stringdata_InbetweenDialog = {
    {
QT_MOC_LITERAL(0, 0, 15) // "InbetweenDialog"

    },
    "InbetweenDialog"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_InbetweenDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void InbetweenDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject InbetweenDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_InbetweenDialog.data,
    qt_meta_data_InbetweenDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *InbetweenDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InbetweenDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_InbetweenDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int InbetweenDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
