/****************************************************************************
** Meta object code from reading C++ file 'cleanupsettingsmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/cleanupsettingsmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cleanupsettingsmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CleanupSettingsModel_t {
    QByteArrayData data[14];
    char stringdata0[200];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CleanupSettingsModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CleanupSettingsModel_t qt_meta_stringdata_CleanupSettingsModel = {
    {
QT_MOC_LITERAL(0, 0, 20), // "CleanupSettingsModel"
QT_MOC_LITERAL(1, 21, 12), // "modelChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 16), // "needsPostProcess"
QT_MOC_LITERAL(4, 52, 13), // "imageSwitched"
QT_MOC_LITERAL(5, 66, 18), // "previewDataChanged"
QT_MOC_LITERAL(6, 85, 9), // "clnLoaded"
QT_MOC_LITERAL(7, 95, 13), // "commitChanges"
QT_MOC_LITERAL(8, 109, 10), // "promptSave"
QT_MOC_LITERAL(9, 120, 10), // "promptLoad"
QT_MOC_LITERAL(10, 131, 21), // "restoreGlobalSettings"
QT_MOC_LITERAL(11, 153, 15), // "onSceneSwitched"
QT_MOC_LITERAL(12, 169, 13), // "onCellChanged"
QT_MOC_LITERAL(13, 183, 16) // "onPaletteChanged"

    },
    "CleanupSettingsModel\0modelChanged\0\0"
    "needsPostProcess\0imageSwitched\0"
    "previewDataChanged\0clnLoaded\0commitChanges\0"
    "promptSave\0promptLoad\0restoreGlobalSettings\0"
    "onSceneSwitched\0onCellChanged\0"
    "onPaletteChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CleanupSettingsModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       4,    0,   72,    2, 0x06 /* Public */,
       5,    0,   73,    2, 0x06 /* Public */,
       6,    0,   74,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,   75,    2, 0x0a /* Public */,
       8,    0,   76,    2, 0x0a /* Public */,
       9,    0,   77,    2, 0x0a /* Public */,
      10,    0,   78,    2, 0x0a /* Public */,
      11,    0,   79,    2, 0x08 /* Private */,
      12,    0,   80,    2, 0x08 /* Private */,
      13,    0,   81,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void CleanupSettingsModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CleanupSettingsModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->modelChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->imageSwitched(); break;
        case 2: _t->previewDataChanged(); break;
        case 3: _t->clnLoaded(); break;
        case 4: _t->commitChanges(); break;
        case 5: _t->promptSave(); break;
        case 6: _t->promptLoad(); break;
        case 7: _t->restoreGlobalSettings(); break;
        case 8: _t->onSceneSwitched(); break;
        case 9: _t->onCellChanged(); break;
        case 10: _t->onPaletteChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CleanupSettingsModel::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CleanupSettingsModel::modelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CleanupSettingsModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CleanupSettingsModel::imageSwitched)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (CleanupSettingsModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CleanupSettingsModel::previewDataChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (CleanupSettingsModel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CleanupSettingsModel::clnLoaded)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CleanupSettingsModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_CleanupSettingsModel.data,
    qt_meta_data_CleanupSettingsModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CleanupSettingsModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CleanupSettingsModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CleanupSettingsModel.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CleanupSettingsModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void CleanupSettingsModel::modelChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void CleanupSettingsModel::imageSwitched()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CleanupSettingsModel::previewDataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CleanupSettingsModel::clnLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
