/****************************************************************************
** Meta object code from reading C++ file 'colorfield.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/colorfield.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'colorfield.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__CommonChessboard_t {
    QByteArrayData data[1];
    char stringdata0[24];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__CommonChessboard_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__CommonChessboard_t qt_meta_stringdata_DVGui__CommonChessboard = {
    {
QT_MOC_LITERAL(0, 0, 23) // "DVGui::CommonChessboard"

    },
    "DVGui::CommonChessboard"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__CommonChessboard[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void DVGui::CommonChessboard::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::CommonChessboard::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DVGui__CommonChessboard.data,
    qt_meta_data_DVGui__CommonChessboard,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::CommonChessboard::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::CommonChessboard::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__CommonChessboard.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DVGui::CommonChessboard::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_DVGui__StyleSample_t {
    QByteArrayData data[3];
    char stringdata0[28];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__StyleSample_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__StyleSample_t qt_meta_stringdata_DVGui__StyleSample = {
    {
QT_MOC_LITERAL(0, 0, 18), // "DVGui::StyleSample"
QT_MOC_LITERAL(1, 19, 7), // "clicked"
QT_MOC_LITERAL(2, 27, 0) // ""

    },
    "DVGui::StyleSample\0clicked\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__StyleSample[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void DVGui::StyleSample::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StyleSample *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StyleSample::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StyleSample::clicked)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVGui::StyleSample::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__StyleSample.data,
    qt_meta_data_DVGui__StyleSample,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::StyleSample::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::StyleSample::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__StyleSample.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::StyleSample::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DVGui::StyleSample::clicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_DVGui__ChannelField_t {
    QByteArrayData data[10];
    char stringdata0[117];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__ChannelField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__ChannelField_t qt_meta_stringdata_DVGui__ChannelField = {
    {
QT_MOC_LITERAL(0, 0, 19), // "DVGui::ChannelField"
QT_MOC_LITERAL(1, 20, 12), // "valueChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 5), // "value"
QT_MOC_LITERAL(4, 40, 10), // "isDragging"
QT_MOC_LITERAL(5, 51, 15), // "onSliderChanged"
QT_MOC_LITERAL(6, 67, 16), // "onSliderReleased"
QT_MOC_LITERAL(7, 84, 13), // "onEditChanged"
QT_MOC_LITERAL(8, 98, 3), // "str"
QT_MOC_LITERAL(9, 102, 14) // "onEditFinished"

    },
    "DVGui::ChannelField\0valueChanged\0\0"
    "value\0isDragging\0onSliderChanged\0"
    "onSliderReleased\0onEditChanged\0str\0"
    "onEditFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__ChannelField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   39,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   44,    2, 0x09 /* Protected */,
       6,    0,   47,    2, 0x09 /* Protected */,
       7,    1,   48,    2, 0x09 /* Protected */,
       9,    0,   51,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    3,    4,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,

       0        // eod
};

void DVGui::ChannelField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChannelField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->onSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onSliderReleased(); break;
        case 3: _t->onEditChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->onEditFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ChannelField::*)(int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChannelField::valueChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::ChannelField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__ChannelField.data,
    qt_meta_data_DVGui__ChannelField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::ChannelField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::ChannelField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__ChannelField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::ChannelField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void DVGui::ChannelField::valueChanged(int _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_DVGui__ColorField_t {
    QByteArrayData data[14];
    char stringdata0[193];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__ColorField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__ColorField_t qt_meta_stringdata_DVGui__ColorField = {
    {
QT_MOC_LITERAL(0, 0, 17), // "DVGui::ColorField"
QT_MOC_LITERAL(1, 18, 14), // "editingChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 8), // "TPixel32"
QT_MOC_LITERAL(4, 43, 9), // "isEditing"
QT_MOC_LITERAL(5, 53, 12), // "colorChanged"
QT_MOC_LITERAL(6, 66, 10), // "isDragging"
QT_MOC_LITERAL(7, 77, 19), // "onRedChannelChanged"
QT_MOC_LITERAL(8, 97, 5), // "value"
QT_MOC_LITERAL(9, 103, 21), // "onGreenChannelChanged"
QT_MOC_LITERAL(10, 125, 20), // "onBlueChannelChanged"
QT_MOC_LITERAL(11, 146, 21), // "onAlphaChannelChanged"
QT_MOC_LITERAL(12, 168, 12), // "onPasteColor"
QT_MOC_LITERAL(13, 181, 11) // "onCopyColor"

    },
    "DVGui::ColorField\0editingChanged\0\0"
    "TPixel32\0isEditing\0colorChanged\0"
    "isDragging\0onRedChannelChanged\0value\0"
    "onGreenChannelChanged\0onBlueChannelChanged\0"
    "onAlphaChannelChanged\0onPasteColor\0"
    "onCopyColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__ColorField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   54,    2, 0x06 /* Public */,
       5,    2,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,   64,    2, 0x09 /* Protected */,
       9,    2,   69,    2, 0x09 /* Protected */,
      10,    2,   74,    2, 0x09 /* Protected */,
      11,    2,   79,    2, 0x09 /* Protected */,
      12,    0,   84,    2, 0x09 /* Protected */,
      13,    0,   85,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    6,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    6,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    6,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    6,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    6,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void DVGui::ColorField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editingChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->colorChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->onRedChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->onGreenChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->onBlueChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->onAlphaChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->onPasteColor(); break;
        case 7: _t->onCopyColor(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorField::*)(const TPixel32 & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorField::editingChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ColorField::*)(const TPixel32 & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorField::colorChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::ColorField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__ColorField.data,
    qt_meta_data_DVGui__ColorField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::ColorField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::ColorField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__ColorField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::ColorField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void DVGui::ColorField::editingChanged(const TPixel32 & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::ColorField::colorChanged(const TPixel32 & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_DVGui__CleanupColorField_t {
    QByteArrayData data[15];
    char stringdata0[259];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__CleanupColorField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__CleanupColorField_t qt_meta_stringdata_DVGui__CleanupColorField = {
    {
QT_MOC_LITERAL(0, 0, 24), // "DVGui::CleanupColorField"
QT_MOC_LITERAL(1, 25, 14), // "editingChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 8), // "TPixel32"
QT_MOC_LITERAL(4, 50, 9), // "isEditing"
QT_MOC_LITERAL(5, 60, 13), // "StyleSelected"
QT_MOC_LITERAL(6, 74, 14), // "TCleanupStyle*"
QT_MOC_LITERAL(7, 89, 26), // "onBrightnessChannelChanged"
QT_MOC_LITERAL(8, 116, 5), // "value"
QT_MOC_LITERAL(9, 122, 8), // "dragging"
QT_MOC_LITERAL(10, 131, 24), // "onContrastChannelChanged"
QT_MOC_LITERAL(11, 156, 26), // "onCThresholdChannelChanged"
QT_MOC_LITERAL(12, 183, 26), // "onWThresholdChannelChanged"
QT_MOC_LITERAL(13, 210, 22), // "onHRangeChannelChanged"
QT_MOC_LITERAL(14, 233, 25) // "onLineWidthChannelChanged"

    },
    "DVGui::CleanupColorField\0editingChanged\0"
    "\0TPixel32\0isEditing\0StyleSelected\0"
    "TCleanupStyle*\0onBrightnessChannelChanged\0"
    "value\0dragging\0onContrastChannelChanged\0"
    "onCThresholdChannelChanged\0"
    "onWThresholdChannelChanged\0"
    "onHRangeChannelChanged\0onLineWidthChannelChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__CleanupColorField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   54,    2, 0x06 /* Public */,
       5,    1,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,   62,    2, 0x09 /* Protected */,
      10,    2,   67,    2, 0x09 /* Protected */,
      11,    2,   72,    2, 0x09 /* Protected */,
      12,    2,   77,    2, 0x09 /* Protected */,
      13,    2,   82,    2, 0x09 /* Protected */,
      14,    2,   87,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    4,
    QMetaType::Void, 0x80000000 | 6,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    9,

       0        // eod
};

void DVGui::CleanupColorField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CleanupColorField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editingChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->StyleSelected((*reinterpret_cast< TCleanupStyle*(*)>(_a[1]))); break;
        case 2: _t->onBrightnessChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->onContrastChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->onCThresholdChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->onWThresholdChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->onHRangeChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 7: _t->onLineWidthChannelChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CleanupColorField::*)(const TPixel32 & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CleanupColorField::editingChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CleanupColorField::*)(TCleanupStyle * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CleanupColorField::StyleSelected)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::CleanupColorField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__CleanupColorField.data,
    qt_meta_data_DVGui__CleanupColorField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::CleanupColorField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::CleanupColorField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__CleanupColorField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::CleanupColorField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void DVGui::CleanupColorField::editingChanged(const TPixel32 & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::CleanupColorField::StyleSelected(TCleanupStyle * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
