/****************************************************************************
** Meta object code from reading C++ file 'studiopaletteviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/studiopaletteviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'studiopaletteviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StudioPaletteTreeViewer_t {
    QByteArrayData data[24];
    char stringdata0[388];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StudioPaletteTreeViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StudioPaletteTreeViewer_t qt_meta_stringdata_StudioPaletteTreeViewer = {
    {
QT_MOC_LITERAL(0, 0, 23), // "StudioPaletteTreeViewer"
QT_MOC_LITERAL(1, 24, 7), // "refresh"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 11), // "refreshItem"
QT_MOC_LITERAL(4, 45, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(5, 62, 25), // "resetProjectPaletteFolder"
QT_MOC_LITERAL(6, 88, 13), // "onItemClicked"
QT_MOC_LITERAL(7, 102, 4), // "item"
QT_MOC_LITERAL(8, 107, 6), // "column"
QT_MOC_LITERAL(9, 114, 13), // "onItemChanged"
QT_MOC_LITERAL(10, 128, 20), // "onCurrentItemChanged"
QT_MOC_LITERAL(11, 149, 7), // "current"
QT_MOC_LITERAL(12, 157, 8), // "previous"
QT_MOC_LITERAL(13, 166, 30), // "onRefreshTreeShortcutTriggered"
QT_MOC_LITERAL(14, 197, 13), // "addNewPalette"
QT_MOC_LITERAL(15, 211, 12), // "addNewFolder"
QT_MOC_LITERAL(16, 224, 11), // "deleteItems"
QT_MOC_LITERAL(17, 236, 16), // "searchForPalette"
QT_MOC_LITERAL(18, 253, 12), // "setAsDefault"
QT_MOC_LITERAL(19, 266, 20), // "loadInCurrentPalette"
QT_MOC_LITERAL(20, 287, 33), // "loadInCurrentPaletteAndAdaptL..."
QT_MOC_LITERAL(21, 321, 21), // "replaceCurrentPalette"
QT_MOC_LITERAL(22, 343, 21), // "mergeToCurrentPalette"
QT_MOC_LITERAL(23, 365, 22) // "convertToStudioPalette"

    },
    "StudioPaletteTreeViewer\0refresh\0\0"
    "refreshItem\0QTreeWidgetItem*\0"
    "resetProjectPaletteFolder\0onItemClicked\0"
    "item\0column\0onItemChanged\0"
    "onCurrentItemChanged\0current\0previous\0"
    "onRefreshTreeShortcutTriggered\0"
    "addNewPalette\0addNewFolder\0deleteItems\0"
    "searchForPalette\0setAsDefault\0"
    "loadInCurrentPalette\0"
    "loadInCurrentPaletteAndAdaptLevel\0"
    "replaceCurrentPalette\0mergeToCurrentPalette\0"
    "convertToStudioPalette"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StudioPaletteTreeViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x09 /* Protected */,
       3,    1,  100,    2, 0x09 /* Protected */,
       5,    0,  103,    2, 0x09 /* Protected */,
       6,    2,  104,    2, 0x09 /* Protected */,
       9,    2,  109,    2, 0x09 /* Protected */,
      10,    2,  114,    2, 0x09 /* Protected */,
      13,    0,  119,    2, 0x09 /* Protected */,
      14,    0,  120,    2, 0x0a /* Public */,
      15,    0,  121,    2, 0x0a /* Public */,
      16,    0,  122,    2, 0x0a /* Public */,
      17,    0,  123,    2, 0x0a /* Public */,
      18,    0,  124,    2, 0x0a /* Public */,
      19,    0,  125,    2, 0x0a /* Public */,
      20,    0,  126,    2, 0x0a /* Public */,
      21,    0,  127,    2, 0x0a /* Public */,
      22,    0,  128,    2, 0x0a /* Public */,
      23,    0,  129,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int,    7,    8,
    QMetaType::Void, 0x80000000 | 4, QMetaType::Int,    7,    8,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 4,   11,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StudioPaletteTreeViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StudioPaletteTreeViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refresh(); break;
        case 1: _t->refreshItem((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 2: _t->resetProjectPaletteFolder(); break;
        case 3: _t->onItemClicked((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->onItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->onCurrentItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 6: _t->onRefreshTreeShortcutTriggered(); break;
        case 7: _t->addNewPalette(); break;
        case 8: _t->addNewFolder(); break;
        case 9: _t->deleteItems(); break;
        case 10: _t->searchForPalette(); break;
        case 11: _t->setAsDefault(); break;
        case 12: _t->loadInCurrentPalette(); break;
        case 13: _t->loadInCurrentPaletteAndAdaptLevel(); break;
        case 14: _t->replaceCurrentPalette(); break;
        case 15: _t->mergeToCurrentPalette(); break;
        case 16: _t->convertToStudioPalette(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StudioPaletteTreeViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeWidget::staticMetaObject>(),
    qt_meta_stringdata_StudioPaletteTreeViewer.data,
    qt_meta_data_StudioPaletteTreeViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StudioPaletteTreeViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StudioPaletteTreeViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StudioPaletteTreeViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "StudioPalette::Listener"))
        return static_cast< StudioPalette::Listener*>(this);
    if (!strcmp(_clname, "TProjectManager::Listener"))
        return static_cast< TProjectManager::Listener*>(this);
    return QTreeWidget::qt_metacast(_clname);
}

int StudioPaletteTreeViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}
struct qt_meta_stringdata_StudioPaletteViewer_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StudioPaletteViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StudioPaletteViewer_t qt_meta_stringdata_StudioPaletteViewer = {
    {
QT_MOC_LITERAL(0, 0, 19) // "StudioPaletteViewer"

    },
    "StudioPaletteViewer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StudioPaletteViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void StudioPaletteViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StudioPaletteViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_meta_stringdata_StudioPaletteViewer.data,
    qt_meta_data_StudioPaletteViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StudioPaletteViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StudioPaletteViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StudioPaletteViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SaveLoadQSettings"))
        return static_cast< SaveLoadQSettings*>(this);
    return QSplitter::qt_metacast(_clname);
}

int StudioPaletteViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_AdjustPaletteDialog_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AdjustPaletteDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AdjustPaletteDialog_t qt_meta_stringdata_AdjustPaletteDialog = {
    {
QT_MOC_LITERAL(0, 0, 19) // "AdjustPaletteDialog"

    },
    "AdjustPaletteDialog"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AdjustPaletteDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void AdjustPaletteDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject AdjustPaletteDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_AdjustPaletteDialog.data,
    qt_meta_data_AdjustPaletteDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AdjustPaletteDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AdjustPaletteDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AdjustPaletteDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int AdjustPaletteDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
