/****************************************************************************
** Meta object code from reading C++ file 'tonecurvefield.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/tonecurvefield.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tonecurvefield.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__ChennelCurveEditor_t {
    QByteArrayData data[10];
    char stringdata0[158];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__ChennelCurveEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__ChennelCurveEditor_t qt_meta_stringdata_DVGui__ChennelCurveEditor = {
    {
QT_MOC_LITERAL(0, 0, 25), // "DVGui::ChennelCurveEditor"
QT_MOC_LITERAL(1, 26, 8), // "focusOut"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 19), // "controlPointChanged"
QT_MOC_LITERAL(4, 56, 10), // "isDragging"
QT_MOC_LITERAL(5, 67, 17), // "controlPointAdded"
QT_MOC_LITERAL(6, 85, 5), // "index"
QT_MOC_LITERAL(7, 91, 19), // "controlPointRemoved"
QT_MOC_LITERAL(8, 111, 24), // "firstLastXPostionChanged"
QT_MOC_LITERAL(9, 136, 21) // "updateCurrentPosition"

    },
    "DVGui::ChennelCurveEditor\0focusOut\0\0"
    "controlPointChanged\0isDragging\0"
    "controlPointAdded\0index\0controlPointRemoved\0"
    "firstLastXPostionChanged\0updateCurrentPosition"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__ChennelCurveEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    1,   45,    2, 0x06 /* Public */,
       5,    1,   48,    2, 0x06 /* Public */,
       7,    1,   51,    2, 0x06 /* Public */,
       8,    2,   54,    2, 0x06 /* Public */,
       9,    2,   59,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    4,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::QPointF,    2,    2,

       0        // eod
};

void DVGui::ChennelCurveEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ChennelCurveEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->focusOut(); break;
        case 1: _t->controlPointChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->controlPointAdded((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->controlPointRemoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->firstLastXPostionChanged((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 5: _t->updateCurrentPosition((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QPointF(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ChennelCurveEditor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChennelCurveEditor::focusOut)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ChennelCurveEditor::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChennelCurveEditor::controlPointChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ChennelCurveEditor::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChennelCurveEditor::controlPointAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ChennelCurveEditor::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChennelCurveEditor::controlPointRemoved)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ChennelCurveEditor::*)(double , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChennelCurveEditor::firstLastXPostionChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ChennelCurveEditor::*)(int , QPointF );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ChennelCurveEditor::updateCurrentPosition)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::ChennelCurveEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__ChennelCurveEditor.data,
    qt_meta_data_DVGui__ChennelCurveEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::ChennelCurveEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::ChennelCurveEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__ChennelCurveEditor.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::ChennelCurveEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DVGui::ChennelCurveEditor::focusOut()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DVGui::ChennelCurveEditor::controlPointChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DVGui::ChennelCurveEditor::controlPointAdded(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DVGui::ChennelCurveEditor::controlPointRemoved(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void DVGui::ChennelCurveEditor::firstLastXPostionChanged(double _t1, double _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void DVGui::ChennelCurveEditor::updateCurrentPosition(int _t1, QPointF _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
struct qt_meta_stringdata_DVGui__ToneCurveField_t {
    QByteArrayData data[14];
    char stringdata0[255];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__ToneCurveField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__ToneCurveField_t qt_meta_stringdata_DVGui__ToneCurveField = {
    {
QT_MOC_LITERAL(0, 0, 21), // "DVGui::ToneCurveField"
QT_MOC_LITERAL(1, 22, 26), // "currentChannelIndexChanged"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 15), // "isLinearChanged"
QT_MOC_LITERAL(4, 66, 11), // "sizeChanged"
QT_MOC_LITERAL(5, 78, 18), // "sliderValueChanged"
QT_MOC_LITERAL(6, 97, 26), // "onFirstLastXPostionChanged"
QT_MOC_LITERAL(7, 124, 23), // "onUpdateCurrentPosition"
QT_MOC_LITERAL(8, 148, 21), // "onCurrentPointEditted"
QT_MOC_LITERAL(9, 170, 24), // "onCurrentChannelSwitched"
QT_MOC_LITERAL(10, 195, 19), // "onRangeModeSwitched"
QT_MOC_LITERAL(11, 215, 9), // "setLinear"
QT_MOC_LITERAL(12, 225, 11), // "setEnlarged"
QT_MOC_LITERAL(13, 237, 17) // "setLinearManually"

    },
    "DVGui::ToneCurveField\0currentChannelIndexChanged\0"
    "\0isLinearChanged\0sizeChanged\0"
    "sliderValueChanged\0onFirstLastXPostionChanged\0"
    "onUpdateCurrentPosition\0onCurrentPointEditted\0"
    "onCurrentChannelSwitched\0onRangeModeSwitched\0"
    "setLinear\0setEnlarged\0setLinearManually"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__ToneCurveField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       3,    1,   77,    2, 0x06 /* Public */,
       4,    0,   80,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   81,    2, 0x09 /* Protected */,
       6,    2,   84,    2, 0x09 /* Protected */,
       7,    2,   89,    2, 0x09 /* Protected */,
       8,    0,   94,    2, 0x09 /* Protected */,
       9,    1,   95,    2, 0x09 /* Protected */,
      10,    1,   98,    2, 0x09 /* Protected */,
      11,    1,  101,    2, 0x0a /* Public */,
      12,    1,  104,    2, 0x0a /* Public */,
      13,    1,  107,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::QPointF,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void DVGui::ToneCurveField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToneCurveField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentChannelIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->isLinearChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->sizeChanged(); break;
        case 3: _t->sliderValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->onFirstLastXPostionChanged((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 5: _t->onUpdateCurrentPosition((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QPointF(*)>(_a[2]))); break;
        case 6: _t->onCurrentPointEditted(); break;
        case 7: _t->onCurrentChannelSwitched((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->onRangeModeSwitched((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->setLinear((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->setEnlarged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->setLinearManually((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ToneCurveField::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToneCurveField::currentChannelIndexChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ToneCurveField::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToneCurveField::isLinearChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ToneCurveField::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ToneCurveField::sizeChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::ToneCurveField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__ToneCurveField.data,
    qt_meta_data_DVGui__ToneCurveField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::ToneCurveField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::ToneCurveField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__ToneCurveField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::ToneCurveField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void DVGui::ToneCurveField::currentChannelIndexChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::ToneCurveField::isLinearChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DVGui::ToneCurveField::sizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
