/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Room_t {
    QByteArrayData data[1];
    char stringdata0[5];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Room_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Room_t qt_meta_stringdata_Room = {
    {
QT_MOC_LITERAL(0, 0, 4) // "Room"

    },
    "Room"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Room[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Room::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Room::staticMetaObject = { {
    QMetaObject::SuperData::link<TMainWindow::staticMetaObject>(),
    qt_meta_stringdata_Room.data,
    qt_meta_data_Room,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Room::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Room::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Room.stringdata0))
        return static_cast<void*>(this);
    return TMainWindow::qt_metacast(_clname);
}

int Room::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TMainWindow::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[22];
    char stringdata0[277];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 4), // "exit"
QT_MOC_LITERAL(2, 16, 0), // ""
QT_MOC_LITERAL(3, 17, 5), // "bool&"
QT_MOC_LITERAL(4, 23, 20), // "onCurrentRoomChanged"
QT_MOC_LITERAL(5, 44, 12), // "newRoomIndex"
QT_MOC_LITERAL(6, 57, 14), // "onIndexSwapped"
QT_MOC_LITERAL(7, 72, 10), // "firstIndex"
QT_MOC_LITERAL(8, 83, 11), // "secondIndex"
QT_MOC_LITERAL(9, 95, 13), // "insertNewRoom"
QT_MOC_LITERAL(10, 109, 10), // "deleteRoom"
QT_MOC_LITERAL(11, 120, 5), // "index"
QT_MOC_LITERAL(12, 126, 10), // "renameRoom"
QT_MOC_LITERAL(13, 137, 4), // "name"
QT_MOC_LITERAL(14, 142, 21), // "onMenuCheckboxChanged"
QT_MOC_LITERAL(15, 164, 19), // "onInkCheckTriggered"
QT_MOC_LITERAL(16, 184, 2), // "on"
QT_MOC_LITERAL(17, 187, 20), // "onInk1CheckTriggered"
QT_MOC_LITERAL(18, 208, 19), // "onUpdateCheckerDone"
QT_MOC_LITERAL(19, 228, 21), // "onActiveViewerChanged"
QT_MOC_LITERAL(20, 250, 17), // "changeWindowTitle"
QT_MOC_LITERAL(21, 268, 8) // "QString&"

    },
    "MainWindow\0exit\0\0bool&\0onCurrentRoomChanged\0"
    "newRoomIndex\0onIndexSwapped\0firstIndex\0"
    "secondIndex\0insertNewRoom\0deleteRoom\0"
    "index\0renameRoom\0name\0onMenuCheckboxChanged\0"
    "onInkCheckTriggered\0on\0onInk1CheckTriggered\0"
    "onUpdateCheckerDone\0onActiveViewerChanged\0"
    "changeWindowTitle\0QString&"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   82,    2, 0x09 /* Protected */,
       6,    2,   85,    2, 0x09 /* Protected */,
       9,    0,   90,    2, 0x09 /* Protected */,
      10,    1,   91,    2, 0x09 /* Protected */,
      12,    2,   94,    2, 0x09 /* Protected */,
      14,    0,   99,    2, 0x09 /* Protected */,
      15,    1,  100,    2, 0x09 /* Protected */,
      17,    1,  103,    2, 0x09 /* Protected */,
      18,    1,  106,    2, 0x09 /* Protected */,
      19,    0,  109,    2, 0x09 /* Protected */,
      20,    0,  110,    2, 0x0a /* Public */,
      20,    1,  111,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    7,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   11,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21,    2,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->exit((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onCurrentRoomChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onIndexSwapped((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->insertNewRoom(); break;
        case 4: _t->deleteRoom((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->renameRoom((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->onMenuCheckboxChanged(); break;
        case 7: _t->onInkCheckTriggered((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->onInk1CheckTriggered((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->onUpdateCheckerDone((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->onActiveViewerChanged(); break;
        case 11: _t->changeWindowTitle(); break;
        case 12: _t->changeWindowTitle((*reinterpret_cast< QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)(bool & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::exit)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::exit(bool & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
