/****************************************************************************
** Meta object code from reading C++ file 'intpairfield.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/intpairfield.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'intpairfield.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__IntPairField_t {
    QByteArrayData data[14];
    char stringdata0[233];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__IntPairField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__IntPairField_t qt_meta_stringdata_DVGui__IntPairField = {
    {
QT_MOC_LITERAL(0, 0, 19), // "DVGui::IntPairField"
QT_MOC_LITERAL(1, 20, 13), // "valuesChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 10), // "isDragging"
QT_MOC_LITERAL(4, 46, 21), // "onLeftEditingFinished"
QT_MOC_LITERAL(5, 68, 22), // "onRightEditingFinished"
QT_MOC_LITERAL(6, 91, 16), // "HandleLeftPixmap"
QT_MOC_LITERAL(7, 108, 17), // "HandleRightPixmap"
QT_MOC_LITERAL(8, 126, 20), // "HandleLeftGrayPixmap"
QT_MOC_LITERAL(9, 147, 21), // "HandleRightGrayPixmap"
QT_MOC_LITERAL(10, 169, 14), // "LightLineColor"
QT_MOC_LITERAL(11, 184, 13), // "DarkLineColor"
QT_MOC_LITERAL(12, 198, 15), // "MiddleLineColor"
QT_MOC_LITERAL(13, 214, 18) // "LightLineEdgeColor"

    },
    "DVGui::IntPairField\0valuesChanged\0\0"
    "isDragging\0onLeftEditingFinished\0"
    "onRightEditingFinished\0HandleLeftPixmap\0"
    "HandleRightPixmap\0HandleLeftGrayPixmap\0"
    "HandleRightGrayPixmap\0LightLineColor\0"
    "DarkLineColor\0MiddleLineColor\0"
    "LightLineEdgeColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__IntPairField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       8,   34, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   32,    2, 0x09 /* Protected */,
       5,    0,   33,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       6, QMetaType::QPixmap, 0x00095103,
       7, QMetaType::QPixmap, 0x00095103,
       8, QMetaType::QPixmap, 0x00095103,
       9, QMetaType::QPixmap, 0x00095103,
      10, QMetaType::QColor, 0x00095103,
      11, QMetaType::QColor, 0x00095103,
      12, QMetaType::QColor, 0x00095103,
      13, QMetaType::QColor, 0x00095103,

       0        // eod
};

void DVGui::IntPairField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<IntPairField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->valuesChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onLeftEditingFinished(); break;
        case 2: _t->onRightEditingFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (IntPairField::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&IntPairField::valuesChanged)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<IntPairField *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QPixmap*>(_v) = _t->getHandleLeftPixmap(); break;
        case 1: *reinterpret_cast< QPixmap*>(_v) = _t->getHandleRightPixmap(); break;
        case 2: *reinterpret_cast< QPixmap*>(_v) = _t->getHandleLeftGrayPixmap(); break;
        case 3: *reinterpret_cast< QPixmap*>(_v) = _t->getHandleRightGrayPixmap(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getLightLineColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getDarkLineColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getMiddleLineColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->getLightLineEdgeColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<IntPairField *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHandleLeftPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 1: _t->setHandleRightPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 2: _t->setHandleLeftGrayPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 3: _t->setHandleRightGrayPixmap(*reinterpret_cast< QPixmap*>(_v)); break;
        case 4: _t->setLightLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setDarkLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setMiddleLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setLightLineEdgeColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DVGui::IntPairField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__IntPairField.data,
    qt_meta_data_DVGui__IntPairField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::IntPairField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::IntPairField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__IntPairField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::IntPairField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DVGui::IntPairField::valuesChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
