/****************************************************************************
** Meta object code from reading C++ file 'projectpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/projectpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'projectpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ProjectPopup_t {
    QByteArrayData data[3];
    char stringdata0[38];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ProjectPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ProjectPopup_t qt_meta_stringdata_ProjectPopup = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ProjectPopup"
QT_MOC_LITERAL(1, 13, 23), // "onRulePreferenceToggled"
QT_MOC_LITERAL(2, 37, 0) // ""

    },
    "ProjectPopup\0onRulePreferenceToggled\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ProjectPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    2,    2,

       0        // eod
};

void ProjectPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ProjectPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onRulePreferenceToggled((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ProjectPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_ProjectPopup.data,
    qt_meta_data_ProjectPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ProjectPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProjectPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ProjectPopup.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TProjectManager::Listener"))
        return static_cast< TProjectManager::Listener*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int ProjectPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ProjectSettingsPopup_t {
    QByteArrayData data[4];
    char stringdata0[64];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ProjectSettingsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ProjectSettingsPopup_t qt_meta_stringdata_ProjectSettingsPopup = {
    {
QT_MOC_LITERAL(0, 0, 20), // "ProjectSettingsPopup"
QT_MOC_LITERAL(1, 21, 18), // "onSomethingChanged"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 22) // "onChooseProjectChanged"

    },
    "ProjectSettingsPopup\0onSomethingChanged\0"
    "\0onChooseProjectChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ProjectSettingsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    1,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void ProjectSettingsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ProjectSettingsPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onSomethingChanged(); break;
        case 1: _t->onChooseProjectChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ProjectSettingsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<ProjectPopup::staticMetaObject>(),
    qt_meta_stringdata_ProjectSettingsPopup.data,
    qt_meta_data_ProjectSettingsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ProjectSettingsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProjectSettingsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ProjectSettingsPopup.stringdata0))
        return static_cast<void*>(this);
    return ProjectPopup::qt_metacast(_clname);
}

int ProjectSettingsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ProjectPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ProjectCreatePopup_t {
    QByteArrayData data[3];
    char stringdata0[34];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ProjectCreatePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ProjectCreatePopup_t qt_meta_stringdata_ProjectCreatePopup = {
    {
QT_MOC_LITERAL(0, 0, 18), // "ProjectCreatePopup"
QT_MOC_LITERAL(1, 19, 13), // "createProject"
QT_MOC_LITERAL(2, 33, 0) // ""

    },
    "ProjectCreatePopup\0createProject\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ProjectCreatePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void ProjectCreatePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ProjectCreatePopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->createProject(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ProjectCreatePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<ProjectPopup::staticMetaObject>(),
    qt_meta_stringdata_ProjectCreatePopup.data,
    qt_meta_data_ProjectCreatePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ProjectCreatePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProjectCreatePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ProjectCreatePopup.stringdata0))
        return static_cast<void*>(this);
    return ProjectPopup::qt_metacast(_clname);
}

int ProjectCreatePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ProjectPopup::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
