/****************************************************************************
** Meta object code from reading C++ file 'fxschematicscene.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/fxschematicscene.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fxschematicscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FxSchematicScene_t {
    QByteArrayData data[60];
    char stringdata0[911];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FxSchematicScene_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FxSchematicScene_t qt_meta_stringdata_FxSchematicScene = {
    {
QT_MOC_LITERAL(0, 0, 16), // "FxSchematicScene"
QT_MOC_LITERAL(1, 17, 11), // "showPreview"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 4), // "TFxP"
QT_MOC_LITERAL(4, 35, 7), // "cacheFx"
QT_MOC_LITERAL(5, 43, 10), // "doCollapse"
QT_MOC_LITERAL(6, 54, 11), // "QList<TFxP>"
QT_MOC_LITERAL(7, 66, 14), // "doExplodeChild"
QT_MOC_LITERAL(8, 81, 10), // "editObject"
QT_MOC_LITERAL(9, 92, 27), // "requestDisconnectFromXSheet"
QT_MOC_LITERAL(10, 120, 22), // "requestConnectToXSheet"
QT_MOC_LITERAL(11, 143, 14), // "requestPreview"
QT_MOC_LITERAL(12, 158, 14), // "requestCacheFx"
QT_MOC_LITERAL(13, 173, 16), // "requestUncacheFx"
QT_MOC_LITERAL(14, 190, 15), // "requestDeleteFx"
QT_MOC_LITERAL(15, 206, 18), // "requestDuplicateFx"
QT_MOC_LITERAL(16, 225, 15), // "requestUnlinkFx"
QT_MOC_LITERAL(17, 241, 14), // "requestMacroFx"
QT_MOC_LITERAL(18, 256, 21), // "requestExplodeMacroFx"
QT_MOC_LITERAL(19, 278, 18), // "requestOpenMacroFx"
QT_MOC_LITERAL(20, 297, 19), // "requestSavePresetFx"
QT_MOC_LITERAL(21, 317, 21), // "requestActivateOutput"
QT_MOC_LITERAL(22, 339, 18), // "requestInsertPaste"
QT_MOC_LITERAL(23, 358, 15), // "requestAddPaste"
QT_MOC_LITERAL(24, 374, 19), // "requestReplacePaste"
QT_MOC_LITERAL(25, 394, 16), // "requestEditGroup"
QT_MOC_LITERAL(26, 411, 19), // "onSelectionSwitched"
QT_MOC_LITERAL(27, 431, 11), // "TSelection*"
QT_MOC_LITERAL(28, 443, 6), // "oldSel"
QT_MOC_LITERAL(29, 450, 6), // "newSel"
QT_MOC_LITERAL(30, 457, 18), // "onSelectionChanged"
QT_MOC_LITERAL(31, 476, 22), // "onDisconnectFromXSheet"
QT_MOC_LITERAL(32, 499, 17), // "onConnectToXSheet"
QT_MOC_LITERAL(33, 517, 10), // "onDeleteFx"
QT_MOC_LITERAL(34, 528, 13), // "onDuplicateFx"
QT_MOC_LITERAL(35, 542, 10), // "onUnlinkFx"
QT_MOC_LITERAL(36, 553, 9), // "onMacroFx"
QT_MOC_LITERAL(37, 563, 16), // "onExplodeMacroFx"
QT_MOC_LITERAL(38, 580, 13), // "onOpenMacroFx"
QT_MOC_LITERAL(39, 594, 14), // "onSavePresetFx"
QT_MOC_LITERAL(40, 609, 16), // "onActivateOutput"
QT_MOC_LITERAL(41, 626, 9), // "onPreview"
QT_MOC_LITERAL(42, 636, 9), // "onCacheFx"
QT_MOC_LITERAL(43, 646, 11), // "onUncacheFx"
QT_MOC_LITERAL(44, 658, 10), // "onCollapse"
QT_MOC_LITERAL(45, 669, 15), // "onXsheetChanged"
QT_MOC_LITERAL(46, 685, 14), // "onSceneChanged"
QT_MOC_LITERAL(47, 700, 17), // "onSwitchCurrentFx"
QT_MOC_LITERAL(48, 718, 4), // "TFx*"
QT_MOC_LITERAL(49, 723, 19), // "onCurrentFxSwitched"
QT_MOC_LITERAL(50, 743, 22), // "onCurrentColumnChanged"
QT_MOC_LITERAL(51, 766, 21), // "onFxNodeDoubleClicked"
QT_MOC_LITERAL(52, 788, 13), // "onInsertPaste"
QT_MOC_LITERAL(53, 802, 10), // "onAddPaste"
QT_MOC_LITERAL(54, 813, 14), // "onReplacePaste"
QT_MOC_LITERAL(55, 828, 20), // "onAltModifierChanged"
QT_MOC_LITERAL(56, 849, 11), // "onEditGroup"
QT_MOC_LITERAL(57, 861, 21), // "onIconifyNodesToggled"
QT_MOC_LITERAL(58, 883, 9), // "iconified"
QT_MOC_LITERAL(59, 893, 17) // "onNodeChangedSize"

    },
    "FxSchematicScene\0showPreview\0\0TFxP\0"
    "cacheFx\0doCollapse\0QList<TFxP>\0"
    "doExplodeChild\0editObject\0"
    "requestDisconnectFromXSheet\0"
    "requestConnectToXSheet\0requestPreview\0"
    "requestCacheFx\0requestUncacheFx\0"
    "requestDeleteFx\0requestDuplicateFx\0"
    "requestUnlinkFx\0requestMacroFx\0"
    "requestExplodeMacroFx\0requestOpenMacroFx\0"
    "requestSavePresetFx\0requestActivateOutput\0"
    "requestInsertPaste\0requestAddPaste\0"
    "requestReplacePaste\0requestEditGroup\0"
    "onSelectionSwitched\0TSelection*\0oldSel\0"
    "newSel\0onSelectionChanged\0"
    "onDisconnectFromXSheet\0onConnectToXSheet\0"
    "onDeleteFx\0onDuplicateFx\0onUnlinkFx\0"
    "onMacroFx\0onExplodeMacroFx\0onOpenMacroFx\0"
    "onSavePresetFx\0onActivateOutput\0"
    "onPreview\0onCacheFx\0onUncacheFx\0"
    "onCollapse\0onXsheetChanged\0onSceneChanged\0"
    "onSwitchCurrentFx\0TFx*\0onCurrentFxSwitched\0"
    "onCurrentColumnChanged\0onFxNodeDoubleClicked\0"
    "onInsertPaste\0onAddPaste\0onReplacePaste\0"
    "onAltModifierChanged\0onEditGroup\0"
    "onIconifyNodesToggled\0iconified\0"
    "onNodeChangedSize"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FxSchematicScene[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      51,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  269,    2, 0x06 /* Public */,
       4,    1,  272,    2, 0x06 /* Public */,
       5,    1,  275,    2, 0x06 /* Public */,
       7,    1,  278,    2, 0x06 /* Public */,
       8,    0,  281,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,  282,    2, 0x0a /* Public */,
      10,    0,  283,    2, 0x0a /* Public */,
      11,    0,  284,    2, 0x0a /* Public */,
      12,    0,  285,    2, 0x0a /* Public */,
      13,    0,  286,    2, 0x0a /* Public */,
      14,    0,  287,    2, 0x0a /* Public */,
      15,    0,  288,    2, 0x0a /* Public */,
      16,    0,  289,    2, 0x0a /* Public */,
      17,    0,  290,    2, 0x0a /* Public */,
      18,    0,  291,    2, 0x0a /* Public */,
      19,    0,  292,    2, 0x0a /* Public */,
      20,    0,  293,    2, 0x0a /* Public */,
      21,    0,  294,    2, 0x0a /* Public */,
      22,    0,  295,    2, 0x0a /* Public */,
      23,    0,  296,    2, 0x0a /* Public */,
      24,    0,  297,    2, 0x0a /* Public */,
      25,    0,  298,    2, 0x0a /* Public */,
      26,    2,  299,    2, 0x09 /* Protected */,
      30,    0,  304,    2, 0x09 /* Protected */,
      31,    0,  305,    2, 0x09 /* Protected */,
      32,    0,  306,    2, 0x09 /* Protected */,
      33,    0,  307,    2, 0x09 /* Protected */,
      34,    0,  308,    2, 0x09 /* Protected */,
      35,    0,  309,    2, 0x09 /* Protected */,
      36,    0,  310,    2, 0x09 /* Protected */,
      37,    0,  311,    2, 0x09 /* Protected */,
      38,    0,  312,    2, 0x09 /* Protected */,
      39,    0,  313,    2, 0x09 /* Protected */,
      40,    0,  314,    2, 0x09 /* Protected */,
      41,    0,  315,    2, 0x09 /* Protected */,
      42,    0,  316,    2, 0x09 /* Protected */,
      43,    0,  317,    2, 0x09 /* Protected */,
      44,    1,  318,    2, 0x09 /* Protected */,
      45,    0,  321,    2, 0x09 /* Protected */,
      46,    0,  322,    2, 0x09 /* Protected */,
      47,    1,  323,    2, 0x09 /* Protected */,
      49,    0,  326,    2, 0x09 /* Protected */,
      50,    1,  327,    2, 0x09 /* Protected */,
      51,    0,  330,    2, 0x09 /* Protected */,
      52,    0,  331,    2, 0x09 /* Protected */,
      53,    0,  332,    2, 0x09 /* Protected */,
      54,    0,  333,    2, 0x09 /* Protected */,
      55,    1,  334,    2, 0x09 /* Protected */,
      56,    0,  337,    2, 0x09 /* Protected */,
      57,    1,  338,    2, 0x09 /* Protected */,
      59,    0,  341,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 27, 0x80000000 | 27,   28,   29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 6,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 48,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   58,
    QMetaType::Void,

       0        // eod
};

void FxSchematicScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FxSchematicScene *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->showPreview((*reinterpret_cast< TFxP(*)>(_a[1]))); break;
        case 1: _t->cacheFx((*reinterpret_cast< TFxP(*)>(_a[1]))); break;
        case 2: _t->doCollapse((*reinterpret_cast< const QList<TFxP>(*)>(_a[1]))); break;
        case 3: _t->doExplodeChild((*reinterpret_cast< const QList<TFxP>(*)>(_a[1]))); break;
        case 4: _t->editObject(); break;
        case 5: _t->requestDisconnectFromXSheet(); break;
        case 6: _t->requestConnectToXSheet(); break;
        case 7: _t->requestPreview(); break;
        case 8: _t->requestCacheFx(); break;
        case 9: _t->requestUncacheFx(); break;
        case 10: _t->requestDeleteFx(); break;
        case 11: _t->requestDuplicateFx(); break;
        case 12: _t->requestUnlinkFx(); break;
        case 13: _t->requestMacroFx(); break;
        case 14: _t->requestExplodeMacroFx(); break;
        case 15: _t->requestOpenMacroFx(); break;
        case 16: _t->requestSavePresetFx(); break;
        case 17: _t->requestActivateOutput(); break;
        case 18: _t->requestInsertPaste(); break;
        case 19: _t->requestAddPaste(); break;
        case 20: _t->requestReplacePaste(); break;
        case 21: _t->requestEditGroup(); break;
        case 22: _t->onSelectionSwitched((*reinterpret_cast< TSelection*(*)>(_a[1])),(*reinterpret_cast< TSelection*(*)>(_a[2]))); break;
        case 23: _t->onSelectionChanged(); break;
        case 24: _t->onDisconnectFromXSheet(); break;
        case 25: _t->onConnectToXSheet(); break;
        case 26: _t->onDeleteFx(); break;
        case 27: _t->onDuplicateFx(); break;
        case 28: _t->onUnlinkFx(); break;
        case 29: _t->onMacroFx(); break;
        case 30: _t->onExplodeMacroFx(); break;
        case 31: _t->onOpenMacroFx(); break;
        case 32: _t->onSavePresetFx(); break;
        case 33: _t->onActivateOutput(); break;
        case 34: _t->onPreview(); break;
        case 35: _t->onCacheFx(); break;
        case 36: _t->onUncacheFx(); break;
        case 37: _t->onCollapse((*reinterpret_cast< const QList<TFxP>(*)>(_a[1]))); break;
        case 38: _t->onXsheetChanged(); break;
        case 39: _t->onSceneChanged(); break;
        case 40: _t->onSwitchCurrentFx((*reinterpret_cast< TFx*(*)>(_a[1]))); break;
        case 41: _t->onCurrentFxSwitched(); break;
        case 42: _t->onCurrentColumnChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 43: _t->onFxNodeDoubleClicked(); break;
        case 44: _t->onInsertPaste(); break;
        case 45: _t->onAddPaste(); break;
        case 46: _t->onReplacePaste(); break;
        case 47: _t->onAltModifierChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 48: _t->onEditGroup(); break;
        case 49: _t->onIconifyNodesToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 50: _t->onNodeChangedSize(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FxSchematicScene::*)(TFxP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicScene::showPreview)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FxSchematicScene::*)(TFxP );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicScene::cacheFx)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (FxSchematicScene::*)(const QList<TFxP> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicScene::doCollapse)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (FxSchematicScene::*)(const QList<TFxP> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicScene::doExplodeChild)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (FxSchematicScene::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FxSchematicScene::editObject)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject FxSchematicScene::staticMetaObject = { {
    QMetaObject::SuperData::link<SchematicScene::staticMetaObject>(),
    qt_meta_stringdata_FxSchematicScene.data,
    qt_meta_data_FxSchematicScene,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FxSchematicScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FxSchematicScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FxSchematicScene.stringdata0))
        return static_cast<void*>(this);
    return SchematicScene::qt_metacast(_clname);
}

int FxSchematicScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SchematicScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 51)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 51;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 51)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 51;
    }
    return _id;
}

// SIGNAL 0
void FxSchematicScene::showPreview(TFxP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void FxSchematicScene::cacheFx(TFxP _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void FxSchematicScene::doCollapse(const QList<TFxP> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void FxSchematicScene::doExplodeChild(const QList<TFxP> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void FxSchematicScene::editObject()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
