/****************************************************************************
** Meta object code from reading C++ file 'dvitemview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/dvitemview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dvitemview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ItemViewPlayWidget_t {
    QByteArrayData data[1];
    char stringdata0[19];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ItemViewPlayWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ItemViewPlayWidget_t qt_meta_stringdata_ItemViewPlayWidget = {
    {
QT_MOC_LITERAL(0, 0, 18) // "ItemViewPlayWidget"

    },
    "ItemViewPlayWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ItemViewPlayWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ItemViewPlayWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ItemViewPlayWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ItemViewPlayWidget.data,
    qt_meta_data_ItemViewPlayWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ItemViewPlayWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ItemViewPlayWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ItemViewPlayWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ItemViewPlayWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_DVItemViewPlayDelegate_t {
    QByteArrayData data[3];
    char stringdata0[40];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVItemViewPlayDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVItemViewPlayDelegate_t qt_meta_stringdata_DVItemViewPlayDelegate = {
    {
QT_MOC_LITERAL(0, 0, 22), // "DVItemViewPlayDelegate"
QT_MOC_LITERAL(1, 23, 15), // "resetPlayWidget"
QT_MOC_LITERAL(2, 39, 0) // ""

    },
    "DVItemViewPlayDelegate\0resetPlayWidget\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVItemViewPlayDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void DVItemViewPlayDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DVItemViewPlayDelegate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resetPlayWidget(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DVItemViewPlayDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DVItemViewPlayDelegate.data,
    qt_meta_data_DVItemViewPlayDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVItemViewPlayDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVItemViewPlayDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVItemViewPlayDelegate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DVItemViewPlayDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_DvItemSelection_t {
    QByteArrayData data[3];
    char stringdata0[38];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DvItemSelection_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DvItemSelection_t qt_meta_stringdata_DvItemSelection = {
    {
QT_MOC_LITERAL(0, 0, 15), // "DvItemSelection"
QT_MOC_LITERAL(1, 16, 20), // "itemSelectionChanged"
QT_MOC_LITERAL(2, 37, 0) // ""

    },
    "DvItemSelection\0itemSelectionChanged\0"
    ""
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DvItemSelection[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void DvItemSelection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvItemSelection *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->itemSelectionChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DvItemSelection::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemSelection::itemSelectionChanged)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject DvItemSelection::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DvItemSelection.data,
    qt_meta_data_DvItemSelection,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DvItemSelection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DvItemSelection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DvItemSelection.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSelection"))
        return static_cast< TSelection*>(this);
    return QObject::qt_metacast(_clname);
}

int DvItemSelection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void DvItemSelection::itemSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_DvItemViewerPanel_t {
    QByteArrayData data[15];
    char stringdata0[223];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DvItemViewerPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DvItemViewerPanel_t qt_meta_stringdata_DvItemViewerPanel = {
    {
QT_MOC_LITERAL(0, 0, 17), // "DvItemViewerPanel"
QT_MOC_LITERAL(1, 18, 14), // "viewTypeChange"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 27), // "DvItemViewerPanel::ViewType"
QT_MOC_LITERAL(4, 62, 8), // "viewType"
QT_MOC_LITERAL(5, 71, 11), // "setListView"
QT_MOC_LITERAL(6, 83, 12), // "setTableView"
QT_MOC_LITERAL(7, 96, 17), // "setThumbnailsView"
QT_MOC_LITERAL(8, 114, 6), // "rename"
QT_MOC_LITERAL(9, 121, 14), // "exportFileList"
QT_MOC_LITERAL(10, 136, 19), // "AlternateBackground"
QT_MOC_LITERAL(11, 156, 9), // "TextColor"
QT_MOC_LITERAL(12, 166, 17), // "SelectedTextColor"
QT_MOC_LITERAL(13, 184, 15), // "FolderTextColor"
QT_MOC_LITERAL(14, 200, 22) // "SelectedItemBackground"

    },
    "DvItemViewerPanel\0viewTypeChange\0\0"
    "DvItemViewerPanel::ViewType\0viewType\0"
    "setListView\0setTableView\0setThumbnailsView\0"
    "rename\0exportFileList\0AlternateBackground\0"
    "TextColor\0SelectedTextColor\0FolderTextColor\0"
    "SelectedItemBackground"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DvItemViewerPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       5,   52, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   47,    2, 0x0a /* Public */,
       6,    0,   48,    2, 0x0a /* Public */,
       7,    0,   49,    2, 0x0a /* Public */,
       8,    0,   50,    2, 0x0a /* Public */,
       9,    0,   51,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      10, QMetaType::QColor, 0x00095103,
      11, QMetaType::QColor, 0x00095103,
      12, QMetaType::QColor, 0x00095103,
      13, QMetaType::QColor, 0x00095103,
      14, QMetaType::QColor, 0x00095103,

       0        // eod
};

void DvItemViewerPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvItemViewerPanel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->viewTypeChange((*reinterpret_cast< DvItemViewerPanel::ViewType(*)>(_a[1]))); break;
        case 1: _t->setListView(); break;
        case 2: _t->setTableView(); break;
        case 3: _t->setThumbnailsView(); break;
        case 4: _t->rename(); break;
        case 5: _t->exportFileList(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DvItemViewerPanel::*)(DvItemViewerPanel::ViewType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewerPanel::viewTypeChange)) {
                *result = 0;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DvItemViewerPanel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getAlternateBackground(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getSelectedTextColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getFolderTextColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getSelectedItemBackground(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DvItemViewerPanel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAlternateBackground(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setSelectedTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setFolderTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setSelectedItemBackground(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DvItemViewerPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_DvItemViewerPanel.data,
    qt_meta_data_DvItemViewerPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DvItemViewerPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DvItemViewerPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DvItemViewerPanel.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TSelection::View"))
        return static_cast< TSelection::View*>(this);
    return QFrame::qt_metacast(_clname);
}

int DvItemViewerPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void DvItemViewerPanel::viewTypeChange(DvItemViewerPanel::ViewType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_DvItemViewer_t {
    QByteArrayData data[8];
    char stringdata0[86];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DvItemViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DvItemViewer_t qt_meta_stringdata_DvItemViewer = {
    {
QT_MOC_LITERAL(0, 0, 12), // "DvItemViewer"
QT_MOC_LITERAL(1, 13, 11), // "clickedItem"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 5), // "index"
QT_MOC_LITERAL(4, 32, 17), // "doubleClickedItem"
QT_MOC_LITERAL(5, 50, 13), // "selectedItems"
QT_MOC_LITERAL(6, 64, 13), // "std::set<int>"
QT_MOC_LITERAL(7, 78, 7) // "indexes"

    },
    "DvItemViewer\0clickedItem\0\0index\0"
    "doubleClickedItem\0selectedItems\0"
    "std::set<int>\0indexes"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DvItemViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       4,    1,   32,    2, 0x06 /* Public */,
       5,    1,   35,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, 0x80000000 | 6,    7,

       0        // eod
};

void DvItemViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvItemViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clickedItem((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->doubleClickedItem((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->selectedItems((*reinterpret_cast< const std::set<int>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DvItemViewer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewer::clickedItem)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DvItemViewer::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewer::doubleClickedItem)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DvItemViewer::*)(const std::set<int> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewer::selectedItems)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DvItemViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_DvItemViewer.data,
    qt_meta_data_DvItemViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DvItemViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DvItemViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DvItemViewer.stringdata0))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int DvItemViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void DvItemViewer::clickedItem(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DvItemViewer::doubleClickedItem(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DvItemViewer::selectedItems(const std::set<int> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
struct qt_meta_stringdata_DvItemViewerTitleBar_t {
    QByteArrayData data[10];
    char stringdata0[144];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DvItemViewerTitleBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DvItemViewerTitleBar_t qt_meta_stringdata_DvItemViewerTitleBar = {
    {
QT_MOC_LITERAL(0, 0, 20), // "DvItemViewerTitleBar"
QT_MOC_LITERAL(1, 21, 17), // "onViewTypeChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 27), // "DvItemViewerPanel::ViewType"
QT_MOC_LITERAL(4, 68, 8), // "viewType"
QT_MOC_LITERAL(5, 77, 8), // "ColColor"
QT_MOC_LITERAL(6, 86, 14), // "ColSortedColor"
QT_MOC_LITERAL(7, 101, 12), // "ColTextColor"
QT_MOC_LITERAL(8, 114, 14), // "ColBorderColor"
QT_MOC_LITERAL(9, 129, 14) // "ColRaisedColor"

    },
    "DvItemViewerTitleBar\0onViewTypeChanged\0"
    "\0DvItemViewerPanel::ViewType\0viewType\0"
    "ColColor\0ColSortedColor\0ColTextColor\0"
    "ColBorderColor\0ColRaisedColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DvItemViewerTitleBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       5,   22, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // properties: name, type, flags
       5, QMetaType::QColor, 0x00095103,
       6, QMetaType::QColor, 0x00095103,
       7, QMetaType::QColor, 0x00095103,
       8, QMetaType::QColor, 0x00095103,
       9, QMetaType::QColor, 0x00095103,

       0        // eod
};

void DvItemViewerTitleBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvItemViewerTitleBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onViewTypeChanged((*reinterpret_cast< DvItemViewerPanel::ViewType(*)>(_a[1]))); break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<DvItemViewerTitleBar *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getColColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getColColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getColColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getColColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getColRaisedColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<DvItemViewerTitleBar *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setColSortedColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setColTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setColBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setColRaisedColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject DvItemViewerTitleBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DvItemViewerTitleBar.data,
    qt_meta_data_DvItemViewerTitleBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DvItemViewerTitleBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DvItemViewerTitleBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DvItemViewerTitleBar.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DvItemViewerTitleBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_DvItemViewerButtonBar_t {
    QByteArrayData data[8];
    char stringdata0[100];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DvItemViewerButtonBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DvItemViewerButtonBar_t qt_meta_stringdata_DvItemViewerButtonBar = {
    {
QT_MOC_LITERAL(0, 0, 21), // "DvItemViewerButtonBar"
QT_MOC_LITERAL(1, 22, 8), // "folderUp"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 9), // "newFolder"
QT_MOC_LITERAL(4, 42, 10), // "folderBack"
QT_MOC_LITERAL(5, 53, 9), // "folderFwd"
QT_MOC_LITERAL(6, 63, 16), // "onHistoryChanged"
QT_MOC_LITERAL(7, 80, 19) // "onPreferenceChanged"

    },
    "DvItemViewerButtonBar\0folderUp\0\0"
    "newFolder\0folderBack\0folderFwd\0"
    "onHistoryChanged\0onPreferenceChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DvItemViewerButtonBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x06 /* Public */,
       3,    0,   45,    2, 0x06 /* Public */,
       4,    0,   46,    2, 0x06 /* Public */,
       5,    0,   47,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    2,   48,    2, 0x0a /* Public */,
       7,    1,   53,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void DvItemViewerButtonBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DvItemViewerButtonBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->folderUp(); break;
        case 1: _t->newFolder(); break;
        case 2: _t->folderBack(); break;
        case 3: _t->folderFwd(); break;
        case 4: _t->onHistoryChanged((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DvItemViewerButtonBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewerButtonBar::folderUp)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (DvItemViewerButtonBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewerButtonBar::newFolder)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (DvItemViewerButtonBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewerButtonBar::folderBack)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (DvItemViewerButtonBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DvItemViewerButtonBar::folderFwd)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DvItemViewerButtonBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolBar::staticMetaObject>(),
    qt_meta_stringdata_DvItemViewerButtonBar.data,
    qt_meta_data_DvItemViewerButtonBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DvItemViewerButtonBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DvItemViewerButtonBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DvItemViewerButtonBar.stringdata0))
        return static_cast<void*>(this);
    return QToolBar::qt_metacast(_clname);
}

int DvItemViewerButtonBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void DvItemViewerButtonBar::folderUp()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DvItemViewerButtonBar::newFolder()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DvItemViewerButtonBar::folderBack()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DvItemViewerButtonBar::folderFwd()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
