/****************************************************************************
** Meta object code from reading C++ file 'svncommitdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/svncommitdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'svncommitdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SVNCommitDialog_t {
    QByteArrayData data[16];
    char stringdata0[259];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNCommitDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNCommitDialog_t qt_meta_stringdata_SVNCommitDialog = {
    {
QT_MOC_LITERAL(0, 0, 15), // "SVNCommitDialog"
QT_MOC_LITERAL(1, 16, 4), // "done"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 21), // "onCommitButtonClicked"
QT_MOC_LITERAL(4, 44, 9), // "onAddDone"
QT_MOC_LITERAL(5, 54, 17), // "onSetPropertyDone"
QT_MOC_LITERAL(6, 72, 12), // "onCommitDone"
QT_MOC_LITERAL(7, 85, 13), // "onItemChanged"
QT_MOC_LITERAL(8, 99, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(9, 116, 26), // "onSelectionCheckBoxClicked"
QT_MOC_LITERAL(10, 143, 7), // "checked"
QT_MOC_LITERAL(11, 151, 7), // "onError"
QT_MOC_LITERAL(12, 159, 17), // "onStatusRetrieved"
QT_MOC_LITERAL(13, 177, 25), // "onStatusRetrievedAfterAdd"
QT_MOC_LITERAL(14, 203, 26), // "onResourcesStatusRetrieved"
QT_MOC_LITERAL(15, 230, 28) // "onCommitSceneContentsToggled"

    },
    "SVNCommitDialog\0done\0\0onCommitButtonClicked\0"
    "onAddDone\0onSetPropertyDone\0onCommitDone\0"
    "onItemChanged\0QTreeWidgetItem*\0"
    "onSelectionCheckBoxClicked\0checked\0"
    "onError\0onStatusRetrieved\0"
    "onStatusRetrievedAfterAdd\0"
    "onResourcesStatusRetrieved\0"
    "onCommitSceneContentsToggled"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNCommitDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   77,    2, 0x08 /* Private */,
       4,    0,   78,    2, 0x08 /* Private */,
       5,    0,   79,    2, 0x08 /* Private */,
       6,    0,   80,    2, 0x08 /* Private */,
       7,    1,   81,    2, 0x08 /* Private */,
       9,    1,   84,    2, 0x08 /* Private */,
      11,    1,   87,    2, 0x08 /* Private */,
      12,    1,   90,    2, 0x08 /* Private */,
      13,    1,   93,    2, 0x08 /* Private */,
      14,    1,   96,    2, 0x08 /* Private */,
      15,    1,   99,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void, QMetaType::Bool,   10,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void SVNCommitDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNCommitDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->onCommitButtonClicked(); break;
        case 2: _t->onAddDone(); break;
        case 3: _t->onSetPropertyDone(); break;
        case 4: _t->onCommitDone(); break;
        case 5: _t->onItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 6: _t->onSelectionCheckBoxClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->onStatusRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->onStatusRetrievedAfterAdd((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->onResourcesStatusRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->onCommitSceneContentsToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SVNCommitDialog::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SVNCommitDialog::done)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNCommitDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNCommitDialog.data,
    qt_meta_data_SVNCommitDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNCommitDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNCommitDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNCommitDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNCommitDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void SVNCommitDialog::done(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_SVNCommitFrameRangeDialog_t {
    QByteArrayData data[11];
    char stringdata0[143];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNCommitFrameRangeDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNCommitFrameRangeDialog_t qt_meta_stringdata_SVNCommitFrameRangeDialog = {
    {
QT_MOC_LITERAL(0, 0, 25), // "SVNCommitFrameRangeDialog"
QT_MOC_LITERAL(1, 26, 4), // "done"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 7), // "onError"
QT_MOC_LITERAL(4, 40, 13), // "onPropGetDone"
QT_MOC_LITERAL(5, 54, 13), // "onPropSetDone"
QT_MOC_LITERAL(6, 68, 15), // "onHookFileAdded"
QT_MOC_LITERAL(7, 84, 21), // "onCommitButtonClicked"
QT_MOC_LITERAL(8, 106, 12), // "onCommitDone"
QT_MOC_LITERAL(9, 119, 12), // "onUpdateDone"
QT_MOC_LITERAL(10, 132, 10) // "onLockDone"

    },
    "SVNCommitFrameRangeDialog\0done\0\0onError\0"
    "onPropGetDone\0onPropSetDone\0onHookFileAdded\0"
    "onCommitButtonClicked\0onCommitDone\0"
    "onUpdateDone\0onLockDone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNCommitFrameRangeDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   62,    2, 0x08 /* Private */,
       4,    1,   65,    2, 0x08 /* Private */,
       5,    0,   68,    2, 0x08 /* Private */,
       6,    0,   69,    2, 0x08 /* Private */,
       7,    0,   70,    2, 0x08 /* Private */,
       8,    0,   71,    2, 0x08 /* Private */,
       9,    0,   72,    2, 0x08 /* Private */,
      10,    0,   73,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SVNCommitFrameRangeDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNCommitFrameRangeDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onPropGetDone((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->onPropSetDone(); break;
        case 4: _t->onHookFileAdded(); break;
        case 5: _t->onCommitButtonClicked(); break;
        case 6: _t->onCommitDone(); break;
        case 7: _t->onUpdateDone(); break;
        case 8: _t->onLockDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SVNCommitFrameRangeDialog::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SVNCommitFrameRangeDialog::done)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNCommitFrameRangeDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNCommitFrameRangeDialog.data,
    qt_meta_data_SVNCommitFrameRangeDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNCommitFrameRangeDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNCommitFrameRangeDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNCommitFrameRangeDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNCommitFrameRangeDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void SVNCommitFrameRangeDialog::done(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
