/****************************************************************************
** Meta object code from reading C++ file 'penciltestpopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/penciltestpopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'penciltestpopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MyVideoWidget_t {
    QByteArrayData data[8];
    char stringdata0[89];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MyVideoWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MyVideoWidget_t qt_meta_stringdata_MyVideoWidget = {
    {
QT_MOC_LITERAL(0, 0, 13), // "MyVideoWidget"
QT_MOC_LITERAL(1, 14, 11), // "startCamera"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 10), // "stopCamera"
QT_MOC_LITERAL(4, 38, 16), // "subCameraChanged"
QT_MOC_LITERAL(5, 55, 10), // "isDragging"
QT_MOC_LITERAL(6, 66, 19), // "onUpsideDownChecked"
QT_MOC_LITERAL(7, 86, 2) // "on"

    },
    "MyVideoWidget\0startCamera\0\0stopCamera\0"
    "subCameraChanged\0isDragging\0"
    "onUpsideDownChecked\0on"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MyVideoWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x06 /* Public */,
       3,    0,   35,    2, 0x06 /* Public */,
       4,    1,   36,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   39,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    5,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    7,

       0        // eod
};

void MyVideoWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MyVideoWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startCamera(); break;
        case 1: _t->stopCamera(); break;
        case 2: _t->subCameraChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onUpsideDownChecked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MyVideoWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MyVideoWidget::startCamera)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MyVideoWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MyVideoWidget::stopCamera)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MyVideoWidget::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MyVideoWidget::subCameraChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MyVideoWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_MyVideoWidget.data,
    qt_meta_data_MyVideoWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MyVideoWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MyVideoWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MyVideoWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MyVideoWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void MyVideoWidget::startCamera()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MyVideoWidget::stopCamera()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MyVideoWidget::subCameraChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
struct qt_meta_stringdata_FrameNumberLineEdit_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FrameNumberLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FrameNumberLineEdit_t qt_meta_stringdata_FrameNumberLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 19) // "FrameNumberLineEdit"

    },
    "FrameNumberLineEdit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FrameNumberLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void FrameNumberLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FrameNumberLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::LineEdit::staticMetaObject>(),
    qt_meta_stringdata_FrameNumberLineEdit.data,
    qt_meta_data_FrameNumberLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FrameNumberLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FrameNumberLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FrameNumberLineEdit.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "TProjectManager::Listener"))
        return static_cast< TProjectManager::Listener*>(this);
    return DVGui::LineEdit::qt_metacast(_clname);
}

int FrameNumberLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::LineEdit::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_LevelNameLineEdit_t {
    QByteArrayData data[4];
    char stringdata0[53];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LevelNameLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LevelNameLineEdit_t qt_meta_stringdata_LevelNameLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 17), // "LevelNameLineEdit"
QT_MOC_LITERAL(1, 18, 15), // "levelNameEdited"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 17) // "onEditingFinished"

    },
    "LevelNameLineEdit\0levelNameEdited\0\0"
    "onEditingFinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LevelNameLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   25,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void LevelNameLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LevelNameLineEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->levelNameEdited(); break;
        case 1: _t->onEditingFinished(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (LevelNameLineEdit::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LevelNameLineEdit::levelNameEdited)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject LevelNameLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_meta_stringdata_LevelNameLineEdit.data,
    qt_meta_data_LevelNameLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LevelNameLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LevelNameLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LevelNameLineEdit.stringdata0))
        return static_cast<void*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int LevelNameLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void LevelNameLineEdit::levelNameEdited()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_PencilTestSaveInFolderPopup_t {
    QByteArrayData data[8];
    char stringdata0[169];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PencilTestSaveInFolderPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PencilTestSaveInFolderPopup_t qt_meta_stringdata_PencilTestSaveInFolderPopup = {
    {
QT_MOC_LITERAL(0, 0, 27), // "PencilTestSaveInFolderPopup"
QT_MOC_LITERAL(1, 28, 19), // "updateSubFolderName"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 22), // "onAutoSubNameCBClicked"
QT_MOC_LITERAL(4, 72, 28), // "onShowPopupOnLaunchCBClicked"
QT_MOC_LITERAL(5, 101, 30), // "onCreateSceneInFolderCBClicked"
QT_MOC_LITERAL(6, 132, 24), // "onSetAsDefaultBtnPressed"
QT_MOC_LITERAL(7, 157, 11) // "onOkPressed"

    },
    "PencilTestSaveInFolderPopup\0"
    "updateSubFolderName\0\0onAutoSubNameCBClicked\0"
    "onShowPopupOnLaunchCBClicked\0"
    "onCreateSceneInFolderCBClicked\0"
    "onSetAsDefaultBtnPressed\0onOkPressed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PencilTestSaveInFolderPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   44,    2, 0x09 /* Protected */,
       3,    1,   45,    2, 0x09 /* Protected */,
       4,    1,   48,    2, 0x09 /* Protected */,
       5,    1,   51,    2, 0x09 /* Protected */,
       6,    0,   54,    2, 0x09 /* Protected */,
       7,    0,   55,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PencilTestSaveInFolderPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PencilTestSaveInFolderPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateSubFolderName(); break;
        case 1: _t->onAutoSubNameCBClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->onShowPopupOnLaunchCBClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->onCreateSceneInFolderCBClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->onSetAsDefaultBtnPressed(); break;
        case 5: _t->onOkPressed(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PencilTestSaveInFolderPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_PencilTestSaveInFolderPopup.data,
    qt_meta_data_PencilTestSaveInFolderPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PencilTestSaveInFolderPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PencilTestSaveInFolderPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PencilTestSaveInFolderPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int PencilTestSaveInFolderPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
struct qt_meta_stringdata_SubCameraButton_t {
    QByteArrayData data[7];
    char stringdata0[91];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SubCameraButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SubCameraButton_t qt_meta_stringdata_SubCameraButton = {
    {
QT_MOC_LITERAL(0, 0, 15), // "SubCameraButton"
QT_MOC_LITERAL(1, 16, 23), // "subCameraPresetSelected"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 3), // "dpi"
QT_MOC_LITERAL(4, 45, 14), // "onSubCameraAct"
QT_MOC_LITERAL(5, 60, 15), // "onSaveSubCamera"
QT_MOC_LITERAL(6, 76, 14) // "onDeletePreset"

    },
    "SubCameraButton\0subCameraPresetSelected\0"
    "\0dpi\0onSubCameraAct\0onSaveSubCamera\0"
    "onDeletePreset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SubCameraButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   39,    2, 0x09 /* Protected */,
       5,    0,   40,    2, 0x09 /* Protected */,
       6,    0,   41,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QRect, QMetaType::Double,    2,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SubCameraButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SubCameraButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->subCameraPresetSelected((*reinterpret_cast< const QRect(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 1: _t->onSubCameraAct(); break;
        case 2: _t->onSaveSubCamera(); break;
        case 3: _t->onDeletePreset(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SubCameraButton::*)(const QRect & , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SubCameraButton::subCameraPresetSelected)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SubCameraButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QPushButton::staticMetaObject>(),
    qt_meta_stringdata_SubCameraButton.data,
    qt_meta_data_SubCameraButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SubCameraButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SubCameraButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SubCameraButton.stringdata0))
        return static_cast<void*>(this);
    return QPushButton::qt_metacast(_clname);
}

int SubCameraButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPushButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void SubCameraButton::subCameraPresetSelected(const QRect & _t1, double _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_PencilTestPopup_t {
    QByteArrayData data[42];
    char stringdata0[801];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PencilTestPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PencilTestPopup_t qt_meta_stringdata_PencilTestPopup = {
    {
QT_MOC_LITERAL(0, 0, 15), // "PencilTestPopup"
QT_MOC_LITERAL(1, 16, 17), // "refreshCameraList"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 26), // "onCameraListComboActivated"
QT_MOC_LITERAL(4, 62, 5), // "index"
QT_MOC_LITERAL(5, 68, 26), // "onResolutionComboActivated"
QT_MOC_LITERAL(6, 95, 31), // "onFileFormatOptionButtonPressed"
QT_MOC_LITERAL(7, 127, 17), // "onLevelNameEdited"
QT_MOC_LITERAL(8, 145, 10), // "onNextName"
QT_MOC_LITERAL(9, 156, 14), // "onPreviousName"
QT_MOC_LITERAL(10, 171, 23), // "onColorTypeComboChanged"
QT_MOC_LITERAL(11, 195, 15), // "onFrameCaptured"
QT_MOC_LITERAL(12, 211, 8), // "cv::Mat&"
QT_MOC_LITERAL(13, 220, 5), // "image"
QT_MOC_LITERAL(14, 226, 29), // "onCaptureWhiteBGButtonPressed"
QT_MOC_LITERAL(15, 256, 16), // "onOnionCBToggled"
QT_MOC_LITERAL(16, 273, 24), // "onLoadImageButtonPressed"
QT_MOC_LITERAL(17, 298, 23), // "onOnionOpacityFldEdited"
QT_MOC_LITERAL(18, 322, 16), // "onTimerCBToggled"
QT_MOC_LITERAL(19, 339, 21), // "onCaptureTimerTimeout"
QT_MOC_LITERAL(20, 361, 11), // "onCountDown"
QT_MOC_LITERAL(21, 373, 22), // "onCaptureButtonClicked"
QT_MOC_LITERAL(22, 396, 33), // "onCaptureFilterSettingsBtnPre..."
QT_MOC_LITERAL(23, 430, 16), // "refreshFrameInfo"
QT_MOC_LITERAL(24, 447, 18), // "onSaveInPathEdited"
QT_MOC_LITERAL(25, 466, 17), // "onFileTypeChanged"
QT_MOC_LITERAL(26, 484, 15), // "onSceneSwitched"
QT_MOC_LITERAL(27, 500, 18), // "onSubCameraToggled"
QT_MOC_LITERAL(28, 519, 18), // "onSubCameraChanged"
QT_MOC_LITERAL(29, 538, 10), // "isDragging"
QT_MOC_LITERAL(30, 549, 21), // "onSubCameraRectEdited"
QT_MOC_LITERAL(31, 571, 25), // "onSubCameraPresetSelected"
QT_MOC_LITERAL(32, 597, 9), // "onTimeout"
QT_MOC_LITERAL(33, 607, 20), // "onCalibCapBtnClicked"
QT_MOC_LITERAL(34, 628, 20), // "onCalibNewBtnClicked"
QT_MOC_LITERAL(35, 649, 26), // "resetCalibSettingsFromFile"
QT_MOC_LITERAL(36, 676, 21), // "onCalibLoadBtnClicked"
QT_MOC_LITERAL(37, 698, 23), // "onCalibExportBtnClicked"
QT_MOC_LITERAL(38, 722, 13), // "onCalibReadme"
QT_MOC_LITERAL(39, 736, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(40, 756, 22), // "saveImageAdjustDefault"
QT_MOC_LITERAL(41, 779, 21) // "openSaveInFolderPopup"

    },
    "PencilTestPopup\0refreshCameraList\0\0"
    "onCameraListComboActivated\0index\0"
    "onResolutionComboActivated\0"
    "onFileFormatOptionButtonPressed\0"
    "onLevelNameEdited\0onNextName\0"
    "onPreviousName\0onColorTypeComboChanged\0"
    "onFrameCaptured\0cv::Mat&\0image\0"
    "onCaptureWhiteBGButtonPressed\0"
    "onOnionCBToggled\0onLoadImageButtonPressed\0"
    "onOnionOpacityFldEdited\0onTimerCBToggled\0"
    "onCaptureTimerTimeout\0onCountDown\0"
    "onCaptureButtonClicked\0"
    "onCaptureFilterSettingsBtnPressed\0"
    "refreshFrameInfo\0onSaveInPathEdited\0"
    "onFileTypeChanged\0onSceneSwitched\0"
    "onSubCameraToggled\0onSubCameraChanged\0"
    "isDragging\0onSubCameraRectEdited\0"
    "onSubCameraPresetSelected\0onTimeout\0"
    "onCalibCapBtnClicked\0onCalibNewBtnClicked\0"
    "resetCalibSettingsFromFile\0"
    "onCalibLoadBtnClicked\0onCalibExportBtnClicked\0"
    "onCalibReadme\0onPreferenceChanged\0"
    "saveImageAdjustDefault\0openSaveInFolderPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PencilTestPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      36,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  194,    2, 0x09 /* Protected */,
       3,    1,  195,    2, 0x09 /* Protected */,
       5,    0,  198,    2, 0x09 /* Protected */,
       6,    0,  199,    2, 0x09 /* Protected */,
       7,    0,  200,    2, 0x09 /* Protected */,
       8,    0,  201,    2, 0x09 /* Protected */,
       9,    0,  202,    2, 0x09 /* Protected */,
      10,    1,  203,    2, 0x09 /* Protected */,
      11,    1,  206,    2, 0x09 /* Protected */,
      14,    0,  209,    2, 0x09 /* Protected */,
      15,    1,  210,    2, 0x09 /* Protected */,
      16,    0,  213,    2, 0x09 /* Protected */,
      17,    0,  214,    2, 0x09 /* Protected */,
      18,    1,  215,    2, 0x09 /* Protected */,
      19,    0,  218,    2, 0x09 /* Protected */,
      20,    0,  219,    2, 0x09 /* Protected */,
      21,    1,  220,    2, 0x09 /* Protected */,
      22,    0,  223,    2, 0x09 /* Protected */,
      23,    0,  224,    2, 0x09 /* Protected */,
      24,    0,  225,    2, 0x09 /* Protected */,
      25,    0,  226,    2, 0x09 /* Protected */,
      26,    0,  227,    2, 0x09 /* Protected */,
      27,    1,  228,    2, 0x09 /* Protected */,
      28,    1,  231,    2, 0x09 /* Protected */,
      30,    0,  234,    2, 0x09 /* Protected */,
      31,    2,  235,    2, 0x09 /* Protected */,
      32,    0,  240,    2, 0x09 /* Protected */,
      33,    0,  241,    2, 0x09 /* Protected */,
      34,    0,  242,    2, 0x09 /* Protected */,
      35,    0,  243,    2, 0x09 /* Protected */,
      36,    0,  244,    2, 0x09 /* Protected */,
      37,    0,  245,    2, 0x09 /* Protected */,
      38,    0,  246,    2, 0x09 /* Protected */,
      39,    1,  247,    2, 0x09 /* Protected */,
      40,    0,  250,    2, 0x09 /* Protected */,
      41,    0,  251,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,   29,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRect, QMetaType::Double,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PencilTestPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PencilTestPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refreshCameraList(); break;
        case 1: _t->onCameraListComboActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->onResolutionComboActivated(); break;
        case 3: _t->onFileFormatOptionButtonPressed(); break;
        case 4: _t->onLevelNameEdited(); break;
        case 5: _t->onNextName(); break;
        case 6: _t->onPreviousName(); break;
        case 7: _t->onColorTypeComboChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->onFrameCaptured((*reinterpret_cast< cv::Mat(*)>(_a[1]))); break;
        case 9: _t->onCaptureWhiteBGButtonPressed(); break;
        case 10: _t->onOnionCBToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->onLoadImageButtonPressed(); break;
        case 12: _t->onOnionOpacityFldEdited(); break;
        case 13: _t->onTimerCBToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->onCaptureTimerTimeout(); break;
        case 15: _t->onCountDown(); break;
        case 16: _t->onCaptureButtonClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->onCaptureFilterSettingsBtnPressed(); break;
        case 18: _t->refreshFrameInfo(); break;
        case 19: _t->onSaveInPathEdited(); break;
        case 20: _t->onFileTypeChanged(); break;
        case 21: _t->onSceneSwitched(); break;
        case 22: _t->onSubCameraToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->onSubCameraChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->onSubCameraRectEdited(); break;
        case 25: _t->onSubCameraPresetSelected((*reinterpret_cast< const QRect(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 26: _t->onTimeout(); break;
        case 27: _t->onCalibCapBtnClicked(); break;
        case 28: _t->onCalibNewBtnClicked(); break;
        case 29: _t->resetCalibSettingsFromFile(); break;
        case 30: _t->onCalibLoadBtnClicked(); break;
        case 31: _t->onCalibExportBtnClicked(); break;
        case 32: _t->onCalibReadme(); break;
        case 33: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 34: _t->saveImageAdjustDefault(); break;
        case 35: _t->openSaveInFolderPopup(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PencilTestPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_PencilTestPopup.data,
    qt_meta_data_PencilTestPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PencilTestPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PencilTestPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PencilTestPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int PencilTestPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 36)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 36)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 36;
    }
    return _id;
}
struct qt_meta_stringdata_ExportCalibrationFilePopup_t {
    QByteArrayData data[1];
    char stringdata0[27];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ExportCalibrationFilePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ExportCalibrationFilePopup_t qt_meta_stringdata_ExportCalibrationFilePopup = {
    {
QT_MOC_LITERAL(0, 0, 26) // "ExportCalibrationFilePopup"

    },
    "ExportCalibrationFilePopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ExportCalibrationFilePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ExportCalibrationFilePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ExportCalibrationFilePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericSaveFilePopup::staticMetaObject>(),
    qt_meta_stringdata_ExportCalibrationFilePopup.data,
    qt_meta_data_ExportCalibrationFilePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ExportCalibrationFilePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportCalibrationFilePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ExportCalibrationFilePopup.stringdata0))
        return static_cast<void*>(this);
    return GenericSaveFilePopup::qt_metacast(_clname);
}

int ExportCalibrationFilePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericSaveFilePopup::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_LoadCalibrationFilePopup_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LoadCalibrationFilePopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LoadCalibrationFilePopup_t qt_meta_stringdata_LoadCalibrationFilePopup = {
    {
QT_MOC_LITERAL(0, 0, 24) // "LoadCalibrationFilePopup"

    },
    "LoadCalibrationFilePopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LoadCalibrationFilePopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void LoadCalibrationFilePopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject LoadCalibrationFilePopup::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericLoadFilePopup::staticMetaObject>(),
    qt_meta_stringdata_LoadCalibrationFilePopup.data,
    qt_meta_data_LoadCalibrationFilePopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LoadCalibrationFilePopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LoadCalibrationFilePopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LoadCalibrationFilePopup.stringdata0))
        return static_cast<void*>(this);
    return GenericLoadFilePopup::qt_metacast(_clname);
}

int LoadCalibrationFilePopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericLoadFilePopup::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
