/****************************************************************************
** Meta object code from reading C++ file 'stopmotioncontroller.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../opentoonz/toonz/sources/stopmotion/stopmotioncontroller.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'stopmotioncontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StopMotionController_t {
    QByteArrayData data[145];
    char stringdata0[2982];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StopMotionController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StopMotionController_t qt_meta_stringdata_StopMotionController = {
    {
QT_MOC_LITERAL(0, 0, 20), // "StopMotionController"
QT_MOC_LITERAL(1, 21, 17), // "refreshCameraList"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 12), // "activeCamera"
QT_MOC_LITERAL(4, 53, 23), // "refreshCameraListCalled"
QT_MOC_LITERAL(5, 77, 19), // "refreshOptionsLists"
QT_MOC_LITERAL(6, 97, 26), // "onCameraListComboActivated"
QT_MOC_LITERAL(7, 124, 5), // "index"
QT_MOC_LITERAL(8, 130, 26), // "onResolutionComboActivated"
QT_MOC_LITERAL(9, 157, 8), // "itemText"
QT_MOC_LITERAL(10, 166, 33), // "onCaptureFilterSettingsBtnPre..."
QT_MOC_LITERAL(11, 200, 31), // "onFileFormatOptionButtonPressed"
QT_MOC_LITERAL(12, 232, 17), // "onLevelNameEdited"
QT_MOC_LITERAL(13, 250, 10), // "onNextName"
QT_MOC_LITERAL(14, 261, 14), // "onPreviousName"
QT_MOC_LITERAL(15, 276, 11), // "onNextFrame"
QT_MOC_LITERAL(16, 288, 15), // "onPreviousFrame"
QT_MOC_LITERAL(17, 304, 14), // "onNextNewLevel"
QT_MOC_LITERAL(18, 319, 11), // "onLastFrame"
QT_MOC_LITERAL(19, 331, 19), // "onFileTypeActivated"
QT_MOC_LITERAL(20, 351, 20), // "onFrameNumberChanged"
QT_MOC_LITERAL(21, 372, 26), // "onXSheetFrameNumberChanged"
QT_MOC_LITERAL(22, 399, 15), // "onFrameCaptured"
QT_MOC_LITERAL(23, 415, 7), // "QImage&"
QT_MOC_LITERAL(24, 423, 5), // "image"
QT_MOC_LITERAL(25, 429, 23), // "onOnionOpacityFldEdited"
QT_MOC_LITERAL(26, 453, 27), // "onOnionOpacitySliderChanged"
QT_MOC_LITERAL(27, 481, 6), // "ignore"
QT_MOC_LITERAL(28, 488, 23), // "onLiveViewToggleClicked"
QT_MOC_LITERAL(29, 512, 22), // "onCaptureButtonClicked"
QT_MOC_LITERAL(30, 535, 7), // "setPage"
QT_MOC_LITERAL(31, 543, 28), // "onScaleFullSizeImagesChanged"
QT_MOC_LITERAL(32, 572, 7), // "checked"
QT_MOC_LITERAL(33, 580, 30), // "onBlackScreenForCaptureChanged"
QT_MOC_LITERAL(34, 611, 22), // "onPlaceOnXSheetChanged"
QT_MOC_LITERAL(35, 634, 16), // "onUseMjpgChanged"
QT_MOC_LITERAL(36, 651, 22), // "onUseDirectShowChanged"
QT_MOC_LITERAL(37, 674, 28), // "onLiveViewOnAllFramesChanged"
QT_MOC_LITERAL(38, 703, 18), // "onUseNumpadChanged"
QT_MOC_LITERAL(39, 722, 20), // "onDrawBeneathChanged"
QT_MOC_LITERAL(40, 743, 16), // "updateDimensions"
QT_MOC_LITERAL(41, 760, 18), // "onSaveInPathEdited"
QT_MOC_LITERAL(42, 779, 15), // "onSceneSwitched"
QT_MOC_LITERAL(43, 795, 21), // "onPreviousXSheetFrame"
QT_MOC_LITERAL(44, 817, 17), // "onNextXSheetFrame"
QT_MOC_LITERAL(45, 835, 23), // "setToCurrentXSheetFrame"
QT_MOC_LITERAL(46, 859, 17), // "serialPortChanged"
QT_MOC_LITERAL(47, 877, 24), // "onIntervalTimerCBToggled"
QT_MOC_LITERAL(48, 902, 28), // "onIntervalSliderValueChanged"
QT_MOC_LITERAL(49, 931, 29), // "onIntervalCaptureTimerTimeout"
QT_MOC_LITERAL(50, 961, 26), // "onIntervalCountDownTimeout"
QT_MOC_LITERAL(51, 988, 23), // "onIntervalAmountChanged"
QT_MOC_LITERAL(52, 1012, 17), // "onIntervalToggled"
QT_MOC_LITERAL(53, 1030, 17), // "onIntervalStarted"
QT_MOC_LITERAL(54, 1048, 17), // "onIntervalStopped"
QT_MOC_LITERAL(55, 1066, 18), // "onPlaySoundToggled"
QT_MOC_LITERAL(56, 1085, 15), // "setScreen1Color"
QT_MOC_LITERAL(57, 1101, 8), // "TPixel32"
QT_MOC_LITERAL(58, 1110, 5), // "value"
QT_MOC_LITERAL(59, 1116, 10), // "isDragging"
QT_MOC_LITERAL(60, 1127, 15), // "setScreen2Color"
QT_MOC_LITERAL(61, 1143, 15), // "setScreen3Color"
QT_MOC_LITERAL(62, 1159, 23), // "onScreen1OverlayToggled"
QT_MOC_LITERAL(63, 1183, 23), // "onScreen2OverlayToggled"
QT_MOC_LITERAL(64, 1207, 23), // "onScreen3OverlayToggled"
QT_MOC_LITERAL(65, 1231, 19), // "onTestLightsPressed"
QT_MOC_LITERAL(66, 1251, 19), // "onTestLightsTimeout"
QT_MOC_LITERAL(67, 1271, 21), // "onScreen1ColorChanged"
QT_MOC_LITERAL(68, 1293, 21), // "onScreen2ColorChanged"
QT_MOC_LITERAL(69, 1315, 21), // "onScreen3ColorChanged"
QT_MOC_LITERAL(70, 1337, 23), // "onScreen1OverlayChanged"
QT_MOC_LITERAL(71, 1361, 23), // "onScreen2OverlayChanged"
QT_MOC_LITERAL(72, 1385, 23), // "onScreen3OverlayChanged"
QT_MOC_LITERAL(73, 1409, 17), // "onApertureChanged"
QT_MOC_LITERAL(74, 1427, 21), // "onShutterSpeedChanged"
QT_MOC_LITERAL(75, 1449, 12), // "onIsoChanged"
QT_MOC_LITERAL(76, 1462, 17), // "onExposureChanged"
QT_MOC_LITERAL(77, 1480, 21), // "onWhiteBalanceChanged"
QT_MOC_LITERAL(78, 1502, 25), // "onColorTemperatureChanged"
QT_MOC_LITERAL(79, 1528, 21), // "onImageQualityChanged"
QT_MOC_LITERAL(80, 1550, 21), // "onPictureStyleChanged"
QT_MOC_LITERAL(81, 1572, 13), // "onZoomPressed"
QT_MOC_LITERAL(82, 1586, 17), // "onPickZoomPressed"
QT_MOC_LITERAL(83, 1604, 11), // "onFocusNear"
QT_MOC_LITERAL(84, 1616, 10), // "onFocusFar"
QT_MOC_LITERAL(85, 1627, 12), // "onFocusNear2"
QT_MOC_LITERAL(86, 1640, 11), // "onFocusFar2"
QT_MOC_LITERAL(87, 1652, 12), // "onFocusNear3"
QT_MOC_LITERAL(88, 1665, 11), // "onFocusFar3"
QT_MOC_LITERAL(89, 1677, 23), // "onApertureChangedSignal"
QT_MOC_LITERAL(90, 1701, 18), // "onIsoChangedSignal"
QT_MOC_LITERAL(91, 1720, 27), // "onShutterSpeedChangedSignal"
QT_MOC_LITERAL(92, 1748, 23), // "onExposureChangedSignal"
QT_MOC_LITERAL(93, 1772, 27), // "onWhiteBalanceChangedSignal"
QT_MOC_LITERAL(94, 1800, 31), // "onColorTemperatureChangedSignal"
QT_MOC_LITERAL(95, 1832, 27), // "onImageQualityChangedSignal"
QT_MOC_LITERAL(96, 1860, 27), // "onPictureStyleChangedSignal"
QT_MOC_LITERAL(97, 1888, 19), // "refreshApertureList"
QT_MOC_LITERAL(98, 1908, 23), // "refreshShutterSpeedList"
QT_MOC_LITERAL(99, 1932, 14), // "refreshIsoList"
QT_MOC_LITERAL(100, 1947, 19), // "refreshExposureList"
QT_MOC_LITERAL(101, 1967, 23), // "refreshWhiteBalanceList"
QT_MOC_LITERAL(102, 1991, 27), // "refreshColorTemperatureList"
QT_MOC_LITERAL(103, 2019, 23), // "refreshImageQualityList"
QT_MOC_LITERAL(104, 2043, 23), // "refreshPictureStyleList"
QT_MOC_LITERAL(105, 2067, 11), // "refreshMode"
QT_MOC_LITERAL(106, 2079, 19), // "onFocusCheckToggled"
QT_MOC_LITERAL(107, 2099, 2), // "on"
QT_MOC_LITERAL(108, 2102, 23), // "onPickFocusCheckToggled"
QT_MOC_LITERAL(109, 2126, 38), // "onAlwaysUseLiveViewImagesButt..."
QT_MOC_LITERAL(110, 2165, 32), // "onAlwaysUseLiveViewImagesToggled"
QT_MOC_LITERAL(111, 2198, 24), // "onCaptureReviewFldEdited"
QT_MOC_LITERAL(112, 2223, 28), // "onCaptureReviewSliderChanged"
QT_MOC_LITERAL(113, 2252, 22), // "onSubsamplingFldEdited"
QT_MOC_LITERAL(114, 2275, 26), // "onSubsamplingSliderChanged"
QT_MOC_LITERAL(115, 2302, 20), // "onSubsamplingChanged"
QT_MOC_LITERAL(116, 2323, 17), // "onFilePathChanged"
QT_MOC_LITERAL(117, 2341, 18), // "onLevelNameChanged"
QT_MOC_LITERAL(118, 2360, 17), // "onFileTypeChanged"
QT_MOC_LITERAL(119, 2378, 22), // "onFrameInfoTextChanged"
QT_MOC_LITERAL(120, 2401, 16), // "onOpacityChanged"
QT_MOC_LITERAL(121, 2418, 7), // "opacity"
QT_MOC_LITERAL(122, 2426, 27), // "onScaleFullSizeImagesSignal"
QT_MOC_LITERAL(123, 2454, 20), // "onBlackCaptureSignal"
QT_MOC_LITERAL(124, 2475, 27), // "onLiveViewOnAllFramesSignal"
QT_MOC_LITERAL(125, 2503, 21), // "onPlaceOnXSheetSignal"
QT_MOC_LITERAL(126, 2525, 15), // "onUseMjpgSignal"
QT_MOC_LITERAL(127, 2541, 21), // "onUseDirectShowSignal"
QT_MOC_LITERAL(128, 2563, 25), // "onReviewTimeChangedSignal"
QT_MOC_LITERAL(129, 2589, 19), // "onPlayCaptureSignal"
QT_MOC_LITERAL(130, 2609, 17), // "onUseNumpadSignal"
QT_MOC_LITERAL(131, 2627, 19), // "onDrawBeneathSignal"
QT_MOC_LITERAL(132, 2647, 17), // "onLiveViewChanged"
QT_MOC_LITERAL(133, 2665, 19), // "onNewCameraSelected"
QT_MOC_LITERAL(134, 2685, 26), // "onWebcamResolutionsChanged"
QT_MOC_LITERAL(135, 2712, 29), // "onNewWebcamResolutionSelected"
QT_MOC_LITERAL(136, 2742, 24), // "onWebcamAutofocusToggled"
QT_MOC_LITERAL(137, 2767, 26), // "onWebcamFocusSliderChanged"
QT_MOC_LITERAL(138, 2794, 29), // "onWebcamExposureSliderChanged"
QT_MOC_LITERAL(139, 2824, 31), // "onWebcamBrightnessSliderChanged"
QT_MOC_LITERAL(140, 2856, 29), // "onWebcamContrastSliderChanged"
QT_MOC_LITERAL(141, 2886, 25), // "onWebcamGainSliderChanged"
QT_MOC_LITERAL(142, 2912, 31), // "onWebcamSaturationSliderChanged"
QT_MOC_LITERAL(143, 2944, 15), // "getWebcamStatus"
QT_MOC_LITERAL(144, 2960, 21) // "openSaveInFolderPopup"

    },
    "StopMotionController\0refreshCameraList\0"
    "\0activeCamera\0refreshCameraListCalled\0"
    "refreshOptionsLists\0onCameraListComboActivated\0"
    "index\0onResolutionComboActivated\0"
    "itemText\0onCaptureFilterSettingsBtnPressed\0"
    "onFileFormatOptionButtonPressed\0"
    "onLevelNameEdited\0onNextName\0"
    "onPreviousName\0onNextFrame\0onPreviousFrame\0"
    "onNextNewLevel\0onLastFrame\0"
    "onFileTypeActivated\0onFrameNumberChanged\0"
    "onXSheetFrameNumberChanged\0onFrameCaptured\0"
    "QImage&\0image\0onOnionOpacityFldEdited\0"
    "onOnionOpacitySliderChanged\0ignore\0"
    "onLiveViewToggleClicked\0onCaptureButtonClicked\0"
    "setPage\0onScaleFullSizeImagesChanged\0"
    "checked\0onBlackScreenForCaptureChanged\0"
    "onPlaceOnXSheetChanged\0onUseMjpgChanged\0"
    "onUseDirectShowChanged\0"
    "onLiveViewOnAllFramesChanged\0"
    "onUseNumpadChanged\0onDrawBeneathChanged\0"
    "updateDimensions\0onSaveInPathEdited\0"
    "onSceneSwitched\0onPreviousXSheetFrame\0"
    "onNextXSheetFrame\0setToCurrentXSheetFrame\0"
    "serialPortChanged\0onIntervalTimerCBToggled\0"
    "onIntervalSliderValueChanged\0"
    "onIntervalCaptureTimerTimeout\0"
    "onIntervalCountDownTimeout\0"
    "onIntervalAmountChanged\0onIntervalToggled\0"
    "onIntervalStarted\0onIntervalStopped\0"
    "onPlaySoundToggled\0setScreen1Color\0"
    "TPixel32\0value\0isDragging\0setScreen2Color\0"
    "setScreen3Color\0onScreen1OverlayToggled\0"
    "onScreen2OverlayToggled\0onScreen3OverlayToggled\0"
    "onTestLightsPressed\0onTestLightsTimeout\0"
    "onScreen1ColorChanged\0onScreen2ColorChanged\0"
    "onScreen3ColorChanged\0onScreen1OverlayChanged\0"
    "onScreen2OverlayChanged\0onScreen3OverlayChanged\0"
    "onApertureChanged\0onShutterSpeedChanged\0"
    "onIsoChanged\0onExposureChanged\0"
    "onWhiteBalanceChanged\0onColorTemperatureChanged\0"
    "onImageQualityChanged\0onPictureStyleChanged\0"
    "onZoomPressed\0onPickZoomPressed\0"
    "onFocusNear\0onFocusFar\0onFocusNear2\0"
    "onFocusFar2\0onFocusNear3\0onFocusFar3\0"
    "onApertureChangedSignal\0onIsoChangedSignal\0"
    "onShutterSpeedChangedSignal\0"
    "onExposureChangedSignal\0"
    "onWhiteBalanceChangedSignal\0"
    "onColorTemperatureChangedSignal\0"
    "onImageQualityChangedSignal\0"
    "onPictureStyleChangedSignal\0"
    "refreshApertureList\0refreshShutterSpeedList\0"
    "refreshIsoList\0refreshExposureList\0"
    "refreshWhiteBalanceList\0"
    "refreshColorTemperatureList\0"
    "refreshImageQualityList\0refreshPictureStyleList\0"
    "refreshMode\0onFocusCheckToggled\0on\0"
    "onPickFocusCheckToggled\0"
    "onAlwaysUseLiveViewImagesButtonClicked\0"
    "onAlwaysUseLiveViewImagesToggled\0"
    "onCaptureReviewFldEdited\0"
    "onCaptureReviewSliderChanged\0"
    "onSubsamplingFldEdited\0"
    "onSubsamplingSliderChanged\0"
    "onSubsamplingChanged\0onFilePathChanged\0"
    "onLevelNameChanged\0onFileTypeChanged\0"
    "onFrameInfoTextChanged\0onOpacityChanged\0"
    "opacity\0onScaleFullSizeImagesSignal\0"
    "onBlackCaptureSignal\0onLiveViewOnAllFramesSignal\0"
    "onPlaceOnXSheetSignal\0onUseMjpgSignal\0"
    "onUseDirectShowSignal\0onReviewTimeChangedSignal\0"
    "onPlayCaptureSignal\0onUseNumpadSignal\0"
    "onDrawBeneathSignal\0onLiveViewChanged\0"
    "onNewCameraSelected\0onWebcamResolutionsChanged\0"
    "onNewWebcamResolutionSelected\0"
    "onWebcamAutofocusToggled\0"
    "onWebcamFocusSliderChanged\0"
    "onWebcamExposureSliderChanged\0"
    "onWebcamBrightnessSliderChanged\0"
    "onWebcamContrastSliderChanged\0"
    "onWebcamGainSliderChanged\0"
    "onWebcamSaturationSliderChanged\0"
    "getWebcamStatus\0openSaveInFolderPopup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StopMotionController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     134,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  684,    2, 0x09 /* Protected */,
       1,    0,  687,    2, 0x29 /* Protected | MethodCloned */,
       4,    0,  688,    2, 0x09 /* Protected */,
       5,    0,  689,    2, 0x09 /* Protected */,
       6,    1,  690,    2, 0x09 /* Protected */,
       8,    1,  693,    2, 0x09 /* Protected */,
      10,    0,  696,    2, 0x09 /* Protected */,
      11,    0,  697,    2, 0x09 /* Protected */,
      12,    0,  698,    2, 0x09 /* Protected */,
      13,    0,  699,    2, 0x09 /* Protected */,
      14,    0,  700,    2, 0x09 /* Protected */,
      15,    0,  701,    2, 0x09 /* Protected */,
      16,    0,  702,    2, 0x09 /* Protected */,
      17,    0,  703,    2, 0x09 /* Protected */,
      18,    0,  704,    2, 0x09 /* Protected */,
      19,    0,  705,    2, 0x09 /* Protected */,
      20,    0,  706,    2, 0x09 /* Protected */,
      21,    0,  707,    2, 0x09 /* Protected */,
      22,    1,  708,    2, 0x09 /* Protected */,
      25,    0,  711,    2, 0x09 /* Protected */,
      26,    1,  712,    2, 0x09 /* Protected */,
      28,    0,  715,    2, 0x09 /* Protected */,
      29,    1,  716,    2, 0x09 /* Protected */,
      30,    1,  719,    2, 0x09 /* Protected */,
      31,    1,  722,    2, 0x09 /* Protected */,
      33,    1,  725,    2, 0x09 /* Protected */,
      34,    1,  728,    2, 0x09 /* Protected */,
      35,    1,  731,    2, 0x09 /* Protected */,
      36,    1,  734,    2, 0x09 /* Protected */,
      37,    1,  737,    2, 0x09 /* Protected */,
      38,    1,  740,    2, 0x09 /* Protected */,
      39,    1,  743,    2, 0x09 /* Protected */,
      40,    0,  746,    2, 0x09 /* Protected */,
      41,    0,  747,    2, 0x09 /* Protected */,
      42,    0,  748,    2, 0x09 /* Protected */,
      43,    0,  749,    2, 0x09 /* Protected */,
      44,    0,  750,    2, 0x09 /* Protected */,
      45,    0,  751,    2, 0x09 /* Protected */,
      46,    1,  752,    2, 0x09 /* Protected */,
      47,    1,  755,    2, 0x09 /* Protected */,
      48,    1,  758,    2, 0x09 /* Protected */,
      49,    0,  761,    2, 0x09 /* Protected */,
      50,    0,  762,    2, 0x09 /* Protected */,
      51,    1,  763,    2, 0x09 /* Protected */,
      52,    1,  766,    2, 0x09 /* Protected */,
      53,    0,  769,    2, 0x09 /* Protected */,
      54,    0,  770,    2, 0x09 /* Protected */,
      55,    1,  771,    2, 0x09 /* Protected */,
      56,    2,  774,    2, 0x09 /* Protected */,
      60,    2,  779,    2, 0x09 /* Protected */,
      61,    2,  784,    2, 0x09 /* Protected */,
      62,    1,  789,    2, 0x09 /* Protected */,
      63,    1,  792,    2, 0x09 /* Protected */,
      64,    1,  795,    2, 0x09 /* Protected */,
      65,    0,  798,    2, 0x09 /* Protected */,
      66,    0,  799,    2, 0x09 /* Protected */,
      67,    1,  800,    2, 0x09 /* Protected */,
      68,    1,  803,    2, 0x09 /* Protected */,
      69,    1,  806,    2, 0x09 /* Protected */,
      70,    1,  809,    2, 0x09 /* Protected */,
      71,    1,  812,    2, 0x09 /* Protected */,
      72,    1,  815,    2, 0x09 /* Protected */,
      73,    1,  818,    2, 0x09 /* Protected */,
      74,    1,  821,    2, 0x09 /* Protected */,
      75,    1,  824,    2, 0x09 /* Protected */,
      76,    1,  827,    2, 0x09 /* Protected */,
      77,    1,  830,    2, 0x09 /* Protected */,
      78,    1,  833,    2, 0x09 /* Protected */,
      79,    1,  836,    2, 0x09 /* Protected */,
      80,    1,  839,    2, 0x09 /* Protected */,
      81,    0,  842,    2, 0x09 /* Protected */,
      82,    0,  843,    2, 0x09 /* Protected */,
      83,    0,  844,    2, 0x09 /* Protected */,
      84,    0,  845,    2, 0x09 /* Protected */,
      85,    0,  846,    2, 0x09 /* Protected */,
      86,    0,  847,    2, 0x09 /* Protected */,
      87,    0,  848,    2, 0x09 /* Protected */,
      88,    0,  849,    2, 0x09 /* Protected */,
      89,    1,  850,    2, 0x09 /* Protected */,
      90,    1,  853,    2, 0x09 /* Protected */,
      91,    1,  856,    2, 0x09 /* Protected */,
      92,    1,  859,    2, 0x09 /* Protected */,
      93,    1,  862,    2, 0x09 /* Protected */,
      94,    1,  865,    2, 0x09 /* Protected */,
      95,    1,  868,    2, 0x09 /* Protected */,
      96,    1,  871,    2, 0x09 /* Protected */,
      97,    0,  874,    2, 0x09 /* Protected */,
      98,    0,  875,    2, 0x09 /* Protected */,
      99,    0,  876,    2, 0x09 /* Protected */,
     100,    0,  877,    2, 0x09 /* Protected */,
     101,    0,  878,    2, 0x09 /* Protected */,
     102,    0,  879,    2, 0x09 /* Protected */,
     103,    0,  880,    2, 0x09 /* Protected */,
     104,    0,  881,    2, 0x09 /* Protected */,
     105,    0,  882,    2, 0x09 /* Protected */,
     106,    1,  883,    2, 0x09 /* Protected */,
     108,    1,  886,    2, 0x09 /* Protected */,
     109,    0,  889,    2, 0x09 /* Protected */,
     110,    1,  890,    2, 0x09 /* Protected */,
     111,    0,  893,    2, 0x09 /* Protected */,
     112,    1,  894,    2, 0x09 /* Protected */,
     113,    0,  897,    2, 0x09 /* Protected */,
     114,    1,  898,    2, 0x09 /* Protected */,
     115,    1,  901,    2, 0x09 /* Protected */,
     116,    1,  904,    2, 0x09 /* Protected */,
     117,    1,  907,    2, 0x09 /* Protected */,
     118,    1,  910,    2, 0x09 /* Protected */,
      21,    1,  913,    2, 0x09 /* Protected */,
      20,    1,  916,    2, 0x09 /* Protected */,
     119,    1,  919,    2, 0x09 /* Protected */,
     120,    1,  922,    2, 0x09 /* Protected */,
     122,    1,  925,    2, 0x09 /* Protected */,
     123,    1,  928,    2, 0x09 /* Protected */,
     124,    1,  931,    2, 0x09 /* Protected */,
     125,    1,  934,    2, 0x09 /* Protected */,
     126,    1,  937,    2, 0x09 /* Protected */,
     127,    1,  940,    2, 0x09 /* Protected */,
     128,    1,  943,    2, 0x09 /* Protected */,
     129,    1,  946,    2, 0x09 /* Protected */,
     130,    1,  949,    2, 0x09 /* Protected */,
     131,    1,  952,    2, 0x09 /* Protected */,
     132,    1,  955,    2, 0x09 /* Protected */,
     133,    2,  958,    2, 0x09 /* Protected */,
     134,    0,  963,    2, 0x09 /* Protected */,
     135,    1,  964,    2, 0x09 /* Protected */,
     136,    1,  967,    2, 0x09 /* Protected */,
     137,    1,  970,    2, 0x09 /* Protected */,
     138,    1,  973,    2, 0x09 /* Protected */,
     139,    1,  976,    2, 0x09 /* Protected */,
     140,    1,  979,    2, 0x09 /* Protected */,
     141,    1,  982,    2, 0x09 /* Protected */,
     142,    1,  985,    2, 0x09 /* Protected */,
     143,    0,  988,    2, 0x09 /* Protected */,
     144,    0,  989,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   27,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void, QMetaType::Int,   32,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, 0x80000000 | 57, QMetaType::Bool,   58,   59,
    QMetaType::Void, 0x80000000 | 57, QMetaType::Bool,   58,   59,
    QMetaType::Void, 0x80000000 | 57, QMetaType::Bool,   58,   59,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 57,    2,
    QMetaType::Void, 0x80000000 | 57,    2,
    QMetaType::Void, 0x80000000 | 57,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  107,
    QMetaType::Void, QMetaType::Bool,  107,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   27,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   27,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,  121,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void, QMetaType::Int,   58,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void StopMotionController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StopMotionController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->refreshCameraList((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->refreshCameraList(); break;
        case 2: _t->refreshCameraListCalled(); break;
        case 3: _t->refreshOptionsLists(); break;
        case 4: _t->onCameraListComboActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onResolutionComboActivated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->onCaptureFilterSettingsBtnPressed(); break;
        case 7: _t->onFileFormatOptionButtonPressed(); break;
        case 8: _t->onLevelNameEdited(); break;
        case 9: _t->onNextName(); break;
        case 10: _t->onPreviousName(); break;
        case 11: _t->onNextFrame(); break;
        case 12: _t->onPreviousFrame(); break;
        case 13: _t->onNextNewLevel(); break;
        case 14: _t->onLastFrame(); break;
        case 15: _t->onFileTypeActivated(); break;
        case 16: _t->onFrameNumberChanged(); break;
        case 17: _t->onXSheetFrameNumberChanged(); break;
        case 18: _t->onFrameCaptured((*reinterpret_cast< QImage(*)>(_a[1]))); break;
        case 19: _t->onOnionOpacityFldEdited(); break;
        case 20: _t->onOnionOpacitySliderChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->onLiveViewToggleClicked(); break;
        case 22: _t->onCaptureButtonClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->setPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->onScaleFullSizeImagesChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->onBlackScreenForCaptureChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->onPlaceOnXSheetChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->onUseMjpgChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->onUseDirectShowChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->onLiveViewOnAllFramesChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 30: _t->onUseNumpadChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->onDrawBeneathChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 32: _t->updateDimensions(); break;
        case 33: _t->onSaveInPathEdited(); break;
        case 34: _t->onSceneSwitched(); break;
        case 35: _t->onPreviousXSheetFrame(); break;
        case 36: _t->onNextXSheetFrame(); break;
        case 37: _t->setToCurrentXSheetFrame(); break;
        case 38: _t->serialPortChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 39: _t->onIntervalTimerCBToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 40: _t->onIntervalSliderValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 41: _t->onIntervalCaptureTimerTimeout(); break;
        case 42: _t->onIntervalCountDownTimeout(); break;
        case 43: _t->onIntervalAmountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 44: _t->onIntervalToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 45: _t->onIntervalStarted(); break;
        case 46: _t->onIntervalStopped(); break;
        case 47: _t->onPlaySoundToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 48: _t->setScreen1Color((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 49: _t->setScreen2Color((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 50: _t->setScreen3Color((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 51: _t->onScreen1OverlayToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 52: _t->onScreen2OverlayToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 53: _t->onScreen3OverlayToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 54: _t->onTestLightsPressed(); break;
        case 55: _t->onTestLightsTimeout(); break;
        case 56: _t->onScreen1ColorChanged((*reinterpret_cast< TPixel32(*)>(_a[1]))); break;
        case 57: _t->onScreen2ColorChanged((*reinterpret_cast< TPixel32(*)>(_a[1]))); break;
        case 58: _t->onScreen3ColorChanged((*reinterpret_cast< TPixel32(*)>(_a[1]))); break;
        case 59: _t->onScreen1OverlayChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 60: _t->onScreen2OverlayChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 61: _t->onScreen3OverlayChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 62: _t->onApertureChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 63: _t->onShutterSpeedChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 64: _t->onIsoChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 65: _t->onExposureChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 66: _t->onWhiteBalanceChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 67: _t->onColorTemperatureChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 68: _t->onImageQualityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 69: _t->onPictureStyleChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 70: _t->onZoomPressed(); break;
        case 71: _t->onPickZoomPressed(); break;
        case 72: _t->onFocusNear(); break;
        case 73: _t->onFocusFar(); break;
        case 74: _t->onFocusNear2(); break;
        case 75: _t->onFocusFar2(); break;
        case 76: _t->onFocusNear3(); break;
        case 77: _t->onFocusFar3(); break;
        case 78: _t->onApertureChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 79: _t->onIsoChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 80: _t->onShutterSpeedChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 81: _t->onExposureChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 82: _t->onWhiteBalanceChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 83: _t->onColorTemperatureChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 84: _t->onImageQualityChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 85: _t->onPictureStyleChangedSignal((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 86: _t->refreshApertureList(); break;
        case 87: _t->refreshShutterSpeedList(); break;
        case 88: _t->refreshIsoList(); break;
        case 89: _t->refreshExposureList(); break;
        case 90: _t->refreshWhiteBalanceList(); break;
        case 91: _t->refreshColorTemperatureList(); break;
        case 92: _t->refreshImageQualityList(); break;
        case 93: _t->refreshPictureStyleList(); break;
        case 94: _t->refreshMode(); break;
        case 95: _t->onFocusCheckToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 96: _t->onPickFocusCheckToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 97: _t->onAlwaysUseLiveViewImagesButtonClicked(); break;
        case 98: _t->onAlwaysUseLiveViewImagesToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 99: _t->onCaptureReviewFldEdited(); break;
        case 100: _t->onCaptureReviewSliderChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 101: _t->onSubsamplingFldEdited(); break;
        case 102: _t->onSubsamplingSliderChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 103: _t->onSubsamplingChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 104: _t->onFilePathChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 105: _t->onLevelNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 106: _t->onFileTypeChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 107: _t->onXSheetFrameNumberChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 108: _t->onFrameNumberChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 109: _t->onFrameInfoTextChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 110: _t->onOpacityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 111: _t->onScaleFullSizeImagesSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 112: _t->onBlackCaptureSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 113: _t->onLiveViewOnAllFramesSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 114: _t->onPlaceOnXSheetSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 115: _t->onUseMjpgSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 116: _t->onUseDirectShowSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 117: _t->onReviewTimeChangedSignal((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 118: _t->onPlayCaptureSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 119: _t->onUseNumpadSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 120: _t->onDrawBeneathSignal((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 121: _t->onLiveViewChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 122: _t->onNewCameraSelected((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 123: _t->onWebcamResolutionsChanged(); break;
        case 124: _t->onNewWebcamResolutionSelected((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 125: _t->onWebcamAutofocusToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 126: _t->onWebcamFocusSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 127: _t->onWebcamExposureSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 128: _t->onWebcamBrightnessSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 129: _t->onWebcamContrastSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 130: _t->onWebcamGainSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 131: _t->onWebcamSaturationSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 132: _t->getWebcamStatus(); break;
        case 133: _t->openSaveInFolderPopup(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StopMotionController::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_StopMotionController.data,
    qt_meta_data_StopMotionController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StopMotionController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StopMotionController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StopMotionController.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int StopMotionController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 134)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 134;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 134)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 134;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
