/****************************************************************************
** Meta object code from reading C++ file 'tooloptionsshortcutinvoker.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/tooloptionsshortcutinvoker.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tooloptionsshortcutinvoker.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ToolOptionsShortcutWorker__DoubleWorker_t {
    QByteArrayData data[7];
    char stringdata0[102];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionsShortcutWorker__DoubleWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionsShortcutWorker__DoubleWorker_t qt_meta_stringdata_ToolOptionsShortcutWorker__DoubleWorker = {
    {
QT_MOC_LITERAL(0, 0, 39), // "ToolOptionsShortcutWorker::Do..."
QT_MOC_LITERAL(1, 40, 8), // "increase"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 4), // "step"
QT_MOC_LITERAL(4, 55, 18), // "increaseFractional"
QT_MOC_LITERAL(5, 74, 8), // "decrease"
QT_MOC_LITERAL(6, 83, 18) // "decreaseFractional"

    },
    "ToolOptionsShortcutWorker::DoubleWorker\0"
    "increase\0\0step\0increaseFractional\0"
    "decrease\0decreaseFractional"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionsShortcutWorker__DoubleWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x0a /* Public */,
       1,    0,   47,    2, 0x2a /* Public | MethodCloned */,
       4,    0,   48,    2, 0x0a /* Public */,
       5,    1,   49,    2, 0x0a /* Public */,
       5,    0,   52,    2, 0x2a /* Public | MethodCloned */,
       6,    0,   53,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ToolOptionsShortcutWorker::DoubleWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DoubleWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->increase((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->increase(); break;
        case 2: _t->increaseFractional(); break;
        case 3: _t->decrease((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 4: _t->decrease(); break;
        case 5: _t->decreaseFractional(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionsShortcutWorker::DoubleWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionsShortcutWorker__DoubleWorker.data,
    qt_meta_data_ToolOptionsShortcutWorker__DoubleWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionsShortcutWorker::DoubleWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionsShortcutWorker::DoubleWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionsShortcutWorker__DoubleWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolOptionsShortcutWorker::DoubleWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionsShortcutWorker__DoublePairWorker_t {
    QByteArrayData data[6];
    char stringdata0[113];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionsShortcutWorker__DoublePairWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionsShortcutWorker__DoublePairWorker_t qt_meta_stringdata_ToolOptionsShortcutWorker__DoublePairWorker = {
    {
QT_MOC_LITERAL(0, 0, 43), // "ToolOptionsShortcutWorker::Do..."
QT_MOC_LITERAL(1, 44, 16), // "increaseMaxValue"
QT_MOC_LITERAL(2, 61, 0), // ""
QT_MOC_LITERAL(3, 62, 16), // "decreaseMaxValue"
QT_MOC_LITERAL(4, 79, 16), // "increaseMinValue"
QT_MOC_LITERAL(5, 96, 16) // "decreaseMinValue"

    },
    "ToolOptionsShortcutWorker::DoublePairWorker\0"
    "increaseMaxValue\0\0decreaseMaxValue\0"
    "increaseMinValue\0decreaseMinValue"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionsShortcutWorker__DoublePairWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x0a /* Public */,
       3,    0,   35,    2, 0x0a /* Public */,
       4,    0,   36,    2, 0x0a /* Public */,
       5,    0,   37,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ToolOptionsShortcutWorker::DoublePairWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DoublePairWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->increaseMaxValue(); break;
        case 1: _t->decreaseMaxValue(); break;
        case 2: _t->increaseMinValue(); break;
        case 3: _t->decreaseMinValue(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionsShortcutWorker::DoublePairWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionsShortcutWorker__DoublePairWorker.data,
    qt_meta_data_ToolOptionsShortcutWorker__DoublePairWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionsShortcutWorker::DoublePairWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionsShortcutWorker::DoublePairWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionsShortcutWorker__DoublePairWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolOptionsShortcutWorker::DoublePairWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionsShortcutWorker__IntPairWorker_t {
    QByteArrayData data[6];
    char stringdata0[110];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionsShortcutWorker__IntPairWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionsShortcutWorker__IntPairWorker_t qt_meta_stringdata_ToolOptionsShortcutWorker__IntPairWorker = {
    {
QT_MOC_LITERAL(0, 0, 40), // "ToolOptionsShortcutWorker::In..."
QT_MOC_LITERAL(1, 41, 16), // "increaseMaxValue"
QT_MOC_LITERAL(2, 58, 0), // ""
QT_MOC_LITERAL(3, 59, 16), // "decreaseMaxValue"
QT_MOC_LITERAL(4, 76, 16), // "increaseMinValue"
QT_MOC_LITERAL(5, 93, 16) // "decreaseMinValue"

    },
    "ToolOptionsShortcutWorker::IntPairWorker\0"
    "increaseMaxValue\0\0decreaseMaxValue\0"
    "increaseMinValue\0decreaseMinValue"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionsShortcutWorker__IntPairWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   34,    2, 0x0a /* Public */,
       3,    0,   35,    2, 0x0a /* Public */,
       4,    0,   36,    2, 0x0a /* Public */,
       5,    0,   37,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ToolOptionsShortcutWorker::IntPairWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<IntPairWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->increaseMaxValue(); break;
        case 1: _t->decreaseMaxValue(); break;
        case 2: _t->increaseMinValue(); break;
        case 3: _t->decreaseMinValue(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionsShortcutWorker::IntPairWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionsShortcutWorker__IntPairWorker.data,
    qt_meta_data_ToolOptionsShortcutWorker__IntPairWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionsShortcutWorker::IntPairWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionsShortcutWorker::IntPairWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionsShortcutWorker__IntPairWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolOptionsShortcutWorker::IntPairWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionsShortcutWorker__IntWorker_t {
    QByteArrayData data[4];
    char stringdata0[56];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionsShortcutWorker__IntWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionsShortcutWorker__IntWorker_t qt_meta_stringdata_ToolOptionsShortcutWorker__IntWorker = {
    {
QT_MOC_LITERAL(0, 0, 36), // "ToolOptionsShortcutWorker::In..."
QT_MOC_LITERAL(1, 37, 8), // "increase"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 8) // "decrease"

    },
    "ToolOptionsShortcutWorker::IntWorker\0"
    "increase\0\0decrease"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionsShortcutWorker__IntWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    0,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ToolOptionsShortcutWorker::IntWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<IntWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->increase(); break;
        case 1: _t->decrease(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionsShortcutWorker::IntWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionsShortcutWorker__IntWorker.data,
    qt_meta_data_ToolOptionsShortcutWorker__IntWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionsShortcutWorker::IntWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionsShortcutWorker::IntWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionsShortcutWorker__IntWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolOptionsShortcutWorker::IntWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionsShortcutWorker__BoolWorker_t {
    QByteArrayData data[3];
    char stringdata0[47];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionsShortcutWorker__BoolWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionsShortcutWorker__BoolWorker_t qt_meta_stringdata_ToolOptionsShortcutWorker__BoolWorker = {
    {
QT_MOC_LITERAL(0, 0, 37), // "ToolOptionsShortcutWorker::Bo..."
QT_MOC_LITERAL(1, 38, 7), // "doCheck"
QT_MOC_LITERAL(2, 46, 0) // ""

    },
    "ToolOptionsShortcutWorker::BoolWorker\0"
    "doCheck\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionsShortcutWorker__BoolWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,

       0        // eod
};

void ToolOptionsShortcutWorker::BoolWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BoolWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->doCheck((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionsShortcutWorker::BoolWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionsShortcutWorker__BoolWorker.data,
    qt_meta_data_ToolOptionsShortcutWorker__BoolWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionsShortcutWorker::BoolWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionsShortcutWorker::BoolWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionsShortcutWorker__BoolWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolOptionsShortcutWorker::BoolWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionsShortcutWorker__EnumWorker_t {
    QByteArrayData data[4];
    char stringdata0[66];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionsShortcutWorker__EnumWorker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionsShortcutWorker__EnumWorker_t qt_meta_stringdata_ToolOptionsShortcutWorker__EnumWorker = {
    {
QT_MOC_LITERAL(0, 0, 37), // "ToolOptionsShortcutWorker::En..."
QT_MOC_LITERAL(1, 38, 12), // "cycleOptions"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 13) // "doOnActivated"

    },
    "ToolOptionsShortcutWorker::EnumWorker\0"
    "cycleOptions\0\0doOnActivated"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionsShortcutWorker__EnumWorker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    1,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void ToolOptionsShortcutWorker::EnumWorker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<EnumWorker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cycleOptions(); break;
        case 1: _t->doOnActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionsShortcutWorker::EnumWorker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionsShortcutWorker__EnumWorker.data,
    qt_meta_data_ToolOptionsShortcutWorker__EnumWorker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionsShortcutWorker::EnumWorker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionsShortcutWorker::EnumWorker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionsShortcutWorker__EnumWorker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolOptionsShortcutWorker::EnumWorker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
struct qt_meta_stringdata_ToolOptionsShortcutInvoker_t {
    QByteArrayData data[74];
    char stringdata0[1601];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ToolOptionsShortcutInvoker_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ToolOptionsShortcutInvoker_t qt_meta_stringdata_ToolOptionsShortcutInvoker = {
    {
QT_MOC_LITERAL(0, 0, 26), // "ToolOptionsShortcutInvoker"
QT_MOC_LITERAL(1, 27, 14), // "onToolSwitched"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 18), // "toggleEditNextMode"
QT_MOC_LITERAL(4, 62, 18), // "toggleEditPosition"
QT_MOC_LITERAL(5, 81, 18), // "toggleEditRotation"
QT_MOC_LITERAL(6, 100, 19), // "toggleEditNextScale"
QT_MOC_LITERAL(7, 120, 19), // "toggleEditNextShear"
QT_MOC_LITERAL(8, 140, 20), // "toggleEditNextCenter"
QT_MOC_LITERAL(9, 161, 17), // "toggleEditNextAll"
QT_MOC_LITERAL(10, 179, 23), // "toggleSelectionNextType"
QT_MOC_LITERAL(11, 203, 26), // "toggleSelectionRectangular"
QT_MOC_LITERAL(12, 230, 23), // "toggleSelectionFreehand"
QT_MOC_LITERAL(13, 254, 23), // "toggleSelectionPolyline"
QT_MOC_LITERAL(14, 278, 24), // "toggleGeometricNextShape"
QT_MOC_LITERAL(15, 303, 24), // "toggleGeometricRectangle"
QT_MOC_LITERAL(16, 328, 21), // "toggleGeometricCircle"
QT_MOC_LITERAL(17, 350, 22), // "toggleGeometricEllipse"
QT_MOC_LITERAL(18, 373, 19), // "toggleGeometricLine"
QT_MOC_LITERAL(19, 393, 23), // "toggleGeometricPolyline"
QT_MOC_LITERAL(20, 417, 18), // "toggleGeometricArc"
QT_MOC_LITERAL(21, 436, 23), // "toggleGeometricMultiArc"
QT_MOC_LITERAL(22, 460, 22), // "toggleGeometricPolygon"
QT_MOC_LITERAL(23, 483, 19), // "toggleTypeNextStyle"
QT_MOC_LITERAL(24, 503, 17), // "toggleTypeOblique"
QT_MOC_LITERAL(25, 521, 17), // "toggleTypeRegular"
QT_MOC_LITERAL(26, 539, 21), // "toggleTypeBoldOblique"
QT_MOC_LITERAL(27, 561, 14), // "toggleTypeBold"
QT_MOC_LITERAL(28, 576, 24), // "togglePaintBrushNextMode"
QT_MOC_LITERAL(29, 601, 21), // "togglePaintBrushAreas"
QT_MOC_LITERAL(30, 623, 21), // "togglePaintBrushLines"
QT_MOC_LITERAL(31, 645, 29), // "togglePaintBrushLinesAndAreas"
QT_MOC_LITERAL(32, 675, 18), // "toggleFillNextType"
QT_MOC_LITERAL(33, 694, 16), // "toggleFillNormal"
QT_MOC_LITERAL(34, 711, 21), // "toggleFillRectangular"
QT_MOC_LITERAL(35, 733, 18), // "toggleFillFreehand"
QT_MOC_LITERAL(36, 752, 18), // "toggleFillPolyline"
QT_MOC_LITERAL(37, 771, 18), // "toggleFillFreepick"
QT_MOC_LITERAL(38, 790, 18), // "toggleFillNextMode"
QT_MOC_LITERAL(39, 809, 15), // "toggleFillAreas"
QT_MOC_LITERAL(40, 825, 15), // "toggleFillLines"
QT_MOC_LITERAL(41, 841, 23), // "toggleFillLinesAndAreas"
QT_MOC_LITERAL(42, 865, 20), // "toggleEraserNextType"
QT_MOC_LITERAL(43, 886, 18), // "toggleEraserNormal"
QT_MOC_LITERAL(44, 905, 23), // "toggleEraserRectangular"
QT_MOC_LITERAL(45, 929, 20), // "toggleEraserFreehand"
QT_MOC_LITERAL(46, 950, 20), // "toggleEraserPolyline"
QT_MOC_LITERAL(47, 971, 19), // "toggleEraserSegment"
QT_MOC_LITERAL(48, 991, 20), // "toggleEraserMultiArc"
QT_MOC_LITERAL(49, 1012, 18), // "toggleTapeNextType"
QT_MOC_LITERAL(50, 1031, 16), // "toggleTapeNormal"
QT_MOC_LITERAL(51, 1048, 21), // "toggleTapeRectangular"
QT_MOC_LITERAL(52, 1070, 18), // "toggleTapeNextMode"
QT_MOC_LITERAL(53, 1089, 28), // "toggleTapeEndpointToEndpoint"
QT_MOC_LITERAL(54, 1118, 24), // "toggleTapeEndpointToLine"
QT_MOC_LITERAL(55, 1143, 20), // "toggleTapeLineToLine"
QT_MOC_LITERAL(56, 1164, 23), // "togglePickStyleNextMode"
QT_MOC_LITERAL(57, 1188, 20), // "togglePickStyleAreas"
QT_MOC_LITERAL(58, 1209, 20), // "togglePickStyleLines"
QT_MOC_LITERAL(59, 1230, 28), // "togglePickStyleLinesAndAreas"
QT_MOC_LITERAL(60, 1259, 23), // "toggleRGBPickerNextType"
QT_MOC_LITERAL(61, 1283, 21), // "toggleRGBPickerNormal"
QT_MOC_LITERAL(62, 1305, 26), // "toggleRGBPickerRectangular"
QT_MOC_LITERAL(63, 1332, 23), // "toggleRGBPickerFreehand"
QT_MOC_LITERAL(64, 1356, 23), // "toggleRGBPickerPolyline"
QT_MOC_LITERAL(65, 1380, 22), // "ToggleSkeletonNextMode"
QT_MOC_LITERAL(66, 1403, 27), // "ToggleSkeletonBuildSkeleton"
QT_MOC_LITERAL(67, 1431, 21), // "ToggleSkeletonAnimate"
QT_MOC_LITERAL(68, 1453, 31), // "ToggleSkeletonInverseKinematics"
QT_MOC_LITERAL(69, 1485, 21), // "TogglePlasticNextMode"
QT_MOC_LITERAL(70, 1507, 21), // "TogglePlasticEditMesh"
QT_MOC_LITERAL(71, 1529, 23), // "TogglePlasticPaintRigid"
QT_MOC_LITERAL(72, 1553, 26), // "TogglePlasticBuildSkeleton"
QT_MOC_LITERAL(73, 1580, 20) // "TogglePlasticAnimate"

    },
    "ToolOptionsShortcutInvoker\0onToolSwitched\0"
    "\0toggleEditNextMode\0toggleEditPosition\0"
    "toggleEditRotation\0toggleEditNextScale\0"
    "toggleEditNextShear\0toggleEditNextCenter\0"
    "toggleEditNextAll\0toggleSelectionNextType\0"
    "toggleSelectionRectangular\0"
    "toggleSelectionFreehand\0toggleSelectionPolyline\0"
    "toggleGeometricNextShape\0"
    "toggleGeometricRectangle\0toggleGeometricCircle\0"
    "toggleGeometricEllipse\0toggleGeometricLine\0"
    "toggleGeometricPolyline\0toggleGeometricArc\0"
    "toggleGeometricMultiArc\0toggleGeometricPolygon\0"
    "toggleTypeNextStyle\0toggleTypeOblique\0"
    "toggleTypeRegular\0toggleTypeBoldOblique\0"
    "toggleTypeBold\0togglePaintBrushNextMode\0"
    "togglePaintBrushAreas\0togglePaintBrushLines\0"
    "togglePaintBrushLinesAndAreas\0"
    "toggleFillNextType\0toggleFillNormal\0"
    "toggleFillRectangular\0toggleFillFreehand\0"
    "toggleFillPolyline\0toggleFillFreepick\0"
    "toggleFillNextMode\0toggleFillAreas\0"
    "toggleFillLines\0toggleFillLinesAndAreas\0"
    "toggleEraserNextType\0toggleEraserNormal\0"
    "toggleEraserRectangular\0toggleEraserFreehand\0"
    "toggleEraserPolyline\0toggleEraserSegment\0"
    "toggleEraserMultiArc\0toggleTapeNextType\0"
    "toggleTapeNormal\0toggleTapeRectangular\0"
    "toggleTapeNextMode\0toggleTapeEndpointToEndpoint\0"
    "toggleTapeEndpointToLine\0toggleTapeLineToLine\0"
    "togglePickStyleNextMode\0togglePickStyleAreas\0"
    "togglePickStyleLines\0togglePickStyleLinesAndAreas\0"
    "toggleRGBPickerNextType\0toggleRGBPickerNormal\0"
    "toggleRGBPickerRectangular\0"
    "toggleRGBPickerFreehand\0toggleRGBPickerPolyline\0"
    "ToggleSkeletonNextMode\0"
    "ToggleSkeletonBuildSkeleton\0"
    "ToggleSkeletonAnimate\0"
    "ToggleSkeletonInverseKinematics\0"
    "TogglePlasticNextMode\0TogglePlasticEditMesh\0"
    "TogglePlasticPaintRigid\0"
    "TogglePlasticBuildSkeleton\0"
    "TogglePlasticAnimate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ToolOptionsShortcutInvoker[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      72,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  374,    2, 0x09 /* Protected */,
       3,    0,  375,    2, 0x09 /* Protected */,
       4,    0,  376,    2, 0x09 /* Protected */,
       5,    0,  377,    2, 0x09 /* Protected */,
       6,    0,  378,    2, 0x09 /* Protected */,
       7,    0,  379,    2, 0x09 /* Protected */,
       8,    0,  380,    2, 0x09 /* Protected */,
       9,    0,  381,    2, 0x09 /* Protected */,
      10,    0,  382,    2, 0x09 /* Protected */,
      11,    0,  383,    2, 0x09 /* Protected */,
      12,    0,  384,    2, 0x09 /* Protected */,
      13,    0,  385,    2, 0x09 /* Protected */,
      14,    0,  386,    2, 0x09 /* Protected */,
      15,    0,  387,    2, 0x09 /* Protected */,
      16,    0,  388,    2, 0x09 /* Protected */,
      17,    0,  389,    2, 0x09 /* Protected */,
      18,    0,  390,    2, 0x09 /* Protected */,
      19,    0,  391,    2, 0x09 /* Protected */,
      20,    0,  392,    2, 0x09 /* Protected */,
      21,    0,  393,    2, 0x09 /* Protected */,
      22,    0,  394,    2, 0x09 /* Protected */,
      23,    0,  395,    2, 0x09 /* Protected */,
      24,    0,  396,    2, 0x09 /* Protected */,
      25,    0,  397,    2, 0x09 /* Protected */,
      26,    0,  398,    2, 0x09 /* Protected */,
      27,    0,  399,    2, 0x09 /* Protected */,
      28,    0,  400,    2, 0x09 /* Protected */,
      29,    0,  401,    2, 0x09 /* Protected */,
      30,    0,  402,    2, 0x09 /* Protected */,
      31,    0,  403,    2, 0x09 /* Protected */,
      32,    0,  404,    2, 0x09 /* Protected */,
      33,    0,  405,    2, 0x09 /* Protected */,
      34,    0,  406,    2, 0x09 /* Protected */,
      35,    0,  407,    2, 0x09 /* Protected */,
      36,    0,  408,    2, 0x09 /* Protected */,
      37,    0,  409,    2, 0x09 /* Protected */,
      38,    0,  410,    2, 0x09 /* Protected */,
      39,    0,  411,    2, 0x09 /* Protected */,
      40,    0,  412,    2, 0x09 /* Protected */,
      41,    0,  413,    2, 0x09 /* Protected */,
      42,    0,  414,    2, 0x09 /* Protected */,
      43,    0,  415,    2, 0x09 /* Protected */,
      44,    0,  416,    2, 0x09 /* Protected */,
      45,    0,  417,    2, 0x09 /* Protected */,
      46,    0,  418,    2, 0x09 /* Protected */,
      47,    0,  419,    2, 0x09 /* Protected */,
      48,    0,  420,    2, 0x09 /* Protected */,
      49,    0,  421,    2, 0x09 /* Protected */,
      50,    0,  422,    2, 0x09 /* Protected */,
      51,    0,  423,    2, 0x09 /* Protected */,
      52,    0,  424,    2, 0x09 /* Protected */,
      53,    0,  425,    2, 0x09 /* Protected */,
      54,    0,  426,    2, 0x09 /* Protected */,
      55,    0,  427,    2, 0x09 /* Protected */,
      56,    0,  428,    2, 0x09 /* Protected */,
      57,    0,  429,    2, 0x09 /* Protected */,
      58,    0,  430,    2, 0x09 /* Protected */,
      59,    0,  431,    2, 0x09 /* Protected */,
      60,    0,  432,    2, 0x09 /* Protected */,
      61,    0,  433,    2, 0x09 /* Protected */,
      62,    0,  434,    2, 0x09 /* Protected */,
      63,    0,  435,    2, 0x09 /* Protected */,
      64,    0,  436,    2, 0x09 /* Protected */,
      65,    0,  437,    2, 0x09 /* Protected */,
      66,    0,  438,    2, 0x09 /* Protected */,
      67,    0,  439,    2, 0x09 /* Protected */,
      68,    0,  440,    2, 0x09 /* Protected */,
      69,    0,  441,    2, 0x09 /* Protected */,
      70,    0,  442,    2, 0x09 /* Protected */,
      71,    0,  443,    2, 0x09 /* Protected */,
      72,    0,  444,    2, 0x09 /* Protected */,
      73,    0,  445,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ToolOptionsShortcutInvoker::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ToolOptionsShortcutInvoker *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onToolSwitched(); break;
        case 1: _t->toggleEditNextMode(); break;
        case 2: _t->toggleEditPosition(); break;
        case 3: _t->toggleEditRotation(); break;
        case 4: _t->toggleEditNextScale(); break;
        case 5: _t->toggleEditNextShear(); break;
        case 6: _t->toggleEditNextCenter(); break;
        case 7: _t->toggleEditNextAll(); break;
        case 8: _t->toggleSelectionNextType(); break;
        case 9: _t->toggleSelectionRectangular(); break;
        case 10: _t->toggleSelectionFreehand(); break;
        case 11: _t->toggleSelectionPolyline(); break;
        case 12: _t->toggleGeometricNextShape(); break;
        case 13: _t->toggleGeometricRectangle(); break;
        case 14: _t->toggleGeometricCircle(); break;
        case 15: _t->toggleGeometricEllipse(); break;
        case 16: _t->toggleGeometricLine(); break;
        case 17: _t->toggleGeometricPolyline(); break;
        case 18: _t->toggleGeometricArc(); break;
        case 19: _t->toggleGeometricMultiArc(); break;
        case 20: _t->toggleGeometricPolygon(); break;
        case 21: _t->toggleTypeNextStyle(); break;
        case 22: _t->toggleTypeOblique(); break;
        case 23: _t->toggleTypeRegular(); break;
        case 24: _t->toggleTypeBoldOblique(); break;
        case 25: _t->toggleTypeBold(); break;
        case 26: _t->togglePaintBrushNextMode(); break;
        case 27: _t->togglePaintBrushAreas(); break;
        case 28: _t->togglePaintBrushLines(); break;
        case 29: _t->togglePaintBrushLinesAndAreas(); break;
        case 30: _t->toggleFillNextType(); break;
        case 31: _t->toggleFillNormal(); break;
        case 32: _t->toggleFillRectangular(); break;
        case 33: _t->toggleFillFreehand(); break;
        case 34: _t->toggleFillPolyline(); break;
        case 35: _t->toggleFillFreepick(); break;
        case 36: _t->toggleFillNextMode(); break;
        case 37: _t->toggleFillAreas(); break;
        case 38: _t->toggleFillLines(); break;
        case 39: _t->toggleFillLinesAndAreas(); break;
        case 40: _t->toggleEraserNextType(); break;
        case 41: _t->toggleEraserNormal(); break;
        case 42: _t->toggleEraserRectangular(); break;
        case 43: _t->toggleEraserFreehand(); break;
        case 44: _t->toggleEraserPolyline(); break;
        case 45: _t->toggleEraserSegment(); break;
        case 46: _t->toggleEraserMultiArc(); break;
        case 47: _t->toggleTapeNextType(); break;
        case 48: _t->toggleTapeNormal(); break;
        case 49: _t->toggleTapeRectangular(); break;
        case 50: _t->toggleTapeNextMode(); break;
        case 51: _t->toggleTapeEndpointToEndpoint(); break;
        case 52: _t->toggleTapeEndpointToLine(); break;
        case 53: _t->toggleTapeLineToLine(); break;
        case 54: _t->togglePickStyleNextMode(); break;
        case 55: _t->togglePickStyleAreas(); break;
        case 56: _t->togglePickStyleLines(); break;
        case 57: _t->togglePickStyleLinesAndAreas(); break;
        case 58: _t->toggleRGBPickerNextType(); break;
        case 59: _t->toggleRGBPickerNormal(); break;
        case 60: _t->toggleRGBPickerRectangular(); break;
        case 61: _t->toggleRGBPickerFreehand(); break;
        case 62: _t->toggleRGBPickerPolyline(); break;
        case 63: _t->ToggleSkeletonNextMode(); break;
        case 64: _t->ToggleSkeletonBuildSkeleton(); break;
        case 65: _t->ToggleSkeletonAnimate(); break;
        case 66: _t->ToggleSkeletonInverseKinematics(); break;
        case 67: _t->TogglePlasticNextMode(); break;
        case 68: _t->TogglePlasticEditMesh(); break;
        case 69: _t->TogglePlasticPaintRigid(); break;
        case 70: _t->TogglePlasticBuildSkeleton(); break;
        case 71: _t->TogglePlasticAnimate(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ToolOptionsShortcutInvoker::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ToolOptionsShortcutInvoker.data,
    qt_meta_data_ToolOptionsShortcutInvoker,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ToolOptionsShortcutInvoker::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ToolOptionsShortcutInvoker::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ToolOptionsShortcutInvoker.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ToolOptionsShortcutInvoker::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 72)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 72;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 72)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 72;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
