/****************************************************************************
** Meta object code from reading C++ file 'dvwidgets.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/dvwidgets.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dvwidgets.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__PropertyComboBox_t {
    QByteArrayData data[4];
    char stringdata0[52];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__PropertyComboBox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__PropertyComboBox_t qt_meta_stringdata_DVGui__PropertyComboBox = {
    {
QT_MOC_LITERAL(0, 0, 23), // "DVGui::PropertyComboBox"
QT_MOC_LITERAL(1, 24, 21), // "onCurrentIndexChanged"
QT_MOC_LITERAL(2, 46, 0), // ""
QT_MOC_LITERAL(3, 47, 4) // "text"

    },
    "DVGui::PropertyComboBox\0onCurrentIndexChanged\0"
    "\0text"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__PropertyComboBox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,

       0        // eod
};

void DVGui::PropertyComboBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyComboBox *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onCurrentIndexChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::PropertyComboBox::staticMetaObject = { {
    QMetaObject::SuperData::link<QComboBox::staticMetaObject>(),
    qt_meta_stringdata_DVGui__PropertyComboBox.data,
    qt_meta_data_DVGui__PropertyComboBox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::PropertyComboBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::PropertyComboBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__PropertyComboBox.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PropertyWidget"))
        return static_cast< PropertyWidget*>(this);
    return QComboBox::qt_metacast(_clname);
}

int DVGui::PropertyComboBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__PropertyCheckBox_t {
    QByteArrayData data[4];
    char stringdata0[46];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__PropertyCheckBox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__PropertyCheckBox_t qt_meta_stringdata_DVGui__PropertyCheckBox = {
    {
QT_MOC_LITERAL(0, 0, 23), // "DVGui::PropertyCheckBox"
QT_MOC_LITERAL(1, 24, 14), // "onStateChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 5) // "state"

    },
    "DVGui::PropertyCheckBox\0onStateChanged\0"
    "\0state"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__PropertyCheckBox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,

       0        // eod
};

void DVGui::PropertyCheckBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyCheckBox *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onStateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::PropertyCheckBox::staticMetaObject = { {
    QMetaObject::SuperData::link<CheckBox::staticMetaObject>(),
    qt_meta_stringdata_DVGui__PropertyCheckBox.data,
    qt_meta_data_DVGui__PropertyCheckBox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::PropertyCheckBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::PropertyCheckBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__PropertyCheckBox.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PropertyWidget"))
        return static_cast< PropertyWidget*>(this);
    return CheckBox::qt_metacast(_clname);
}

int DVGui::PropertyCheckBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = CheckBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__PropertyLineEdit_t {
    QByteArrayData data[4];
    char stringdata0[44];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__PropertyLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__PropertyLineEdit_t qt_meta_stringdata_DVGui__PropertyLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 23), // "DVGui::PropertyLineEdit"
QT_MOC_LITERAL(1, 24, 13), // "onTextChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 4) // "text"

    },
    "DVGui::PropertyLineEdit\0onTextChanged\0"
    "\0text"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__PropertyLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,

       0        // eod
};

void DVGui::PropertyLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyLineEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::PropertyLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<LineEdit::staticMetaObject>(),
    qt_meta_stringdata_DVGui__PropertyLineEdit.data,
    qt_meta_data_DVGui__PropertyLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::PropertyLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::PropertyLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__PropertyLineEdit.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PropertyWidget"))
        return static_cast< PropertyWidget*>(this);
    return LineEdit::qt_metacast(_clname);
}

int DVGui::PropertyLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__PropertyIntField_t {
    QByteArrayData data[4];
    char stringdata0[51];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__PropertyIntField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__PropertyIntField_t qt_meta_stringdata_DVGui__PropertyIntField = {
    {
QT_MOC_LITERAL(0, 0, 23), // "DVGui::PropertyIntField"
QT_MOC_LITERAL(1, 24, 14), // "onValueChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 10) // "isDragging"

    },
    "DVGui::PropertyIntField\0onValueChanged\0"
    "\0isDragging"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__PropertyIntField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void DVGui::PropertyIntField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyIntField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::PropertyIntField::staticMetaObject = { {
    QMetaObject::SuperData::link<IntField::staticMetaObject>(),
    qt_meta_stringdata_DVGui__PropertyIntField.data,
    qt_meta_data_DVGui__PropertyIntField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::PropertyIntField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::PropertyIntField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__PropertyIntField.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PropertyWidget"))
        return static_cast< PropertyWidget*>(this);
    return IntField::qt_metacast(_clname);
}

int DVGui::PropertyIntField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = IntField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__PropertyDoubleField_t {
    QByteArrayData data[4];
    char stringdata0[54];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__PropertyDoubleField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__PropertyDoubleField_t qt_meta_stringdata_DVGui__PropertyDoubleField = {
    {
QT_MOC_LITERAL(0, 0, 26), // "DVGui::PropertyDoubleField"
QT_MOC_LITERAL(1, 27, 14), // "onValueChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 10) // "isDragging"

    },
    "DVGui::PropertyDoubleField\0onValueChanged\0"
    "\0isDragging"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__PropertyDoubleField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   19,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,

       0        // eod
};

void DVGui::PropertyDoubleField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyDoubleField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::PropertyDoubleField::staticMetaObject = { {
    QMetaObject::SuperData::link<DoubleField::staticMetaObject>(),
    qt_meta_stringdata_DVGui__PropertyDoubleField.data,
    qt_meta_data_DVGui__PropertyDoubleField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::PropertyDoubleField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::PropertyDoubleField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__PropertyDoubleField.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PropertyWidget"))
        return static_cast< PropertyWidget*>(this);
    return DoubleField::qt_metacast(_clname);
}

int DVGui::PropertyDoubleField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DoubleField::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
