/****************************************************************************
** Meta object code from reading C++ file 'spreadsheetviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/spreadsheetviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'spreadsheetviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Spreadsheet__FrameScroller_t {
    QByteArrayData data[10];
    char stringdata0[116];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Spreadsheet__FrameScroller_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Spreadsheet__FrameScroller_t qt_meta_stringdata_Spreadsheet__FrameScroller = {
    {
QT_MOC_LITERAL(0, 0, 26), // "Spreadsheet::FrameScroller"
QT_MOC_LITERAL(1, 27, 21), // "prepareToScrollOffset"
QT_MOC_LITERAL(2, 49, 0), // ""
QT_MOC_LITERAL(3, 50, 6), // "offset"
QT_MOC_LITERAL(4, 57, 16), // "zoomScrollAdjust"
QT_MOC_LITERAL(5, 74, 8), // "QPointF&"
QT_MOC_LITERAL(6, 83, 6), // "toZoom"
QT_MOC_LITERAL(7, 90, 9), // "onVScroll"
QT_MOC_LITERAL(8, 100, 5), // "value"
QT_MOC_LITERAL(9, 106, 9) // "onHScroll"

    },
    "Spreadsheet::FrameScroller\0"
    "prepareToScrollOffset\0\0offset\0"
    "zoomScrollAdjust\0QPointF&\0toZoom\0"
    "onVScroll\0value\0onHScroll"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Spreadsheet__FrameScroller[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,
       4,    2,   37,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,   42,    2, 0x08 /* Private */,
       9,    1,   45,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QPointF,    3,
    QMetaType::Void, 0x80000000 | 5, QMetaType::Bool,    3,    6,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,

       0        // eod
};

void Spreadsheet::FrameScroller::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FrameScroller *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->prepareToScrollOffset((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 1: _t->zoomScrollAdjust((*reinterpret_cast< QPointF(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 2: _t->onVScroll((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->onHScroll((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FrameScroller::*)(const QPointF & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FrameScroller::prepareToScrollOffset)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (FrameScroller::*)(QPointF & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FrameScroller::zoomScrollAdjust)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Spreadsheet::FrameScroller::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_Spreadsheet__FrameScroller.data,
    qt_meta_data_Spreadsheet__FrameScroller,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Spreadsheet::FrameScroller::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Spreadsheet::FrameScroller::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Spreadsheet__FrameScroller.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Spreadsheet::FrameScroller::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Spreadsheet::FrameScroller::prepareToScrollOffset(const QPointF & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Spreadsheet::FrameScroller::zoomScrollAdjust(QPointF & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_Spreadsheet__ScrollArea_t {
    QByteArrayData data[1];
    char stringdata0[24];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Spreadsheet__ScrollArea_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Spreadsheet__ScrollArea_t qt_meta_stringdata_Spreadsheet__ScrollArea = {
    {
QT_MOC_LITERAL(0, 0, 23) // "Spreadsheet::ScrollArea"

    },
    "Spreadsheet::ScrollArea"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Spreadsheet__ScrollArea[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Spreadsheet::ScrollArea::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Spreadsheet::ScrollArea::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_Spreadsheet__ScrollArea.data,
    qt_meta_data_Spreadsheet__ScrollArea,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Spreadsheet::ScrollArea::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Spreadsheet::ScrollArea::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Spreadsheet__ScrollArea.stringdata0))
        return static_cast<void*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int Spreadsheet::ScrollArea::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Spreadsheet__GenericPanel_t {
    QByteArrayData data[1];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Spreadsheet__GenericPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Spreadsheet__GenericPanel_t qt_meta_stringdata_Spreadsheet__GenericPanel = {
    {
QT_MOC_LITERAL(0, 0, 25) // "Spreadsheet::GenericPanel"

    },
    "Spreadsheet::GenericPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Spreadsheet__GenericPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Spreadsheet::GenericPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Spreadsheet::GenericPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_Spreadsheet__GenericPanel.data,
    qt_meta_data_Spreadsheet__GenericPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Spreadsheet::GenericPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Spreadsheet::GenericPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Spreadsheet__GenericPanel.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Spreadsheet::GenericPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Spreadsheet__RowPanel_t {
    QByteArrayData data[1];
    char stringdata0[22];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Spreadsheet__RowPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Spreadsheet__RowPanel_t qt_meta_stringdata_Spreadsheet__RowPanel = {
    {
QT_MOC_LITERAL(0, 0, 21) // "Spreadsheet::RowPanel"

    },
    "Spreadsheet::RowPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Spreadsheet__RowPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Spreadsheet::RowPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Spreadsheet::RowPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericPanel::staticMetaObject>(),
    qt_meta_stringdata_Spreadsheet__RowPanel.data,
    qt_meta_data_Spreadsheet__RowPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Spreadsheet::RowPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Spreadsheet::RowPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Spreadsheet__RowPanel.stringdata0))
        return static_cast<void*>(this);
    return GenericPanel::qt_metacast(_clname);
}

int Spreadsheet::RowPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Spreadsheet__ColumnPanel_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Spreadsheet__ColumnPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Spreadsheet__ColumnPanel_t qt_meta_stringdata_Spreadsheet__ColumnPanel = {
    {
QT_MOC_LITERAL(0, 0, 24) // "Spreadsheet::ColumnPanel"

    },
    "Spreadsheet::ColumnPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Spreadsheet__ColumnPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Spreadsheet::ColumnPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Spreadsheet::ColumnPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericPanel::staticMetaObject>(),
    qt_meta_stringdata_Spreadsheet__ColumnPanel.data,
    qt_meta_data_Spreadsheet__ColumnPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Spreadsheet::ColumnPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Spreadsheet::ColumnPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Spreadsheet__ColumnPanel.stringdata0))
        return static_cast<void*>(this);
    return GenericPanel::qt_metacast(_clname);
}

int Spreadsheet::ColumnPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_Spreadsheet__CellPanel_t {
    QByteArrayData data[1];
    char stringdata0[23];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Spreadsheet__CellPanel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Spreadsheet__CellPanel_t qt_meta_stringdata_Spreadsheet__CellPanel = {
    {
QT_MOC_LITERAL(0, 0, 22) // "Spreadsheet::CellPanel"

    },
    "Spreadsheet::CellPanel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Spreadsheet__CellPanel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void Spreadsheet::CellPanel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject Spreadsheet::CellPanel::staticMetaObject = { {
    QMetaObject::SuperData::link<GenericPanel::staticMetaObject>(),
    qt_meta_stringdata_Spreadsheet__CellPanel.data,
    qt_meta_data_Spreadsheet__CellPanel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Spreadsheet::CellPanel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Spreadsheet::CellPanel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Spreadsheet__CellPanel.stringdata0))
        return static_cast<void*>(this);
    return GenericPanel::qt_metacast(_clname);
}

int Spreadsheet::CellPanel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GenericPanel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SpreadsheetViewer_t {
    QByteArrayData data[41];
    char stringdata0[697];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SpreadsheetViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SpreadsheetViewer_t qt_meta_stringdata_SpreadsheetViewer = {
    {
QT_MOC_LITERAL(0, 0, 17), // "SpreadsheetViewer"
QT_MOC_LITERAL(1, 18, 11), // "setRowCount"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 8), // "rowCount"
QT_MOC_LITERAL(4, 40, 14), // "setColumnCount"
QT_MOC_LITERAL(5, 55, 11), // "columnCount"
QT_MOC_LITERAL(6, 67, 13), // "frameSwitched"
QT_MOC_LITERAL(7, 81, 11), // "updateAreas"
QT_MOC_LITERAL(8, 93, 16), // "onVSliderChanged"
QT_MOC_LITERAL(9, 110, 16), // "onHSliderChanged"
QT_MOC_LITERAL(10, 127, 23), // "onPrepareToScrollOffset"
QT_MOC_LITERAL(11, 151, 6), // "offset"
QT_MOC_LITERAL(12, 158, 18), // "onZoomScrollAdjust"
QT_MOC_LITERAL(13, 177, 8), // "QPointF&"
QT_MOC_LITERAL(14, 186, 6), // "toZoom"
QT_MOC_LITERAL(15, 193, 17), // "LightLightBGColor"
QT_MOC_LITERAL(16, 211, 7), // "BGColor"
QT_MOC_LITERAL(17, 219, 14), // "LightLineColor"
QT_MOC_LITERAL(18, 234, 17), // "CurrentRowBgColor"
QT_MOC_LITERAL(19, 252, 15), // "MarkerLineColor"
QT_MOC_LITERAL(20, 268, 18), // "SecMarkerLineColor"
QT_MOC_LITERAL(21, 287, 9), // "TextColor"
QT_MOC_LITERAL(22, 297, 19), // "CurrentRowTextColor"
QT_MOC_LITERAL(23, 317, 14), // "FrameTextColor"
QT_MOC_LITERAL(24, 332, 18), // "GroupNameTextColor"
QT_MOC_LITERAL(25, 351, 20), // "ChannelNameTextColor"
QT_MOC_LITERAL(26, 372, 17), // "KeyframeLineColor"
QT_MOC_LITERAL(27, 390, 17), // "VerticalLineColor"
QT_MOC_LITERAL(28, 408, 13), // "KeyFrameColor"
QT_MOC_LITERAL(29, 422, 19), // "KeyFrameBorderColor"
QT_MOC_LITERAL(30, 442, 21), // "SelectedKeyFrameColor"
QT_MOC_LITERAL(31, 464, 20), // "IgnoredKeyFrameColor"
QT_MOC_LITERAL(32, 485, 28), // "SelectedIgnoredKeyFrameColor"
QT_MOC_LITERAL(33, 514, 14), // "InBetweenColor"
QT_MOC_LITERAL(34, 529, 20), // "InBetweenBorderColor"
QT_MOC_LITERAL(35, 550, 22), // "SelectedInBetweenColor"
QT_MOC_LITERAL(36, 573, 21), // "IgnoredInBetweenColor"
QT_MOC_LITERAL(37, 595, 29), // "SelectedIgnoredInBetweenColor"
QT_MOC_LITERAL(38, 625, 18), // "SelectedEmptyColor"
QT_MOC_LITERAL(39, 644, 28), // "SelectedSceneRangeEmptyColor"
QT_MOC_LITERAL(40, 673, 23) // "ColumnHeaderBorderColor"

    },
    "SpreadsheetViewer\0setRowCount\0\0rowCount\0"
    "setColumnCount\0columnCount\0frameSwitched\0"
    "updateAreas\0onVSliderChanged\0"
    "onHSliderChanged\0onPrepareToScrollOffset\0"
    "offset\0onZoomScrollAdjust\0QPointF&\0"
    "toZoom\0LightLightBGColor\0BGColor\0"
    "LightLineColor\0CurrentRowBgColor\0"
    "MarkerLineColor\0SecMarkerLineColor\0"
    "TextColor\0CurrentRowTextColor\0"
    "FrameTextColor\0GroupNameTextColor\0"
    "ChannelNameTextColor\0KeyframeLineColor\0"
    "VerticalLineColor\0KeyFrameColor\0"
    "KeyFrameBorderColor\0SelectedKeyFrameColor\0"
    "IgnoredKeyFrameColor\0SelectedIgnoredKeyFrameColor\0"
    "InBetweenColor\0InBetweenBorderColor\0"
    "SelectedInBetweenColor\0IgnoredInBetweenColor\0"
    "SelectedIgnoredInBetweenColor\0"
    "SelectedEmptyColor\0SelectedSceneRangeEmptyColor\0"
    "ColumnHeaderBorderColor"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SpreadsheetViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
      26,   76, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x0a /* Public */,
       4,    1,   57,    2, 0x0a /* Public */,
       6,    0,   60,    2, 0x0a /* Public */,
       7,    0,   61,    2, 0x0a /* Public */,
       8,    1,   62,    2, 0x0a /* Public */,
       9,    1,   65,    2, 0x0a /* Public */,
      10,    1,   68,    2, 0x0a /* Public */,
      12,    2,   71,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QPointF,   11,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Bool,   11,   14,

 // properties: name, type, flags
      15, QMetaType::QColor, 0x00095103,
      16, QMetaType::QColor, 0x00095103,
      17, QMetaType::QColor, 0x00095103,
      18, QMetaType::QColor, 0x00095103,
      19, QMetaType::QColor, 0x00095103,
      20, QMetaType::QColor, 0x00095103,
      21, QMetaType::QColor, 0x00095103,
      22, QMetaType::QColor, 0x00095103,
      23, QMetaType::QColor, 0x00095103,
      24, QMetaType::QColor, 0x00095103,
      25, QMetaType::QColor, 0x00095103,
      26, QMetaType::QColor, 0x00095103,
      27, QMetaType::QColor, 0x00095103,
      28, QMetaType::QColor, 0x00095103,
      29, QMetaType::QColor, 0x00095103,
      30, QMetaType::QColor, 0x00095103,
      31, QMetaType::QColor, 0x00095103,
      32, QMetaType::QColor, 0x00095103,
      33, QMetaType::QColor, 0x00095103,
      34, QMetaType::QColor, 0x00095103,
      35, QMetaType::QColor, 0x00095103,
      36, QMetaType::QColor, 0x00095103,
      37, QMetaType::QColor, 0x00095103,
      38, QMetaType::QColor, 0x00095103,
      39, QMetaType::QColor, 0x00095103,
      40, QMetaType::QColor, 0x00095103,

       0        // eod
};

void SpreadsheetViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SpreadsheetViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setRowCount((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setColumnCount((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->frameSwitched(); break;
        case 3: _t->updateAreas(); break;
        case 4: _t->onVSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->onHSliderChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->onPrepareToScrollOffset((*reinterpret_cast< const QPointF(*)>(_a[1]))); break;
        case 7: _t->onZoomScrollAdjust((*reinterpret_cast< QPointF(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<SpreadsheetViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->getLightLightBGColor(); break;
        case 1: *reinterpret_cast< QColor*>(_v) = _t->getBGColor(); break;
        case 2: *reinterpret_cast< QColor*>(_v) = _t->getLightLineColor(); break;
        case 3: *reinterpret_cast< QColor*>(_v) = _t->getCurrentRowBgColor(); break;
        case 4: *reinterpret_cast< QColor*>(_v) = _t->getMarkerLineColor(); break;
        case 5: *reinterpret_cast< QColor*>(_v) = _t->getSecMarkerLineColor(); break;
        case 6: *reinterpret_cast< QColor*>(_v) = _t->getTextColor(); break;
        case 7: *reinterpret_cast< QColor*>(_v) = _t->getCurrentRowTextColor(); break;
        case 8: *reinterpret_cast< QColor*>(_v) = _t->getFrameTextColor(); break;
        case 9: *reinterpret_cast< QColor*>(_v) = _t->getGroupNameTextColor(); break;
        case 10: *reinterpret_cast< QColor*>(_v) = _t->getChannelNameTextColor(); break;
        case 11: *reinterpret_cast< QColor*>(_v) = _t->getKeyframeLineColor(); break;
        case 12: *reinterpret_cast< QColor*>(_v) = _t->getVerticalLineColor(); break;
        case 13: *reinterpret_cast< QColor*>(_v) = _t->getKeyFrameColor(); break;
        case 14: *reinterpret_cast< QColor*>(_v) = _t->getKeyFrameBorderColor(); break;
        case 15: *reinterpret_cast< QColor*>(_v) = _t->getSelectedKeyFrameColor(); break;
        case 16: *reinterpret_cast< QColor*>(_v) = _t->getIgnoredKeyFrameColor(); break;
        case 17: *reinterpret_cast< QColor*>(_v) = _t->getSelectedIgnoredKeyFrameColor(); break;
        case 18: *reinterpret_cast< QColor*>(_v) = _t->getInBetweenColor(); break;
        case 19: *reinterpret_cast< QColor*>(_v) = _t->getInBetweenBorderColor(); break;
        case 20: *reinterpret_cast< QColor*>(_v) = _t->getSelectedInBetweenColor(); break;
        case 21: *reinterpret_cast< QColor*>(_v) = _t->getIgnoredInBetweenColor(); break;
        case 22: *reinterpret_cast< QColor*>(_v) = _t->getSelectedIgnoredInBetweenColor(); break;
        case 23: *reinterpret_cast< QColor*>(_v) = _t->getSelectedEmptyColor(); break;
        case 24: *reinterpret_cast< QColor*>(_v) = _t->getSelectedSceneRangeEmptyColor(); break;
        case 25: *reinterpret_cast< QColor*>(_v) = _t->getColumnHeaderBorderColor(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<SpreadsheetViewer *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLightLightBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 1: _t->setBGColor(*reinterpret_cast< QColor*>(_v)); break;
        case 2: _t->setLightLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 3: _t->setCurrentRowBgColor(*reinterpret_cast< QColor*>(_v)); break;
        case 4: _t->setMarkerLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 5: _t->setSecMarkerLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 6: _t->setTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 7: _t->setCurrentRowTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 8: _t->setFrameTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 9: _t->setGroupNameTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 10: _t->setChannelNameTextColor(*reinterpret_cast< QColor*>(_v)); break;
        case 11: _t->setKeyframeLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 12: _t->setVerticalLineColor(*reinterpret_cast< QColor*>(_v)); break;
        case 13: _t->setKeyFrameColor(*reinterpret_cast< QColor*>(_v)); break;
        case 14: _t->setKeyFrameBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 15: _t->setSelectedKeyFrameColor(*reinterpret_cast< QColor*>(_v)); break;
        case 16: _t->setIgnoredKeyFrameColor(*reinterpret_cast< QColor*>(_v)); break;
        case 17: _t->setSelectedIgnoredKeyFrameColor(*reinterpret_cast< QColor*>(_v)); break;
        case 18: _t->setInBetweenColor(*reinterpret_cast< QColor*>(_v)); break;
        case 19: _t->setInBetweenBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        case 20: _t->setSelectedInBetweenColor(*reinterpret_cast< QColor*>(_v)); break;
        case 21: _t->setIgnoredInBetweenColor(*reinterpret_cast< QColor*>(_v)); break;
        case 22: _t->setSelectedIgnoredInBetweenColor(*reinterpret_cast< QColor*>(_v)); break;
        case 23: _t->setSelectedEmptyColor(*reinterpret_cast< QColor*>(_v)); break;
        case 24: _t->setSelectedSceneRangeEmptyColor(*reinterpret_cast< QColor*>(_v)); break;
        case 25: _t->setColumnHeaderBorderColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject SpreadsheetViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_SpreadsheetViewer.data,
    qt_meta_data_SpreadsheetViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SpreadsheetViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpreadsheetViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SpreadsheetViewer.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int SpreadsheetViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 26;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 26;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
