/****************************************************************************
** Meta object code from reading C++ file 'hexcolornames.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/hexcolornames.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'hexcolornames.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__HexColorNames_t {
    QByteArrayData data[4];
    char stringdata0[56];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__HexColorNames_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__HexColorNames_t qt_meta_stringdata_DVGui__HexColorNames = {
    {
QT_MOC_LITERAL(0, 0, 20), // "DVGui::HexColorNames"
QT_MOC_LITERAL(1, 21, 19), // "autoCompleteChanged"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 13) // "colorsChanged"

    },
    "DVGui::HexColorNames\0autoCompleteChanged\0"
    "\0colorsChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__HexColorNames[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,
       3,    0,   27,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

       0        // eod
};

void DVGui::HexColorNames::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HexColorNames *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->autoCompleteChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->colorsChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HexColorNames::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HexColorNames::autoCompleteChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HexColorNames::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HexColorNames::colorsChanged)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::HexColorNames::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_DVGui__HexColorNames.data,
    qt_meta_data_DVGui__HexColorNames,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::HexColorNames::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::HexColorNames::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__HexColorNames.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int DVGui::HexColorNames::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DVGui::HexColorNames::autoCompleteChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::HexColorNames::colorsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_DVGui__HexLineEdit_t {
    QByteArrayData data[5];
    char stringdata0[78];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__HexLineEdit_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__HexLineEdit_t qt_meta_stringdata_DVGui__HexLineEdit = {
    {
QT_MOC_LITERAL(0, 0, 18), // "DVGui::HexLineEdit"
QT_MOC_LITERAL(1, 19, 21), // "onAutoCompleteChanged"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 15), // "onColorsChanged"
QT_MOC_LITERAL(4, 58, 19) // "updateCompleterList"

    },
    "DVGui::HexLineEdit\0onAutoCompleteChanged\0"
    "\0onColorsChanged\0updateCompleterList"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__HexLineEdit[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x08 /* Private */,
       3,    0,   32,    2, 0x08 /* Private */,
       4,    0,   33,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void DVGui::HexLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HexLineEdit *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onAutoCompleteChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->onColorsChanged(); break;
        case 2: _t->updateCompleterList(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::HexLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_meta_stringdata_DVGui__HexLineEdit.data,
    qt_meta_data_DVGui__HexLineEdit,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::HexLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::HexLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__HexLineEdit.stringdata0))
        return static_cast<void*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int DVGui::HexLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}
struct qt_meta_stringdata_DVGui__HexColorNamesEditingDelegate_t {
    QByteArrayData data[11];
    char stringdata0[164];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__HexColorNamesEditingDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__HexColorNamesEditingDelegate_t qt_meta_stringdata_DVGui__HexColorNamesEditingDelegate = {
    {
QT_MOC_LITERAL(0, 0, 35), // "DVGui::HexColorNamesEditingDe..."
QT_MOC_LITERAL(1, 36, 14), // "editingStarted"
QT_MOC_LITERAL(2, 51, 0), // ""
QT_MOC_LITERAL(3, 52, 11), // "QModelIndex"
QT_MOC_LITERAL(4, 64, 15), // "editingFinished"
QT_MOC_LITERAL(5, 80, 13), // "editingClosed"
QT_MOC_LITERAL(6, 94, 13), // "onCloseEditor"
QT_MOC_LITERAL(7, 108, 8), // "QWidget*"
QT_MOC_LITERAL(8, 117, 6), // "editor"
QT_MOC_LITERAL(9, 124, 34), // "QAbstractItemDelegate::EndEdi..."
QT_MOC_LITERAL(10, 159, 4) // "hint"

    },
    "DVGui::HexColorNamesEditingDelegate\0"
    "editingStarted\0\0QModelIndex\0editingFinished\0"
    "editingClosed\0onCloseEditor\0QWidget*\0"
    "editor\0QAbstractItemDelegate::EndEditHint\0"
    "hint"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__HexColorNamesEditingDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   39,    2, 0x06 /* Public */,
       4,    1,   42,    2, 0x06 /* Public */,
       5,    0,   45,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    2,   46,    2, 0x08 /* Private */,
       6,    1,   51,    2, 0x28 /* Private | MethodCloned */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7, 0x80000000 | 9,    8,   10,
    QMetaType::Void, 0x80000000 | 7,    8,

       0        // eod
};

void DVGui::HexColorNamesEditingDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HexColorNamesEditingDelegate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->editingStarted((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 1: _t->editingFinished((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 2: _t->editingClosed(); break;
        case 3: _t->onCloseEditor((*reinterpret_cast< QWidget*(*)>(_a[1])),(*reinterpret_cast< QAbstractItemDelegate::EndEditHint(*)>(_a[2]))); break;
        case 4: _t->onCloseEditor((*reinterpret_cast< QWidget*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HexColorNamesEditingDelegate::*)(const QModelIndex & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HexColorNamesEditingDelegate::editingStarted)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HexColorNamesEditingDelegate::*)(const QModelIndex & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HexColorNamesEditingDelegate::editingFinished)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HexColorNamesEditingDelegate::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HexColorNamesEditingDelegate::editingClosed)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::HexColorNamesEditingDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QStyledItemDelegate::staticMetaObject>(),
    qt_meta_stringdata_DVGui__HexColorNamesEditingDelegate.data,
    qt_meta_data_DVGui__HexColorNamesEditingDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::HexColorNamesEditingDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::HexColorNamesEditingDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__HexColorNamesEditingDelegate.stringdata0))
        return static_cast<void*>(this);
    return QStyledItemDelegate::qt_metacast(_clname);
}

int DVGui::HexColorNamesEditingDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStyledItemDelegate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void DVGui::HexColorNamesEditingDelegate::editingStarted(const QModelIndex & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< DVGui::HexColorNamesEditingDelegate *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::HexColorNamesEditingDelegate::editingFinished(const QModelIndex & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< DVGui::HexColorNamesEditingDelegate *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DVGui::HexColorNamesEditingDelegate::editingClosed()const
{
    QMetaObject::activate(const_cast< DVGui::HexColorNamesEditingDelegate *>(this), &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_DVGui__HexColorNamesEditor_t {
    QByteArrayData data[26];
    char stringdata0[310];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__HexColorNamesEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__HexColorNamesEditor_t qt_meta_stringdata_DVGui__HexColorNamesEditor = {
    {
QT_MOC_LITERAL(0, 0, 26), // "DVGui::HexColorNamesEditor"
QT_MOC_LITERAL(1, 27, 20), // "onCurrentItemChanged"
QT_MOC_LITERAL(2, 48, 0), // ""
QT_MOC_LITERAL(3, 49, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(4, 66, 7), // "current"
QT_MOC_LITERAL(5, 74, 8), // "previous"
QT_MOC_LITERAL(6, 83, 16), // "onEditingStarted"
QT_MOC_LITERAL(7, 100, 11), // "QModelIndex"
QT_MOC_LITERAL(8, 112, 17), // "onEditingFinished"
QT_MOC_LITERAL(9, 130, 15), // "onEditingClosed"
QT_MOC_LITERAL(10, 146, 13), // "onItemStarted"
QT_MOC_LITERAL(11, 160, 4), // "item"
QT_MOC_LITERAL(12, 165, 6), // "column"
QT_MOC_LITERAL(13, 172, 14), // "onItemFinished"
QT_MOC_LITERAL(14, 187, 19), // "onColorFieldChanged"
QT_MOC_LITERAL(15, 207, 8), // "TPixel32"
QT_MOC_LITERAL(16, 216, 5), // "color"
QT_MOC_LITERAL(17, 222, 10), // "isDragging"
QT_MOC_LITERAL(18, 233, 12), // "onHexChanged"
QT_MOC_LITERAL(19, 246, 10), // "onAddColor"
QT_MOC_LITERAL(20, 257, 10), // "onDelColor"
QT_MOC_LITERAL(21, 268, 10), // "onDeselect"
QT_MOC_LITERAL(22, 279, 8), // "onImport"
QT_MOC_LITERAL(23, 288, 8), // "onExport"
QT_MOC_LITERAL(24, 297, 4), // "onOK"
QT_MOC_LITERAL(25, 302, 7) // "onApply"

    },
    "DVGui::HexColorNamesEditor\0"
    "onCurrentItemChanged\0\0QTreeWidgetItem*\0"
    "current\0previous\0onEditingStarted\0"
    "QModelIndex\0onEditingFinished\0"
    "onEditingClosed\0onItemStarted\0item\0"
    "column\0onItemFinished\0onColorFieldChanged\0"
    "TPixel32\0color\0isDragging\0onHexChanged\0"
    "onAddColor\0onDelColor\0onDeselect\0"
    "onImport\0onExport\0onOK\0onApply"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__HexColorNamesEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   89,    2, 0x08 /* Private */,
       6,    1,   94,    2, 0x08 /* Private */,
       8,    1,   97,    2, 0x08 /* Private */,
       9,    0,  100,    2, 0x08 /* Private */,
      10,    2,  101,    2, 0x08 /* Private */,
      13,    2,  106,    2, 0x08 /* Private */,
      14,    2,  111,    2, 0x08 /* Private */,
      18,    0,  116,    2, 0x08 /* Private */,
      19,    0,  117,    2, 0x08 /* Private */,
      20,    0,  118,    2, 0x08 /* Private */,
      21,    0,  119,    2, 0x08 /* Private */,
      22,    0,  120,    2, 0x08 /* Private */,
      23,    0,  121,    2, 0x08 /* Private */,
      24,    0,  122,    2, 0x08 /* Private */,
      25,    0,  123,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    4,    5,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void, 0x80000000 | 7,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,   11,   12,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Int,   11,   12,
    QMetaType::Void, 0x80000000 | 15, QMetaType::Bool,   16,   17,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void DVGui::HexColorNamesEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HexColorNamesEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onCurrentItemChanged((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 1: _t->onEditingStarted((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 2: _t->onEditingFinished((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 3: _t->onEditingClosed(); break;
        case 4: _t->onItemStarted((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 5: _t->onItemFinished((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->onColorFieldChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 7: _t->onHexChanged(); break;
        case 8: _t->onAddColor(); break;
        case 9: _t->onDelColor(); break;
        case 10: _t->onDeselect(); break;
        case 11: _t->onImport(); break;
        case 12: _t->onExport(); break;
        case 13: _t->onOK(); break;
        case 14: _t->onApply(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::HexColorNamesEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_DVGui__HexColorNamesEditor.data,
    qt_meta_data_DVGui__HexColorNamesEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::HexColorNamesEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::HexColorNamesEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__HexColorNamesEditor.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int DVGui::HexColorNamesEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
