/****************************************************************************
** Meta object code from reading C++ file 'menubar.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/menubar.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'menubar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_RoomTabWidget_t {
    QByteArrayData data[17];
    char stringdata0[194];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_RoomTabWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_RoomTabWidget_t qt_meta_stringdata_RoomTabWidget = {
    {
QT_MOC_LITERAL(0, 0, 13), // "RoomTabWidget"
QT_MOC_LITERAL(1, 14, 12), // "indexSwapped"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 10), // "firstIndex"
QT_MOC_LITERAL(4, 39, 11), // "secondIndex"
QT_MOC_LITERAL(5, 51, 16), // "insertNewTabRoom"
QT_MOC_LITERAL(6, 68, 13), // "deleteTabRoom"
QT_MOC_LITERAL(7, 82, 5), // "index"
QT_MOC_LITERAL(8, 88, 13), // "renameTabRoom"
QT_MOC_LITERAL(9, 102, 4), // "name"
QT_MOC_LITERAL(10, 107, 16), // "customizeMenuBar"
QT_MOC_LITERAL(11, 124, 13), // "updateTabName"
QT_MOC_LITERAL(12, 138, 9), // "addNewTab"
QT_MOC_LITERAL(13, 148, 9), // "deleteTab"
QT_MOC_LITERAL(14, 158, 11), // "setIsLocked"
QT_MOC_LITERAL(15, 170, 4), // "lock"
QT_MOC_LITERAL(16, 175, 18) // "onCustomizeMenuBar"

    },
    "RoomTabWidget\0indexSwapped\0\0firstIndex\0"
    "secondIndex\0insertNewTabRoom\0deleteTabRoom\0"
    "index\0renameTabRoom\0name\0customizeMenuBar\0"
    "updateTabName\0addNewTab\0deleteTab\0"
    "setIsLocked\0lock\0onCustomizeMenuBar"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_RoomTabWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   64,    2, 0x06 /* Public */,
       5,    0,   69,    2, 0x06 /* Public */,
       6,    1,   70,    2, 0x06 /* Public */,
       8,    2,   73,    2, 0x06 /* Public */,
      10,    1,   78,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,   81,    2, 0x09 /* Protected */,
      12,    0,   82,    2, 0x09 /* Protected */,
      13,    0,   83,    2, 0x09 /* Protected */,
      14,    1,   84,    2, 0x09 /* Protected */,
      16,    0,   87,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    3,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,    7,    9,
    QMetaType::Void, QMetaType::Int,    7,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   15,
    QMetaType::Void,

       0        // eod
};

void RoomTabWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<RoomTabWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->indexSwapped((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->insertNewTabRoom(); break;
        case 2: _t->deleteTabRoom((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->renameTabRoom((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->customizeMenuBar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->updateTabName(); break;
        case 6: _t->addNewTab(); break;
        case 7: _t->deleteTab(); break;
        case 8: _t->setIsLocked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->onCustomizeMenuBar(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (RoomTabWidget::*)(int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RoomTabWidget::indexSwapped)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (RoomTabWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RoomTabWidget::insertNewTabRoom)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (RoomTabWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RoomTabWidget::deleteTabRoom)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (RoomTabWidget::*)(int , const QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RoomTabWidget::renameTabRoom)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (RoomTabWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&RoomTabWidget::customizeMenuBar)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject RoomTabWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabBar::staticMetaObject>(),
    qt_meta_stringdata_RoomTabWidget.data,
    qt_meta_data_RoomTabWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *RoomTabWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RoomTabWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_RoomTabWidget.stringdata0))
        return static_cast<void*>(this);
    return QTabBar::qt_metacast(_clname);
}

int RoomTabWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabBar::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void RoomTabWidget::indexSwapped(int _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void RoomTabWidget::insertNewTabRoom()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void RoomTabWidget::deleteTabRoom(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void RoomTabWidget::renameTabRoom(int _t1, const QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void RoomTabWidget::customizeMenuBar(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_UrlOpener_t {
    QByteArrayData data[3];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UrlOpener_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UrlOpener_t qt_meta_stringdata_UrlOpener = {
    {
QT_MOC_LITERAL(0, 0, 9), // "UrlOpener"
QT_MOC_LITERAL(1, 10, 4), // "open"
QT_MOC_LITERAL(2, 15, 0) // ""

    },
    "UrlOpener\0open\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UrlOpener[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void UrlOpener::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UrlOpener *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->open(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject UrlOpener::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_UrlOpener.data,
    qt_meta_data_UrlOpener,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UrlOpener::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UrlOpener::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UrlOpener.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int UrlOpener::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_StackedMenuBar_t {
    QByteArrayData data[9];
    char stringdata0[110];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StackedMenuBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StackedMenuBar_t qt_meta_stringdata_StackedMenuBar = {
    {
QT_MOC_LITERAL(0, 0, 14), // "StackedMenuBar"
QT_MOC_LITERAL(1, 15, 14), // "onIndexSwapped"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 10), // "firstIndex"
QT_MOC_LITERAL(4, 42, 11), // "secondIndex"
QT_MOC_LITERAL(5, 54, 16), // "insertNewMenuBar"
QT_MOC_LITERAL(6, 71, 13), // "deleteMenuBar"
QT_MOC_LITERAL(7, 85, 5), // "index"
QT_MOC_LITERAL(8, 91, 18) // "doCustomizeMenuBar"

    },
    "StackedMenuBar\0onIndexSwapped\0\0"
    "firstIndex\0secondIndex\0insertNewMenuBar\0"
    "deleteMenuBar\0index\0doCustomizeMenuBar"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StackedMenuBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   34,    2, 0x09 /* Protected */,
       5,    0,   39,    2, 0x09 /* Protected */,
       6,    1,   40,    2, 0x09 /* Protected */,
       8,    1,   43,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    3,    4,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    7,

       0        // eod
};

void StackedMenuBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StackedMenuBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onIndexSwapped((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 1: _t->insertNewMenuBar(); break;
        case 2: _t->deleteMenuBar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->doCustomizeMenuBar((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StackedMenuBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QStackedWidget::staticMetaObject>(),
    qt_meta_stringdata_StackedMenuBar.data,
    qt_meta_data_StackedMenuBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StackedMenuBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StackedMenuBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StackedMenuBar.stringdata0))
        return static_cast<void*>(this);
    return QStackedWidget::qt_metacast(_clname);
}

int StackedMenuBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStackedWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}
struct qt_meta_stringdata_TopBar_t {
    QByteArrayData data[1];
    char stringdata0[7];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TopBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TopBar_t qt_meta_stringdata_TopBar = {
    {
QT_MOC_LITERAL(0, 0, 6) // "TopBar"

    },
    "TopBar"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TopBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void TopBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TopBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolBar::staticMetaObject>(),
    qt_meta_stringdata_TopBar.data,
    qt_meta_data_TopBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TopBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TopBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TopBar.stringdata0))
        return static_cast<void*>(this);
    return QToolBar::qt_metacast(_clname);
}

int TopBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolBar::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
