/****************************************************************************
** Meta object code from reading C++ file 'svnupdateandlockdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/svnupdateandlockdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'svnupdateandlockdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SVNUpdateAndLockDialog_t {
    QByteArrayData data[14];
    char stringdata0[215];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SVNUpdateAndLockDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SVNUpdateAndLockDialog_t qt_meta_stringdata_SVNUpdateAndLockDialog = {
    {
QT_MOC_LITERAL(0, 0, 22), // "SVNUpdateAndLockDialog"
QT_MOC_LITERAL(1, 23, 4), // "done"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 7), // "onError"
QT_MOC_LITERAL(4, 37, 28), // "onUpdateAndLockButtonClicked"
QT_MOC_LITERAL(5, 66, 19), // "onLockButtonClicked"
QT_MOC_LITERAL(6, 86, 12), // "onUpdateDone"
QT_MOC_LITERAL(7, 99, 10), // "onLockDone"
QT_MOC_LITERAL(8, 110, 6), // "finish"
QT_MOC_LITERAL(9, 117, 17), // "onStatusRetrieved"
QT_MOC_LITERAL(10, 135, 26), // "onEditSceneContentsToggled"
QT_MOC_LITERAL(11, 162, 7), // "checked"
QT_MOC_LITERAL(12, 170, 31), // "onSceneResourcesStatusRetrieved"
QT_MOC_LITERAL(13, 202, 12) // "reverseMerge"

    },
    "SVNUpdateAndLockDialog\0done\0\0onError\0"
    "onUpdateAndLockButtonClicked\0"
    "onLockButtonClicked\0onUpdateDone\0"
    "onLockDone\0finish\0onStatusRetrieved\0"
    "onEditSceneContentsToggled\0checked\0"
    "onSceneResourcesStatusRetrieved\0"
    "reverseMerge"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SVNUpdateAndLockDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   72,    2, 0x09 /* Protected */,
       4,    0,   75,    2, 0x09 /* Protected */,
       5,    0,   76,    2, 0x09 /* Protected */,
       6,    0,   77,    2, 0x09 /* Protected */,
       7,    0,   78,    2, 0x09 /* Protected */,
       8,    0,   79,    2, 0x09 /* Protected */,
       9,    1,   80,    2, 0x09 /* Protected */,
      10,    1,   83,    2, 0x09 /* Protected */,
      12,    1,   86,    2, 0x09 /* Protected */,
      13,    0,   89,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QStringList,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,

       0        // eod
};

void SVNUpdateAndLockDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SVNUpdateAndLockDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->done((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 1: _t->onError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->onUpdateAndLockButtonClicked(); break;
        case 3: _t->onLockButtonClicked(); break;
        case 4: _t->onUpdateDone(); break;
        case 5: _t->onLockDone(); break;
        case 6: _t->finish(); break;
        case 7: _t->onStatusRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->onEditSceneContentsToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->onSceneResourcesStatusRetrieved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->reverseMerge(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SVNUpdateAndLockDialog::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SVNUpdateAndLockDialog::done)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SVNUpdateAndLockDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SVNUpdateAndLockDialog.data,
    qt_meta_data_SVNUpdateAndLockDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SVNUpdateAndLockDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SVNUpdateAndLockDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SVNUpdateAndLockDialog.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SVNUpdateAndLockDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void SVNUpdateAndLockDialog::done(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
