/****************************************************************************
** Meta object code from reading C++ file 'outputsettingspopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/outputsettingspopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'outputsettingspopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AnimatedLabel_t {
    QByteArrayData data[2];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AnimatedLabel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AnimatedLabel_t qt_meta_stringdata_AnimatedLabel = {
    {
QT_MOC_LITERAL(0, 0, 13), // "AnimatedLabel"
QT_MOC_LITERAL(1, 14, 5) // "color"

    },
    "AnimatedLabel\0color"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AnimatedLabel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       1,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QColor, 0x00095103,

       0        // eod
};

void AnimatedLabel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<AnimatedLabel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QColor*>(_v) = _t->color(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<AnimatedLabel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setColor(*reinterpret_cast< QColor*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject AnimatedLabel::staticMetaObject = { {
    QMetaObject::SuperData::link<QLabel::staticMetaObject>(),
    qt_meta_stringdata_AnimatedLabel.data,
    qt_meta_data_AnimatedLabel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AnimatedLabel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AnimatedLabel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AnimatedLabel.stringdata0))
        return static_cast<void*>(this);
    return QLabel::qt_metacast(_clname);
}

int AnimatedLabel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLabel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 1;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 1;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_OutputSettingsPopup_t {
    QByteArrayData data[39];
    char stringdata0[709];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OutputSettingsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OutputSettingsPopup_t qt_meta_stringdata_OutputSettingsPopup = {
    {
QT_MOC_LITERAL(0, 0, 19), // "OutputSettingsPopup"
QT_MOC_LITERAL(1, 20, 11), // "updateField"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 13), // "onPathChanged"
QT_MOC_LITERAL(4, 47, 13), // "onNameChanged"
QT_MOC_LITERAL(5, 61, 15), // "onFormatChanged"
QT_MOC_LITERAL(6, 77, 3), // "str"
QT_MOC_LITERAL(7, 81, 17), // "openSettingsPopup"
QT_MOC_LITERAL(8, 99, 15), // "onCameraChanged"
QT_MOC_LITERAL(9, 115, 22), // "onFrameFldEditFinished"
QT_MOC_LITERAL(10, 138, 17), // "onResampleChanged"
QT_MOC_LITERAL(11, 156, 4), // "type"
QT_MOC_LITERAL(12, 161, 21), // "onChannelWidthChanged"
QT_MOC_LITERAL(13, 183, 25), // "onLinearColorSpaceClicked"
QT_MOC_LITERAL(14, 209, 7), // "checked"
QT_MOC_LITERAL(15, 217, 23), // "onColorSpaceGammaEdited"
QT_MOC_LITERAL(16, 241, 22), // "onGammaFldEditFinished"
QT_MOC_LITERAL(17, 264, 22), // "onDominantFieldChanged"
QT_MOC_LITERAL(18, 287, 24), // "onStretchFldEditFinished"
QT_MOC_LITERAL(19, 312, 20), // "onApplyShrinkChecked"
QT_MOC_LITERAL(20, 333, 5), // "state"
QT_MOC_LITERAL(21, 339, 18), // "onSubcameraChecked"
QT_MOC_LITERAL(22, 358, 19), // "onMultimediaChanged"
QT_MOC_LITERAL(23, 378, 4), // "mode"
QT_MOC_LITERAL(24, 383, 21), // "onThreadsComboChanged"
QT_MOC_LITERAL(25, 405, 26), // "onRasterGranularityChanged"
QT_MOC_LITERAL(26, 432, 15), // "onStereoChecked"
QT_MOC_LITERAL(27, 448, 15), // "onStereoChanged"
QT_MOC_LITERAL(28, 464, 26), // "onSyncColorSettingsChecked"
QT_MOC_LITERAL(29, 491, 15), // "onRenderClicked"
QT_MOC_LITERAL(30, 507, 24), // "onAddPresetButtonPressed"
QT_MOC_LITERAL(31, 532, 27), // "onRemovePresetButtonPressed"
QT_MOC_LITERAL(32, 560, 16), // "onPresetSelected"
QT_MOC_LITERAL(33, 577, 23), // "onCameraSettingsChanged"
QT_MOC_LITERAL(34, 601, 26), // "onFrameRateEditingFinished"
QT_MOC_LITERAL(35, 628, 17), // "onAddBoardChecked"
QT_MOC_LITERAL(36, 646, 25), // "onBoardSettingsBtnClicked"
QT_MOC_LITERAL(37, 672, 19), // "onCategoryActivated"
QT_MOC_LITERAL(38, 692, 16) // "QListWidgetItem*"

    },
    "OutputSettingsPopup\0updateField\0\0"
    "onPathChanged\0onNameChanged\0onFormatChanged\0"
    "str\0openSettingsPopup\0onCameraChanged\0"
    "onFrameFldEditFinished\0onResampleChanged\0"
    "type\0onChannelWidthChanged\0"
    "onLinearColorSpaceClicked\0checked\0"
    "onColorSpaceGammaEdited\0onGammaFldEditFinished\0"
    "onDominantFieldChanged\0onStretchFldEditFinished\0"
    "onApplyShrinkChecked\0state\0"
    "onSubcameraChecked\0onMultimediaChanged\0"
    "mode\0onThreadsComboChanged\0"
    "onRasterGranularityChanged\0onStereoChecked\0"
    "onStereoChanged\0onSyncColorSettingsChecked\0"
    "onRenderClicked\0onAddPresetButtonPressed\0"
    "onRemovePresetButtonPressed\0"
    "onPresetSelected\0onCameraSettingsChanged\0"
    "onFrameRateEditingFinished\0onAddBoardChecked\0"
    "onBoardSettingsBtnClicked\0onCategoryActivated\0"
    "QListWidgetItem*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OutputSettingsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      31,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  169,    2, 0x09 /* Protected */,
       3,    0,  170,    2, 0x09 /* Protected */,
       4,    0,  171,    2, 0x09 /* Protected */,
       5,    1,  172,    2, 0x09 /* Protected */,
       7,    0,  175,    2, 0x09 /* Protected */,
       8,    1,  176,    2, 0x09 /* Protected */,
       9,    0,  179,    2, 0x09 /* Protected */,
      10,    1,  180,    2, 0x09 /* Protected */,
      12,    1,  183,    2, 0x09 /* Protected */,
      13,    1,  186,    2, 0x09 /* Protected */,
      15,    0,  189,    2, 0x09 /* Protected */,
      16,    0,  190,    2, 0x09 /* Protected */,
      17,    1,  191,    2, 0x09 /* Protected */,
      18,    0,  194,    2, 0x09 /* Protected */,
      19,    1,  195,    2, 0x09 /* Protected */,
      21,    1,  198,    2, 0x09 /* Protected */,
      22,    1,  201,    2, 0x09 /* Protected */,
      24,    1,  204,    2, 0x09 /* Protected */,
      25,    1,  207,    2, 0x09 /* Protected */,
      26,    1,  210,    2, 0x09 /* Protected */,
      27,    0,  213,    2, 0x09 /* Protected */,
      28,    1,  214,    2, 0x09 /* Protected */,
      29,    0,  217,    2, 0x09 /* Protected */,
      30,    0,  218,    2, 0x09 /* Protected */,
      31,    0,  219,    2, 0x09 /* Protected */,
      32,    1,  220,    2, 0x09 /* Protected */,
      33,    0,  223,    2, 0x09 /* Protected */,
      34,    0,  224,    2, 0x09 /* Protected */,
      35,    1,  225,    2, 0x09 /* Protected */,
      36,    0,  228,    2, 0x09 /* Protected */,
      37,    1,  229,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Bool,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   23,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 38,    2,

       0        // eod
};

void OutputSettingsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OutputSettingsPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateField(); break;
        case 1: _t->onPathChanged(); break;
        case 2: _t->onNameChanged(); break;
        case 3: _t->onFormatChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->openSettingsPopup(); break;
        case 5: _t->onCameraChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->onFrameFldEditFinished(); break;
        case 7: _t->onResampleChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->onChannelWidthChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->onLinearColorSpaceClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->onColorSpaceGammaEdited(); break;
        case 11: _t->onGammaFldEditFinished(); break;
        case 12: _t->onDominantFieldChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->onStretchFldEditFinished(); break;
        case 14: _t->onApplyShrinkChecked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->onSubcameraChecked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->onMultimediaChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->onThreadsComboChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->onRasterGranularityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->onStereoChecked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->onStereoChanged(); break;
        case 21: _t->onSyncColorSettingsChecked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->onRenderClicked(); break;
        case 23: _t->onAddPresetButtonPressed(); break;
        case 24: _t->onRemovePresetButtonPressed(); break;
        case 25: _t->onPresetSelected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 26: _t->onCameraSettingsChanged(); break;
        case 27: _t->onFrameRateEditingFinished(); break;
        case 28: _t->onAddBoardChecked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->onBoardSettingsBtnClicked(); break;
        case 30: _t->onCategoryActivated((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OutputSettingsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_OutputSettingsPopup.data,
    qt_meta_data_OutputSettingsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OutputSettingsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputSettingsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OutputSettingsPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int OutputSettingsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 31;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
