/****************************************************************************
** Meta object code from reading C++ file 'separatecolorspopup.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/separatecolorspopup.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'separatecolorspopup.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SeparateColorsPopup_t {
    QByteArrayData data[18];
    char stringdata0[251];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SeparateColorsPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SeparateColorsPopup_t qt_meta_stringdata_SeparateColorsPopup = {
    {
QT_MOC_LITERAL(0, 0, 19), // "SeparateColorsPopup"
QT_MOC_LITERAL(1, 20, 8), // "separate"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 18), // "onSeparateFinished"
QT_MOC_LITERAL(4, 49, 19), // "onPreviewBtnPressed"
QT_MOC_LITERAL(5, 69, 31), // "onPreviewFrameFieldValueChanged"
QT_MOC_LITERAL(6, 101, 10), // "isDragging"
QT_MOC_LITERAL(7, 112, 8), // "onChange"
QT_MOC_LITERAL(8, 121, 8), // "onToggle"
QT_MOC_LITERAL(9, 130, 13), // "onColorChange"
QT_MOC_LITERAL(10, 144, 8), // "TPixel32"
QT_MOC_LITERAL(11, 153, 14), // "onSaveSettings"
QT_MOC_LITERAL(12, 168, 14), // "onLoadSettings"
QT_MOC_LITERAL(13, 183, 14), // "doSaveSettings"
QT_MOC_LITERAL(14, 198, 9), // "TFilePath"
QT_MOC_LITERAL(15, 208, 14), // "doLoadSettings"
QT_MOC_LITERAL(16, 223, 7), // "loadAll"
QT_MOC_LITERAL(17, 231, 19) // "onSaveInPathChanged"

    },
    "SeparateColorsPopup\0separate\0\0"
    "onSeparateFinished\0onPreviewBtnPressed\0"
    "onPreviewFrameFieldValueChanged\0"
    "isDragging\0onChange\0onToggle\0onColorChange\0"
    "TPixel32\0onSaveSettings\0onLoadSettings\0"
    "doSaveSettings\0TFilePath\0doLoadSettings\0"
    "loadAll\0onSaveInPathChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SeparateColorsPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x0a /* Public */,
       3,    0,   75,    2, 0x0a /* Public */,
       4,    0,   76,    2, 0x0a /* Public */,
       5,    1,   77,    2, 0x0a /* Public */,
       7,    1,   80,    2, 0x0a /* Public */,
       8,    0,   83,    2, 0x0a /* Public */,
       9,    2,   84,    2, 0x0a /* Public */,
      11,    0,   89,    2, 0x0a /* Public */,
      12,    0,   90,    2, 0x0a /* Public */,
      13,    1,   91,    2, 0x0a /* Public */,
      15,    2,   94,    2, 0x0a /* Public */,
      17,    0,   99,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Bool,    2,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,    2,
    QMetaType::Void, 0x80000000 | 14, QMetaType::Bool,    2,   16,
    QMetaType::Void,

       0        // eod
};

void SeparateColorsPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SeparateColorsPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->separate(); break;
        case 1: _t->onSeparateFinished(); break;
        case 2: _t->onPreviewBtnPressed(); break;
        case 3: _t->onPreviewFrameFieldValueChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->onChange((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->onToggle(); break;
        case 6: _t->onColorChange((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 7: _t->onSaveSettings(); break;
        case 8: _t->onLoadSettings(); break;
        case 9: _t->doSaveSettings((*reinterpret_cast< const TFilePath(*)>(_a[1]))); break;
        case 10: _t->doLoadSettings((*reinterpret_cast< const TFilePath(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 11: _t->onSaveInPathChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SeparateColorsPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<DVGui::Dialog::staticMetaObject>(),
    qt_meta_stringdata_SeparateColorsPopup.data,
    qt_meta_data_SeparateColorsPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SeparateColorsPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SeparateColorsPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SeparateColorsPopup.stringdata0))
        return static_cast<void*>(this);
    return DVGui::Dialog::qt_metacast(_clname);
}

int SeparateColorsPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DVGui::Dialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
