/****************************************************************************
** Meta object code from reading C++ file 'spectrumfield.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonzqt/spectrumfield.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'spectrumfield.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_DVGui__SpectrumBar_t {
    QByteArrayData data[13];
    char stringdata0[159];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__SpectrumBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__SpectrumBar_t qt_meta_stringdata_DVGui__SpectrumBar = {
    {
QT_MOC_LITERAL(0, 0, 18), // "DVGui::SpectrumBar"
QT_MOC_LITERAL(1, 19, 17), // "currentPosChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 10), // "isDragging"
QT_MOC_LITERAL(4, 49, 17), // "currentKeyChanged"
QT_MOC_LITERAL(5, 67, 15), // "currentKeyAdded"
QT_MOC_LITERAL(6, 83, 17), // "currentKeyRemoved"
QT_MOC_LITERAL(7, 101, 13), // "setCurrentPos"
QT_MOC_LITERAL(8, 115, 3), // "pos"
QT_MOC_LITERAL(9, 119, 15), // "setCurrentColor"
QT_MOC_LITERAL(10, 135, 8), // "TPixel32"
QT_MOC_LITERAL(11, 144, 5), // "color"
QT_MOC_LITERAL(12, 150, 8) // "addKeyAt"

    },
    "DVGui::SpectrumBar\0currentPosChanged\0"
    "\0isDragging\0currentKeyChanged\0"
    "currentKeyAdded\0currentKeyRemoved\0"
    "setCurrentPos\0pos\0setCurrentColor\0"
    "TPixel32\0color\0addKeyAt"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__SpectrumBar[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       4,    0,   52,    2, 0x06 /* Public */,
       5,    1,   53,    2, 0x06 /* Public */,
       6,    1,   56,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    2,   59,    2, 0x0a /* Public */,
       9,    1,   64,    2, 0x0a /* Public */,
      12,    1,   67,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,    8,    3,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::Int,    8,

       0        // eod
};

void DVGui::SpectrumBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SpectrumBar *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentPosChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->currentKeyChanged(); break;
        case 2: _t->currentKeyAdded((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->currentKeyRemoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->setCurrentPos((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 5: _t->setCurrentColor((*reinterpret_cast< const TPixel32(*)>(_a[1]))); break;
        case 6: _t->addKeyAt((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SpectrumBar::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumBar::currentPosChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SpectrumBar::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumBar::currentKeyChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SpectrumBar::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumBar::currentKeyAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SpectrumBar::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumBar::currentKeyRemoved)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::SpectrumBar::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__SpectrumBar.data,
    qt_meta_data_DVGui__SpectrumBar,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::SpectrumBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::SpectrumBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__SpectrumBar.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::SpectrumBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void DVGui::SpectrumBar::currentPosChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::SpectrumBar::currentKeyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DVGui::SpectrumBar::currentKeyAdded(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DVGui::SpectrumBar::currentKeyRemoved(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_DVGui__SpectrumField_t {
    QByteArrayData data[12];
    char stringdata0[158];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DVGui__SpectrumField_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DVGui__SpectrumField_t qt_meta_stringdata_DVGui__SpectrumField = {
    {
QT_MOC_LITERAL(0, 0, 20), // "DVGui::SpectrumField"
QT_MOC_LITERAL(1, 21, 15), // "keyColorChanged"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 18), // "keyPositionChanged"
QT_MOC_LITERAL(4, 57, 8), // "keyAdded"
QT_MOC_LITERAL(5, 66, 10), // "keyRemoved"
QT_MOC_LITERAL(6, 77, 19), // "onCurrentPosChanged"
QT_MOC_LITERAL(7, 97, 10), // "isDragging"
QT_MOC_LITERAL(8, 108, 19), // "onCurrentKeyChanged"
QT_MOC_LITERAL(9, 128, 14), // "onColorChanged"
QT_MOC_LITERAL(10, 143, 8), // "TPixel32"
QT_MOC_LITERAL(11, 152, 5) // "color"

    },
    "DVGui::SpectrumField\0keyColorChanged\0"
    "\0keyPositionChanged\0keyAdded\0keyRemoved\0"
    "onCurrentPosChanged\0isDragging\0"
    "onCurrentKeyChanged\0onColorChanged\0"
    "TPixel32\0color"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DVGui__SpectrumField[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       3,    1,   52,    2, 0x06 /* Public */,
       4,    1,   55,    2, 0x06 /* Public */,
       5,    1,   58,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,   61,    2, 0x09 /* Protected */,
       8,    0,   64,    2, 0x09 /* Protected */,
       9,    2,   65,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10, QMetaType::Bool,   11,    7,

       0        // eod
};

void DVGui::SpectrumField::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SpectrumField *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->keyColorChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->keyPositionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->keyAdded((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->keyRemoved((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->onCurrentPosChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->onCurrentKeyChanged(); break;
        case 6: _t->onColorChanged((*reinterpret_cast< const TPixel32(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SpectrumField::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumField::keyColorChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SpectrumField::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumField::keyPositionChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SpectrumField::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumField::keyAdded)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SpectrumField::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SpectrumField::keyRemoved)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DVGui::SpectrumField::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_DVGui__SpectrumField.data,
    qt_meta_data_DVGui__SpectrumField,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DVGui::SpectrumField::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DVGui::SpectrumField::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DVGui__SpectrumField.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int DVGui::SpectrumField::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void DVGui::SpectrumField::keyColorChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void DVGui::SpectrumField::keyPositionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void DVGui::SpectrumField::keyAdded(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void DVGui::SpectrumField::keyRemoved(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
