/****************************************************************************
** Meta object code from reading C++ file 'stylemanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../opentoonz/toonz/sources/include/toonz/stylemanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'stylemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_FavoritesManager_t {
    QByteArrayData data[3];
    char stringdata0[35];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_FavoritesManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_FavoritesManager_t qt_meta_stringdata_FavoritesManager = {
    {
QT_MOC_LITERAL(0, 0, 16), // "FavoritesManager"
QT_MOC_LITERAL(1, 17, 16), // "pinsToTopChanged"
QT_MOC_LITERAL(2, 34, 0) // ""

    },
    "FavoritesManager\0pinsToTopChanged\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_FavoritesManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void FavoritesManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<FavoritesManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->pinsToTopChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (FavoritesManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&FavoritesManager::pinsToTopChanged)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject FavoritesManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_FavoritesManager.data,
    qt_meta_data_FavoritesManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *FavoritesManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FavoritesManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_FavoritesManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FavoritesManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void FavoritesManager::pinsToTopChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_BaseStyleManager_t {
    QByteArrayData data[1];
    char stringdata0[17];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BaseStyleManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BaseStyleManager_t qt_meta_stringdata_BaseStyleManager = {
    {
QT_MOC_LITERAL(0, 0, 16) // "BaseStyleManager"

    },
    "BaseStyleManager"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BaseStyleManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void BaseStyleManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject BaseStyleManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_BaseStyleManager.data,
    qt_meta_data_BaseStyleManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BaseStyleManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BaseStyleManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BaseStyleManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int BaseStyleManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_CustomStyleManager_t {
    QByteArrayData data[3];
    char stringdata0[33];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CustomStyleManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CustomStyleManager_t qt_meta_stringdata_CustomStyleManager = {
    {
QT_MOC_LITERAL(0, 0, 18), // "CustomStyleManager"
QT_MOC_LITERAL(1, 19, 12), // "patternAdded"
QT_MOC_LITERAL(2, 32, 0) // ""

    },
    "CustomStyleManager\0patternAdded\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CustomStyleManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

       0        // eod
};

void CustomStyleManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CustomStyleManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->patternAdded(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CustomStyleManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CustomStyleManager::patternAdded)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject CustomStyleManager::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseStyleManager::staticMetaObject>(),
    qt_meta_stringdata_CustomStyleManager.data,
    qt_meta_data_CustomStyleManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CustomStyleManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomStyleManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CustomStyleManager.stringdata0))
        return static_cast<void*>(this);
    return BaseStyleManager::qt_metacast(_clname);
}

int CustomStyleManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseStyleManager::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void CustomStyleManager::patternAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_TextureStyleManager_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TextureStyleManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TextureStyleManager_t qt_meta_stringdata_TextureStyleManager = {
    {
QT_MOC_LITERAL(0, 0, 19) // "TextureStyleManager"

    },
    "TextureStyleManager"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TextureStyleManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void TextureStyleManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject TextureStyleManager::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseStyleManager::staticMetaObject>(),
    qt_meta_stringdata_TextureStyleManager.data,
    qt_meta_data_TextureStyleManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TextureStyleManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextureStyleManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TextureStyleManager.stringdata0))
        return static_cast<void*>(this);
    return BaseStyleManager::qt_metacast(_clname);
}

int TextureStyleManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseStyleManager::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_MyPaintBrushStyleManager_t {
    QByteArrayData data[1];
    char stringdata0[25];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MyPaintBrushStyleManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MyPaintBrushStyleManager_t qt_meta_stringdata_MyPaintBrushStyleManager = {
    {
QT_MOC_LITERAL(0, 0, 24) // "MyPaintBrushStyleManager"

    },
    "MyPaintBrushStyleManager"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MyPaintBrushStyleManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void MyPaintBrushStyleManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject MyPaintBrushStyleManager::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseStyleManager::staticMetaObject>(),
    qt_meta_stringdata_MyPaintBrushStyleManager.data,
    qt_meta_data_MyPaintBrushStyleManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MyPaintBrushStyleManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MyPaintBrushStyleManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MyPaintBrushStyleManager.stringdata0))
        return static_cast<void*>(this);
    return BaseStyleManager::qt_metacast(_clname);
}

int MyPaintBrushStyleManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseStyleManager::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SpecialStyleManager_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SpecialStyleManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SpecialStyleManager_t qt_meta_stringdata_SpecialStyleManager = {
    {
QT_MOC_LITERAL(0, 0, 19) // "SpecialStyleManager"

    },
    "SpecialStyleManager"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SpecialStyleManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SpecialStyleManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SpecialStyleManager::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseStyleManager::staticMetaObject>(),
    qt_meta_stringdata_SpecialStyleManager.data,
    qt_meta_data_SpecialStyleManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SpecialStyleManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SpecialStyleManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SpecialStyleManager.stringdata0))
        return static_cast<void*>(this);
    return BaseStyleManager::qt_metacast(_clname);
}

int SpecialStyleManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseStyleManager::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
