/****************************************************************************
** Meta object code from reading C++ file 'sceneviewer.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../opentoonz/toonz/sources/toonz/sceneviewer.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sceneviewer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SceneViewer_t {
    QByteArrayData data[52];
    char stringdata0[781];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SceneViewer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SceneViewer_t qt_meta_stringdata_SceneViewer = {
    {
QT_MOC_LITERAL(0, 0, 11), // "SceneViewer"
QT_MOC_LITERAL(1, 12, 13), // "onZoomChanged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 14), // "onFlipHChanged"
QT_MOC_LITERAL(4, 42, 14), // "onFlipVChanged"
QT_MOC_LITERAL(5, 57, 18), // "freezeStateChanged"
QT_MOC_LITERAL(6, 76, 20), // "previewStatusChanged"
QT_MOC_LITERAL(7, 97, 11), // "refreshNavi"
QT_MOC_LITERAL(8, 109, 14), // "previewToggled"
QT_MOC_LITERAL(9, 124, 18), // "aboutToBeDestroyed"
QT_MOC_LITERAL(10, 143, 16), // "resetSceneViewer"
QT_MOC_LITERAL(11, 160, 9), // "resetZoom"
QT_MOC_LITERAL(12, 170, 13), // "resetRotation"
QT_MOC_LITERAL(13, 184, 13), // "resetPosition"
QT_MOC_LITERAL(14, 198, 18), // "setActualPixelSize"
QT_MOC_LITERAL(15, 217, 5), // "flipX"
QT_MOC_LITERAL(16, 223, 5), // "flipY"
QT_MOC_LITERAL(17, 229, 10), // "rotateLeft"
QT_MOC_LITERAL(18, 240, 11), // "rotateRight"
QT_MOC_LITERAL(19, 252, 6), // "zoomIn"
QT_MOC_LITERAL(20, 259, 7), // "zoomOut"
QT_MOC_LITERAL(21, 267, 15), // "onXsheetChanged"
QT_MOC_LITERAL(22, 283, 16), // "onObjectSwitched"
QT_MOC_LITERAL(23, 300, 13), // "onToolChanged"
QT_MOC_LITERAL(24, 314, 14), // "onToolSwitched"
QT_MOC_LITERAL(25, 329, 14), // "onSceneChanged"
QT_MOC_LITERAL(26, 344, 14), // "onLevelChanged"
QT_MOC_LITERAL(27, 359, 15), // "onLevelSwitched"
QT_MOC_LITERAL(28, 375, 15), // "onFrameSwitched"
QT_MOC_LITERAL(29, 391, 22), // "onOnionSkinMaskChanged"
QT_MOC_LITERAL(30, 414, 16), // "setReferenceMode"
QT_MOC_LITERAL(31, 431, 13), // "referenceMode"
QT_MOC_LITERAL(32, 445, 13), // "enablePreview"
QT_MOC_LITERAL(33, 459, 11), // "previewMode"
QT_MOC_LITERAL(34, 471, 6), // "freeze"
QT_MOC_LITERAL(35, 478, 2), // "on"
QT_MOC_LITERAL(36, 481, 15), // "onButtonPressed"
QT_MOC_LITERAL(37, 497, 20), // "FlipConsole::EGadget"
QT_MOC_LITERAL(38, 518, 6), // "button"
QT_MOC_LITERAL(39, 525, 11), // "fitToCamera"
QT_MOC_LITERAL(40, 537, 18), // "fitToCameraOutline"
QT_MOC_LITERAL(41, 556, 12), // "swapCompared"
QT_MOC_LITERAL(42, 569, 22), // "regeneratePreviewFrame"
QT_MOC_LITERAL(43, 592, 17), // "regeneratePreview"
QT_MOC_LITERAL(44, 610, 17), // "doDeleteSubCamera"
QT_MOC_LITERAL(45, 628, 17), // "resetTabletStatus"
QT_MOC_LITERAL(46, 646, 23), // "releaseBusyOnTabletMove"
QT_MOC_LITERAL(47, 670, 27), // "onContextAboutToBeDestroyed"
QT_MOC_LITERAL(48, 698, 25), // "onNewStopMotionImageReady"
QT_MOC_LITERAL(49, 724, 27), // "onStopMotionLiveViewStopped"
QT_MOC_LITERAL(50, 752, 19), // "onPreferenceChanged"
QT_MOC_LITERAL(51, 772, 8) // "prefName"

    },
    "SceneViewer\0onZoomChanged\0\0onFlipHChanged\0"
    "onFlipVChanged\0freezeStateChanged\0"
    "previewStatusChanged\0refreshNavi\0"
    "previewToggled\0aboutToBeDestroyed\0"
    "resetSceneViewer\0resetZoom\0resetRotation\0"
    "resetPosition\0setActualPixelSize\0flipX\0"
    "flipY\0rotateLeft\0rotateRight\0zoomIn\0"
    "zoomOut\0onXsheetChanged\0onObjectSwitched\0"
    "onToolChanged\0onToolSwitched\0"
    "onSceneChanged\0onLevelChanged\0"
    "onLevelSwitched\0onFrameSwitched\0"
    "onOnionSkinMaskChanged\0setReferenceMode\0"
    "referenceMode\0enablePreview\0previewMode\0"
    "freeze\0on\0onButtonPressed\0"
    "FlipConsole::EGadget\0button\0fitToCamera\0"
    "fitToCameraOutline\0swapCompared\0"
    "regeneratePreviewFrame\0regeneratePreview\0"
    "doDeleteSubCamera\0resetTabletStatus\0"
    "releaseBusyOnTabletMove\0"
    "onContextAboutToBeDestroyed\0"
    "onNewStopMotionImageReady\0"
    "onStopMotionLiveViewStopped\0"
    "onPreferenceChanged\0prefName"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SceneViewer[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      44,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  234,    2, 0x06 /* Public */,
       3,    1,  235,    2, 0x06 /* Public */,
       4,    1,  238,    2, 0x06 /* Public */,
       5,    1,  241,    2, 0x06 /* Public */,
       6,    0,  244,    2, 0x06 /* Public */,
       7,    0,  245,    2, 0x06 /* Public */,
       8,    0,  246,    2, 0x06 /* Public */,
       9,    0,  247,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,  248,    2, 0x0a /* Public */,
      11,    0,  249,    2, 0x0a /* Public */,
      12,    0,  250,    2, 0x0a /* Public */,
      13,    0,  251,    2, 0x0a /* Public */,
      14,    0,  252,    2, 0x0a /* Public */,
      15,    0,  253,    2, 0x0a /* Public */,
      16,    0,  254,    2, 0x0a /* Public */,
      17,    0,  255,    2, 0x0a /* Public */,
      18,    0,  256,    2, 0x0a /* Public */,
      19,    0,  257,    2, 0x0a /* Public */,
      20,    0,  258,    2, 0x0a /* Public */,
      21,    0,  259,    2, 0x0a /* Public */,
      22,    0,  260,    2, 0x0a /* Public */,
      23,    0,  261,    2, 0x0a /* Public */,
      24,    0,  262,    2, 0x0a /* Public */,
      25,    0,  263,    2, 0x0a /* Public */,
      26,    0,  264,    2, 0x0a /* Public */,
      27,    0,  265,    2, 0x0a /* Public */,
      28,    0,  266,    2, 0x0a /* Public */,
      29,    0,  267,    2, 0x0a /* Public */,
      30,    1,  268,    2, 0x0a /* Public */,
      32,    1,  271,    2, 0x0a /* Public */,
      34,    1,  274,    2, 0x0a /* Public */,
      36,    1,  277,    2, 0x0a /* Public */,
      39,    0,  280,    2, 0x0a /* Public */,
      40,    0,  281,    2, 0x0a /* Public */,
      41,    0,  282,    2, 0x0a /* Public */,
      42,    0,  283,    2, 0x0a /* Public */,
      43,    0,  284,    2, 0x0a /* Public */,
      44,    0,  285,    2, 0x0a /* Public */,
      45,    0,  286,    2, 0x0a /* Public */,
      46,    0,  287,    2, 0x0a /* Public */,
      47,    0,  288,    2, 0x0a /* Public */,
      48,    0,  289,    2, 0x0a /* Public */,
      49,    0,  290,    2, 0x0a /* Public */,
      50,    1,  291,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   31,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void, 0x80000000 | 37,   38,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   51,

       0        // eod
};

void SceneViewer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SceneViewer *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->onZoomChanged(); break;
        case 1: _t->onFlipHChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->onFlipVChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->freezeStateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->previewStatusChanged(); break;
        case 5: _t->refreshNavi(); break;
        case 6: _t->previewToggled(); break;
        case 7: _t->aboutToBeDestroyed(); break;
        case 8: _t->resetSceneViewer(); break;
        case 9: _t->resetZoom(); break;
        case 10: _t->resetRotation(); break;
        case 11: _t->resetPosition(); break;
        case 12: _t->setActualPixelSize(); break;
        case 13: _t->flipX(); break;
        case 14: _t->flipY(); break;
        case 15: _t->rotateLeft(); break;
        case 16: _t->rotateRight(); break;
        case 17: _t->zoomIn(); break;
        case 18: _t->zoomOut(); break;
        case 19: _t->onXsheetChanged(); break;
        case 20: _t->onObjectSwitched(); break;
        case 21: _t->onToolChanged(); break;
        case 22: _t->onToolSwitched(); break;
        case 23: _t->onSceneChanged(); break;
        case 24: _t->onLevelChanged(); break;
        case 25: _t->onLevelSwitched(); break;
        case 26: _t->onFrameSwitched(); break;
        case 27: _t->onOnionSkinMaskChanged(); break;
        case 28: _t->setReferenceMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->enablePreview((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 30: _t->freeze((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 31: _t->onButtonPressed((*reinterpret_cast< FlipConsole::EGadget(*)>(_a[1]))); break;
        case 32: _t->fitToCamera(); break;
        case 33: _t->fitToCameraOutline(); break;
        case 34: _t->swapCompared(); break;
        case 35: _t->regeneratePreviewFrame(); break;
        case 36: _t->regeneratePreview(); break;
        case 37: _t->doDeleteSubCamera(); break;
        case 38: _t->resetTabletStatus(); break;
        case 39: _t->releaseBusyOnTabletMove(); break;
        case 40: _t->onContextAboutToBeDestroyed(); break;
        case 41: _t->onNewStopMotionImageReady(); break;
        case 42: _t->onStopMotionLiveViewStopped(); break;
        case 43: _t->onPreferenceChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SceneViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::onZoomChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SceneViewer::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::onFlipHChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SceneViewer::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::onFlipVChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (SceneViewer::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::freezeStateChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (SceneViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::previewStatusChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (SceneViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::refreshNavi)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (SceneViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::previewToggled)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (SceneViewer::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SceneViewer::aboutToBeDestroyed)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SceneViewer::staticMetaObject = { {
    QMetaObject::SuperData::link<TToolViewer::staticMetaObject>(),
    qt_meta_stringdata_SceneViewer.data,
    qt_meta_data_SceneViewer,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SceneViewer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SceneViewer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SceneViewer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Previewer::Listener"))
        return static_cast< Previewer::Listener*>(this);
    return TToolViewer::qt_metacast(_clname);
}

int SceneViewer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TToolViewer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 44)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 44;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 44)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 44;
    }
    return _id;
}

// SIGNAL 0
void SceneViewer::onZoomChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SceneViewer::onFlipHChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void SceneViewer::onFlipVChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void SceneViewer::freezeStateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void SceneViewer::previewStatusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void SceneViewer::refreshNavi()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void SceneViewer::previewToggled()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void SceneViewer::aboutToBeDestroyed()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
